searchData={"items":[{"type":"module","title":"Ecto","doc":"Ecto is split into 4 main components:\n\n  * `Ecto.Repo` - repositories are wrappers around the data store.\n    Via the repository, we can create, update, destroy and query\n    existing entries. A repository needs an adapter and credentials\n    to communicate to the database\n\n  * `Ecto.Schema` - schemas are used to map external data into Elixir\n    structs. We often use them to map database tables to Elixir data but\n    they have many other use cases\n\n  * `Ecto.Query` - written in Elixir syntax, queries are used to retrieve\n    information from a given repository. Ecto queries are secure and composable\n\n  * `Ecto.Changeset` - changesets provide a way to track and validate changes\n    before they are applied to the data\n\nIn summary:\n\n  * `Ecto.Repo` - **where** the data is\n  * `Ecto.Schema` - **what** the data is\n  * `Ecto.Query` - **how to read** the data\n  * `Ecto.Changeset` - **how to change** the data\n\nBesides the four components above, most developers use Ecto to interact\nwith SQL databases, such as PostgreSQL and MySQL via the\n[`ecto_sql`](https://hexdocs.pm/ecto_sql) project. `ecto_sql` provides many\nconveniences for working with SQL databases as well as the ability to version\nhow your database changes through time via\n[database migrations](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.html#module-migrations).\n\nIf you want to quickly check a sample application using Ecto, please check\nthe [getting started guide](https://hexdocs.pm/ecto/getting-started.html) and\nthe accompanying sample application. [Ecto's README](https://github.com/elixir-ecto/ecto)\nalso links to other resources.\n\nIn the following sections, we will provide an overview of those components and\nhow they interact with each other. Feel free to access their respective module\ndocumentation for more specific examples, options and configuration.","ref":"Ecto.html"},{"type":"module","title":"Repositories - Ecto","doc":"`Ecto.Repo` is a wrapper around the database. We can define a\nrepository as follows:\n\n    defmodule Repo do\n      use Ecto.Repo,\n        otp_app: :my_app,\n        adapter: Ecto.Adapters.Postgres\n    end\n\nWhere the configuration for the Repo must be in your application\nenvironment, usually defined in your `config/config.exs`:\n\n    config :my_app, Repo,\n      database: \"ecto_simple\",\n      username: \"postgres\",\n      password: \"postgres\",\n      hostname: \"localhost\",\n      # OR use a URL to connect instead\n      url: \"postgres://postgres:postgres@localhost/ecto_simple\"\n\nEach repository in Ecto defines a `start_link/0` function that needs to be invoked\nbefore using the repository. In general, this function is not called directly,\nbut is used as part of your application supervision tree.\n\nIf your application was generated with a supervisor (by passing `--sup` to `mix new`)\nyou will have a `lib/my_app/application.ex` file containing the application start\ncallback that defines and starts your supervisor.  You just need to edit the `start/2`\nfunction to start the repo as a supervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        MyApp.Repo,\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","ref":"Ecto.html#module-repositories"},{"type":"module","title":"Schema - Ecto","doc":"Schemas allow developers to define the shape of their data.\nLet's see an example:\n\n    defmodule Weather do\n      use Ecto.Schema\n\n      # weather is the DB table\n      schema \"weather\" do\n        field :city,    :string\n        field :temp_lo, :integer\n        field :temp_hi, :integer\n        field :prcp,    :float, default: 0.0\n      end\n    end\n\nBy defining a schema, Ecto automatically defines a struct with\nthe schema fields:\n\n    iex> weather = %Weather{temp_lo: 30}\n    iex> weather.temp_lo\n    30\n\nThe schema also allows us to interact with a repository:\n\n    iex> weather = %Weather{temp_lo: 0, temp_hi: 23}\n    iex> Repo.insert!(weather)\n    %Weather{...}\n\nAfter persisting `weather` to the database, it will return a new copy of\n`%Weather{}` with the primary key (the `id`) set. We can use this value\nto read a struct back from the repository:\n\n    # Get the struct back\n    iex> weather = Repo.get Weather, 1\n    %Weather{id: 1, ...}\n\n    # Delete it\n    iex> Repo.delete!(weather)\n    %Weather{...}\n\n> NOTE: by using `Ecto.Schema`, an `:id` field with type `:id` (:id means :integer) is\n> generated by default, which is the primary key of the schema. If you want\n> to use a different primary key, you can declare custom `@primary_key`\n> before the `schema/2` call. Consult the `Ecto.Schema` documentation\n> for more information.\n\nNotice how the storage (repository) and the data are decoupled. This provides\ntwo main benefits:\n\n  * By having structs as data, we guarantee they are light-weight,\n    serializable structures. In many languages, the data is often represented\n    by large, complex objects, with entwined state transactions, which makes\n    serialization, maintenance and understanding hard;\n\n  * You do not need to define schemas in order to interact with repositories,\n    operations like `all`, `insert_all` and so on allow developers to directly\n    access and modify the data, keeping the database at your fingertips when\n    necessary;","ref":"Ecto.html#module-schema"},{"type":"module","title":"Changesets - Ecto","doc":"Although in the example above we have directly inserted and deleted the\nstruct in the repository, operations on top of schemas are done through\nchangesets so Ecto can efficiently track changes.\n\nChangesets allow developers to filter, cast, and validate changes before\nwe apply them to the data. Imagine the given schema:\n\n    defmodule User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :name\n        field :email\n        field :age, :integer\n      end\n\n      def changeset(user, params \\\\ %{}) do\n        user\n        |> cast(params, [:name, :email, :age])\n        |> validate_required([:name, :email])\n        |> validate_format(:email, ~r/@/)\n        |> validate_inclusion(:age, 18..100)\n      end\n    end\n\nThe `changeset/2` function first invokes `Ecto.Changeset.cast/4` with\nthe struct, the parameters and a list of allowed fields; this returns a changeset.\nThe parameters is a map with binary keys and values that will be cast based\non the type defined by the schema.\n\nAny parameter that was not explicitly listed in the fields list will be ignored.\n\nAfter casting, the changeset is given to many `Ecto.Changeset.validate_*`\nfunctions that validate only the **changed fields**. In other words:\nif a field was not given as a parameter, it won't be validated at all.\nFor example, if the params map contain only the \"name\" and \"email\" keys,\nthe \"age\" validation won't run.\n\nOnce a changeset is built, it can be given to functions like `insert` and\n`update` in the repository that will return an `:ok` or `:error` tuple:\n\n    case Repo.update(changeset) do\n      {:ok, user} ->\n        # user updated\n      {:error, changeset} ->\n        # an error occurred\n    end\n\nThe benefit of having explicit changesets is that we can easily provide\ndifferent changesets for different use cases. For example, one\ncould easily provide specific changesets for registering and updating\nusers:\n\n    def registration_changeset(user, params) do\n      # Changeset on create\n    end\n\n    def update_changeset(user, params) do\n      # Changeset on update\n    end\n\nChangesets are also capable of transforming database constraints,\nlike unique indexes and foreign key checks, into errors. Allowing\ndevelopers to keep their database consistent while still providing\nproper feedback to end users. Check `Ecto.Changeset.unique_constraint/3`\nfor some examples as well as the other `_constraint` functions.","ref":"Ecto.html#module-changesets"},{"type":"module","title":"Query - Ecto","doc":"Last but not least, Ecto allows you to write queries in Elixir and send\nthem to the repository, which translates them to the underlying database.\nLet's see an example:\n\n    import Ecto.Query, only: [from: 2]\n\n    query = from u in User,\n              where: u.age > 18 or is_nil(u.email),\n              select: u\n\n    # Returns %User{} structs matching the query\n    Repo.all(query)\n\nIn the example above we relied on our schema but queries can also be\nmade directly against a table by giving the table name as a string. In\nsuch cases, the data to be fetched must be explicitly outlined:\n\n    query = from u in \"users\",\n              where: u.age > 18 or is_nil(u.email),\n              select: %{name: u.name, age: u.age}\n\n    # Returns maps as defined in select\n    Repo.all(query)\n\nQueries are defined and extended with the `from` macro. The supported\nkeywords are:\n\n  * `:distinct`\n  * `:where`\n  * `:order_by`\n  * `:offset`\n  * `:limit`\n  * `:lock`\n  * `:group_by`\n  * `:having`\n  * `:join`\n  * `:select`\n  * `:preload`\n\nExamples and detailed documentation for each of those are available\nin the `Ecto.Query` module. Functions supported in queries are listed\nin `Ecto.Query.API`.\n\nWhen writing a query, you are inside Ecto's query syntax. In order to\naccess params values or invoke Elixir functions, you need to use the `^`\noperator, which is overloaded by Ecto:\n\n    def min_age(min) do\n      from u in User, where: u.age > ^min\n    end\n\nBesides `Repo.all/1` which returns all entries, repositories also\nprovide `Repo.one/1` which returns one entry or nil, `Repo.one!/1`\nwhich returns one entry or raises, `Repo.get/2` which fetches\nentries for a particular ID and more.\n\nFinally, if you need an escape hatch, Ecto provides fragments\n(see `Ecto.Query.API.fragment/1`) to inject SQL (and non-SQL)\nfragments into queries. Also, most adapters provide direct\nAPIs for queries, like `Ecto.Adapters.SQL.query/4`, allowing\ndevelopers to completely bypass Ecto queries.","ref":"Ecto.html#module-query"},{"type":"module","title":"Other topics - Ecto","doc":"#","ref":"Ecto.html#module-other-topics"},{"type":"module","title":"Associations - Ecto","doc":"Ecto supports defining associations on schemas:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        has_many :comments, Comment\n      end\n    end\n\n    defmodule Comment do\n      use Ecto.Schema\n\n      schema \"comments\" do\n        field :title, :string\n        belongs_to :post, Post\n      end\n    end\n\nWhen an association is defined, Ecto also defines a field in the schema\nwith the association name. By default, associations are not loaded into\nthis field:\n\n    iex> post = Repo.get(Post, 42)\n    iex> post.comments\n    #Ecto.Association.NotLoaded \n\nHowever, developers can use the preload functionality in queries to\nautomatically pre-populate the field:\n\n    Repo.all from p in Post, preload: [:comments]\n\nPreloading can also be done with a pre-defined join value:\n\n    Repo.all from p in Post,\n              join: c in assoc(p, :comments),\n              where: c.votes > p.votes,\n              preload: [comments: c]\n\nFinally, for the simple cases, preloading can also be done after\na collection was fetched:\n\n    posts = Repo.all(Post) |> Repo.preload(:comments)\n\nThe `Ecto` module also provides conveniences for working\nwith associations. For example, `Ecto.assoc/3` returns a query\nwith all associated data to a given struct:\n\n    import Ecto\n\n    # Get all comments for the given post\n    Repo.all assoc(post, :comments)\n\n    # Or build a query on top of the associated comments\n    query = from c in assoc(post, :comments), where: not is_nil(c.title)\n    Repo.all(query)\n\nAnother function in `Ecto` is `build_assoc/3`, which allows\nsomeone to build an associated struct with the proper fields:\n\n    Repo.transaction fn ->\n      post = Repo.insert!(%Post{title: \"Hello\", body: \"world\"})\n\n      # Build a comment from post\n      comment = Ecto.build_assoc(post, :comments, body: \"Excellent!\")\n\n      Repo.insert!(comment)\n    end\n\nIn the example above, `Ecto.build_assoc/3` is equivalent to:\n\n    %Comment{post_id: post.id, body: \"Excellent!\"}\n\nYou can find more information about defining associations and each\nrespective association module in `Ecto.Schema` docs.\n\n> NOTE: Ecto does not lazy load associations. While lazily loading\n> associations may sound convenient at first, in the long run it\n> becomes a source of confusion and performance issues.\n\n#","ref":"Ecto.html#module-associations"},{"type":"module","title":"Embeds - Ecto","doc":"Ecto also supports embeds. While associations keep parent and child\nentries in different tables, embeds stores the child along side the\nparent.\n\nDatabases like MongoDB have native support for embeds. Databases\nlike PostgreSQL uses a mixture of JSONB (`embeds_one/3`) and ARRAY\ncolumns to provide this functionality.\n\nCheck `Ecto.Schema.embeds_one/3` and `Ecto.Schema.embeds_many/3`\nfor more information.\n\n#","ref":"Ecto.html#module-embeds"},{"type":"module","title":"Mix tasks and generators - Ecto","doc":"Ecto provides many tasks to help your workflow as well as code generators.\nYou can find all available tasks by typing `mix help` inside a project\nwith Ecto listed as a dependency.\n\nEcto generators will automatically open the generated files if you have\n`ECTO_EDITOR` set in your environment variable.\n\n##","ref":"Ecto.html#module-mix-tasks-and-generators"},{"type":"module","title":"Repo resolution - Ecto","doc":"Ecto requires developers to specify the key `:ecto_repos` in their\napplication configuration before using tasks like `ecto.create` and\n`ecto.migrate`. For example:\n\n    config :my_app, :ecto_repos, [MyApp.Repo]\n\n    config :my_app, MyApp.Repo,\n      database: \"ecto_simple\",\n      username: \"postgres\",\n      password: \"postgres\",\n      hostname: \"localhost\"","ref":"Ecto.html#module-repo-resolution"},{"type":"function","title":"Ecto.assoc/3","doc":"Builds a query for the association in the given struct or structs.","ref":"Ecto.html#assoc/3"},{"type":"function","title":"Examples - Ecto.assoc/3","doc":"In the example below, we get all comments associated to the given\npost:\n\n    post = Repo.get Post, 1\n    Repo.all Ecto.assoc(post, :comments)\n\n`assoc/3` can also receive a list of posts, as long as the posts are\nnot empty:\n\n    posts = Repo.all from p in Post, where: is_nil(p.published_at)\n    Repo.all Ecto.assoc(posts, :comments)\n\nThis function can also be used to dynamically load through associations\nby giving it a list. For example, to get all authors for all comments for\nthe given posts, do:\n\n    posts = Repo.all from p in Post, where: is_nil(p.published_at)\n    Repo.all Ecto.assoc(posts, [:comments, :author])","ref":"Ecto.html#assoc/3-examples"},{"type":"function","title":"Options - Ecto.assoc/3","doc":"* `:prefix` - the prefix to fetch assocs from. By default, queries\n    will use the same prefix as the first struct in the given collection.\n    This option allows the prefix to be changed.","ref":"Ecto.html#assoc/3-options"},{"type":"function","title":"Ecto.assoc_loaded?/1","doc":"Checks if an association is loaded.","ref":"Ecto.html#assoc_loaded?/1"},{"type":"function","title":"Examples - Ecto.assoc_loaded?/1","doc":"iex> post = Repo.get(Post, 1)\n    iex> Ecto.assoc_loaded?(post.comments)\n    false\n    iex> post = post |> Repo.preload(:comments)\n    iex> Ecto.assoc_loaded?(post.comments)\n    true","ref":"Ecto.html#assoc_loaded?/1-examples"},{"type":"function","title":"Ecto.build_assoc/3","doc":"Builds a struct from the given `assoc` in `struct`.","ref":"Ecto.html#build_assoc/3"},{"type":"function","title":"Examples - Ecto.build_assoc/3","doc":"If the relationship is a `has_one` or `has_many` and\nthe primary key is set in the parent struct, the key will\nautomatically be set in the built association:\n\n    iex> post = Repo.get(Post, 13)\n    %Post{id: 13}\n    iex> build_assoc(post, :comments)\n    %Comment{id: nil, post_id: 13}\n\nNote though it doesn't happen with `belongs_to` cases, as the\nkey is often the primary key and such is usually generated\ndynamically:\n\n    iex> comment = Repo.get(Comment, 13)\n    %Comment{id: 13, post_id: 25}\n    iex> build_assoc(comment, :post)\n    %Post{id: nil}\n\nYou can also pass the attributes, which can be a map or\na keyword list, to set the struct's fields except the\nassociation key.\n\n    iex> build_assoc(post, :comments, text: \"cool\")\n    %Comment{id: nil, post_id: 13, text: \"cool\"}\n\n    iex> build_assoc(post, :comments, %{text: \"cool\"})\n    %Comment{id: nil, post_id: 13, text: \"cool\"}\n\n    iex> build_assoc(post, :comments, post_id: 1)\n    %Comment{id: nil, post_id: 13}\n\nThe given attributes are expected to be structured data.\nIf you want to build an association with external data,\nsuch as a request parameters, you can use `Ecto.Changeset.cast/3`\nafter `build_assoc/3`:\n\n    parent\n    |> Ecto.build_assoc(:child)\n    |> Ecto.Changeset.cast(params, [:field1, :field2])","ref":"Ecto.html#build_assoc/3-examples"},{"type":"function","title":"Ecto.embedded_dump/2","doc":"Dumps the given struct defined by an embedded schema.\n\nThis converts the given embedded schema to a map to be serialized\nwith the given format. For example:\n\n    iex> Ecto.embedded_dump(%Post{}, :json)\n    %{title: \"hello\"}","ref":"Ecto.html#embedded_dump/2"},{"type":"function","title":"Ecto.embedded_load/3","doc":"Loads previously dumped `data` in the given `format` into a schema.\n\nThe first argument can be an embedded schema module, or a map (of types) and\ndetermines the return value: a struct or a map, respectively.\n\nThe second argument `data` specifies fields and values that are to be loaded.\nIt can be a map, a keyword list, or a `{fields, values}` tuple. Fields can be\natoms or strings.\n\nThe third argument `format` is the format the data has been dumped as. For\nexample, databases may dump embedded to `:json`, this function allows such\ndumped data to be put back into the schemas.\n\nFields that are not present in the schema (or `types` map) are ignored.\nIf any of the values has invalid type, an error is raised.\n\nNote that if you want to load data into a non-embedded schema that was\ndirectly persisted into a given repository, then use `c:Ecto.Repo.load/2`.","ref":"Ecto.html#embedded_load/3"},{"type":"function","title":"Examples - Ecto.embedded_load/3","doc":"iex> result = Ecto.Adapters.SQL.query!(MyRepo, \"SELECT users.settings FROM users\", [])\n    iex> Enum.map(result.rows, fn [settings] -> Ecto.embedded_load(Setting, Jason.decode!(settings), :json) end)\n    [%Setting{...}, ...]","ref":"Ecto.html#embedded_load/3-examples"},{"type":"function","title":"Ecto.get_meta/2","doc":"Gets the metadata from the given struct.","ref":"Ecto.html#get_meta/2"},{"type":"function","title":"Ecto.primary_key/1","doc":"Returns the schema primary keys as a keyword list.","ref":"Ecto.html#primary_key/1"},{"type":"function","title":"Ecto.primary_key!/1","doc":"Returns the schema primary keys as a keyword list.\n\nRaises `Ecto.NoPrimaryKeyFieldError` if the schema has no\nprimary key field.","ref":"Ecto.html#primary_key!/1"},{"type":"function","title":"Ecto.put_meta/2","doc":"Returns a new struct with updated metadata.\n\nIt is possible to set:\n\n  * `:source` - changes the struct query source\n  * `:prefix` - changes the struct query prefix\n  * `:context` - changes the struct meta context\n  * `:state` - changes the struct state\n\nPlease refer to the `Ecto.Schema.Metadata` module for more information.","ref":"Ecto.html#put_meta/2"},{"type":"function","title":"Ecto.reset_fields/2","doc":"Resets fields in a struct to their default values.","ref":"Ecto.html#reset_fields/2"},{"type":"function","title":"Examples - Ecto.reset_fields/2","doc":"iex> post = post |> Repo.preload(:author)\n    %Post{title: \"hello world\", author: %Author{}}\n    iex> Ecto.reset_fields(post, [:title, :author])\n    %Post{\n      title: \"default title\",\n      author: #Ecto.Association.NotLoaded \n    }","ref":"Ecto.html#reset_fields/2-examples"},{"type":"module","title":"Ecto.Changeset","doc":"Changesets allow filtering, casting, validation and\ndefinition of constraints when manipulating structs.\n\nThere is an example of working with changesets in the introductory\ndocumentation in the `Ecto` module. The functions `cast/4` and\n`change/2` are the usual entry points for creating changesets.\nThe first one is used to cast and validate external parameters,\nsuch as parameters sent through a form, API, command line, etc.\nThe second one is used to change data directly from your application.\n\nThe remaining functions in this module, such as validations,\nconstraints, association handling, are about manipulating\nchangesets. Let's discuss some of this extra functionality.","ref":"Ecto.Changeset.html"},{"type":"module","title":"External vs internal data - Ecto.Changeset","doc":"Changesets allow working with both kinds of data:\n\n  * internal to the application - for example programmatically generated,\n    or coming from other subsystems. This use case is primarily covered\n    by the `change/2` and `put_change/3` functions.\n\n  * external to the application - for example data provided by the user in\n    a form that needs to be type-converted and properly validated. This\n    use case is primarily covered by the `cast/4` function.","ref":"Ecto.Changeset.html#module-external-vs-internal-data"},{"type":"module","title":"Validations and constraints - Ecto.Changeset","doc":"Ecto changesets provide both validations and constraints which\nare ultimately turned into errors in case something goes wrong.\n\nThe difference between them is that most validations can be\nexecuted without a need to interact with the database and, therefore,\nare always executed before attempting to insert or update the entry\nin the database. Validations run immediately when a validation function\nis called on the data that is contained in the changeset at that time.\n\nSome validations may happen against the database but\nthey are inherently unsafe. Those validations start with a `unsafe_`\nprefix, such as `unsafe_validate_unique/4`.\n\nOn the other hand, constraints rely on the database and are always safe.\nAs a consequence, validations are always checked before constraints.\nConstraints won't even be checked in case validations failed.\n\nLet's see an example:\n\n    defmodule User do\n      use Ecto.Schema\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :name\n        field :email\n        field :age, :integer\n      end\n\n      def changeset(user, params \\\\ %{}) do\n        user\n        |> cast(params, [:name, :email, :age])\n        |> validate_required([:name, :email])\n        |> validate_format(:email, ~r/@/)\n        |> validate_inclusion(:age, 18..100)\n        |> unique_constraint(:email)\n      end\n    end\n\nIn the `changeset/2` function above, we define three validations.\nThey check that `name` and `email` fields are present in the\nchangeset, the e-mail is of the specified format, and the age is\nbetween 18 and 100 - as well as a unique constraint in the email\nfield.\n\nLet's suppose the e-mail is given but the age is invalid. The\nchangeset would have the following errors:\n\n    changeset = User.changeset(%User{}, %{age: 0, email: \"mary@example.com\"})\n    {:error, changeset} = Repo.insert(changeset)\n    changeset.errors #=> [age: {\"is invalid\", []}, name: {\"can't be blank\", []}]\n\nIn this case, we haven't checked the unique constraint in the\ne-mail field because the data did not validate. Let's fix the\nage and the name, and assume that the e-mail already exists in the\ndatabase:\n\n    changeset = User.changeset(%User{}, %{age: 42, name: \"Mary\", email: \"mary@example.com\"})\n    {:error, changeset} = Repo.insert(changeset)\n    changeset.errors #=> [email: {\"has already been taken\", []}]\n\nValidations and constraints define an explicit boundary when the check\nhappens. By moving constraints to the database, we also provide a safe,\ncorrect and data-race free means of checking the user input.\n\n#","ref":"Ecto.Changeset.html#module-validations-and-constraints"},{"type":"module","title":"Deferred constraints - Ecto.Changeset","doc":"Some databases support deferred constraints, i.e., constraints which are\nchecked at the end of the transaction rather than at the end of each statement.\n\nChangesets do not support this type of constraints. When working with deferred\nconstraints, a violation while invoking `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2` won't\nreturn `{:error, changeset}`, but rather raise an error at the end of the\ntransaction.","ref":"Ecto.Changeset.html#module-deferred-constraints"},{"type":"module","title":"Empty values - Ecto.Changeset","doc":"Many times, the data given on cast needs to be further pruned, specially\nregarding empty values. For example, if you are gathering data to be\ncast from the command line or through an HTML form or any other text-based\nformat, it is likely those means cannot express nil values. For\nthose reasons, changesets include the concept of empty values.\n\nWhen applying changes using `cast/4`, an empty value will be automatically\nconverted to the field's default value. If the field is an array type, any\nempty value inside the array will be removed.\n\nEmpty values are stored as a list in the changeset's `:empty_values` field.\nThe list contains elements of type `t:empty_value/0`. Those are either values,\nwhich will be considered empty if they\nmatch, or a function that must return a boolean if the value is empty or\nnot. By default, Ecto uses `Ecto.Changeset.empty_values/0` which will mark\na field as empty if it is a string made only of whitespace characters.\nYou can also pass the `:empty_values` option to `cast/4` in case you want\nto change how a particular `cast/4` work.","ref":"Ecto.Changeset.html#module-empty-values"},{"type":"module","title":"Associations, embeds, and on replace - Ecto.Changeset","doc":"Using changesets you can work with associations as well as with embedded\nstructs. There are two primary APIs:\n\n  * `cast_assoc/3` and `cast_embed/3` - those functions are used when\n    working with external data. In particular, they allow you to change\n    associations and embeds alongside the parent struct, all at once.\n\n  * `put_assoc/4` and `put_embed/4` - it allows you to replace the\n    association or embed as a whole. This can be used to move associated\n    data from one entry to another, to completely remove or replace\n    existing entries.\n\nThese functions are opinionated on how it works with associations.\nIf you need different behaviour or explicit control over the associated\ndata, you can skip this functionality and use `Ecto.Multi` to encode how\nseveral database operations will happen on several schemas and changesets\nat once.\n\nYou can learn more about working with associations in our documentation,\nincluding cheatsheets and practical examples. Check out:\n\n  * The docs for `cast_assoc/3` and `put_assoc/3`\n  * The [associations cheatsheet](associations.html)\n  * The [Constraints and Upserts guide](constraints-and-upserts.html)\n  * The [Polymorphic associations with many to many guide](polymorphic-associations-with-many-to-many.html)\n\n#","ref":"Ecto.Changeset.html#module-associations-embeds-and-on-replace"},{"type":"module","title":"The `:on_replace` option - Ecto.Changeset","doc":"When using any of those APIs, you may run into situations where Ecto sees\ndata is being replaced. For example, imagine a Post has many Comments where\nthe comments have IDs 1, 2 and 3. If you call `cast_assoc/3` passing only\nthe IDs 1 and 2, Ecto will consider 3 is being \"replaced\" and it will raise\nby default. Such behaviour can be changed when defining the relation by\nsetting `:on_replace` option when defining your association/embed according\nto the values below:\n\n  * `:raise` (default) - do not allow removing association or embedded\n    data via parent changesets\n  * `:mark_as_invalid` - if attempting to remove the association or\n    embedded data via parent changeset - an error will be added to the parent\n    changeset, and it will be marked as invalid\n  * `:nilify` - sets owner reference column to `nil` (available only for\n    associations). Use this on a `belongs_to` column to allow the association\n    to be cleared out so that it can be set to a new value. Will set `action`\n    on associated changesets to `:replace`\n  * `:update` - updates the association, available only for `has_one`, `belongs_to`\n    and `embeds_one`. This option will update all the fields given to the changeset\n    including the id for the association\n  * `:delete` - removes the association or related data from the database.\n    This option has to be used carefully (see below). Will set `action` on associated\n    changesets to `:replace`\n  * `:delete_if_exists` - like `:delete` except that it ignores any stale entry\n    error. For instance, if you set `on_replace: :delete` but the replaced\n    resource was already deleted by a separate request, it will raise a\n    `Ecto.StaleEntryError`. `:delete_if_exists` makes it so it will only delete\n    if the entry still exists\n\nThe `:delete` and `:delete_if_exists` options must be used carefully as they allow\nusers to delete any associated data by simply setting it to `nil` or an empty list.\nIf you need deletion, it is often preferred to add a separate boolean virtual field\nin the schema and manually mark the changeset for deletion if the `:delete` field is\nset in the params, as in the example below. Note that we don't call `cast/4` in this\ncase because we don't want to prevent deletion if a change is invalid (changes are\nirrelevant if the entity needs to be deleted).\n\n    defmodule Comment do\n      use Ecto.Schema\n      import Ecto.Changeset\n\n      schema \"comments\" do\n        field :body, :string\n        field :delete, :boolean, virtual: true\n      end\n\n      def changeset(comment, %{\"delete\" => \"true\"}) do\n        %{Ecto.Changeset.change(comment, delete: true) | action: :delete}\n      end\n\n      def changeset(comment, params) do\n        cast(comment, params, [:body])\n      end\n    end","ref":"Ecto.Changeset.html#module-the-on_replace-option"},{"type":"module","title":"Schemaless changesets - Ecto.Changeset","doc":"In the changeset examples so far, we have always used changesets to validate\nand cast data contained in a struct defined by an Ecto schema, such as the `%User{}`\nstruct defined by the `User` module.\n\nHowever, changesets can also be used with \"regular\" structs too by passing a tuple\nwith the data and its types:\n\n    user = %User{}\n    types = %{name: :string, email: :string, age: :integer}\n    params = %{name: \"Callum\", email: \"callum@example.com\", age: 27}\n    changeset =\n      {user, types}\n      |> Ecto.Changeset.cast(params, Map.keys(types))\n      |> Ecto.Changeset.validate_required(...)\n      |> Ecto.Changeset.validate_length(...)\n\nwhere the user struct refers to the definition in the following module:\n\n    defmodule User do\n      defstruct [:name, :email, :age]\n    end\n\nChangesets can also be used with data in a plain map, by following the same API:\n\n    data  = %{}\n    types = %{name: :string, email: :string, age: :integer}\n    params = %{name: \"Callum\", email: \"callum@example.com\", age: 27}\n    changeset =\n      {data, types}\n      |> Ecto.Changeset.cast(params, Map.keys(types))\n      |> Ecto.Changeset.validate_required(...)\n      |> Ecto.Changeset.validate_length(...)\n\nBesides the basic types which are mentioned above, such as `:boolean` and `:string`,\nparameterized types can also be used in schemaless changesets. They implement\nthe `Ecto.ParameterizedType` behaviour and we can create the necessary type info by\ncalling the `init/2` function.\n\nFor example, to use `Ecto.Enum` in a schemaless changeset:\n\n    types = %{\n      name: :string,\n      role: Ecto.ParameterizedType.init(Ecto.Enum, values: [:reader, :editor, :admin])\n    }\n\n    data  = %{}\n    params = %{name: \"Callum\", role: \"reader\"}\n\n    changeset =\n      {data, types}\n      |> Ecto.Changeset.cast(params, Map.keys(types))\n      |> Ecto.Changeset.validate_required(...)\n      |> Ecto.Changeset.validate_length(...)\n\nSchemaless changesets make Ecto extremely useful to cast, validate and prune data even\nif it is not meant to be persisted to the database.\n\n#","ref":"Ecto.Changeset.html#module-schemaless-changesets"},{"type":"module","title":"Changeset actions - Ecto.Changeset","doc":"Changesets have an action field which is usually set by `Ecto.Repo`\nwhenever one of the operations such as `insert` or `update` is called:\n\n    changeset = User.changeset(%User{}, %{age: 42, email: \"mary@example.com\"})\n    {:error, changeset} = Repo.insert(changeset)\n    changeset.action\n    #=> :insert\n\nThis means that when working with changesets that are not meant to be\npersisted to the database, such as schemaless changesets, you may need\nto explicitly set the action to one specific value. Frameworks such as\nPhoenix [use the action value to define how HTML forms should\nact](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#form/1-a-note-on-errors).\n\nInstead of setting the action manually, you may use `apply_action/2` that\nemulates operations such as `c:Ecto.Repo.insert`. `apply_action/2` will return\n`{:ok, changes}` if the changeset is valid or `{:error, changeset}`, with\nthe given `action` set in the changeset in case of errors.","ref":"Ecto.Changeset.html#module-changeset-actions"},{"type":"module","title":"The Ecto.Changeset struct - Ecto.Changeset","doc":"The public fields are:\n\n  * `valid?`       - Stores if the changeset is valid\n  * `data`         - The changeset source data, for example, a struct\n  * `params`       - The parameters as given on changeset creation\n  * `changes`      - The `changes` from parameters that were approved in casting\n  * `errors`       - All errors from validations\n  * `required`     - All required fields as a list of atoms\n  * `action`       - The action to be performed with the changeset\n  * `types`        - Cache of the data's field types\n  * `empty_values` - A list of values to be considered empty\n  * `repo`         - The repository applying the changeset (only set after a Repo function is called)\n  * `repo_opts`    - A keyword list of options given to the underlying repository operation\n\nThe following fields are private and must not be accessed directly.\n\n  * `validations`\n  * `constraints`\n  * `filters`\n  * `prepare`\n\n#","ref":"Ecto.Changeset.html#module-the-ecto-changeset-struct"},{"type":"module","title":"Redacting fields in inspect - Ecto.Changeset","doc":"To hide a field's value from the inspect protocol of `Ecto.Changeset`, mark\nthe field as `redact: true` in the schema, and it will display with the\nvalue `**redacted**`.","ref":"Ecto.Changeset.html#module-redacting-fields-in-inspect"},{"type":"function","title":"Ecto.Changeset.add_error/4","doc":"Adds an error to the changeset.\n\nAn additional keyword list `keys` can be passed to provide additional\ncontextual information for the error. This is useful when using\n`traverse_errors/2` and when translating errors with `Gettext`","ref":"Ecto.Changeset.html#add_error/4"},{"type":"function","title":"Examples - Ecto.Changeset.add_error/4","doc":"iex> changeset = change(%Post{}, %{title: \"\"})\n    iex> changeset = add_error(changeset, :title, \"empty\")\n    iex> changeset.errors\n    [title: {\"empty\", []}]\n    iex> changeset.valid?\n    false\n\n    iex> changeset = change(%Post{}, %{title: \"\"})\n    iex> changeset = add_error(changeset, :title, \"empty\", additional: \"info\")\n    iex> changeset.errors\n    [title: {\"empty\", [additional: \"info\"]}]\n    iex> changeset.valid?\n    false\n\n    iex> changeset = change(%Post{}, %{tags: [\"ecto\", \"elixir\", \"x\"]})\n    iex> changeset = add_error(changeset, :tags, \"tag '%{val}' is too short\", val: \"x\")\n    iex> changeset.errors\n    [tags: {\"tag '%{val}' is too short\", [val: \"x\"]}]\n    iex> changeset.valid?\n    false","ref":"Ecto.Changeset.html#add_error/4-examples"},{"type":"function","title":"Ecto.Changeset.apply_action/2","doc":"Applies the changeset action only if the changes are valid.\n\nIf the changes are valid, all changes are applied to the changeset data.\nIf the changes are invalid, no changes are applied, and an error tuple\nis returned with the changeset containing the action that was attempted\nto be applied.\n\nThe action may be any atom.","ref":"Ecto.Changeset.html#apply_action/2"},{"type":"function","title":"Examples - Ecto.Changeset.apply_action/2","doc":"iex> {:ok, data} = apply_action(changeset, :update)\n\n    iex> {:ok, data} = apply_action(changeset, :my_action)\n\n    iex> {:error, changeset} = apply_action(changeset, :update)\n    %Ecto.Changeset{action: :update}","ref":"Ecto.Changeset.html#apply_action/2-examples"},{"type":"function","title":"Ecto.Changeset.apply_action!/2","doc":"Applies the changeset action if the changes are valid or raises an error.","ref":"Ecto.Changeset.html#apply_action!/2"},{"type":"function","title":"Examples - Ecto.Changeset.apply_action!/2","doc":"iex> changeset = change(%Post{author: \"bar\"}, %{title: \"foo\"})\n    iex> apply_action!(changeset, :update)\n    %Post{author: \"bar\", title: \"foo\"}\n\n    iex> changeset = change(%Post{author: \"bar\"}, %{title: :bad})\n    iex> apply_action!(changeset, :update)\n    ** (Ecto.InvalidChangesetError) could not perform update because changeset is invalid.\n\nSee `apply_action/2` for more information.","ref":"Ecto.Changeset.html#apply_action!/2-examples"},{"type":"function","title":"Ecto.Changeset.apply_changes/1","doc":"Applies the changeset changes to the changeset data.\n\nThis operation will return the underlying data with changes\nregardless if the changeset is valid or not. See `apply_action/2`\nfor a similar function that ensures the changeset is valid.","ref":"Ecto.Changeset.html#apply_changes/1"},{"type":"function","title":"Examples - Ecto.Changeset.apply_changes/1","doc":"iex> changeset = change(%Post{author: \"bar\"}, %{title: \"foo\"})\n    iex> apply_changes(changeset)\n    %Post{author: \"bar\", title: \"foo\"}","ref":"Ecto.Changeset.html#apply_changes/1-examples"},{"type":"function","title":"Ecto.Changeset.assoc_constraint/3","doc":"Checks the associated field exists.\n\nThis is similar to `foreign_key_constraint/3` except that the\nfield is inferred from the association definition. This is useful\nto guarantee that a child will only be created if the parent exists\nin the database too. Therefore, it only applies to `belongs_to`\nassociations.\n\nAs the name says, a constraint is required in the database for\nthis function to work. Such constraint is often added as a\nreference to the child table:\n\n    create table(:comments) do\n      add :post_id, references(:posts)\n    end\n\nNow, when inserting a comment, it is possible to forbid any\ncomment to be added if the associated post does not exist:\n\n    comment\n    |> Ecto.Changeset.cast(params, [:post_id])\n    |> Ecto.Changeset.assoc_constraint(:post)\n    |> Repo.insert","ref":"Ecto.Changeset.html#assoc_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.assoc_constraint/3","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"does not exist\"\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.","ref":"Ecto.Changeset.html#assoc_constraint/3-options"},{"type":"function","title":"Ecto.Changeset.cast/4","doc":"Applies the given `params` as changes on the `data` according to\nthe set of `permitted` keys. Returns a changeset.\n\n`data` may be either a changeset, a schema struct or a `{data, types}`\ntuple. The second argument is a map of `params` that are cast according\nto the type information from `data`. `params` is a map with string keys\nor a map with atom keys, containing potentially invalid data. Mixed keys\nare not allowed.\n\nDuring casting, all `permitted` parameters whose values match the specified\ntype information will have their key name converted to an atom and stored\ntogether with the value as a change in the `:changes` field of the changeset.\nIf the cast value matches the current value for the field, it will not be\nincluded in `:changes` unless the `force_changes: true` option is\nprovided. All parameters that are not explicitly permitted are ignored.\n\nIf casting of all fields is successful, the changeset is returned as valid.\n\nNote that `cast/4` validates the types in the `params`, but not in the given\n`data`.","ref":"Ecto.Changeset.html#cast/4"},{"type":"function","title":"Options - Ecto.Changeset.cast/4","doc":"* `:empty_values` - a list of values to be considered as empty when casting.\n    Empty values are always replaced by the default value of the respective field.\n    If the field is an array type, any empty value inside of the array will be removed.\n    To set this option while keeping the current default, use `empty_values/0` and add\n    your additional empty values\n\n  * `:force_changes` - a boolean indicating whether to include values that don't alter\n    the current data in `:changes`. See `force_change/3` for more information, Defaults\n    to `false`\n\n  * `:message` - a function of arity 2 that is used to create the error message when\n    casting fails. It is called for every field that cannot be casted and receives the\n    field name as the first argument and the error metadata as the second argument. It\n    must return a string or `nil`. If a string is returned it will be used as the error\n    message. If `nil` is returned the default error message will be used. The field type\n    is given under the `:type` key in the metadata","ref":"Ecto.Changeset.html#cast/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.cast/4","doc":"iex> changeset = cast(post, params, [:title])\n    iex> if changeset.valid? do\n    ...>   Repo.update!(changeset)\n    ...> end\n\nPassing a changeset as the first argument:\n\n    iex> changeset = cast(post, %{title: \"Hello\"}, [:title])\n    iex> new_changeset = cast(changeset, %{title: \"Foo\", body: \"World\"}, [:body])\n    iex> new_changeset.params\n    %{\"title\" => \"Hello\", \"body\" => \"World\"}\n\nOr creating a changeset from a simple map with types:\n\n    iex> data = %{title: \"hello\"}\n    iex> types = %{title: :string}\n    iex> changeset = cast({data, types}, %{title: \"world\"}, [:title])\n    iex> apply_changes(changeset)\n    %{title: \"world\"}\n\nYou can use empty values (and even cast multiple times) to change\nwhat is considered an empty value:\n\n    # Using default\n    iex> params = %{title: \"\", topics: []}\n    iex> changeset = cast(post, params, [:title, :topics])\n    iex> changeset.params\n    %{topics: []}\n\n    # Changing default\n    iex> params = %{title: \"\", topics: []}\n    iex> changeset = cast(post, params, [:topics], empty_values: [[], nil])\n    iex> changeset.params\n    %{title: \"\"}\n\n    # Augmenting default\n    iex> params = %{title: \"\", topics: []}\n    iex> changeset =\n    ...>   cast(post, params, [:topics], empty_values: [[], nil] ++ Ecto.Changeset.empty_values())\n    iex> changeset.params\n    %{}\n\nYou can define a custom error message function.\n\n    # Using field name\n    iex> params = %{title: 1, body: 2}\n    iex> custom_errors = [title: \"must be a string\"]\n    iex> msg_func = fn field, _meta -> custom_errors[field] end\n    iex> changeset = cast(post, params, [:title, :body], message: msg_func)\n    iex> changeset.errors\n    [\n      title: {\"must be a string\", [type: :string, validation: :cast]},\n      body: {\"is_invalid\", [type: :string, validation: :cast]}\n    ]\n\n    # Using field type\n    iex> params = %{title: 1, body: 2}\n    iex> custom_errors = [string: \"must be a string\"]\n    iex> msg_func = fn _field, meta ->\n    ...    type = meta[:type]\n    ...    custom_errors[type]\n    ...  end\n    iex> changeset = cast(post, params, [:title, :body], message: msg_func)\n    iex> changeset.errors\n    [\n      title: {\"must be a string\", [type: :string, validation: :cast]},\n      body: {\"must be a string\", [type: :string, validation: :cast]}\n    ]","ref":"Ecto.Changeset.html#cast/4-examples"},{"type":"function","title":"Composing casts - Ecto.Changeset.cast/4","doc":"`cast/4` also accepts a changeset as its first argument. In such cases, all\nthe effects caused by the call to `cast/4` (additional errors and changes)\nare simply added to the ones already present in the argument changeset.\nParameters are merged (**not deep-merged**) and the ones passed to `cast/4`\ntake precedence over the ones already in the changeset.","ref":"Ecto.Changeset.html#cast/4-composing-casts"},{"type":"function","title":"Ecto.Changeset.cast_assoc/3","doc":"Casts the given association with the changeset parameters.\n\nThis function should be used when working with the entire association at\nonce (and not a single element of a many-style association) and receiving\ndata external to the application.\n\n`cast_assoc/3` matches the records extracted from the database\nand compares it with the parameters received from an external source.\nTherefore, it is expected that the data in the changeset has explicitly\npreloaded the association being cast and that all of the IDs exist and\nare unique.\n\nFor example, imagine a user has many addresses relationship where\npost data is sent as follows\n\n    %{\"name\" => \"john doe\", \"addresses\" => [\n      %{\"street\" => \"somewhere\", \"country\" => \"brazil\", \"id\" => 1},\n      %{\"street\" => \"elsewhere\", \"country\" => \"poland\"},\n    ]}\n\nand then\n\n    User\n    |> Repo.get!(id)\n    |> Repo.preload(:addresses) # Only required when updating data\n    |> Ecto.Changeset.cast(params, [])\n    |> Ecto.Changeset.cast_assoc(:addresses, with: &MyApp.Address.changeset/2)\n\nThe parameters for the given association will be retrieved\nfrom `changeset.params`. Those parameters are expected to be\na map with attributes, similar to the ones passed to `cast/4`.\nOnce parameters are retrieved, `cast_assoc/3` will match those\nparameters with the associations already in the changeset data.\n\nOnce `cast_assoc/3` is called, Ecto will compare each parameter\nwith the user's already preloaded addresses and act as follows:\n\n  * If the parameter does not contain an ID, the parameter data\n    will be passed to `MyApp.Address.changeset/2` with a new struct\n    and become an insert operation. We only consider the ID as not\n    given if there is no \"id\" key or if its value is strictly `nil`\n\n  * If the parameter contains an ID and there is no associated child\n    with such ID, the parameter data will be passed to\n    `MyApp.Address.changeset/2` with a new struct and become an insert\n    operation\n\n  * If the parameter contains an ID and there is an associated child\n    with such ID, the parameter data will be passed to\n    `MyApp.Address.changeset/2` with the existing struct and become an\n    update operation\n\n  * If there is an associated child with an ID and its ID is not given\n    as parameter, the `:on_replace` callback for that association will\n    be invoked (see the \"On replace\" section on the module documentation)\n\nIf two or more addresses have the same IDs, Ecto will consider that an\nerror and add an error to the changeset saying that there are duplicate\nentries.\n\nEvery time the `MyApp.Address.changeset/2` function is invoked, it must\nreturn a changeset. This changeset will always be included under `changes`\nof the parent changeset, even if there are no changes. This is done for\nreflection purposes, allowing developers to introspect validations and\nother metadata from the association. Once the parent changeset is given\nto an `Ecto.Repo` function, all entries will be inserted/updated/deleted\nwithin the same transaction.\n\nAs you see above, this function is opinionated on how it works. If you\nneed different behaviour or if you need explicit control over the associated\ndata, you can either use `put_assoc/4` or use `Ecto.Multi` to encode how\nseveral database operations will happen on several schemas and changesets\nat once.","ref":"Ecto.Changeset.html#cast_assoc/3"},{"type":"function","title":"Custom actions - Ecto.Changeset.cast_assoc/3","doc":"Developers are allowed to explicitly set the `:action` field of a\nchangeset to instruct Ecto how to act in certain situations. Let's suppose\nthat, if one of the associations has only empty fields, you want to ignore\nthe entry altogether instead of showing an error. The changeset function could\nbe written like this:\n\n    def changeset(struct, params) do\n      struct\n      |> cast(params, [:title, :body])\n      |> validate_required([:title, :body])\n      |> case do\n        %{valid?: false, changes: changes} = changeset when changes == %{} ->\n          # If the changeset is invalid and has no changes, it is\n          # because all required fields are missing, so we ignore it.\n          %{changeset | action: :ignore}\n        changeset ->\n          changeset\n      end\n    end\n\nYou can also set it to delete if you want data to be deleted based on the\nreceived parameters (such as a checkbox or any other indicator).","ref":"Ecto.Changeset.html#cast_assoc/3-custom-actions"},{"type":"function","title":"Partial changes for many-style associations - Ecto.Changeset.cast_assoc/3","doc":"By preloading an association using a custom query you can confine the behavior\nof `cast_assoc/3`. This opens up the possibility to work on a subset of the data,\ninstead of all associations in the database.\n\nTaking the initial example of users having addresses imagine those addresses\nare set up to belong to a country. If you want to allow users to bulk edit all\naddresses that belong to a single country, you can do so by changing the preload\nquery:\n\n    query = from MyApp.Address, where: [country: ^edit_country]\n\n    User\n    |> Repo.get!(id)\n    |> Repo.preload(addresses: query)\n    |> Ecto.Changeset.cast(params, [])\n    |> Ecto.Changeset.cast_assoc(:addresses)\n\nThis will allow you to cast and update only the association for the given country.\nThe important point for partial changes is that any addresses, which were not\npreloaded won't be changed.","ref":"Ecto.Changeset.html#cast_assoc/3-partial-changes-for-many-style-associations"},{"type":"function","title":"Sorting and deleting from -many collections - Ecto.Changeset.cast_assoc/3","doc":"In earlier examples, we passed a -many style association as a list:\n\n    %{\"name\" => \"john doe\", \"addresses\" => [\n      %{\"street\" => \"somewhere\", \"country\" => \"brazil\", \"id\" => 1},\n      %{\"street\" => \"elsewhere\", \"country\" => \"poland\"},\n    ]}\n\nHowever, it is also common to pass the addresses as a map, where each\nkey is an integer representing its position:\n\n    %{\"name\" => \"john doe\", \"addresses\" => %{\n      0 => %{\"street\" => \"somewhere\", \"country\" => \"brazil\", \"id\" => 1},\n      1 => %{\"street\" => \"elsewhere\", \"country\" => \"poland\"}\n    }}\n\nUsing indexes becomes specially useful with two supporting options:\n`:sort_param` and `:drop_param`. These options tell the indexes should\nbe reordered or deleted from the data. For example, if you did:\n\n    cast_embed(changeset, :addresses,\n      sort_param: :addresses_sort,\n      drop_param: :addresses_drop)\n\nYou can now submit this:\n\n    %{\"name\" => \"john doe\", \"addresses\" => %{...}, \"addresses_drop\" => [0]}\n\nAnd now the entry with index 0 will be dropped from the params before casting.\nNote this requires setting the relevant `:on_replace` option on your\nassociations/embeds definition.\n\nSimilar, for sorting, you could do:\n\n    %{\"name\" => \"john doe\", \"addresses\" => %{...}, \"addresses_sort\" => [1, 0]}\n\nAnd that will internally sort the elements so 1 comes before 0. Note that\nany index not present in `\"addresses_sort\"` will come _before_ any of the\nsorted indexes. If an index is not found, an empty entry is added in its\nplace.\n\nFor embeds, this guarantees the embeds will be rewritten in the given order.\nHowever, for associations, this is not enough. You will have to add a\n`field :position, :integer` to the schema and add a with function of arity 3\nto add the position to your children changeset. For example, you could implement:\n\n    defp child_changeset(child, _changes, position) do\n      child\n      |> change(position: position)\n    end\n\nAnd by passing it to `:with`, it will be called with the final position of the\nitem:\n\n    changeset\n    |> cast_assoc(:children, sort_param: ..., with: &child_changeset/3)\n\nThese parameters can be powerful in certain UIs as it allows you to decouple\nthe sorting and replacement of the data from its representation.","ref":"Ecto.Changeset.html#cast_assoc/3-sorting-and-deleting-from-many-collections"},{"type":"function","title":"More resources - Ecto.Changeset.cast_assoc/3","doc":"You can learn more about working with associations in our documentation,\nincluding cheatsheets and practical examples. Check out:\n\n  * The docs for `put_assoc/3`\n  * The [associations cheatsheet](associations.html)\n  * The [Constraints and Upserts guide](constraints-and-upserts.html)\n  * The [Polymorphic associations with many to many guide](polymorphic-associations-with-many-to-many.html)","ref":"Ecto.Changeset.html#cast_assoc/3-more-resources"},{"type":"function","title":"Options - Ecto.Changeset.cast_assoc/3","doc":"* `:required` - if the association is a required field. For associations of cardinality\n    one, a non-nil value satisfies this validation. For associations with many entries,\n    a non-empty list is satisfactory.\n\n  * `:required_message` - the message on failure, defaults to \"can't be blank\"\n\n  * `:invalid_message` - the message on failure, defaults to \"is invalid\"\n\n  * `:force_update_on_change` - force the parent record to be updated in the\n    repository if there is a change, defaults to `true`\n\n  * `:with` - the function to build the changeset from params. Defaults to the\n    `changeset/2` function of the associated module. It can be an anonymous\n    function that expects two arguments: the associated struct to be cast and its\n    parameters. It must return a changeset. For associations with cardinality `:many`,\n    functions with arity 3 are accepted, and the third argument will be the position\n    of the associated element in the list, or `nil`, if the association is being replaced.\n\n  * `:drop_param` - the parameter name which keeps a list of indexes to drop\n    from the relation parameters\n\n  * `:sort_param` - the parameter name which keeps a list of indexes to sort\n    from the relation parameters. Unknown indexes are considered to be new\n    entries. Non-listed indexes will come before any sorted ones. See\n    `cast_assoc/3` for more information","ref":"Ecto.Changeset.html#cast_assoc/3-options"},{"type":"function","title":"Ecto.Changeset.cast_embed/3","doc":"Casts the given embed with the changeset parameters.\n\nThe parameters for the given embed will be retrieved\nfrom `changeset.params`. Those parameters are expected to be\na map with attributes, similar to the ones passed to `cast/4`.\nOnce parameters are retrieved, `cast_embed/3` will match those\nparameters with the embeds already in the changeset record.\nSee `cast_assoc/3` for an example of working with casts and\nassociations which would also apply for embeds.\n\nThe changeset must have been previously `cast` using\n`cast/4` before this function is invoked.","ref":"Ecto.Changeset.html#cast_embed/3"},{"type":"function","title":"Options - Ecto.Changeset.cast_embed/3","doc":"* `:required` - if the embed is a required field. For embeds of cardinality\n    one, a non-nil value satisfies this validation. For embeds with many entries,\n    a non-empty list is satisfactory.\n\n  * `:required_message` - the message on failure, defaults to \"can't be blank\"\n\n  * `:invalid_message` - the message on failure, defaults to \"is invalid\"\n\n  * `:force_update_on_change` - force the parent record to be updated in the\n    repository if there is a change, defaults to `true`\n\n  * `:with` - the function to build the changeset from params. Defaults to the\n    `changeset/2` function of the associated module. It must be an anonymous\n    function that expects two arguments: the embedded struct to be cast and its\n    parameters. It must return a changeset. For embeds with cardinality `:many`,\n    functions with arity 3 are accepted, and the third argument will be the position\n    of the associated element in the list, or `nil`, if the embed is being replaced.\n\n  * `:drop_param` - the parameter name which keeps a list of indexes to drop\n    from the relation parameters\n\n  * `:sort_param` - the parameter name which keeps a list of indexes to sort\n    from the relation parameters. Unknown indexes are considered to be new\n    entries. Non-listed indexes will come before any sorted ones. See\n    `cast_assoc/3` for more information","ref":"Ecto.Changeset.html#cast_embed/3-options"},{"type":"function","title":"Ecto.Changeset.change/2","doc":"Wraps the given data in a changeset or adds changes to a changeset.\n\n`changes` is a map or keyword where the key is an atom representing a\nfield, association or embed and the value is a term. Note the `value` is\ndirectly stored in the changeset with no validation whatsoever. For this\nreason, this function is meant for working with data internal to the\napplication.\n\nWhen changing embeds and associations, see `put_assoc/4` for a complete\nreference on the accepted values.\n\nThis function is useful for:\n\n  * wrapping a struct inside a changeset\n  * directly changing a struct without performing castings nor validations\n  * directly bulk-adding changes to a changeset\n\nChanged attributes will only be added if the change does not have the\nsame value as the field in the data.\n\nWhen a changeset is passed as the first argument, the changes passed as the\nsecond argument are merged over the changes already in the changeset if they\ndiffer from the values in the struct.\n\nWhen a `{data, types}` is passed as the first argument, a changeset is\ncreated with the given data and types and marked as valid.\n\nSee `cast/4` if you'd prefer to cast and validate external parameters.","ref":"Ecto.Changeset.html#change/2"},{"type":"function","title":"Examples - Ecto.Changeset.change/2","doc":"iex> changeset = change(%Post{})\n    %Ecto.Changeset{...}\n    iex> changeset.valid?\n    true\n    iex> changeset.changes\n    %{}\n\n    iex> changeset = change(%Post{author: \"bar\"}, title: \"title\")\n    iex> changeset.changes\n    %{title: \"title\"}\n\n    iex> changeset = change(%Post{title: \"title\"}, title: \"title\")\n    iex> changeset.changes\n    %{}\n\n    iex> changeset = change(changeset, %{title: \"new title\", body: \"body\"})\n    iex> changeset.changes.title\n    \"new title\"\n    iex> changeset.changes.body\n    \"body\"","ref":"Ecto.Changeset.html#change/2-examples"},{"type":"function","title":"Ecto.Changeset.changed?/3","doc":"Returns true if a field was changed in a changeset.\n\nThis function can check associations and embeds, but doesn't support the `:to`\nand `:from` options for such fields.","ref":"Ecto.Changeset.html#changed?/3"},{"type":"function","title":"Options - Ecto.Changeset.changed?/3","doc":"* `:to` - Check if the field was changed to a specific value\n  * `:from` - Check if the field was changed from a specific value","ref":"Ecto.Changeset.html#changed?/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.changed?/3","doc":"iex> post = %Post{title: \"Foo\", body: \"Old\"}\n    iex> changeset = change(post, %{title: \"New title\", body: \"Old\"})\n\n    iex> changed?(changeset, :body)\n    false\n\n    iex> changed?(changeset, :title)\n    true\n\n    iex> changed?(changeset, :title, to: \"NEW TITLE\")\n    false","ref":"Ecto.Changeset.html#changed?/3-examples"},{"type":"function","title":"Ecto.Changeset.check_constraint/3","doc":"Checks for a check constraint in the given field.\n\nThe check constraint works by relying on the database to check\nif the check constraint has been violated or not and, if so,\nEcto converts it into a changeset error.\n\nIn order to use the check constraint, the first step is\nto define the check constraint in a migration:\n\n    create constraint(\"users\", :age_must_be_positive, check: \"age > 0\")\n\nNow that a constraint exists, when modifying users, we could\nannotate the changeset with a check constraint so Ecto knows\nhow to convert it into an error message:\n\n    cast(user, params, [:age])\n    |> check_constraint(:age, name: :age_must_be_positive)\n\nNow, when invoking `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2`,\nif the age is not positive, the underlying operation will fail\nbut Ecto will convert the database exception into a changeset error\nand return an `{:error, changeset}` tuple. Note that the error will\noccur only after hitting the database, so it will not be visible\nuntil all other validations pass. If the constraint fails inside a\ntransaction, the transaction will be marked as aborted.","ref":"Ecto.Changeset.html#check_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.check_constraint/3","doc":"* `:message` - the message in case the constraint check fails.\n    Defaults to \"is invalid\"\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.","ref":"Ecto.Changeset.html#check_constraint/3-options"},{"type":"function","title":"Ecto.Changeset.constraints/1","doc":"Returns all constraints in a changeset.\n\nA constraint is a map with the following fields:\n\n  * `:type` - the type of the constraint that will be checked in the database,\n    such as `:check`, `:unique`, etc\n  * `:constraint` - the database constraint name as a string or `Regex`. The constraint at\n    the database level will be checked against this according to `:match` type\n  * `:match` - the type of match Ecto will perform on a violated constraint\n    against the `:constraint` value. It is `:exact`, `:suffix` or `:prefix`\n  * `:field` - the field a violated constraint will apply the error to\n  * `:error_message` - the error message in case of violated constraints\n  * `:error_type` - the type of error that identifies the error message","ref":"Ecto.Changeset.html#constraints/1"},{"type":"function","title":"Ecto.Changeset.delete_change/2","doc":"Deletes a change with the given key.","ref":"Ecto.Changeset.html#delete_change/2"},{"type":"function","title":"Examples - Ecto.Changeset.delete_change/2","doc":"iex> changeset = change(%Post{}, %{title: \"foo\"})\n    iex> changeset = delete_change(changeset, :title)\n    iex> get_change(changeset, :title)\n    nil","ref":"Ecto.Changeset.html#delete_change/2-examples"},{"type":"function","title":"Ecto.Changeset.empty_values/0","doc":"Returns the default empty values used by `Ecto.Changeset`.\n\nBy default, Ecto marks a field as empty if it is a string made\nonly of whitespace characters. If you want to provide your\nadditional empty values on top of the default, such as an empty\nlist, you can write:\n\n    @empty_values [[]] ++ Ecto.Changeset.empty_values()\n\nThen, you can pass `empty_values: @empty_values` on `cast/3`.\n\nSee also the [*Empty values* section](#module-empty-values) for more\ninformation.","ref":"Ecto.Changeset.html#empty_values/0"},{"type":"function","title":"Ecto.Changeset.exclusion_constraint/3","doc":"Checks for an exclusion constraint in the given field.\n\nThe exclusion constraint works by relying on the database to check\nif the exclusion constraint has been violated or not and, if so,\nEcto converts it into a changeset error.","ref":"Ecto.Changeset.html#exclusion_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.exclusion_constraint/3","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"violates an exclusion constraint\"\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.","ref":"Ecto.Changeset.html#exclusion_constraint/3-options"},{"type":"function","title":"Ecto.Changeset.fetch_change/2","doc":"Fetches a change from the given changeset.\n\nThis function only looks at the `:changes` field of the given `changeset` and\nreturns `{:ok, value}` if the change is present or `:error` if it's not.","ref":"Ecto.Changeset.html#fetch_change/2"},{"type":"function","title":"Examples - Ecto.Changeset.fetch_change/2","doc":"iex> changeset = change(%Post{body: \"foo\"}, %{title: \"bar\"})\n    iex> fetch_change(changeset, :title)\n    {:ok, \"bar\"}\n    iex> fetch_change(changeset, :body)\n    :error","ref":"Ecto.Changeset.html#fetch_change/2-examples"},{"type":"function","title":"Ecto.Changeset.fetch_change!/2","doc":"Same as `fetch_change/2` but returns the value or raises if the given key was not found.","ref":"Ecto.Changeset.html#fetch_change!/2"},{"type":"function","title":"Examples - Ecto.Changeset.fetch_change!/2","doc":"iex> changeset = change(%Post{body: \"foo\"}, %{title: \"bar\"})\n    iex> fetch_change!(changeset, :title)\n    \"bar\"\n    iex> fetch_change!(changeset, :body)\n    ** (KeyError) key :body not found in: %{title: \"bar\"}","ref":"Ecto.Changeset.html#fetch_change!/2-examples"},{"type":"function","title":"Ecto.Changeset.fetch_field/2","doc":"Fetches the given field from changes or from the data.\n\nWhile `fetch_change/2` only looks at the current `changes`\nto retrieve a value, this function looks at the changes and\nthen falls back on the data, finally returning `:error` if\nno value is available.\n\nFor relations, these functions will return the changeset\noriginal data with changes applied. To retrieve raw changesets,\nplease use `fetch_change/2`.","ref":"Ecto.Changeset.html#fetch_field/2"},{"type":"function","title":"Examples - Ecto.Changeset.fetch_field/2","doc":"iex> post = %Post{title: \"Foo\", body: \"Bar baz bong\"}\n    iex> changeset = change(post, %{title: \"New title\"})\n    iex> fetch_field(changeset, :title)\n    {:changes, \"New title\"}\n    iex> fetch_field(changeset, :body)\n    {:data, \"Bar baz bong\"}\n    iex> fetch_field(changeset, :not_a_field)\n    :error","ref":"Ecto.Changeset.html#fetch_field/2-examples"},{"type":"function","title":"Ecto.Changeset.fetch_field!/2","doc":"Same as `fetch_field/2` but returns the value or raises if the given key was not found.","ref":"Ecto.Changeset.html#fetch_field!/2"},{"type":"function","title":"Examples - Ecto.Changeset.fetch_field!/2","doc":"iex> post = %Post{title: \"Foo\", body: \"Bar baz bong\"}\n    iex> changeset = change(post, %{title: \"New title\"})\n    iex> fetch_field!(changeset, :title)\n    \"New title\"\n    iex> fetch_field!(changeset, :other)\n    ** (KeyError) key :other not found in: %Post{...}","ref":"Ecto.Changeset.html#fetch_field!/2-examples"},{"type":"function","title":"Ecto.Changeset.field_missing?/2","doc":"Determines whether a field is missing in a changeset.\n\nThe field passed into this function will have its presence evaluated\naccording to the same rules as `validate_required/3`.\n\nThis is useful when performing complex validations that are not possible with\n`validate_required/3`. For example, evaluating whether at least one field\nfrom a list is present or evaluating that exactly one field from a list is\npresent.","ref":"Ecto.Changeset.html#field_missing?/2"},{"type":"function","title":"Examples - Ecto.Changeset.field_missing?/2","doc":"iex> changeset = cast(%Post{}, %{color: \"Red\"}, [:color])\n    iex> missing_fields = Enum.filter([:title, :body], &field_missing?(changeset, &1))\n    iex> changeset =\n    ...>   case missing_fields do\n    ...>     [_, _] -> add_error(changeset, :title, \"at least one of `:title` or `:body` must be present\")\n    ...>     _ -> changeset\n    ...>   end\n    ...> changeset.errors\n    [title: {\"at least one of `:title` or `:body` must be present\", []}]","ref":"Ecto.Changeset.html#field_missing?/2-examples"},{"type":"function","title":"Ecto.Changeset.force_change/3","doc":"Forces a change on the given `key` with `value`.\n\nIf the change is already present, it is overridden with\nthe new value. If the value is later modified via\n`put_change/3` and `update_change/3`, revering back to\nits original value, the change will be reverted unless\n`force_change/3` is called once again.","ref":"Ecto.Changeset.html#force_change/3"},{"type":"function","title":"Examples - Ecto.Changeset.force_change/3","doc":"iex> changeset = change(%Post{author: \"bar\"}, %{title: \"foo\"})\n    iex> changeset = force_change(changeset, :title, \"bar\")\n    iex> changeset.changes\n    %{title: \"bar\"}\n\n    iex> changeset = force_change(changeset, :author, \"bar\")\n    iex> changeset.changes\n    %{title: \"bar\", author: \"bar\"}","ref":"Ecto.Changeset.html#force_change/3-examples"},{"type":"function","title":"Ecto.Changeset.foreign_key_constraint/3","doc":"Checks for foreign key constraint in the given field.\n\nThe foreign key constraint works by relying on the database to\ncheck if the associated data exists or not. This is useful to\nguarantee that a child will only be created if the parent exists\nin the database too.\n\nIn order to use the foreign key constraint the first step is\nto define the foreign key in a migration. This is often done\nwith references. For example, imagine you are creating a\ncomments table that belongs to posts. One would have:\n\n    create table(:comments) do\n      add :post_id, references(:posts)\n    end\n\nBy default, Ecto will generate a foreign key constraint with\nname \"comments_post_id_fkey\" (the name is configurable).\n\nNow that a constraint exists, when creating comments, we could\nannotate the changeset with foreign key constraint so Ecto knows\nhow to convert it into an error message:\n\n    cast(comment, params, [:post_id])\n    |> foreign_key_constraint(:post_id)\n\nNow, when invoking `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2`,\nif the associated post does not exist, the underlying operation will\nfail but Ecto will convert the database exception into a changeset\nerror and return an `{:error, changeset}` tuple. Note that the error\nwill occur only after hitting the database, so it will not be visible\nuntil all other validations pass. If the constraint fails inside a\ntransaction, the transaction will be marked as aborted.","ref":"Ecto.Changeset.html#foreign_key_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.foreign_key_constraint/3","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"does not exist\"\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.","ref":"Ecto.Changeset.html#foreign_key_constraint/3-options"},{"type":"function","title":"Ecto.Changeset.get_assoc/3","doc":"Gets the association entry or entries from changes or from the data.\n\nReturned data is normalized to changesets by default. Pass the `:struct`\nflag to retrieve the data as structs with changes applied, similar to `get_field/2`.","ref":"Ecto.Changeset.html#get_assoc/3"},{"type":"function","title":"Examples - Ecto.Changeset.get_assoc/3","doc":"iex> %Author{posts: [%Post{id: 1, title: \"hello\"}]}\n    ...> |> change()\n    ...> |> get_assoc(:posts)\n    [%Ecto.Changeset{data: %Post{id: 1, title: \"hello\"}, changes: %{}}]\n\n    iex> %Author{posts: [%Post{id: 1, title: \"hello\"}]}\n    ...> |> cast(%{posts: [%{id: 1, title: \"world\"}]}, [])\n    ...> |> cast_assoc(:posts)\n    ...> |> get_assoc(:posts, :changeset)\n    [%Ecto.Changeset{data: %Post{id: 1, title: \"hello\"}, changes: %{title: \"world\"}}]\n\n    iex> %Author{posts: [%Post{id: 1, title: \"hello\"}]}\n    ...> |> cast(%{posts: [%{id: 1, title: \"world\"}]}, [])\n    ...> |> cast_assoc(:posts)\n    ...> |> get_assoc(:posts, :struct)\n    [%Post{id: 1, title: \"world\"}]","ref":"Ecto.Changeset.html#get_assoc/3-examples"},{"type":"function","title":"Ecto.Changeset.get_change/3","doc":"Gets a change or returns a default value.\n\nFor associations and embeds, this function always returns\nnil, a changeset, or a list of changesets.","ref":"Ecto.Changeset.html#get_change/3"},{"type":"function","title":"Examples - Ecto.Changeset.get_change/3","doc":"iex> changeset = change(%Post{body: \"foo\"}, %{title: \"bar\"})\n    iex> get_change(changeset, :title)\n    \"bar\"\n    iex> get_change(changeset, :body)\n    nil","ref":"Ecto.Changeset.html#get_change/3-examples"},{"type":"function","title":"Ecto.Changeset.get_embed/3","doc":"Gets the embedded entry or entries from changes or from the data.\n\nReturned data is normalized to changesets by default. Pass the `:struct`\nflag to retrieve the data as structs with changes applied, similar to `get_field/2`.","ref":"Ecto.Changeset.html#get_embed/3"},{"type":"function","title":"Examples - Ecto.Changeset.get_embed/3","doc":"iex> %Post{comments: [%Comment{id: 1, body: \"hello\"}]}\n    ...> |> change()\n    ...> |> get_embed(:comments)\n    [%Ecto.Changeset{data: %Comment{id: 1, body: \"hello\"}, changes: %{}}]\n\n    iex> %Post{comments: [%Comment{id: 1, body: \"hello\"}]}\n    ...> |> cast(%{comments: [%{id: 1, body: \"world\"}]}, [])\n    ...> |> cast_embed(:comments)\n    ...> |> get_embed(:comments, :changeset)\n    [%Ecto.Changeset{data: %Comment{id: 1, body: \"hello\"}, changes: %{body: \"world\"}}]\n\n    iex> %Post{comments: [%Comment{id: 1, body: \"hello\"}]}\n    ...> |> cast(%{comments: [%{id: 1, body: \"world\"}]}, [])\n    ...> |> cast_embed(:comments)\n    ...> |> get_embed(:comments, :struct)\n    [%Comment{id: 1, body: \"world\"}]","ref":"Ecto.Changeset.html#get_embed/3-examples"},{"type":"function","title":"Ecto.Changeset.get_field/3","doc":"Gets a field from changes or from the data.\n\nWhile `get_change/3` only looks at the current `changes`\nto retrieve a value, this function looks at the changes and\nthen falls back on the data, finally returning `default` if\nno value is available.\n\nFor associations and embeds, this function always returns\nnil, a struct, or a list of structs. In case of changes,\nthe changeset data will have all data applies. This guarantees\na consistent result regardless if changes have been applied\nor not. Use `get_change/2` or `get_assoc/3`/`get_embed/3`\nif you want to retrieve the relations as changesets or\nif you want more fine-grained control.\n\n    iex> post = %Post{title: \"A title\", body: \"My body is a cage\"}\n    iex> changeset = change(post, %{title: \"A new title\"})\n    iex> get_field(changeset, :title)\n    \"A new title\"\n    iex> get_field(changeset, :not_a_field, \"Told you, not a field!\")\n    \"Told you, not a field!\"","ref":"Ecto.Changeset.html#get_field/3"},{"type":"function","title":"Ecto.Changeset.merge/2","doc":"Merges two changesets.\n\nThis function merges two changesets provided they have been applied to the\nsame data (their `:data` field is equal); if the data differs, an\n`ArgumentError` exception is raised. If one of the changesets has a `:repo`\nfield which is not `nil`, then the value of that field is used as the `:repo`\nfield of the resulting changeset; if both changesets have a non-`nil` and\ndifferent `:repo` field, an `ArgumentError` exception is raised.\n\nThe other fields are merged with the following criteria:\n\n  * `params` - params are merged (not deep-merged) giving precedence to the\n    params of `changeset2` in case of a conflict. If both changesets have their\n    `:params` fields set to `nil`, the resulting changeset will have its params\n    set to `nil` too.\n  * `changes` - changes are merged giving precedence to the `changeset2`\n    changes.\n  * `errors` and `validations` - they are simply concatenated.\n  * `required` - required fields are merged; all the fields that appear\n    in the required list of both changesets are moved to the required\n    list of the resulting changeset.","ref":"Ecto.Changeset.html#merge/2"},{"type":"function","title":"Examples - Ecto.Changeset.merge/2","doc":"iex> changeset1 = cast(%Post{}, %{title: \"Title\"}, [:title])\n    iex> changeset2 = cast(%Post{}, %{title: \"New title\", body: \"Body\"}, [:title, :body])\n    iex> changeset = merge(changeset1, changeset2)\n    iex> changeset.changes\n    %{body: \"Body\", title: \"New title\"}\n\n    iex> changeset1 = cast(%Post{body: \"Body\"}, %{title: \"Title\"}, [:title])\n    iex> changeset2 = cast(%Post{}, %{title: \"New title\"}, [:title])\n    iex> merge(changeset1, changeset2)\n    ** (ArgumentError) different :data when merging changesets","ref":"Ecto.Changeset.html#merge/2-examples"},{"type":"function","title":"Ecto.Changeset.no_assoc_constraint/3","doc":"Checks the associated field does not exist.\n\nThis is similar to `foreign_key_constraint/3` except that the\nfield is inferred from the association definition. This is useful\nto guarantee that parent can only be deleted (or have its primary\nkey changed) if no child exists in the database. Therefore, it only\napplies to `has_*` associations.\n\nAs the name says, a constraint is required in the database for\nthis function to work. Such constraint is often added as a\nreference to the child table:\n\n    create table(:comments) do\n      add :post_id, references(:posts)\n    end\n\nNow, when deleting the post, it is possible to forbid any post to\nbe deleted if they still have comments attached to it:\n\n    post\n    |> Ecto.Changeset.change\n    |> Ecto.Changeset.no_assoc_constraint(:comments)\n    |> Repo.delete","ref":"Ecto.Changeset.html#no_assoc_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.no_assoc_constraint/3","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"is still associated with this entry\" (for `has_one`)\n    and \"are still associated with this entry\" (for `has_many`)\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.","ref":"Ecto.Changeset.html#no_assoc_constraint/3-options"},{"type":"function","title":"Ecto.Changeset.optimistic_lock/3","doc":"Applies optimistic locking to the changeset.\n\n[Optimistic\nlocking](https://en.wikipedia.org/wiki/Optimistic_concurrency_control) (or\n*optimistic concurrency control*) is a technique that allows concurrent edits\non a single record. While pessimistic locking works by locking a resource for\nan entire transaction, optimistic locking only checks if the resource changed\nbefore updating it.\n\nThis is done by regularly fetching the record from the database, then checking\nwhether another user has made changes to the record *only when updating the\nrecord*. This behaviour is ideal in situations where the chances of concurrent\nupdates to the same record are low; if they're not, pessimistic locking or\nother concurrency patterns may be more suited.","ref":"Ecto.Changeset.html#optimistic_lock/3"},{"type":"function","title":"Usage - Ecto.Changeset.optimistic_lock/3","doc":"Optimistic locking works by keeping a \"version\" counter for each record; this\ncounter gets incremented each time a modification is made to a record. Hence,\nin order to use optimistic locking, a field must exist in your schema for\nversioning purpose. Such field is usually an integer but other types are\nsupported.","ref":"Ecto.Changeset.html#optimistic_lock/3-usage"},{"type":"function","title":"Examples - Ecto.Changeset.optimistic_lock/3","doc":"Assuming we have a `Post` schema (stored in the `posts` table), the first step\nis to add a version column to the `posts` table:\n\n    alter table(:posts) do\n      add :lock_version, :integer, default: 1\n    end\n\nThe column name is arbitrary and doesn't need to be `:lock_version`. Now add\na field to the schema too:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        field :title, :string\n        field :lock_version, :integer, default: 1\n      end\n\n      def changeset(:update, struct, params \\\\ %{}) do\n        struct\n        |> Ecto.Changeset.cast(params, [:title])\n        |> Ecto.Changeset.optimistic_lock(:lock_version)\n      end\n    end\n\nNow let's take optimistic locking for a spin:\n\n    iex> post = Repo.insert!(%Post{title: \"foo\"})\n    %Post{id: 1, title: \"foo\", lock_version: 1}\n    iex> valid_change = Post.changeset(:update, post, %{title: \"bar\"})\n    iex> stale_change = Post.changeset(:update, post, %{title: \"baz\"})\n    iex> Repo.update!(valid_change)\n    %Post{id: 1, title: \"bar\", lock_version: 2}\n    iex> Repo.update!(stale_change)\n    ** (Ecto.StaleEntryError) attempted to update a stale entry:\n\n    %Post{id: 1, title: \"baz\", lock_version: 1}\n\nWhen a conflict happens (a record which has been previously fetched is\nbeing updated, but that same record has been modified since it was\nfetched), an `Ecto.StaleEntryError` exception is raised.\n\nOptimistic locking also works with delete operations. Just call the\n`optimistic_lock/3` function with the data before delete:\n\n    iex> changeset = Ecto.Changeset.optimistic_lock(post, :lock_version)\n    iex> Repo.delete(changeset)\n\n`optimistic_lock/3` by default assumes the field\nbeing used as a lock is an integer. If you want to use another type,\nyou need to pass the third argument customizing how the next value\nis generated:\n\n    iex> Ecto.Changeset.optimistic_lock(post, :lock_uuid, fn _ -> Ecto.UUID.generate end)","ref":"Ecto.Changeset.html#optimistic_lock/3-examples"},{"type":"function","title":"Ecto.Changeset.prepare_changes/2","doc":"Provides a function executed by the repository on insert/update/delete.\n\nIf the changeset given to the repository is valid, the function given to\n`prepare_changes/2` will be called with the changeset and must return a\nchangeset, allowing developers to do final adjustments to the changeset or\nto issue data consistency commands. The repository itself can be accessed\ninside the function under the `repo` field in the changeset. If the\nchangeset given to the repository is invalid, the function will not be\ninvoked.\n\nThe given function is guaranteed to run inside the same transaction\nas the changeset operation for databases that do support transactions.","ref":"Ecto.Changeset.html#prepare_changes/2"},{"type":"function","title":"Example - Ecto.Changeset.prepare_changes/2","doc":"A common use case is updating a counter cache, in this case updating a post's\ncomment count when a comment is created:\n\n    def create_comment(comment, params) do\n      comment\n      |> cast(params, [:body, :post_id])\n      |> prepare_changes(fn changeset ->\n           if post_id = get_change(changeset, :post_id) do\n             query = from Post, where: [id: ^post_id]\n             changeset.repo.update_all(query, inc: [comment_count: 1])\n           end\n           changeset\n         end)\n    end\n\nWe retrieve the repo from the comment changeset itself and use\nupdate_all to update the counter cache in one query. Finally, the original\nchangeset must be returned.","ref":"Ecto.Changeset.html#prepare_changes/2-example"},{"type":"function","title":"Ecto.Changeset.put_assoc/4","doc":"Puts the given association entry or entries as a change in the changeset.\n\nThis function is used to work with associations as a whole. For example,\nif a Post has many Comments, it allows you to add, remove or change all\ncomments at once. If your goal is to simply add a new comment to a post,\nthen it is preferred to do so manually, as we will describe later in the\n\"Example: Adding a comment to a post\" section.\n\nThis function requires the associated data to have been preloaded, except\nwhen the parent changeset has been newly built and not yet persisted.\nMissing data will invoke the `:on_replace` behaviour defined on the\nassociation.\n\nFor associations with cardinality one, `nil` can be used to remove the existing\nentry. For associations with many entries, an empty list may be given instead.\n\nIf the association has no changes, it will be skipped. If the association is\ninvalid, the changeset will be marked as invalid. If the given value is not any\nof values below, it will raise.\n\nThe associated data may be given in different formats:\n\n  * a map or a keyword list representing changes to be applied to the\n    associated data. A map or keyword list can be given to update the\n    associated data as long as they have matching primary keys.\n    For example, `put_assoc(changeset, :comments, [%{id: 1, title: \"changed\"}])`\n    will locate the comment with `:id` of 1 and update its title.\n    If no comment with such id exists, one is created on the fly.\n    Since only a single comment was given, any other associated comment\n    will be replaced. On all cases, it is expected the keys to be atoms.\n    Opposite to `cast_assoc` and `embed_assoc`, the given map (or struct)\n    is not validated in any way and will be inserted as is.\n    This API is mostly used in scripts and tests, to make it straight-\n    forward to create schemas with associations at once, such as:\n\n        Ecto.Changeset.change(\n          %Post{},\n          title: \"foo\",\n          comments: [\n            %{body: \"first\"},\n            %{body: \"second\"}\n          ]\n        )\n\n  * changesets - when changesets are given, they are treated as the canonical\n    data and the associated data currently stored in the association is either\n    updated or replaced. For example, if you call\n    `put_assoc(post_changeset, :comments, [list_of_comments_changesets])`,\n    all comments with matching IDs will be updated according to the changesets.\n    New comments or comments not associated to any post will be correctly\n    associated. Currently associated comments that do not have a matching ID\n    in the list of changesets will act according to the `:on_replace` association\n    configuration (you can chose to raise, ignore the operation, update or delete\n    them). If there are changes in any of the changesets, they will be\n    persisted too.\n\n  * structs - when structs are given, they are treated as the canonical data\n    and the associated data currently stored in the association is replaced.\n    For example, if you call\n    `put_assoc(post_changeset, :comments, [list_of_comments_structs])`,\n    all comments with matching IDs will be replaced by the new structs.\n    New comments or comments not associated to any post will be correctly\n    associated. Currently associated comments that do not have a matching ID\n    in the list of changesets will act according to the `:on_replace`\n    association configuration (you can chose to raise, ignore the operation,\n    update or delete them). Different to passing changesets, structs are not\n    change tracked in any fashion. In other words, if you change a comment\n    struct and give it to `put_assoc/4`, the updates in the struct won't be\n    persisted. You must use changesets instead. `put_assoc/4` with structs\n    only takes care of guaranteeing that the comments and the parent data\n    are associated. This is extremely useful when associating existing data,\n    as we will see in the \"Example: Adding tags to a post\" section.\n\nOnce the parent changeset is given to an `Ecto.Repo` function, all entries\nwill be inserted/updated/deleted within the same transaction.\n\nIf you need different behaviour or explicit control over how this function\nbehaves, you can drop it altogether and use `Ecto.Multi` to encode how several\ndatabase operations will happen on several schemas and changesets at once.","ref":"Ecto.Changeset.html#put_assoc/4"},{"type":"function","title":"Example: Adding a comment to a post - Ecto.Changeset.put_assoc/4","doc":"Imagine a relationship where Post has many comments and you want to add a\nnew comment to an existing post. While it is possible to use `put_assoc/4`\nfor this, it would be unnecessarily complex. Let's see an example.\n\nFirst, let's fetch the post with all existing comments:\n\n    post = Post |> Repo.get!(1) |> Repo.preload(:comments)\n\nThe following approach is **wrong**:\n\n    post\n    |> Ecto.Changeset.change()\n    |> Ecto.Changeset.put_assoc(:comments, [%Comment{body: \"bad example!\"}])\n    |> Repo.update!()\n\nThe reason why the example above is wrong is because `put_assoc/4` always\nworks with the **full data**. So the example above will effectively **erase\nall previous comments** and only keep the comment you are currently adding.\nInstead, you could try:\n\n    post\n    |> Ecto.Changeset.change()\n    |> Ecto.Changeset.put_assoc(:comments, [%Comment{body: \"so-so example!\"} | post.comments])\n    |> Repo.update!()\n\nIn this example, we prepend the new comment to the list of existing comments.\nEcto will diff the list of comments currently in `post` with the list of comments\ngiven, and correctly insert the new comment to the database. Note, however,\nEcto is doing a lot of work just to figure out something we knew since the\nbeginning, which is that there is only one new comment.\n\nIn cases like above, when you want to work only on a single entry, it is\nmuch easier to simply work on the associated directly. For example, we\ncould instead set the `post` association in the comment:\n\n    %Comment{body: \"better example\"}\n    |> Ecto.Changeset.change()\n    |> Ecto.Changeset.put_assoc(:post, post)\n    |> Repo.insert!()\n\nAlternatively, we can make sure that when we create a comment, it is already\nassociated to the post:\n\n    Ecto.build_assoc(post, :comments)\n    |> Ecto.Changeset.change(body: \"great example!\")\n    |> Repo.insert!()\n\nOr we can simply set the post_id in the comment itself:\n\n    %Comment{body: \"better example\", post_id: post.id}\n    |> Repo.insert!()\n\nIn other words, when you find yourself wanting to work only with a subset\nof the data, then using `put_assoc/4` is most likely unnecessary. Instead,\nyou want to work on the other side of the association.\n\nLet's see an example where using `put_assoc/4` is a good fit.","ref":"Ecto.Changeset.html#put_assoc/4-example-adding-a-comment-to-a-post"},{"type":"function","title":"Example: Adding tags to a post - Ecto.Changeset.put_assoc/4","doc":"Imagine you are receiving a set of tags you want to associate to a post.\nLet's imagine that those tags exist upfront and are all persisted to the\ndatabase. Imagine we get the data in this format:\n\n    params = %{\"title\" => \"new post\", \"tags\" => [\"learner\"]}\n\nNow, since the tags already exist, we will bring all of them from the\ndatabase and put them directly in the post:\n\n    tags = Repo.all(from t in Tag, where: t.name in ^params[\"tags\"])\n\n    post\n    |> Repo.preload(:tags)\n    |> Ecto.Changeset.cast(params, [:title]) # No need to allow :tags as we put them directly\n    |> Ecto.Changeset.put_assoc(:tags, tags) # Explicitly set the tags\n\nSince in this case we always require the user to pass all tags\ndirectly, using `put_assoc/4` is a great fit. It will automatically\nremove any tag not given and properly associate all of the given\ntags with the post.\n\nFurthermore, since the tag information is given as structs read directly\nfrom the database, Ecto will treat the data as correct and only do the\nminimum necessary to guarantee that posts and tags are associated,\nwithout trying to update or diff any of the fields in the tag struct.\n\nAlthough it accepts an `opts` argument, there are no options currently\nsupported by `put_assoc/4`.","ref":"Ecto.Changeset.html#put_assoc/4-example-adding-tags-to-a-post"},{"type":"function","title":"More resources - Ecto.Changeset.put_assoc/4","doc":"You can learn more about working with associations in our documentation,\nincluding cheatsheets and practical examples. Check out:\n\n  * The docs for `cast_assoc/3`\n  * The [associations cheatsheet](associations.html)\n  * The [Constraints and Upserts guide](constraints-and-upserts.html)\n  * The [Polymorphic associations with many to many guide](polymorphic-associations-with-many-to-many.html)","ref":"Ecto.Changeset.html#put_assoc/4-more-resources"},{"type":"function","title":"Ecto.Changeset.put_change/3","doc":"Puts a change on the given `key` with `value`.\n\n`key` is an atom that represents any field, embed or\nassociation in the changeset. Note the `value` is directly\nstored in the changeset with no validation whatsoever.\nFor this reason, this function is meant for working with\ndata internal to the application.\n\nIf the change is already present, it is overridden with\nthe new value. If the change has the same value as in the\nchangeset data, no changes are added (and any existing\nchanges are removed).\n\nWhen changing embeds and associations, see `put_assoc/4`\nfor a complete reference on the accepted values.","ref":"Ecto.Changeset.html#put_change/3"},{"type":"function","title":"Examples - Ecto.Changeset.put_change/3","doc":"iex> changeset = change(%Post{}, %{title: \"foo\"})\n    iex> changeset = put_change(changeset, :title, \"bar\")\n    iex> changeset.changes\n    %{title: \"bar\"}\n\n    iex> changeset = change(%Post{title: \"foo\"})\n    iex> changeset = put_change(changeset, :title, \"foo\")\n    iex> changeset.changes\n    %{}","ref":"Ecto.Changeset.html#put_change/3-examples"},{"type":"function","title":"Ecto.Changeset.put_embed/4","doc":"Puts the given embed entry or entries as a change in the changeset.\n\nThis function is used to work with embeds as a whole. For embeds with\ncardinality one, `nil` can be used to remove the existing entry. For\nembeds with many entries, an empty list may be given instead.\n\nIf the embed has no changes, it will be skipped. If the embed is\ninvalid, the changeset will be marked as invalid.\n\nThe list of supported values and their behaviour is described in\n`put_assoc/4`. If the given value is not any of values listed there,\nit will raise.\n\nAlthough this function accepts an `opts` argument, there are no options\ncurrently supported by `put_embed/4`.","ref":"Ecto.Changeset.html#put_embed/4"},{"type":"function","title":"Ecto.Changeset.traverse_errors/2","doc":"Traverses changeset errors and applies the given function to error messages.\n\nThis function is particularly useful when associations and embeds\nare cast in the changeset as it will traverse all associations and\nembeds and place all errors in a series of nested maps.\n\nA changeset is supplied along with a function to apply to each\nerror message as the changeset is traversed. The error message\nfunction receives an error tuple `{msg, opts}`, for example:\n\n    {\"should be at least %{count} characters\", [count: 3, validation: :length, min: 3]}","ref":"Ecto.Changeset.html#traverse_errors/2"},{"type":"function","title":"Examples - Ecto.Changeset.traverse_errors/2","doc":"iex> traverse_errors(changeset, fn {msg, opts} ->\n    ...>   Regex.replace(~r\"%{(\\w+)}\", msg, fn _, key ->\n    ...>     opts |> Keyword.get(String.to_existing_atom(key), key) |> to_string()\n    ...>   end)\n    ...> end)\n    %{title: [\"should be at least 3 characters\"]}\n\nOptionally function can accept three arguments: `changeset`, `field`\nand error tuple `{msg, opts}`. It is useful whenever you want to extract\nvalidations rules from `changeset.validations` to build detailed error\ndescription.","ref":"Ecto.Changeset.html#traverse_errors/2-examples"},{"type":"function","title":"Ecto.Changeset.traverse_validations/2","doc":"Traverses changeset validations and applies the given function to validations.\n\nThis behaves the same as `traverse_errors/2`, but operates on changeset\nvalidations instead of errors.","ref":"Ecto.Changeset.html#traverse_validations/2"},{"type":"function","title":"Examples - Ecto.Changeset.traverse_validations/2","doc":"iex> traverse_validations(changeset, &(&1))\n    %{title: [format: ~r/pattern/, length: [min: 1, max: 20]]}\n\n    iex> traverse_validations(changeset, fn\n    ...>   {:length, opts} -> {:length, \"#{Keyword.get(opts, :min, 0)}-#{Keyword.get(opts, :max, 32)}\"}\n    ...>   {:format, %Regex{source: source}} -> {:format, \"/#{source}/\"}\n    ...>   {other, opts} -> {other, inspect(opts)}\n    ...> end)\n    %{title: [format: \"/pattern/\", length: \"1-20\"]}","ref":"Ecto.Changeset.html#traverse_validations/2-examples"},{"type":"function","title":"Ecto.Changeset.unique_constraint/3","doc":"Checks for a unique constraint in the given field or list of fields.\n\nThe unique constraint works by relying on the database to check\nif the unique constraint has been violated or not and, if so,\nEcto converts it into a changeset error.\n\nIn order to use the uniqueness constraint, the first step is\nto define the unique index in a migration:\n\n    create unique_index(:users, [:email])\n\nNow that a constraint exists, when modifying users, we could\nannotate the changeset with a unique constraint so Ecto knows\nhow to convert it into an error message:\n\n    cast(user, params, [:email])\n    |> unique_constraint(:email)\n\nNow, when invoking `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2`,\nif the email already exists, the underlying operation will fail but\nEcto will convert the database exception into a changeset error and\nreturn an `{:error, changeset}` tuple. Note that the error will occur\nonly after hitting the database, so it will not be visible until all\nother validations pass. If the constraint fails inside a transaction,\nthe transaction will be marked as aborted.","ref":"Ecto.Changeset.html#unique_constraint/3"},{"type":"function","title":"Options - Ecto.Changeset.unique_constraint/3","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"has already been taken\"\n\n  * `:name` - the constraint name. By default, the constraint\n    name is inferred from the table + field. If this option is given,\n    the `field` argument only indicates the field the error will be\n    added to. May be required explicitly for complex cases\n\n  * `:match` - how the changeset constraint name is matched against the\n    repo constraint, may be `:exact`, `:suffix` or `:prefix`. Defaults to\n    `:exact`. `:suffix` matches any repo constraint which `ends_with?` `:name`\n    to this changeset constraint. `:prefix` matches any repo constraint which\n    `starts_with?` `:name` to this changeset constraint.\n\n  * `:error_key` - the key to which changeset error will be added when\n    check fails, defaults to the first field name of the given list of\n    fields.","ref":"Ecto.Changeset.html#unique_constraint/3-options"},{"type":"function","title":"Complex constraints - Ecto.Changeset.unique_constraint/3","doc":"Because the constraint logic is in the database, we can leverage\nall the database functionality when defining them. For example,\nlet's suppose the e-mails are scoped by company id:\n\n    # In migration\n    create unique_index(:users, [:email, :company_id])\n\n    # In the changeset function\n    cast(user, params, [:email])\n    |> unique_constraint([:email, :company_id])\n\nThe first field name, `:email` in this case, will be used as the error\nkey to the changeset errors keyword list. For example, the above\n`unique_constraint/3` would generate something like:\n\n    Repo.insert!(%User{email: \"john@elixir.org\", company_id: 1})\n    changeset = User.changeset(%User{}, %{email: \"john@elixir.org\", company_id: 1})\n    {:error, changeset} = Repo.insert(changeset)\n    changeset.errors #=> [email: {\"has already been taken\", []}]\n\nIn complex cases, instead of relying on name inference, it may be best\nto set the constraint name explicitly:\n\n    # In the migration\n    create unique_index(:users, [:email, :company_id], name: :users_email_company_id_index)\n\n    # In the changeset function\n    cast(user, params, [:email])\n    |> unique_constraint(:email, name: :users_email_company_id_index)\n\n#","ref":"Ecto.Changeset.html#unique_constraint/3-complex-constraints"},{"type":"function","title":"Partitioning - Ecto.Changeset.unique_constraint/3","doc":"If your table is partitioned, then your unique index might look different\nper partition, e.g. Postgres adds p  to the middle of your key, like:\n\n    users_p0_email_key\n    users_p1_email_key\n    ...\n    users_p99_email_key\n\nIn this case you can use the name and suffix options together to match on\nthese dynamic indexes, like:\n\n    cast(user, params, [:email])\n    |> unique_constraint(:email, name: :email_key, match: :suffix)\n\nThere are cases where the index has a number added both for table name and\nindex name, generating an index name such as:\n\n    user_p0_email_idx2\n    user_p1_email_idx3\n    ...\n    user_p99_email_idx101\n\nIn that case, a `Regex` can be used to match:\n\n    cast(user, params, [:email])\n    |> unique_constraint(:email, name: ~r/user_p+_email_idx+/)","ref":"Ecto.Changeset.html#unique_constraint/3-partitioning"},{"type":"function","title":"Case sensitivity - Ecto.Changeset.unique_constraint/3","doc":"Unfortunately, different databases provide different guarantees\nwhen it comes to case-sensitiveness. For example, in MySQL, comparisons\nare case-insensitive by default. In Postgres, users can define case\ninsensitive column by using the `:citext` type/extension. In your migration:\n\n    execute \"CREATE EXTENSION IF NOT EXISTS citext\"\n    create table(:users) do\n      ...\n      add :email, :citext\n      ...\n    end\n\nIf for some reason your database does not support case insensitive columns,\nyou can explicitly downcase values before inserting/updating them:\n\n    cast(data, params, [:email])\n    |> update_change(:email, &String.downcase/1)\n    |> unique_constraint(:email)","ref":"Ecto.Changeset.html#unique_constraint/3-case-sensitivity"},{"type":"function","title":"Ecto.Changeset.unsafe_validate_unique/4","doc":"Validates that no existing record with a different primary key\nhas the same values for these fields.\n\nThis function exists to provide quick feedback to users of your\napplication. It should not be relied on for any data guarantee as it\nhas race conditions and is inherently unsafe. For example, if this\ncheck happens twice in the same time interval (because the user\nsubmitted a form twice), both checks may pass and you may end-up with\nduplicate entries in the database. Therefore, a `unique_constraint/3`\nshould also be used to ensure your data won't get corrupted.\n\nHowever, because constraints are only checked if all validations\nsucceed, this function can be used as an early check to provide\nearly feedback to users, since most conflicting data will have been\ninserted prior to the current validation phase.\n\nWhen applying this validation to a schemas loaded from the database\nthis check will exclude rows having the same primary key as set on\nthe changeset, as those are supposed to be overwritten anyways.","ref":"Ecto.Changeset.html#unsafe_validate_unique/4"},{"type":"function","title":"Options - Ecto.Changeset.unsafe_validate_unique/4","doc":"* `:message` - the message in case the constraint check fails,\n    defaults to \"has already been taken\".\n\n  * `:error_key` - the key to which changeset error will be added when\n    check fails, defaults to the first field name of the given list of\n    fields.\n\n  * `:prefix` - the prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). See `Ecto.Repo` documentation\n    for more information.\n\n  * `:nulls_distinct` - a boolean controlling whether different null values\n    are considered distinct (not equal). If `false`, `nil` values will have\n    their uniqueness checked. Otherwise, the check will not be performed. This\n    is only meaningful when paired with a unique index that treats nulls as equal,\n    such as Postgres 15's `NULLS NOT DISTINCT` option. Defaults to `true`\n\n  * `:repo_opts` - the options to pass to the `Ecto.Repo` call.\n\n  * `:query` - the base query to use for the check. Defaults to the schema of\n    the changeset. If the primary key is set, a clause will be added to exclude\n    the changeset row itself from the check.","ref":"Ecto.Changeset.html#unsafe_validate_unique/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.unsafe_validate_unique/4","doc":"unsafe_validate_unique(changeset, :city_name, repo)\n    unsafe_validate_unique(changeset, [:city_name, :state_name], repo)\n    unsafe_validate_unique(changeset, [:city_name, :state_name], repo, message: \"city must be unique within state\")\n    unsafe_validate_unique(changeset, [:city_name, :state_name], repo, prefix: \"public\")\n    unsafe_validate_unique(changeset, [:city_name, :state_name], repo, query: from(c in City, where: is_nil(c.deleted_at)))","ref":"Ecto.Changeset.html#unsafe_validate_unique/4-examples"},{"type":"function","title":"Ecto.Changeset.update_change/3","doc":"Updates a change.\n\nThe given `function` is invoked with the change value only if there\nis a change for `key`. Once the function is invoked, it behaves as\n`put_changed/3`.\n\nNote that the value of the change can still be `nil` (unless the field\nwas marked as required on `validate_required/3`).","ref":"Ecto.Changeset.html#update_change/3"},{"type":"function","title":"Examples - Ecto.Changeset.update_change/3","doc":"iex> changeset = change(%Post{}, %{impressions: 1})\n    iex> changeset = update_change(changeset, :impressions, &(&1 + 1))\n    iex> changeset.changes.impressions\n    2","ref":"Ecto.Changeset.html#update_change/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_acceptance/3","doc":"Validates the given parameter is true.\n\nNote this validation only checks the parameter itself is true, never\nthe field in the schema. That's because acceptance parameters do not need\nto be persisted, as by definition they would always be stored as `true`.","ref":"Ecto.Changeset.html#validate_acceptance/3"},{"type":"function","title":"Options - Ecto.Changeset.validate_acceptance/3","doc":"* `:message` - the message on failure, defaults to \"must be accepted\"","ref":"Ecto.Changeset.html#validate_acceptance/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_acceptance/3","doc":"validate_acceptance(changeset, :terms_of_service)\n    validate_acceptance(changeset, :rules, message: \"please accept rules\")","ref":"Ecto.Changeset.html#validate_acceptance/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_change/3","doc":"Validates the given `field` change.\n\nIt invokes the `validator` function to perform the validation\nonly if a change for the given `field` exists and the change\nvalue is not `nil`. The function must return a list of errors\n(with an empty list meaning no errors).\n\nIn case there's at least one error, the list of errors will be appended to the\n`:errors` field of the changeset and the `:valid?` flag will be set to\n`false`.","ref":"Ecto.Changeset.html#validate_change/3"},{"type":"function","title":"Examples - Ecto.Changeset.validate_change/3","doc":"iex> changeset = change(%Post{}, %{title: \"foo\"})\n    iex> changeset = validate_change changeset, :title, fn :title, title  ->\n    ...>   # Value must not be \"foo\"!\n    ...>   if title == \"foo\" do\n    ...>     [title: \"cannot be foo\"]\n    ...>   else\n    ...>     []\n    ...>   end\n    ...> end\n    iex> changeset.errors\n    [title: {\"cannot be foo\", []}]\n\n    iex> changeset = change(%Post{}, %{title: \"foo\"})\n    iex> changeset = validate_change changeset, :title, fn :title, title  ->\n    ...>   if title == \"foo\" do\n    ...>     [title: {\"cannot be foo\", additional: \"info\"}]\n    ...>   else\n    ...>     []\n    ...>   end\n    ...> end\n    iex> changeset.errors\n    [title: {\"cannot be foo\", [additional: \"info\"]}]","ref":"Ecto.Changeset.html#validate_change/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_change/4","doc":"Stores the validation `metadata` and validates the given `field` change.\n\nSimilar to `validate_change/3` but stores the validation metadata\ninto the changeset validators. The validator metadata is often used\nas a reflection mechanism, to automatically generate code based on\nthe available validations.","ref":"Ecto.Changeset.html#validate_change/4"},{"type":"function","title":"Examples - Ecto.Changeset.validate_change/4","doc":"iex> changeset = change(%Post{}, %{title: \"foo\"})\n    iex> changeset = validate_change changeset, :title, :useless_validator, fn\n    ...>   _, _ -> []\n    ...> end\n    iex> changeset.validations\n    [title: :useless_validator]","ref":"Ecto.Changeset.html#validate_change/4-examples"},{"type":"function","title":"Ecto.Changeset.validate_confirmation/3","doc":"Validates that the given parameter matches its confirmation.\n\nBy calling `validate_confirmation(changeset, :email)`, this\nvalidation will check if both \"email\" and \"email_confirmation\"\nin the parameter map matches. Note this validation only looks\nat the parameters themselves, never the fields in the schema.\nAs such as, the \"email_confirmation\" field does not need to be\nadded as a virtual field in your schema.\n\nNote that if the confirmation field is missing, this does not\nadd a validation error. This is done on purpose as you do not\ntrigger confirmation validation in places where a confirmation\nis not required (for example, in APIs). You can force the\nconfirmation parameter to be required in the options (see below).","ref":"Ecto.Changeset.html#validate_confirmation/3"},{"type":"function","title":"Options - Ecto.Changeset.validate_confirmation/3","doc":"* `:message` - the message on failure, defaults to \"does not match confirmation\"\n  * `:required` - boolean, sets whether existence of confirmation parameter\n    is required for addition of error. Defaults to false","ref":"Ecto.Changeset.html#validate_confirmation/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_confirmation/3","doc":"validate_confirmation(changeset, :email)\n    validate_confirmation(changeset, :password, message: \"does not match password\")\n\n    cast(data, params, [:password])\n    |> validate_confirmation(:password, message: \"does not match password\")","ref":"Ecto.Changeset.html#validate_confirmation/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_exclusion/4","doc":"Validates a change is not included in the given enumerable.","ref":"Ecto.Changeset.html#validate_exclusion/4"},{"type":"function","title":"Options - Ecto.Changeset.validate_exclusion/4","doc":"* `:message` - the message on failure, defaults to \"is reserved\"","ref":"Ecto.Changeset.html#validate_exclusion/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_exclusion/4","doc":"validate_exclusion(changeset, :name, ~w(admin superadmin))","ref":"Ecto.Changeset.html#validate_exclusion/4-examples"},{"type":"function","title":"Ecto.Changeset.validate_format/4","doc":"Validates a change has the given format.\n\nThe format has to be expressed as a regular expression.","ref":"Ecto.Changeset.html#validate_format/4"},{"type":"function","title":"Options - Ecto.Changeset.validate_format/4","doc":"* `:message` - the message on failure, defaults to \"has invalid format\"","ref":"Ecto.Changeset.html#validate_format/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_format/4","doc":"validate_format(changeset, :email, ~r/@/)","ref":"Ecto.Changeset.html#validate_format/4-examples"},{"type":"function","title":"Ecto.Changeset.validate_inclusion/4","doc":"Validates a change is included in the given enumerable.","ref":"Ecto.Changeset.html#validate_inclusion/4"},{"type":"function","title":"Options - Ecto.Changeset.validate_inclusion/4","doc":"* `:message` - the message on failure, defaults to \"is invalid\"","ref":"Ecto.Changeset.html#validate_inclusion/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_inclusion/4","doc":"validate_inclusion(changeset, :cardinal_direction, [\"north\", \"east\", \"south\", \"west\"])\n    validate_inclusion(changeset, :age, 0..99)","ref":"Ecto.Changeset.html#validate_inclusion/4-examples"},{"type":"function","title":"Ecto.Changeset.validate_length/3","doc":"Validates a change is a string or list of the given length.\n\nNote that the length of a string is counted in graphemes by default. If using\nthis validation to match a character limit of a database backend,\nit's likely that the limit ignores graphemes and limits the number\nof unicode characters. Then consider using the `:count` option to\nlimit the number of codepoints (`:codepoints`), or limit the number of bytes (`:bytes`).","ref":"Ecto.Changeset.html#validate_length/3"},{"type":"function","title":"Options - Ecto.Changeset.validate_length/3","doc":"* `:is` - the length must be exactly this value\n  * `:min` - the length must be greater than or equal to this value\n  * `:max` - the length must be less than or equal to this value\n  * `:count` - what length to count for string, `:graphemes` (default), `:codepoints` or `:bytes`\n  * `:message` - the message on failure, depending on the validation, is one of:\n    * for strings:\n      * \"should be %{count} character(s)\"\n      * \"should be at least %{count} character(s)\"\n      * \"should be at most %{count} character(s)\"\n    * for binary:\n      * \"should be %{count} byte(s)\"\n      * \"should be at least %{count} byte(s)\"\n      * \"should be at most %{count} byte(s)\"\n    * for lists and maps:\n      * \"should have %{count} item(s)\"\n      * \"should have at least %{count} item(s)\"\n      * \"should have at most %{count} item(s)\"","ref":"Ecto.Changeset.html#validate_length/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_length/3","doc":"validate_length(changeset, :title, min: 3)\n    validate_length(changeset, :title, max: 100)\n    validate_length(changeset, :title, min: 3, max: 100)\n    validate_length(changeset, :code, is: 9)\n    validate_length(changeset, :topics, is: 2)\n    validate_length(changeset, :icon, count: :bytes, max: 1024 * 16)","ref":"Ecto.Changeset.html#validate_length/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_number/3","doc":"Validates the properties of a number.","ref":"Ecto.Changeset.html#validate_number/3"},{"type":"function","title":"Options - Ecto.Changeset.validate_number/3","doc":"* `:less_than`\n  * `:greater_than`\n  * `:less_than_or_equal_to`\n  * `:greater_than_or_equal_to`\n  * `:equal_to`\n  * `:not_equal_to`\n  * `:message` - the message on failure, defaults to one of:\n    * \"must be less than %{number}\"\n    * \"must be greater than %{number}\"\n    * \"must be less than or equal to %{number}\"\n    * \"must be greater than or equal to %{number}\"\n    * \"must be equal to %{number}\"\n    * \"must be not equal to %{number}\"","ref":"Ecto.Changeset.html#validate_number/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_number/3","doc":"validate_number(changeset, :count, less_than: 3)\n    validate_number(changeset, :pi, greater_than: 3, less_than: 4)\n    validate_number(changeset, :the_answer_to_life_the_universe_and_everything, equal_to: 42)","ref":"Ecto.Changeset.html#validate_number/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_required/3","doc":"Validates that one or more fields are present in the changeset.\n\nYou can pass a single field name or a list of field names that\nare required.\n\nIf the value of a field is `nil` or a string made only of whitespace,\nthe changeset is marked as invalid, the field is removed from the\nchangeset's changes, and an error is added. An error won't be added if\nthe field already has an error.\n\nIf a field is given to `validate_required/3` but it has not been passed\nas parameter during `cast/3` (i.e. it has not been changed), then\n`validate_required/3` will check for its current value in the data.\nIf the data contains an non-empty value for the field, then no error is\nadded. This allows developers to use `validate_required/3` to perform\npartial updates. For example, on `insert` all fields would be required,\nbecause their default values on the data are all `nil`, but on `update`,\nif you don't want to change a field that has been previously set,\nyou are not required to pass it as a parameter, since `validate_required/3`\nwon't add an error for missing changes as long as the value in the\ndata given to the `changeset` is not empty.\n\nDo not use this function to validate associations that are required,\ninstead pass the `:required` option to `cast_assoc/3` or `cast_embed/3`.\n\nOpposite to other validations, calling this function does not store\nthe validation under the `changeset.validations` key. Instead, it\nstores all required fields under `changeset.required`.","ref":"Ecto.Changeset.html#validate_required/3"},{"type":"function","title":"Options - Ecto.Changeset.validate_required/3","doc":"* `:message` - the message on failure, defaults to \"can't be blank\"","ref":"Ecto.Changeset.html#validate_required/3-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_required/3","doc":"validate_required(changeset, :title)\n    validate_required(changeset, [:title, :body])","ref":"Ecto.Changeset.html#validate_required/3-examples"},{"type":"function","title":"Ecto.Changeset.validate_subset/4","doc":"Validates a change, of type enum, is a subset of the given enumerable.\n\nThis validates if a list of values belongs to the given enumerable.\nIf you need to validate if a single value is inside the given enumerable,\nyou should use `validate_inclusion/4` instead.\n\nType of the field must be array.","ref":"Ecto.Changeset.html#validate_subset/4"},{"type":"function","title":"Options - Ecto.Changeset.validate_subset/4","doc":"* `:message` - the message on failure, defaults to \"has an invalid entry\"","ref":"Ecto.Changeset.html#validate_subset/4-options"},{"type":"function","title":"Examples - Ecto.Changeset.validate_subset/4","doc":"validate_subset(changeset, :pets, [\"cat\", \"dog\", \"parrot\"])\n    validate_subset(changeset, :lottery_numbers, 0..99)","ref":"Ecto.Changeset.html#validate_subset/4-examples"},{"type":"function","title":"Ecto.Changeset.validations/1","doc":"Returns a keyword list of the validations for this changeset.\n\nThe keys in the list are the names of fields, and the values are a\nvalidation associated with the field. A field may occur multiple\ntimes in the list.","ref":"Ecto.Changeset.html#validations/1"},{"type":"function","title":"Example - Ecto.Changeset.validations/1","doc":"%Post{}\n    |> change()\n    |> validate_format(:title, ~r/^\\w+:\\s/, message: \"must start with a topic\")\n    |> validate_length(:title, max: 100)\n    |> validations()\n    #=> [\n      title: {:length, [ max: 100 ]},\n      title: {:format, ~r/^\\w+:\\s/}\n    ]\n\nThe following validations may be included in the result. The list is\nnot necessarily exhaustive. For example, custom validations written\nby the developer will also appear in our return value.\n\nThis first group contains validations that hold a keyword list of validators.\nThis list may also include a `:message` key.\n\n  * `{:length, [option]}`\n\n    * `min: n`\n    * `max: n`\n    * `is: n`\n    * `count: :graphemes | :codepoints`\n\n  * `{:number,  [option]}`\n\n    * `equal_to: n`\n    * `greater_than: n`\n    * `greater_than_or_equal_to: n`\n    * `less_than: n`\n    * `less_than_or_equal_to: n`\n\nThe other validators simply take a value:\n\n  * `{:exclusion, Enum.t}`\n  * `{:format, ~r/pattern/}`\n  * `{:inclusion, Enum.t}`\n  * `{:subset, Enum.t}`\n\nNote that calling `validate_required/3` does not store the validation under the\n`changeset.validations` key (and so won't be included in the result of this\nfunction). The required fields are stored under the `changeset.required` key.","ref":"Ecto.Changeset.html#validations/1-example"},{"type":"type","title":"Ecto.Changeset.action/0","doc":"","ref":"Ecto.Changeset.html#t:action/0"},{"type":"type","title":"Ecto.Changeset.constraint/0","doc":"","ref":"Ecto.Changeset.html#t:constraint/0"},{"type":"type","title":"Ecto.Changeset.data/0","doc":"","ref":"Ecto.Changeset.html#t:data/0"},{"type":"type","title":"Ecto.Changeset.empty_value/0","doc":"A possible value that you can pass to the `:empty_values` option.\n\nSee `empty_values/0` and the [*Empty values* section](#module-empty-values) in\nthe module documentation for more information.","ref":"Ecto.Changeset.html#t:empty_value/0"},{"type":"type","title":"Ecto.Changeset.error/0","doc":"","ref":"Ecto.Changeset.html#t:error/0"},{"type":"type","title":"Ecto.Changeset.t/0","doc":"","ref":"Ecto.Changeset.html#t:t/0"},{"type":"type","title":"Ecto.Changeset.t/1","doc":"","ref":"Ecto.Changeset.html#t:t/1"},{"type":"type","title":"Ecto.Changeset.traverse_result/0","doc":"","ref":"Ecto.Changeset.html#t:traverse_result/0"},{"type":"type","title":"Ecto.Changeset.types/0","doc":"","ref":"Ecto.Changeset.html#t:types/0"},{"type":"module","title":"Ecto.Multi","doc":"`Ecto.Multi` is a data structure for grouping multiple Repo operations.\n\n`Ecto.Multi` makes it possible to pack operations that should be\nperformed in a single database transaction and gives a way to introspect\nthe queued operations without actually performing them. Each operation\nis given a name that is unique and will identify its result in case of\nsuccess or failure.\n\nIf a multi is valid (i.e. all the changesets in it are valid),\nall operations will be executed in the order they were added.\n\nThe `Ecto.Multi` structure should be considered opaque. You can use\n`%Ecto.Multi{}` to pattern match the type, but accessing fields or\ndirectly modifying them is not advised.\n\n`Ecto.Multi.to_list/1` returns a canonical representation of the\nstructure that can be used for introspection.","ref":"Ecto.Multi.html"},{"type":"module","title":"Changesets - Ecto.Multi","doc":"If multi contains operations that accept changesets (like `insert/4`,\n`update/4` or `delete/4`) they will be checked before starting the\ntransaction. If any changeset has errors, the transaction won't even\nbe started and the error will be immediately returned.\n\nNote: `insert/4`, `update/4`, `insert_or_update/4`, and `delete/4`\nvariants that accept a function do not perform these checks since\nthe functions are executed after the transaction has started.","ref":"Ecto.Multi.html#module-changesets"},{"type":"module","title":"Run - Ecto.Multi","doc":"Multi allows you to run arbitrary functions as part of your transaction\nvia `run/3` and `run/5`. This is especially useful when an operation\ndepends on the value of a previous operation. For this reason, the\nfunction given as a callback to `run/3` and `run/5` will receive the repo\nas the first argument, and all changes performed by the multi so far as a\nmap for the second argument.\n\nThe function given to `run` must return `{:ok, value}` or `{:error, value}`\nas its result. Returning an error will abort any further operations\nand make the whole multi fail.","ref":"Ecto.Multi.html#module-run"},{"type":"module","title":"Example - Ecto.Multi","doc":"Let's look at an example definition and usage. The use case we'll be\nlooking into is resetting a password. We need to update the account\nwith proper information, log the request and remove all current sessions:\n\n    defmodule PasswordManager do\n      alias Ecto.Multi\n\n      def reset(account, params) do\n        Multi.new()\n        |> Multi.update(:account, Account.password_reset_changeset(account, params))\n        |> Multi.insert(:log, Log.password_reset_changeset(account, params))\n        |> Multi.delete_all(:sessions, Ecto.assoc(account, :sessions))\n      end\n    end\n\nWe can later execute it in the integration layer using Repo:\n\n    Repo.transaction(PasswordManager.reset(account, params))\n\nBy pattern matching on the result we can differentiate different conditions:\n\n    case result do\n      {:ok, %{account: account, log: log, sessions: sessions}} ->\n        # Operation was successful, we can access results (exactly the same\n        # we would get from running corresponding Repo functions) under keys\n        # we used for naming the operations.\n      {:error, failed_operation, failed_value, changes_so_far} ->\n        # One of the operations failed. We can access the operation's failure\n        # value (like changeset for operations on changesets) to prepare a\n        # proper response. We also get access to the results of any operations\n        # that succeeded before the indicated operation failed. However, any\n        # successful operations would have been rolled back.\n    end\n\nWe can also easily unit test our transaction without actually running it.\nSince changesets can use in-memory-data, we can use an account that is\nconstructed in memory as well (without persisting it to the database):\n\n    test \"dry run password reset\" do\n      account = %Account{password: \"letmein\"}\n      multi = PasswordManager.reset(account, params)\n\n      assert [\n        {:account, {:update, account_changeset, []}},\n        {:log, {:insert, log_changeset, []}},\n        {:sessions, {:delete_all, query, []}}\n      ] = Ecto.Multi.to_list(multi)\n\n      # We can introspect changesets and query to see if everything\n      # is as expected, for example:\n      assert account_changeset.valid?\n      assert log_changeset.valid?\n      assert inspect(query) == \"#Ecto.Query \"\n    end\n\nThe name of each operation does not have to be an atom. This can be particularly\nuseful when you wish to update a collection of changesets at once, and track their\nerrors individually:\n\n    accounts = [%Account{id: 1}, %Account{id: 2}]\n\n    Enum.reduce(accounts, Multi.new(), fn account, multi ->\n      Multi.update(\n        multi,\n        {:account, account.id},\n        Account.password_reset_changeset(account, params)\n      )\n    end)","ref":"Ecto.Multi.html#module-example"},{"type":"function","title":"Ecto.Multi.all/4","doc":"Runs a query and stores all entries in the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.all/2` does.","ref":"Ecto.Multi.html#all/4"},{"type":"function","title":"Example - Ecto.Multi.all/4","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.all(:all, Post)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.all(:all, fn _changes -> Post end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#all/4-example"},{"type":"function","title":"Ecto.Multi.append/2","doc":"Appends the second multi to the first one.\n\nAll names must be unique between both structures.","ref":"Ecto.Multi.html#append/2"},{"type":"function","title":"Example - Ecto.Multi.append/2","doc":"iex> lhs = Ecto.Multi.new() |> Ecto.Multi.run(:left, fn _, changes -> {:ok, changes} end)\n    iex> rhs = Ecto.Multi.new() |> Ecto.Multi.run(:right, fn _, changes -> {:error, changes} end)\n    iex> Ecto.Multi.append(lhs, rhs) |> Ecto.Multi.to_list |> Keyword.keys\n    [:left, :right]","ref":"Ecto.Multi.html#append/2-example"},{"type":"function","title":"Ecto.Multi.delete/4","doc":"Adds a delete operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.delete/2` does.","ref":"Ecto.Multi.html#delete/4"},{"type":"function","title":"Example - Ecto.Multi.delete/4","doc":"post = MyApp.Repo.get!(Post, 1)\n    Ecto.Multi.new()\n    |> Ecto.Multi.delete(:delete, post)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.run(:post, fn repo, _changes ->\n      case repo.get(Post, 1) do\n        nil -> {:error, :not_found}\n        post -> {:ok, post}\n      end\n    end)\n    |> Ecto.Multi.delete(:delete, fn %{post: post} ->\n      # Others validations\n      post\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#delete/4-example"},{"type":"function","title":"Ecto.Multi.delete_all/4","doc":"Adds a delete_all operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.delete_all/2` does.","ref":"Ecto.Multi.html#delete_all/4"},{"type":"function","title":"Example - Ecto.Multi.delete_all/4","doc":"queryable = from(p in Post, where: p.id   Ecto.Multi.delete_all(:delete_all, queryable)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.run(:post, fn repo, _changes ->\n      case repo.get(Post, 1) do\n        nil -> {:error, :not_found}\n        post -> {:ok, post}\n      end\n    end)\n    |> Ecto.Multi.delete_all(:delete_all, fn %{post: post} ->\n      # Others validations\n      from(c in Comment, where: c.post_id == ^post.id)\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#delete_all/4-example"},{"type":"function","title":"Ecto.Multi.error/3","doc":"Causes the multi to fail with the given value.\n\nRunning the multi in a transaction will execute\nno previous steps and returns the value of the first\nerror added.","ref":"Ecto.Multi.html#error/3"},{"type":"function","title":"Ecto.Multi.exists?/4","doc":"Checks if there exists an entry matching the given query and stores a boolean in the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.exists?/2`.","ref":"Ecto.Multi.html#exists?/4"},{"type":"function","title":"Example - Ecto.Multi.exists?/4","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.exists?(:post, Post)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.exists?(:post, fn _changes -> Post end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#exists?/4-example"},{"type":"function","title":"Ecto.Multi.insert/4","doc":"Adds an insert operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.insert/2` does.","ref":"Ecto.Multi.html#insert/4"},{"type":"function","title":"Example - Ecto.Multi.insert/4","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.insert(:insert, %Post{title: \"first\"})\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:post, %Post{title: \"first\"})\n    |> Ecto.Multi.insert(:comment, fn %{post: post} ->\n      Ecto.build_assoc(post, :comments)\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#insert/4-example"},{"type":"function","title":"Ecto.Multi.insert_all/5","doc":"Adds an insert_all operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.insert_all/3` does.","ref":"Ecto.Multi.html#insert_all/5"},{"type":"function","title":"Example - Ecto.Multi.insert_all/5","doc":"posts = [%{title: \"My first post\"}, %{title: \"My second post\"}]\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert_all(:insert_all, Post, posts)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.run(:post, fn repo, _changes ->\n      case repo.get(Post, 1) do\n        nil -> {:error, :not_found}\n        post -> {:ok, post}\n      end\n    end)\n    |> Ecto.Multi.insert_all(:insert_all, Comment, fn %{post: post} ->\n      # Others validations\n\n      entries\n      |> Enum.map(fn comment ->\n        Map.put(comment, :post_id, post.id)\n      end)\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#insert_all/5-example"},{"type":"function","title":"Ecto.Multi.insert_or_update/4","doc":"Inserts or updates a changeset depending on whether the changeset was persisted or not.\n\nAccepts the same arguments and options as `c:Ecto.Repo.insert_or_update/2` does.","ref":"Ecto.Multi.html#insert_or_update/4"},{"type":"function","title":"Example - Ecto.Multi.insert_or_update/4","doc":"changeset = Post.changeset(%Post{}, %{title: \"New title\"})\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert_or_update(:insert_or_update, changeset)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.run(:post, fn repo, _changes ->\n      {:ok, repo.get(Post, 1) || %Post{}}\n    end)\n    |> Ecto.Multi.insert_or_update(:update, fn %{post: post} ->\n      Ecto.Changeset.change(post, title: \"New title\")\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#insert_or_update/4-example"},{"type":"function","title":"Ecto.Multi.inspect/2","doc":"Inspects results from a Multi\n\nBy default, the name is shown as a label to the inspect, custom labels are\nsupported through the `IO.inspect/2` `label` option.","ref":"Ecto.Multi.html#inspect/2"},{"type":"function","title":"Options - Ecto.Multi.inspect/2","doc":"All options for IO.inspect/2 are supported, it also support the following ones:\n\n  * `:only` - A field or a list of fields to inspect, will print the entire\n    map by default.","ref":"Ecto.Multi.html#inspect/2-options"},{"type":"function","title":"Examples - Ecto.Multi.inspect/2","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.insert(:person_a, changeset)\n    |> Ecto.Multi.insert(:person_b, changeset)\n    |> Ecto.Multi.inspect()\n    |> MyApp.Repo.transaction()\n\nPrints:\n    %{person_a: %Person{...}, person_b: %Person{...}}\n\nWe can use the `:only` option to limit which fields will be printed:\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:person_a, changeset)\n    |> Ecto.Multi.insert(:person_b, changeset)\n    |> Ecto.Multi.inspect(only: :person_a)\n    |> MyApp.Repo.transaction()\n\nPrints:\n    %{person_a: %Person{...}}","ref":"Ecto.Multi.html#inspect/2-examples"},{"type":"function","title":"Ecto.Multi.merge/2","doc":"Merges a multi returned dynamically by an anonymous function.\n\nThis function is useful when the multi to be merged requires information\nfrom the original multi. Hence the second argument is an anonymous function\nthat receives the multi changes so far. The anonymous function must return\nanother multi.\n\nIf you would prefer to simply merge two multis together, see `append/2` or\n`prepend/2`.\n\nDuplicated operations are not allowed.","ref":"Ecto.Multi.html#merge/2"},{"type":"function","title":"Example - Ecto.Multi.merge/2","doc":"multi =\n      Ecto.Multi.new()\n      |> Ecto.Multi.insert(:post, %Post{title: \"first\"})\n\n    multi\n    |> Ecto.Multi.merge(fn %{post: post} ->\n      Ecto.Multi.new()\n      |> Ecto.Multi.insert(:comment, Ecto.build_assoc(post, :comments))\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#merge/2-example"},{"type":"function","title":"Ecto.Multi.merge/4","doc":"Merges a multi returned dynamically by calling `module` and `function` with `args`.\n\nSimilar to `merge/2`, but allows to pass module name, function and arguments.\nThe function should return an `Ecto.Multi`, and receives changes so far\nas the first argument (prepended to those passed in the call to the function).\n\nDuplicated operations are not allowed.","ref":"Ecto.Multi.html#merge/4"},{"type":"function","title":"Ecto.Multi.new/0","doc":"Returns an empty `Ecto.Multi` struct.","ref":"Ecto.Multi.html#new/0"},{"type":"function","title":"Example - Ecto.Multi.new/0","doc":"iex> Ecto.Multi.new() |> Ecto.Multi.to_list()\n    []","ref":"Ecto.Multi.html#new/0-example"},{"type":"function","title":"Ecto.Multi.one/4","doc":"Runs a query expecting one result and stores it in the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.one/2`.","ref":"Ecto.Multi.html#one/4"},{"type":"function","title":"Example - Ecto.Multi.one/4","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.one(:post, Post)\n    |> Ecto.Multi.one(:author, fn %{post: post} ->\n      from(a in Author, where: a.id == ^post.author_id)\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#one/4-example"},{"type":"function","title":"Ecto.Multi.prepend/2","doc":"Prepends the second multi to the first one.\n\nAll names must be unique between both structures.","ref":"Ecto.Multi.html#prepend/2"},{"type":"function","title":"Example - Ecto.Multi.prepend/2","doc":"iex> lhs = Ecto.Multi.new() |> Ecto.Multi.run(:left, fn _, changes -> {:ok, changes} end)\n    iex> rhs = Ecto.Multi.new() |> Ecto.Multi.run(:right, fn _, changes -> {:error, changes} end)\n    iex> Ecto.Multi.prepend(lhs, rhs) |> Ecto.Multi.to_list |> Keyword.keys\n    [:right, :left]","ref":"Ecto.Multi.html#prepend/2-example"},{"type":"function","title":"Ecto.Multi.put/3","doc":"Adds a value to the changes so far under the given name.\n\nThe given `value` is added to the multi before the transaction starts.\nIf you would like to run arbitrary functions as part of your transaction,\nsee `run/3` or `run/5`.","ref":"Ecto.Multi.html#put/3"},{"type":"function","title":"Example - Ecto.Multi.put/3","doc":"Imagine there is an existing company schema that you retrieved from\nthe database. You can insert it as a change in the multi using `put/3`:\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.put(:company, company)\n    |> Ecto.Multi.insert(:user, fn changes -> User.changeset(changes.company) end)\n    |> Ecto.Multi.insert(:person, fn changes -> Person.changeset(changes.user, changes.company) end)\n    |> MyApp.Repo.transaction()\n\nIn the example above there isn't a large benefit in putting the\n`company` in the multi, because you could also access the\n`company` variable directly inside the anonymous function.\n\nHowever, the benefit of `put/3` is when composing `Ecto.Multi`s.\nIf the insert operations above were defined in another module,\nyou could use `put(:company, company)` to inject changes that\nwill be accessed by other functions down the chain, removing\nthe need to pass both `multi` and `company` values around.","ref":"Ecto.Multi.html#put/3-example"},{"type":"function","title":"Ecto.Multi.run/3","doc":"Adds a function to run as part of the multi.\n\nThe function should return either `{:ok, value}` or `{:error, value}`,\nand receives the repo as the first argument, and the changes so far\nas the second argument.","ref":"Ecto.Multi.html#run/3"},{"type":"function","title":"Example - Ecto.Multi.run/3","doc":"Ecto.Multi.run(multi, :write, fn _repo, %{image: image} ->\n      with :ok <- File.write(image.name, image.contents) do\n        {:ok, nil}\n      end\n    end)","ref":"Ecto.Multi.html#run/3-example"},{"type":"function","title":"Ecto.Multi.run/5","doc":"Adds a function to run as part of the multi.\n\nSimilar to `run/3`, but allows to pass module name, function and arguments.\nThe function should return either `{:ok, value}` or `{:error, value}`, and\nreceives the repo as the first argument, and the changes so far as the\nsecond argument (prepended to those passed in the call to the function).","ref":"Ecto.Multi.html#run/5"},{"type":"function","title":"Ecto.Multi.to_list/1","doc":"Returns the list of operations stored in `multi`.\n\nAlways use this function when you need to access the operations you\nhave defined in `Ecto.Multi`. Inspecting the `Ecto.Multi` struct internals\ndirectly is discouraged.","ref":"Ecto.Multi.html#to_list/1"},{"type":"function","title":"Ecto.Multi.update/4","doc":"Adds an update operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.update/2` does.","ref":"Ecto.Multi.html#update/4"},{"type":"function","title":"Example - Ecto.Multi.update/4","doc":"post = MyApp.Repo.get!(Post, 1)\n    changeset = Ecto.Changeset.change(post, title: \"New title\")\n    Ecto.Multi.new()\n    |> Ecto.Multi.update(:update, changeset)\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:post, %Post{title: \"first\"})\n    |> Ecto.Multi.update(:fun, fn %{post: post} ->\n      Ecto.Changeset.change(post, title: \"New title\")\n    end)\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#update/4-example"},{"type":"function","title":"Ecto.Multi.update_all/5","doc":"Adds an update_all operation to the multi.\n\nAccepts the same arguments and options as `c:Ecto.Repo.update_all/3` does.","ref":"Ecto.Multi.html#update_all/5"},{"type":"function","title":"Example - Ecto.Multi.update_all/5","doc":"Ecto.Multi.new()\n    |> Ecto.Multi.update_all(:update_all, Post, set: [title: \"New title\"])\n    |> MyApp.Repo.transaction()\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.run(:post, fn repo, _changes ->\n      case repo.get(Post, 1) do\n        nil -> {:error, :not_found}\n        post -> {:ok, post}\n      end\n    end)\n    |> Ecto.Multi.update_all(:update_all, fn %{post: post} ->\n      # Others validations\n      from(c in Comment, where: c.post_id == ^post.id, update: [set: [title: \"New title\"]])\n    end, [])\n    |> MyApp.Repo.transaction()","ref":"Ecto.Multi.html#update_all/5-example"},{"type":"type","title":"Ecto.Multi.changes/0","doc":"Map of changes made so far during the current transaction. For any Multi\nwhich returns `{:ok, result}`, its `t:name/0` is added as a key and its\nresult as the value.","ref":"Ecto.Multi.html#t:changes/0"},{"type":"type","title":"Ecto.Multi.failure/0","doc":"Result of a failed transaction using a Multi.","ref":"Ecto.Multi.html#t:failure/0"},{"type":"type","title":"Ecto.Multi.fun/1","doc":"","ref":"Ecto.Multi.html#t:fun/1"},{"type":"type","title":"Ecto.Multi.merge/0","doc":"","ref":"Ecto.Multi.html#t:merge/0"},{"type":"type","title":"Ecto.Multi.name/0","doc":"Name of an operation in the Multi. Can be any term, as long as it is unique\nwithin the list of operations; for example, `:insert_post` or `{:delete_post,\n5}`.","ref":"Ecto.Multi.html#t:name/0"},{"type":"type","title":"Ecto.Multi.run/0","doc":"","ref":"Ecto.Multi.html#t:run/0"},{"type":"type","title":"Ecto.Multi.t/0","doc":"","ref":"Ecto.Multi.html#t:t/0"},{"type":"module","title":"Ecto.Query","doc":"Provides the Query DSL.\n\nQueries are used to retrieve and manipulate data from a repository\n(see `Ecto.Repo`). Ecto queries come in two flavors: keyword-based\nand macro-based. Most examples will use the keyword-based syntax,\nthe macro one will be explored in later sections.\n\nLet's see a sample query:\n\n    # Imports only from/2 of Ecto.Query\n    import Ecto.Query, only: [from: 2]\n\n    # Create a query\n    query = from u in \"users\",\n              where: u.age > 18,\n              select: u.name\n\n    # Send the query to the repository\n    Repo.all(query)\n\nIn the example above, we are directly querying the \"users\" table\nfrom the database. Queries do not reach out to the data store until\nthey are passed as arguments to a function from `Ecto.Repo`.","ref":"Ecto.Query.html"},{"type":"module","title":"Query expressions - Ecto.Query","doc":"Ecto allows a limited set of expressions inside queries. In the\nquery below, for example, we use `u.age` to access a field, the\n`>` comparison operator and the literal `0`:\n\n    query = from u in \"users\", where: u.age > 0, select: u.name\n\nYou can find the full list of operations in `Ecto.Query.API`.\nBesides the operations listed there, the following literals are\nsupported in queries:\n\n  * Integers: `1`, `2`, `3`\n  * Floats: `1.0`, `2.0`, `3.0`\n  * Booleans: `true`, `false`\n  * Binaries: ` >`\n  * Strings: `\"foo bar\"`, `~s(this is a string)`\n  * Atoms (other than booleans and `nil`): `:foo`, `:bar`\n  * Arrays: `[1, 2, 3]`, `~w(interpolate words)`\n\nAll other types and dynamic values must be passed as a parameter using\ninterpolation as explained below.","ref":"Ecto.Query.html#module-query-expressions"},{"type":"module","title":"Interpolation and casting - Ecto.Query","doc":"External values and Elixir expressions can be injected into a query\nexpression with `^`:\n\n    def with_minimum(age, height_ft) do\n      from u in \"users\",\n        where: u.age > ^age and u.height > ^(height_ft * 3.28),\n        select: u.name\n    end\n\n    with_minimum(18, 5.0)\n\nWhen interpolating values, you may want to explicitly tell Ecto\nwhat is the expected type of the value being interpolated:\n\n    age = \"18\"\n    Repo.all(from u in \"users\",\n              where: u.age > type(^age, :integer),\n              select: u.name)\n\nIn the example above, Ecto will cast the age to type integer. When\na value cannot be cast, `Ecto.Query.CastError` is raised.\n\nTo avoid the repetition of always specifying the types, you may define\nan `Ecto.Schema`. In such cases, Ecto will analyze your queries and\nautomatically cast the interpolated \"age\" when compared to the `u.age`\nfield, as long as the age field is defined with type `:integer` in\nyour schema:\n\n    age = \"18\"\n    Repo.all(from u in User, where: u.age > ^age, select: u.name)\n\nAnother advantage of using schemas is that we no longer need to specify\nthe select option in queries, as by default Ecto will retrieve all\nfields specified in the schema:\n\n    age = \"18\"\n    Repo.all(from u in User, where: u.age > ^age)\n\nFor this reason, we will use schemas on the remaining examples but\nremember Ecto does not require them in order to write queries.\n\n## `nil` comparison\n\n`nil` comparison in filters, such as where and having, is forbidden\nand it will raise an error:\n\n    # Raises if age is nil\n    from u in User, where: u.age == ^age\n\nThis is done as a security measure to avoid attacks that attempt\nto traverse entries with nil columns. To check that value is `nil`,\nuse `is_nil/1` instead:\n\n    from u in User, where: is_nil(u.age)","ref":"Ecto.Query.html#module-interpolation-and-casting"},{"type":"module","title":"Composition - Ecto.Query","doc":"Ecto queries are composable. For example, the query above can\nactually be defined in two parts:\n\n    # Create a query\n    query = from u in User, where: u.age > 18\n\n    # Extend the query\n    query = from u in query, select: u.name\n\nComposing queries uses the same syntax as creating a query.\nThe difference is that, instead of passing a schema like `User`\non the right-hand side of `in`, we passed the query itself.\n\nAny value can be used on the right-hand side of `in` as long as it implements\nthe `Ecto.Queryable` protocol. For now, we know the protocol is\nimplemented for both atoms (like `User`) and strings (like \"users\").\n\nIn any case, regardless if a schema has been given or not, Ecto\nqueries are always composable thanks to its binding system.\n\n#","ref":"Ecto.Query.html#module-composition"},{"type":"module","title":"Positional bindings - Ecto.Query","doc":"On the left-hand side of `in` we specify the query bindings. This is\ndone inside `from` and `join` clauses. In the query below `u` is a\nbinding and `u.age` is a field access using this binding.\n\n    query = from u in User, where: u.age > 18\n\nBindings are not exposed from the query. When composing queries, you\nmust specify bindings again for each refinement query. For example,\nto further narrow down the above query, we again need to tell Ecto what\nbindings to expect:\n\n    query = from u in query, select: u.city\n\nBindings in Ecto are positional, and the names do not have to be\nconsistent between input and refinement queries. For example, the\nquery above could also be written as:\n\n    query = from q in query, select: q.city\n\nIt would make no difference to Ecto. This is important because\nit allows developers to compose queries without caring about\nthe bindings used in the initial query.\n\nWhen using joins, the bindings should be matched in the order they\nare specified:\n\n    # Create a query\n    query = from p in Post,\n              join: c in Comment, on: c.post_id == p.id\n\n    # Extend the query\n    query = from [p, c] in query,\n              select: {p.title, c.body}\n\nYou are not required to specify all bindings when composing.\nFor example, if we would like to order the results above by\npost insertion date, we could further extend it as:\n\n    query = from q in query, order_by: q.inserted_at\n\nThe example above will work if the input query has 1 or 10\nbindings. As long as the number of bindings is less than the\nnumber of `from`s + `join`s, Ecto will match only what you have\nspecified. The first binding always matches the source given\nin `from`.\n\nSimilarly, if you are interested only in the last binding\n(or the last bindings) in a query, you can use `...` to\nspecify \"all bindings before\" and match on the last one.\n\nFor instance, imagine you wrote:\n\n    posts_with_comments =\n      from p in query, join: c in Comment, on: c.post_id == p.id\n\nAnd now we want to make sure to return both the post title\nand the comment body. Although we may not know how many\nbindings there are in the query, we are sure posts is the\nfirst binding and comments are the last one, so we can write:\n\n    from [p, ..., c] in posts_with_comments, select: {p.title, c.body}\n\nIn other words, `...` will include all the bindings between the\nfirst and the last, which may be one, many or no bindings at all.\n\n#","ref":"Ecto.Query.html#module-positional-bindings"},{"type":"module","title":"Named bindings - Ecto.Query","doc":"Another option for flexibly building queries with joins are named\nbindings. Coming back to the previous example, we can use the\n`as: :comment` option to bind the comments join to a concrete name:\n\n    posts_with_comments =\n      from p in Post,\n        join: c in Comment, as: :comment, on: c.post_id == p.id\n\nNow we can refer to it using the following form of a bindings list:\n\n    from [p, comment: c] in posts_with_comments, select: {p.title, c.body}\n\nThis approach lets us not worry about keeping track of the position\nof the bindings when composing the query. The `:as` option can be\ngiven both on joins and on `from`:\n\n    from p in Post, as: :post\n\nOnly atoms are accepted for binding names. Named binding references\nmust always be placed at the end of the bindings list:\n\n    [positional_binding_1, positional_binding_2, named_1: binding, named_2: binding]\n\nNamed bindings can also be used for late binding with the `as/1`\nconstruct, allowing you to refer to a binding that has not been\ndefined yet:\n\n    from c in Comment, where: as(:posts).id == c.post_id\n\nThis is especially useful when working with subqueries, where you\nmay need to refer to a parent binding with `parent_as`, which is\nnot known when writing the subquery:\n\n    child_query = from c in Comment, where: parent_as(:posts).id == c.post_id\n    from p in Post, as: :posts, inner_lateral_join: c in subquery(child_query)\n\nYou can also match on a specific binding when building queries. For\nexample, let's suppose you want to create a generic sort function\nthat will order by a given `field` with a given `as` in `query`:\n\n    # Knowing the name of the binding\n    def sort(query, as, field) do\n      from [{^as, x}] in query, order_by: field(x, ^field)\n    end\n\n#","ref":"Ecto.Query.html#module-named-bindings"},{"type":"module","title":"Bindingless operations - Ecto.Query","doc":"Although bindings are extremely useful when working with joins,\nthey are not necessary when the query has only the `from` clause.\nFor such cases, Ecto supports a way for building queries\nwithout specifying the binding:\n\n    from Post,\n      where: [category: \"fresh and new\"],\n      order_by: [desc: :published_at],\n      select: [:id, :title, :body]\n\nThe query above will select all posts with category \"fresh and new\",\norder by the most recently published, and return Post structs with\nonly the id, title and body fields set. It is equivalent to:\n\n    from p in Post,\n      where: p.category == \"fresh and new\",\n      order_by: [desc: p.published_at],\n      select: struct(p, [:id, :title, :body])\n\nOne advantage of bindingless queries is that they are data-driven\nand therefore useful for dynamically building queries. For example,\nthe query above could also be written as:\n\n    where = [category: \"fresh and new\"]\n    order_by = [desc: :published_at]\n    select = [:id, :title, :body]\n    from Post, where: ^where, order_by: ^order_by, select: ^select\n\nThis feature is very useful when queries need to be built based\non some user input, like web search forms, CLIs and so on.","ref":"Ecto.Query.html#module-bindingless-operations"},{"type":"module","title":"Fragments - Ecto.Query","doc":"If you need an escape hatch, Ecto provides fragments\n(see `Ecto.Query.API.fragment/1`) to inject SQL (and non-SQL)\nfragments into queries.\n\nFor example, to get all posts while running the \"lower(?)\"\nfunction in the database where `p.title` is interpolated\nin place of `?`, one can write:\n\n    from p in Post,\n      where: is_nil(p.published_at) and\n             fragment(\"lower(?)\", p.title) == ^title\n\nAlso, most adapters provide direct APIs for queries, like\n`Ecto.Adapters.SQL.query/4`, allowing developers to\ncompletely bypass Ecto queries.","ref":"Ecto.Query.html#module-fragments"},{"type":"module","title":"Macro API - Ecto.Query","doc":"In all examples so far we have used the **keywords query syntax** to\ncreate a query:\n\n    import Ecto.Query\n    from u in \"users\", where: u.age > 18, select: u.name\n\nDue to the prevalence of the pipe operator in Elixir, Ecto also supports\na pipe-based syntax:\n\n    \"users\"\n    |> where([u], u.age > 18)\n    |> select([u], u.name)\n\nThe keyword-based and pipe-based examples are equivalent. The downside\nof using macros is that the binding must be specified for every operation.\nHowever, since keyword-based and pipe-based examples are equivalent, the\nbindingless syntax also works for macros. Please note that the following\nexample is not completely equivalent to the previous example,\nas it does not return the name but rather the `User` struct:\n\n    \"users\"\n    |> where([u], u.age > 18)\n    |> select([:name])\n\nSuch a syntax allows developers to write queries using bindings only in more\ncomplex query expressions.\n\nThis module documents each of those macros, providing examples in\nboth the keywords query and pipe expression formats.","ref":"Ecto.Query.html#module-macro-api"},{"type":"module","title":"Query prefix - Ecto.Query","doc":"It is possible to set a prefix for the queries. For Postgres users,\nthis will specify the schema where the table is located, while for\nMySQL users this will specify the database where the table is\nlocated. When no prefix is set, Postgres queries are assumed to be\nin the public schema, while MySQL queries are assumed to be in the\ndatabase set in the config for the repo.\n\nThe query prefix may be set either for the whole query or on each\nindividual `from` and `join` expression. If a `prefix` is not given\nto a `from` or a `join`, the prefix of the schema given to the `from`\nor `join` is used. The query prefix is used only if none of the above\nare declared.\n\nLet's see some examples. To see the query prefix globally, the simplest\nmechanism is to pass an option to the repository operation:\n\n    results = Repo.all(query, prefix: \"accounts\")\n\nYou may also set the prefix for the whole query by setting the prefix field:\n\n    results =\n      query # May be User or an Ecto.Query itself\n      |> Ecto.Query.put_query_prefix(\"accounts\")\n      |> Repo.all()\n\nSetting the prefix in the query changes the default prefix of all `from`\nand `join` expressions. You can override the query prefix by either setting\nthe `@schema_prefix` in your schema definitions or by passing the prefix\noption:\n\n    from u in User,\n      prefix: \"accounts\",\n      join: p in assoc(u, :posts),\n      prefix: \"public\"\n\nOverall, here is the prefix lookup precedence:\n\n  1. The `:prefix` option given to `from`/`join` has the highest precedence\n  2. Then it falls back to the `@schema_prefix` attribute declared in the schema\n    given to `from`/`join`\n  3. Then it falls back to the query prefix. The query prefix may be\n     set either on the query with `put_query_prefix/2` or by passing\n     the `:prefix` option when calling the `Repo` module (where the\n     former wins if both methods are used)\n\nThe prefixes set in the query will be preserved when loading data.","ref":"Ecto.Query.html#module-query-prefix"},{"type":"function","title":"Ecto.Query.__struct__/0","doc":"The `Ecto.Query` struct.\n\nUsers of Ecto must consider this struct as opaque\nand not access its field directly. Authors of adapters\nmay read its contents, but never modify them.","ref":"Ecto.Query.html#__struct__/0"},{"type":"macro","title":"Ecto.Query.distinct/3","doc":"A distinct query expression.\n\nWhen true, only keeps distinct values from the resulting\nselect expression.\n\nIf supported by your database, you can also pass query expressions\nto distinct and it will generate a query with DISTINCT ON. In such\ncases, `distinct` accepts exactly the same expressions as `order_by`\nand any `distinct` expression will be automatically prepended to the\n`order_by` expressions in case there is any `order_by` expression.","ref":"Ecto.Query.html#distinct/3"},{"type":"macro","title":"Keywords examples - Ecto.Query.distinct/3","doc":"# Returns the list of different categories in the Post schema\n    from(p in Post, distinct: true, select: p.category)\n\n    # If your database supports DISTINCT ON(),\n    # you can pass expressions to distinct too\n    from(p in Post,\n       distinct: p.category,\n       order_by: [p.date])\n\n    # The DISTINCT ON() also supports ordering similar to ORDER BY.\n    from(p in Post,\n       distinct: [desc: p.category],\n       order_by: [p.date])\n\n    # Using atoms\n    from(p in Post, distinct: :category, order_by: :date)","ref":"Ecto.Query.html#distinct/3-keywords-examples"},{"type":"macro","title":"Expressions example - Ecto.Query.distinct/3","doc":"Post\n    |> distinct(true)\n    |> order_by([p], [p.category, p.author])","ref":"Ecto.Query.html#distinct/3-expressions-example"},{"type":"macro","title":"Ecto.Query.dynamic/2","doc":"Builds a dynamic query expression.\n\nDynamic query expressions allow developers to compose query\nexpressions bit by bit, so that they can be interpolated into\nparts of a query or another dynamic expression later on.","ref":"Ecto.Query.html#dynamic/2"},{"type":"macro","title":"Examples - Ecto.Query.dynamic/2","doc":"Imagine you have a set of conditions you want to build your query on:\n\n    conditions = false\n\n    conditions =\n      if params[\"is_public\"] do\n        dynamic([p], p.is_public or ^conditions)\n      else\n        conditions\n      end\n\n    conditions =\n      if params[\"allow_reviewers\"] do\n        dynamic([p, a], a.reviewer == true or ^conditions)\n      else\n        conditions\n      end\n\n    from query, where: ^conditions\n\nIn the example above, we were able to build the query expressions\nbit by bit, using different bindings, and later interpolate it all\nat once into the actual query.\n\nA dynamic expression can always be interpolated inside another dynamic\nexpression and into the constructs described below.\n\n## `where`, `having` and a `join`'s `on`\n\nThe `dynamic` macro can be interpolated at the root of a `where`,\n`having` or a `join`'s `on`.\n\nFor example, assuming the `conditions` variable defined in the\nprevious section, the following is forbidden because it is not\nat the root of a `where`:\n\n    from q in query, where: q.some_condition and ^conditions\n\nFortunately that's easily solved by simply rewriting it to:\n\n    conditions = dynamic([q], q.some_condition and ^conditions)\n    from query, where: ^conditions\n\n> #","ref":"Ecto.Query.html#dynamic/2-examples"},{"type":"macro","title":"Dynamic boundaries {: .warning} - Ecto.Query.dynamic/2","doc":">\n> Type casting does not cross dynamic boundaries. When you write\n> a dynamic expression, such as `dynamic([p], p.visits > ^param)`,\n> Ecto will automatically cast `^param` to the type of `p.visits`.\n>\n> However, if `p.visits` is in itself dynamic, as in the example\n> below, then Ecto won't be able to propagate its type to `^param`:\n>\n>     field = dynamic([p], p.visits)\n>     dynamic(^field > ^param)\n>\n\n## `order_by`\n\nDynamics can be interpolated inside keyword lists at the root of\n`order_by`. For example, you can write:\n\n    order_by = [\n      asc: :some_field,\n      desc: dynamic([p], fragment(\"?->>?\", p.another_field, \"json_key\"))\n    ]\n\n    from query, order_by: ^order_by\n\nDynamics are also supported in `order_by/2` clauses inside `windows/2`.\n\nAs with `where` and friends, it is not possible to pass dynamics\noutside of a root. For example, this won't work:\n\n    from query, order_by: [asc: ^dynamic(...)]\n\nBut this will:\n\n    from query, order_by: ^[asc: dynamic(...)]\n\n## `group_by`\n\nDynamics can be interpolated inside keyword lists at the root of\n`group_by`. For example, you can write:\n\n    group_by = [\n      :some_field,\n      dynamic([p], fragment(\"?->>?\", p.another_field, \"json_key\"))\n    ]\n\n    from query, group_by: ^group_by\n\nDynamics are also supported in `partition_by/2` clauses inside `windows/2`.\n\nAs with `where` and friends, it is not possible to pass dynamics\noutside of a root. For example, this won't work:\n\n    from query, group_by: [:some_field, ^dynamic(...)]\n\nBut this will:\n\n    from query, group_by: ^[:some_field, dynamic(...)]\n\n## `select` and `select_merge`\n\nDynamics can be inside maps interpolated at the root of a\n`select` or `select_merge`. For example, you can write:\n\n    fields = %{\n      period: dynamic([p], p.month),\n      metric: dynamic([p], p.distance)\n    }\n\n    from query, select: ^fields\n\nAs with `where` and friends, it is not possible to pass dynamics\noutside of a root. For example, this won't work:\n\n    from query, select: %{field: ^dynamic(...)}\n\nBut this will:\n\n    from query, select: ^%{field: dynamic(...)}\n\nMaps with dynamics can also be merged into existing `select` structures,\nenabling a variety of possibilities for partially dynamic selects:\n\n    metric = dynamic([p], p.distance)\n\n    from query, select: [:period, :metric], select_merge: ^%{metric: metric}\n\nAliasing fields with `selected_as/2` and referencing them with `selected_as/1`\nis also allowed:\n\n    fields = %{\n      period: dynamic([p], selected_as(p.month, :month)),\n      metric: dynamic([p], p.distance)\n    }\n\n    order = dynamic(selected_as(:month))\n\n    from query, select: ^fields, order_by: ^order\n\n## `update`\n\nA `dynamic` is also supported inside updates, for example:\n\n    updates = [\n      set: [average: dynamic([p], p.sum / p.count)]\n    ]\n\n    from query, update: ^updates\n\n## `preload`\n\nDynamics can be used with `preload` in order to dynamically\nspecify the binding for a joined association. For example, you can\nwrite:\n\n    preloads = [\n      :non_joined_assoc,\n      joined_assoc: dynamic([joined: j], j)\n    ]\n\n    from x in query,\n      join: assoc(x, :joined_assoc),\n      as: :joined,\n      preload: ^preloads\n\nWhile the example above uses a named binding (`:joined`),\npositional bindings may also be used:\n\n    preloads = [\n      :non_joined_assoc,\n      joined_assoc: dynamic([_, j], j)\n    ]\n\n    from x in query,\n      join: assoc(x, :joined_assoc)\n      preload: ^preloads\n\nAs with `where` and friends, it is not possible to pass dynamics\noutside of an interpolated root. For example, this won't work:\n\n    from query, preload: [comments: ^dynamic(...)]\n\nBut this will:\n\n    from query, preload: ^[comments: dynamic(...)]\n\nDynamic expressions used in `preload` must evaluate to a single\nbinding. For instance, this won't work:\n\n    preloads = dynamic([comments: c, likes: l], [comments: {c, likes: l}])\n\nBut this will:\n\n    dynamic_comments = dynamic([comments: c], c)\n    dynamic_likes = dynamic([likes: l], l)\n\n    preloads = [\n      comments: {dynamic_comments, likes: dynamic_likes}\n    ]","ref":"Ecto.Query.html#dynamic/2-dynamic-boundaries-warning"},{"type":"macro","title":"Ecto.Query.except/2","doc":"An except (set difference) query expression.\n\nTakes the difference of the result sets of multiple queries. The\n`select` of each query must be exactly the same, with the same\ntypes in the same order.\n\n> #","ref":"Ecto.Query.html#except/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.except/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nExcept expression returns only unique rows as if each query returned\ndistinct results. This may cause a performance penalty. If you need\nto take the difference of multiple result sets without\nremoving duplicate rows consider using `except_all/2`.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the set difference. `order_by`\nmust be specified in one of the following ways, since the set difference\nof two or more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the set difference fields.\n  - Wrap the set difference in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#except/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.except/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, except: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    except_query = from c in Customer, select: c.city, except: ^supplier_query\n    from s in subquery(except_query), order_by: s.city","ref":"Ecto.Query.html#except/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.except/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> except(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    except(customer_query, ^supplier_query)","ref":"Ecto.Query.html#except/2-expressions-examples"},{"type":"macro","title":"Ecto.Query.except_all/2","doc":"An except (set difference) query expression.\n\nTakes the difference of the result sets of multiple queries. The\n`select` of each query must be exactly the same, with the same\ntypes in the same order.\n\n> #","ref":"Ecto.Query.html#except_all/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.except_all/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the set difference. `order_by`\nmust be specified in one of the following ways, since the set difference\nof two or more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the set difference fields.\n  - Wrap the set difference in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#except_all/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.except_all/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, except_all: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    except_all_query = from c in Customer, select: c.city, except_all: ^supplier_query\n    from s in subquery(except_all_query), order_by: s.city","ref":"Ecto.Query.html#except_all/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.except_all/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> except_all(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    except_all(customer_query, ^supplier_query)","ref":"Ecto.Query.html#except_all/2-expressions-examples"},{"type":"function","title":"Ecto.Query.exclude/2","doc":"Resets a previously set field on a query.\n\nIt can reset many fields except the query source (`from`). When excluding\na `:join`, it will remove *all* types of joins. If you prefer to remove a\nsingle type of join, please see paragraph below.","ref":"Ecto.Query.html#exclude/2"},{"type":"function","title":"Examples - Ecto.Query.exclude/2","doc":"Ecto.Query.exclude(query, :join)\n    Ecto.Query.exclude(query, :where)\n    Ecto.Query.exclude(query, :order_by)\n    Ecto.Query.exclude(query, :group_by)\n    Ecto.Query.exclude(query, :having)\n    Ecto.Query.exclude(query, :distinct)\n    Ecto.Query.exclude(query, :select)\n    Ecto.Query.exclude(query, :combinations)\n    Ecto.Query.exclude(query, :with_ctes)\n    Ecto.Query.exclude(query, :limit)\n    Ecto.Query.exclude(query, :offset)\n    Ecto.Query.exclude(query, :lock)\n    Ecto.Query.exclude(query, :preload)\n    Ecto.Query.exclude(query, :update)\n\nYou can also remove specific joins as well such as `left_join` and\n`inner_join`:\n\n    Ecto.Query.exclude(query, :inner_join)\n    Ecto.Query.exclude(query, :cross_join)\n    Ecto.Query.exclude(query, :cross_lateral_join)\n    Ecto.Query.exclude(query, :left_join)\n    Ecto.Query.exclude(query, :right_join)\n    Ecto.Query.exclude(query, :full_join)\n    Ecto.Query.exclude(query, :inner_lateral_join)\n    Ecto.Query.exclude(query, :left_lateral_join)\n    Ecto.Query.exclude(query, :array_join)\n    Ecto.Query.exclude(query, :left_array_join)\n\nHowever, keep in mind that if a join is removed and its bindings\nwere referenced elsewhere, the bindings won't be removed, leading\nto a query that won't compile.","ref":"Ecto.Query.html#exclude/2-examples"},{"type":"function","title":"Ecto.Query.first/2","doc":"Restricts the query to return the first result ordered by primary key.\n\nThe query will be automatically ordered by the primary key\nunless `order_by` is given or `order_by` is set in the query.\nLimit is always set to 1.","ref":"Ecto.Query.html#first/2"},{"type":"function","title":"Examples - Ecto.Query.first/2","doc":"Post |> first |> Repo.one\n    query |> first(:inserted_at) |> Repo.one","ref":"Ecto.Query.html#first/2-examples"},{"type":"macro","title":"Ecto.Query.from/2","doc":"Creates a query.\n\nIt can either be a keyword query or a query expression.\n\nIf it is a keyword query the first argument must be\neither an `in` expression, a value that implements\nthe `Ecto.Queryable` protocol, or an `Ecto.Query.API.fragment/1`. If the query needs a\nreference to the data source in any other part of the\nexpression, then an `in` must be used to create a reference\nvariable. The second argument should be a keyword query\nwhere the keys are expression types and the values are\nexpressions.\n\nIf it is a query expression the first argument must be\na value that implements the `Ecto.Queryable` protocol\nand the second argument the expression.","ref":"Ecto.Query.html#from/2"},{"type":"macro","title":"Hints - Ecto.Query.from/2","doc":"The `hints` keyword can be used to specify query hints:\n\n    from p in Post,\n      hints: [\"USE INDEX FOO\"],\n      where: p.title == \"title\"\n\nIt can also be used as a general mechanism for adding statements that\ncome after the `from` clause. For example, it can be used to enable\ntable sampling:\n\n    from p in Post,\n      hints: \"TABLESAMPLE SYSTEM(1)\"\n\n`from` hints must be a (list of) compile-time strings or unsafe fragments. An unsafe\nfragment can be used to specify dynamic hints:\n\n    sample = \"SYSTEM_ROWS(1)\"\n\n    from p in Post,\n      hints: [\"TABLESAMPLE\", unsafe_fragment(^sample)]\n\n> #","ref":"Ecto.Query.html#from/2-hints"},{"type":"macro","title":"Unsafe Fragments {: .warning} - Ecto.Query.from/2","doc":">\n> The output of `unsafe_fragment/1` will be injected directly into the\n> resulting SQL statement without being escaped. For this reason, input\n> from uncontrolled sources, such as user input, should **never** be used.\n> Otherwise, it could lead to harmful SQL injection attacks.","ref":"Ecto.Query.html#from/2-unsafe-fragments-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.from/2","doc":"# `in` expression\n    from(c in City, select: c)\n\n    # Ecto.Queryable\n    from(City, limit: 1)\n\n    # Fragment with user-defined function and predefined columns\n    from(f in fragment(\"my_table_valued_function(arg)\"), select: f.x)\n\n    # Fragment with built-in function and undefined columns\n    from(f in fragment(\"select generate_series(?::integer, ?::integer) as x\", ^0, ^10), select: f.x)","ref":"Ecto.Query.html#from/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.from/2","doc":"# Schema\n    City |> select([c], c)\n\n    # Source\n    \"cities\" |> select([c], c)\n\n    # Source with schema\n    {\"cities\", Source} |> select([c], c)\n\n    # Ecto.Query\n    from(c in Cities) |> select([c], c)","ref":"Ecto.Query.html#from/2-expressions-examples"},{"type":"macro","title":"Examples - Ecto.Query.from/2","doc":"def paginate(query, page, size) do\n      from query,\n        limit: ^size,\n        offset: ^((page-1) * size)\n    end\n\nThe example above does not use `in` because `limit` and `offset`\ndo not require a reference to the data source. However, extending\nthe query with a where expression would require the use of `in`:\n\n    def published(query) do\n      from p in query, where: not(is_nil(p.published_at))\n    end\n\nNotice we have created a `p` variable to reference the query's\noriginal data source. This assumes that the original query\nonly had one source. When the given query has more than one source,\npositional or named bindings may be used to access the additional sources.\n\n    def published_multi(query) do\n      from [p,o] in query,\n      where: not(is_nil(p.published_at)) and not(is_nil(o.published_at))\n    end\n\nNote that the variables `p` and `o` can be named whatever you like\nas they have no importance in the query sent to the database.","ref":"Ecto.Query.html#from/2-examples"},{"type":"macro","title":"Ecto.Query.group_by/3","doc":"A group by query expression.\n\nGroups together rows from the schema that have the same values in the given\nfields. Using `group_by` \"groups\" the query giving it different semantics\nin the `select` expression. If a query is grouped, only fields that were\nreferenced in the `group_by` can be used in the `select` or if the field\nis given as an argument to an aggregate function.\n\n`group_by` also accepts a list of atoms where each atom refers to\na field in source. For more complicated queries you can access fields\ndirectly instead of atoms.","ref":"Ecto.Query.html#group_by/3"},{"type":"macro","title":"Keywords examples - Ecto.Query.group_by/3","doc":"# Returns the number of posts in each category\n    from(p in Post,\n      group_by: p.category,\n      select: {p.category, count(p.id)})\n\n    # Using atoms\n    from(p in Post, group_by: :category, select: {p.category, count(p.id)})\n\n    # Using direct fields access\n    from(p in Post,\n      join: c in assoc(p, :category),\n      group_by: [p.id, c.name])","ref":"Ecto.Query.html#group_by/3-keywords-examples"},{"type":"macro","title":"Expressions example - Ecto.Query.group_by/3","doc":"Post |> group_by([p], p.category) |> select([p], count(p.id))","ref":"Ecto.Query.html#group_by/3-expressions-example"},{"type":"function","title":"Ecto.Query.has_named_binding?/2","doc":"Returns `true` if the query has a binding with the given name, otherwise `false`.\n\nFor more information on named bindings see \"Named bindings\" in this module doc.","ref":"Ecto.Query.html#has_named_binding?/2"},{"type":"macro","title":"Ecto.Query.having/3","doc":"An AND having query expression.\n\nLike `where`, `having` filters rows from the schema, but after the grouping is\nperformed giving it the same semantics as `select` for a grouped query\n(see `group_by/3`). `having` groups the query even if the query has no\n`group_by` expression.","ref":"Ecto.Query.html#having/3"},{"type":"macro","title":"Keywords example - Ecto.Query.having/3","doc":"# Returns the number of posts in each category where the\n    # average number of comments is above ten\n    from(p in Post,\n      group_by: p.category,\n      having: avg(p.num_comments) > 10,\n      select: {p.category, count(p.id)})","ref":"Ecto.Query.html#having/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.having/3","doc":"Post\n    |> group_by([p], p.category)\n    |> having([p], avg(p.num_comments) > 10)\n    |> select([p], count(p.id))","ref":"Ecto.Query.html#having/3-expressions-example"},{"type":"macro","title":"Ecto.Query.intersect/2","doc":"An intersect query expression.\n\nTakes the overlap of the result sets of multiple queries. The\n`select` of each query must be exactly the same, with the same\ntypes in the same order.\n\n> #","ref":"Ecto.Query.html#intersect/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.intersect/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nIntersect expression returns only unique rows as if each query returned\ndistinct results. This may cause a performance penalty. If you need\nto take the intersection of multiple result sets without\nremoving duplicate rows consider using `intersect_all/2`.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the set difference. `order_by`\nmust be specified in one of the following ways, since the intersection\nof two or more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the intersection fields.\n  - Wrap the intersection in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#intersect/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.intersect/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, intersect: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    intersect_query = from c in Customer, select: c.city, intersect: ^supplier_query\n    from s in subquery(intersect_query), order_by: s.city","ref":"Ecto.Query.html#intersect/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.intersect/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> intersect(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    intersect(customer_query, ^supplier_query)","ref":"Ecto.Query.html#intersect/2-expressions-examples"},{"type":"macro","title":"Ecto.Query.intersect_all/2","doc":"An intersect query expression.\n\nTakes the overlap of the result sets of multiple queries. The\n`select` of each query must be exactly the same, with the same\ntypes in the same order.\n\n> #","ref":"Ecto.Query.html#intersect_all/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.intersect_all/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the set difference. `order_by`\nmust be specified in one of the following ways, since the intersection\nof two or more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the intersection fields.\n  - Wrap the intersection in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#intersect_all/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.intersect_all/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, intersect_all: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    intersect_all_query = from c in Customer, select: c.city, intersect_all: ^supplier_query\n    from s in subquery(intersect_all_query), order_by: s.city","ref":"Ecto.Query.html#intersect_all/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.intersect_all/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> intersect_all(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    intersect_all(customer_query, ^supplier_query)","ref":"Ecto.Query.html#intersect_all/2-expressions-examples"},{"type":"macro","title":"Ecto.Query.join/5","doc":"A join query expression.\n\nReceives a source that is to be joined to the query and a condition for\nthe join. The join condition can be any expression that evaluates\nto a boolean value. The qualifier must be one of `:inner`, `:left`,\n`:right`, `:cross`, `:cross_lateral`, `:full`, `:inner_lateral`, `:left_lateral`,\n`:array` or `:left_array`.\n\nFor a keyword query the `:join` keyword can be changed to `:inner_join`,\n`:left_join`, `:right_join`, `:cross_join`, `:cross_lateral_join`, `:full_join`, `:inner_lateral_join`,\n`:left_lateral_join`, `:array_join` or `:left_array_join`. `:join` is equivalent to `:inner_join`.\n\nCurrently it is possible to join on:\n\n  * an `Ecto.Schema`, such as `p in Post`\n  * an interpolated Ecto query with zero or more `where` clauses,\n    such as `c in ^(from \"posts\", where: [public: true])`\n  * an association, such as `c in assoc(post, :comments)`\n  * a subquery, such as `c in subquery(another_query)`\n  * a query fragment, such as `c in fragment(\"SOME COMPLEX QUERY\")`,\n    see \"Joining with fragments\" below.","ref":"Ecto.Query.html#join/5"},{"type":"macro","title":"Options - Ecto.Query.join/5","doc":"Each join accepts the following options:\n\n  * `:on` - a query expression or keyword list to filter the join, defaults to `true`\n  * `:as` - a named binding for the join\n  * `:prefix` - the prefix to be used for the join when issuing a database query\n  * `:hints` - a string or a list of strings to be used as database hints\n\nIn the keyword query syntax, those options must be given immediately\nafter the join. In the expression syntax, the options are given as\nthe fifth argument.\n\n> #","ref":"Ecto.Query.html#join/5-options"},{"type":"macro","title":"Unspecified join condition {: .warning} - Ecto.Query.join/5","doc":">\n> Leaving the `:on` option unspecified while performing a join\n> that is not a cross join will trigger a warning. This is to\n> help users avoid performing expensive cross joins when they don't\n> mean to. If the behaviour is desired, you may remove the warning by\n> changing to a cross join or explicitly setting `on: true`. If\n> the behaviour is not desired, you should specify the appropriate\n> join condition.","ref":"Ecto.Query.html#join/5-unspecified-join-condition-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.join/5","doc":"from c in Comment,\n      join: p in Post,\n      on: p.id == c.post_id,\n      select: {p.title, c.text}\n\n    from p in Post,\n      left_join: c in assoc(p, :comments),\n      select: {p, c}\n\nKeywords can also be given or interpolated as part of `on`:\n\n    from c in Comment,\n      join: p in Post,\n      on: [id: c.post_id],\n      select: {p.title, c.text}\n\nAny key in `on` will apply to the currently joined expression.\n\nIt is also possible to interpolate an Ecto query on the right-hand side\nof `in`. For example, the query above can also be written as:\n\n    posts = Post\n    from c in Comment,\n      join: p in ^posts,\n      on: [id: c.post_id],\n      select: {p.title, c.text}\n\nThe above is specially useful to dynamically join on existing\nqueries, for example, to dynamically choose a source, or by\nchoosing between public posts or posts that have been recently\npublished:\n\n    posts =\n      if params[\"drafts\"] do\n        from p in Post, where: [drafts: true]\n      else\n        from p in Post, where: [public: true]\n      end\n\n    from c in Comment,\n      join: p in ^posts, on: [id: c.post_id],\n      select: {p.title, c.text}\n\nOnly simple queries with `where` expressions can be interpolated\nin a join.","ref":"Ecto.Query.html#join/5-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.join/5","doc":"Comment\n    |> join(:inner, [c], p in Post, on: c.post_id == p.id)\n    |> select([c, p], {p.title, c.text})\n\n    Post\n    |> join(:left, [p], c in assoc(p, :comments))\n    |> select([p, c], {p, c})\n\n    Post\n    |> join(:left, [p], c in Comment, on: c.post_id == p.id and c.is_visible == true)\n    |> select([p, c], {p, c})","ref":"Ecto.Query.html#join/5-expressions-examples"},{"type":"macro","title":"Joining with fragments - Ecto.Query.join/5","doc":"When you need to join on a complex query, Ecto supports fragments in joins:\n\n    Comment\n    |> join(:inner, [c], p in fragment(\"SOME COMPLEX QUERY\", c.id, ^some_param))\n\nAlthough using fragments in joins is discouraged in favor of Ecto\nQuery syntax, they are necessary when writing lateral joins as\nlateral joins require a subquery that refer to previous bindings:\n\n    Game\n    |> join(:inner_lateral, [g], gs in fragment(\"SELECT * FROM games_sold AS gs WHERE gs.game_id = ? ORDER BY gs.sold_on LIMIT 2\", g.id))\n    |> select([g, gs], {g.name, gs.sold_on})\n\nNote that the `join` does not automatically wrap the fragment in\nparentheses, since some expressions require parens and others\nrequire no parens. Therefore, in cases such as common table\nexpressions, you will have to explicitly wrap the fragment content\nin parens.","ref":"Ecto.Query.html#join/5-joining-with-fragments"},{"type":"macro","title":"Hints - Ecto.Query.join/5","doc":"`join` also supports table hints, as found in databases such as\n[MySQL](https://dev.mysql.com/doc/refman/8.0/en/index-hints.html),\n[MSSQL](https://docs.microsoft.com/en-us/sql/t-sql/queries/hints-transact-sql-table?view=sql-server-2017) and\n[Clickhouse](https://clickhouse.tech/docs/en/sql-reference/statements/select/sample/).\n\nFor example, a developer using MySQL may write:\n\n    from p in Post,\n      join: c in Comment,\n      hints: [\"USE INDEX FOO\", \"USE INDEX BAR\"],\n      where: p.id == c.post_id,\n      select: c\n\nKeep in mind you want to use hints rarely, so don't forget to read the database\ndisclaimers about such functionality.\n\nJoin hints must be static compile-time strings when they are specified as (list of) strings.","ref":"Ecto.Query.html#join/5-hints"},{"type":"macro","title":"Array joins - Ecto.Query.join/5","doc":"The `:array` and `:left_array` qualifiers can be used to join with array\ncolumns in [Clickhouse:](https://clickhouse.com/docs/en/sql-reference/statements/select/array-join)\n\n    from at in \"arrays_test\",\n      array_join: a in \"arr\",\n      select: %{s: at.s, arr: a}\n\nNote that only the columns in the base table (i.e. the table referenced in `FROM`) can be used in the array join.","ref":"Ecto.Query.html#join/5-array-joins"},{"type":"function","title":"Ecto.Query.last/2","doc":"Restricts the query to return the last result ordered by primary key.\n\nThe query ordering will be automatically reversed, with ASC\ncolumns becoming DESC columns (and vice-versa) and limit is set\nto 1. If there is no ordering, the query will be automatically\nordered decreasingly by primary key.","ref":"Ecto.Query.html#last/2"},{"type":"function","title":"Examples - Ecto.Query.last/2","doc":"Post |> last |> Repo.one\n    query |> last(:inserted_at) |> Repo.one","ref":"Ecto.Query.html#last/2-examples"},{"type":"macro","title":"Ecto.Query.limit/3","doc":"A limit query expression.\n\nLimits the number of rows returned from the result. Can be any expression but\nhas to evaluate to an integer value and it can't include any field.\n\nIf `limit` is given twice, it overrides the previous value.","ref":"Ecto.Query.html#limit/3"},{"type":"macro","title":"Keywords example - Ecto.Query.limit/3","doc":"from(u in User, where: u.id == ^current_user, limit: 1)","ref":"Ecto.Query.html#limit/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.limit/3","doc":"User |> where([u], u.id == ^current_user) |> limit(1)","ref":"Ecto.Query.html#limit/3-expressions-example"},{"type":"macro","title":"Ecto.Query.lock/3","doc":"A lock query expression.\n\nProvides support for row-level pessimistic locking using\n`SELECT ... FOR UPDATE` or other, database-specific, locking clauses.\n`expr` can be any expression but has to evaluate to a boolean value or to a\nstring and it can't include any fields.\n\nIf `lock` is used more than once, the last one used takes precedence.\n\nEcto also supports [optimistic\nlocking](https://en.wikipedia.org/wiki/Optimistic_concurrency_control) but not\nthrough queries. For more information on optimistic locking, have a look at\nthe `Ecto.Changeset.optimistic_lock/3` function.","ref":"Ecto.Query.html#lock/3"},{"type":"macro","title":"Keywords example - Ecto.Query.lock/3","doc":"from(u in User, where: u.id == ^current_user, lock: \"FOR SHARE NOWAIT\")","ref":"Ecto.Query.html#lock/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.lock/3","doc":"User |> where([u], u.id == ^current_user) |> lock(\"FOR SHARE NOWAIT\")","ref":"Ecto.Query.html#lock/3-expressions-example"},{"type":"macro","title":"Ecto.Query.offset/3","doc":"An offset query expression.\n\nOffsets the number of rows selected from the result. Can be any expression\nbut it must evaluate to an integer value and it can't include any field.\n\nIf `offset` is given twice, it overrides the previous value.","ref":"Ecto.Query.html#offset/3"},{"type":"macro","title":"Keywords example - Ecto.Query.offset/3","doc":"# Get all posts on page 4\n    from(p in Post, limit: 10, offset: 30)","ref":"Ecto.Query.html#offset/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.offset/3","doc":"Post |> limit(10) |> offset(30)","ref":"Ecto.Query.html#offset/3-expressions-example"},{"type":"macro","title":"Ecto.Query.or_having/3","doc":"An OR having query expression.\n\nLike `having` but combines with the previous expression by using\n`OR`. `or_having` behaves for `having` the same way `or_where`\nbehaves for `where`.","ref":"Ecto.Query.html#or_having/3"},{"type":"macro","title":"Keywords example - Ecto.Query.or_having/3","doc":"# Augment a previous group_by with a having condition.\n    from(p in query, or_having: avg(p.num_comments) > 10)","ref":"Ecto.Query.html#or_having/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.or_having/3","doc":"# Augment a previous group_by with a having condition.\n    Post |> or_having([p], avg(p.num_comments) > 10)","ref":"Ecto.Query.html#or_having/3-expressions-example"},{"type":"macro","title":"Ecto.Query.or_where/3","doc":"An OR where query expression.\n\nBehaves exactly the same as `where` except it combines with any previous\nexpression by using an `OR`. All expressions have to evaluate to a boolean\nvalue.\n\n`or_where` also accepts a keyword list where each key is a field to be\ncompared with the given value. Each key-value pair will be combined\nusing `AND`, exactly as in `where`.","ref":"Ecto.Query.html#or_where/3"},{"type":"macro","title":"Keywords example - Ecto.Query.or_where/3","doc":"from(c in City, where: [country: \"Sweden\"], or_where: [country: \"Brazil\"])\n\nIf interpolating keyword lists, the keyword list entries are combined\nusing ANDs and joined to any existing expression with an OR:\n\n    filters = [country: \"USA\", name: \"New York\"]\n    from(c in City, where: [country: \"Sweden\"], or_where: ^filters)\n\nis equivalent to:\n\n    from c in City, where: (c.country == \"Sweden\") or\n                           (c.country == \"USA\" and c.name == \"New York\")\n\nThe behaviour above is by design to keep the changes between `where`\nand `or_where` minimal. Plus, if you have a keyword list and you\nwould like each pair to be combined using `or`, it can be easily done\nwith `Enum.reduce/3`:\n\n    filters = [country: \"USA\", is_tax_exempt: true]\n    Enum.reduce(filters, City, fn {key, value}, query ->\n      from q in query, or_where: field(q, ^key) == ^value\n    end)\n\nwhich will be equivalent to:\n\n    from c in City, or_where: (c.country == \"USA\"), or_where: c.is_tax_exempt == true","ref":"Ecto.Query.html#or_where/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.or_where/3","doc":"City |> where([c], c.country == \"Sweden\") |> or_where([c], c.country == \"Brazil\")","ref":"Ecto.Query.html#or_where/3-expressions-example"},{"type":"macro","title":"Ecto.Query.order_by/3","doc":"An order by query expression.\n\nOrders the fields based on one or more fields. It accepts a single field\nor a list of fields. The default direction is ascending (`:asc`) and can be\ncustomized in a keyword list as one of the following:\n\n  * `:asc`\n  * `:asc_nulls_last`\n  * `:asc_nulls_first`\n  * `:desc`\n  * `:desc_nulls_last`\n  * `:desc_nulls_first`\n\nThe `*_nulls_first` and `*_nulls_last` variants are not supported by all\ndatabases. While all databases default to ascending order, the choice of\n\"nulls first\" or \"nulls last\" is specific to each database implementation.\n\n`order_by` may be invoked or listed in a query many times. New expressions\nare appended to the existing ones.\n\n`order_by` also accepts a list of atoms where each atom refers to a field in\nsource or a keyword list where the direction is given as key and the field\nto order as value.","ref":"Ecto.Query.html#order_by/3"},{"type":"macro","title":"Keywords examples - Ecto.Query.order_by/3","doc":"from(c in City, order_by: c.name, order_by: c.population)\n    from(c in City, order_by: [c.name, c.population])\n    from(c in City, order_by: [asc: c.name, desc: c.population])\n\n    from(c in City, order_by: [:name, :population])\n    from(c in City, order_by: [asc: :name, desc_nulls_first: :population])\n\nA keyword list can also be interpolated:\n\n    values = [asc: :name, desc_nulls_first: :population]\n    from(c in City, order_by: ^values)\n\nA fragment can also be used:\n\n    from c in City, order_by: [\n      # A deterministic shuffled order\n      fragment(\"? % ? DESC\", c.id, ^modulus),\n      desc: c.id,\n    ]\n\nIt's also possible to order by an aliased or calculated column:\n\n    from(c in City,\n      select: %{\n        name: c.name,\n        total_population:\n          fragment(\n            \"COALESCE(?, ?) + ? AS total_population\",\n            c.animal_population,\n            0,\n            c.human_population\n          )\n      },\n      order_by: [\n        # based on `AS total_population` in the previous fragment\n        {:desc, fragment(\"total_population\")}\n      ]\n    )","ref":"Ecto.Query.html#order_by/3-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.order_by/3","doc":"City |> order_by([c], asc: c.name, desc: c.population)\n    City |> order_by(asc: :name) # Sorts by the cities name\n    City |> order_by(^order_by_param) # Keyword list","ref":"Ecto.Query.html#order_by/3-expressions-examples"},{"type":"macro","title":"Ecto.Query.preload/3","doc":"Preloads the associations into the result set.\n\nImagine you have a schema `Post` with a `has_many :comments`\nassociation and you execute the following query:\n\n    Repo.all from p in Post, preload: [:comments]\n\nThe example above will fetch all posts from the database and then do\na separate query returning all comments associated with the given posts.\nThe comments are then processed and associated to each returned `post`\nunder the `comments` field.\n\nOften times, you may want posts and comments to be selected and\nfiltered in the same query. For such cases, you can explicitly tell\nan existing join to be preloaded into the result set:\n\n    Repo.all from p in Post,\n               join: c in assoc(p, :comments),\n               where: c.published_at > p.updated_at,\n               preload: [comments: c]\n\nIn the example above, instead of issuing a separate query to fetch\ncomments, Ecto will fetch posts and comments in a single query and\nthen do a separate pass associating each comment to its parent post.\nTherefore, instead of returning `number_of_posts * number_of_comments`\nresults, like a `join` would, it returns only posts with the `comments`\nfields properly filled in.\n\nNested associations can also be preloaded in both formats:\n\n    Repo.all from p in Post,\n               preload: [comments: :likes]\n\n    Repo.all from p in Post,\n               join: c in assoc(p, :comments),\n               join: l in assoc(c, :likes),\n               where: l.inserted_at > c.updated_at,\n               preload: [comments: {c, likes: l}]\n\nApplying a limit to the association can be achieved with `inner_lateral_join`:\n\n    Repo.all from p in Post, as: :post,\n               join: c in assoc(p, :comments),\n               inner_lateral_join: top_five in subquery(\n                 from Comment,\n                 where: [post_id: parent_as(:post).id],\n                 order_by: :popularity,\n                 limit: 5,\n                 select: [:id]\n               ), on: top_five.id == c.id,\n               preload: [comments: c]\n\nPreloaded joins can also be specified dynamically using `dynamic`:\n\n    preloads = [comments: dynamic([comments: c], c)]\n\n    Repo.all from p in Post,\n               join: c in assoc(p, :comments),\n               as: :comments,\n               where: c.published_at > p.updated_at,\n               preload: ^preloads\n\nSee \"`preload`\" in the documentation for `dynamic/2` for more\ndetails.","ref":"Ecto.Query.html#preload/3"},{"type":"macro","title":"Preload queries - Ecto.Query.preload/3","doc":"Preload also allows queries to be given, allowing you to filter or\ncustomize how the preloads are fetched:\n\n    comments_query = from c in Comment, order_by: c.published_at\n    Repo.all from p in Post, preload: [comments: ^comments_query]\n\nThe example above will issue two queries, one for loading posts and\nthen another for loading the comments associated with the posts.\nComments will be ordered by `published_at`.\n\nWhen specifying a preload query, you can still preload the associations of\nthose records. For instance, you could preload an author's published posts and\nthe comments on those posts:\n\n    posts_query = from p in Post, where: p.state == :published\n    Repo.all from a in Author, preload: [posts: ^{posts_query, [:comments]}]\n\nIf you prefer, you can also add additional preloads directly in the\n`posts_query`:\n\n    posts_query =\n      from p in Post, where: p.state == :published, preload: :related_posts\n\nThe same can be written as pipe based query:\n\n    posts_query =\n      Post |> where([p], p.state == :published) |> preload(:related_posts)\n\nNote: keep in mind operations like limit and offset in the preload\nquery will affect the whole result set and not each association. For\nexample, the query below:\n\n    comments_query = from c in Comment, order_by: c.popularity, limit: 5\n    Repo.all from p in Post, preload: [comments: ^comments_query]\n\nwon't bring the top of comments per post. Rather, it will only bring\nthe 5 top comments across all posts. Instead, use a window:\n\n    ranking_query =\n      from c in Comment,\n      select: %{id: c.id, row_number: over(row_number(), :posts_partition)},\n      windows: [posts_partition: [partition_by: :post_id, order_by: :popularity]]\n\n    comments_query =\n      from c in Comment,\n      join: r in subquery(ranking_query),\n      on: c.id == r.id and r.row_number  comments->authors`), the query will only customize the\nrelationship between comments and authors, even if preloaded through posts.","ref":"Ecto.Query.html#preload/3-preload-queries"},{"type":"macro","title":"Preload functions - Ecto.Query.preload/3","doc":"Preload also allows functions to be given. In such cases, the function\nreceives the IDs of the parent association and it must return the associated\ndata. Ecto then will map this data and sort it by the relationship key:\n\n    comment_preloader = fn post_ids -> fetch_comments_by_post_ids(post_ids) end\n    Repo.all from p in Post, preload: [comments: ^comment_preloader]\n\nThis is useful when the whole dataset was already loaded or must be\nexplicitly fetched from elsewhere. The IDs received by the preloading\nfunction and the result returned depends on the association type:\n\n  * For `has_many` and `belongs_to` - the function receives the IDs of\n    the parent association and it must return a list of maps or structs\n    with the associated entries. The associated map/struct must contain\n    the \"foreign_key\" field. For example, if a post has many comments,\n    when preloading the comments with a custom function, the function\n    will receive a list of \"post_ids\" as the argument and it must return\n    maps or structs representing the comments. The maps/structs must\n    include the `:post_id` field\n\n  * For `has_many :through` - it behaves similarly to a regular `has_many`\n    but note that the IDs received are of the last association. Imagine,\n    for example, a post has many comments and each comment has an author.\n    Therefore, a post may have many comments_authors, written as\n    `has_many :comments_authors, through: [:comments, :author]`. When\n    preloading authors with a custom function via `:comments_authors`,\n    the function will receive the IDs of the authors as the last step\n\n  * For `many_to_many` - the function receives the IDs of the parent\n    association and it must return a tuple with the parent id as the first\n    element and the association map or struct as the second. For example,\n    if a post has many tags, when preloading the tags with a custom\n    function, the function will receive a list of \"post_ids\" as the argument\n    and it must return a tuple in the format of `{post_id, tag}`\n\nIf you want to reset the loaded fields, see `Ecto.reset_fields/2`.","ref":"Ecto.Query.html#preload/3-preload-functions"},{"type":"macro","title":"Keywords example - Ecto.Query.preload/3","doc":"# Returns all posts, their associated comments, and the associated\n    # likes for those comments.\n    from(p in Post,\n      preload: [comments: :likes],\n      select: p\n    )","ref":"Ecto.Query.html#preload/3-keywords-example"},{"type":"macro","title":"Expressions examples - Ecto.Query.preload/3","doc":"Post |> preload(:comments) |> select([p], p)\n\n    Post\n    |> join(:left, [p], c in assoc(p, :comments))\n    |> preload([p, c], [:user, comments: c])\n    |> select([p], p)","ref":"Ecto.Query.html#preload/3-expressions-examples"},{"type":"macro","title":"Ecto.Query.prepend_order_by/3","doc":"Same as `order_by/3` except new expressions will be prepended to existing ones.","ref":"Ecto.Query.html#prepend_order_by/3"},{"type":"function","title":"Ecto.Query.put_query_prefix/2","doc":"Puts the given prefix in a query.","ref":"Ecto.Query.html#put_query_prefix/2"},{"type":"function","title":"Ecto.Query.recursive_ctes/2","doc":"Enables or disables recursive mode for CTEs.\n\nAccording to the SQL standard it affects all CTEs in the query, not individual ones.\n\nSee `with_cte/3` on example of how to build a query with a recursive CTE.","ref":"Ecto.Query.html#recursive_ctes/2"},{"type":"function","title":"Ecto.Query.reverse_order/1","doc":"Reverses the ordering of the query.\n\nASC columns become DESC columns (and vice-versa). If the query\nhas no `order_by`s, it orders by the inverse of the primary key.","ref":"Ecto.Query.html#reverse_order/1"},{"type":"function","title":"Examples - Ecto.Query.reverse_order/1","doc":"query |> reverse_order() |> Repo.one()\n    Post |> order(asc: :id) |> reverse_order() == Post |> order(desc: :id)","ref":"Ecto.Query.html#reverse_order/1-examples"},{"type":"macro","title":"Ecto.Query.select/3","doc":"A select query expression.\n\nSelects which fields will be selected from the schema and any transformations\nthat should be performed on the fields. Any expression that is accepted in a\nquery can be a select field.\n\nSelect also allows each expression to be wrapped in lists, tuples or maps as\nshown in the examples below. A full schema can also be selected.\n\nThere can only be one select expression in a query, if the select expression\nis omitted, the query will by default select the full schema. If `select` is\ngiven more than once, an error is raised. Use `exclude/2` if you would like\nto remove a previous select for overriding or see `select_merge/3` for a\nlimited version of `select` that is composable and can be called multiple\ntimes.\n\n`select` also accepts a list of atoms where each atom refers to a field in\nthe source to be selected.","ref":"Ecto.Query.html#select/3"},{"type":"macro","title":"Keywords examples - Ecto.Query.select/3","doc":"from(c in City, select: c) # returns the schema as a struct\n    from(c in City, select: {c.name, c.population})\n    from(c in City, select: [c.name, c.county])\n    from(c in City, select: %{n: c.name, answer: 42})\n    from(c in City, select: %{c | alternative_name: c.name})\n    from(c in City, select: %Data{name: c.name})\n\nIt is also possible to select a struct and limit the returned\nfields at the same time:\n\n    from(City, select: [:name])\n\nThe syntax above is equivalent to:\n\n    from(city in City, select: struct(city, [:name]))\n\nYou can also write:\n\n    from(city in City, select: map(city, [:name]))\n\nIf you want a map with only the selected fields to be returned.\n\nTo select a struct but omit only given fields, you can\noverride them with `nil` or another default value:\n\n    from(city in City, select: %{city | geojson: nil, text: \" \"})\n\nFor more information, read the docs for `Ecto.Query.API.struct/2`\nand `Ecto.Query.API.map/2`.","ref":"Ecto.Query.html#select/3-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.select/3","doc":"City |> select([c], c)\n    City |> select([c], {c.name, c.country})\n    City |> select([c], %{\"name\" => c.name})\n    City |> select([:name])\n    City |> select([c], struct(c, [:name]))\n    City |> select([c], map(c, [:name]))\n    City |> select([c], %{c | geojson: nil, text: \" \"})","ref":"Ecto.Query.html#select/3-expressions-examples"},{"type":"macro","title":"Dynamic parts - Ecto.Query.select/3","doc":"Dynamics can be part of a `select` as values in a map that must be interpolated\nat the root level:\n\n    period = if monthly?, do: dynamic([p], p.month), else: dynamic([p], p.date)\n    metric = if distance?, do: dynamic([p], p.distance), else: dynamic([p], p.time)\n\n    from(c in City, select: ^%{period: period, metric: metric})","ref":"Ecto.Query.html#select/3-dynamic-parts"},{"type":"macro","title":"Ecto.Query.select_merge/3","doc":"Mergeable select query expression.\n\nThis macro is similar to `select/3` except it may be specified\nmultiple times as long as every entry is a map. This is useful\nfor merging and composing selects. For example:\n\n    query = from p in Post, select: %{}\n\n    query =\n      if include_title? do\n        from p in query, select_merge: %{title: p.title}\n      else\n        query\n      end\n\n    query =\n      if include_visits? do\n        from p in query, select_merge: %{visits: p.visits}\n      else\n        query\n      end\n\nIn the example above, the query is built little by little by merging\ninto a final map. If both conditions above are true, the final query\nwould be equivalent to:\n\n    from p in Post, select: %{title: p.title, visits: p.visits}\n\nIf `:select_merge` is called and there is no value selected previously,\nit will default to the source, `p` in the example above.\n\nThe argument given to `:select_merge` must always be a map. The value\nbeing merged on must be a struct or a map. If it is a struct, the fields\nmerged later on must be part of the struct, otherwise an error is raised.\n\nIf the argument to `:select_merge` is a constructed struct\n(`Ecto.Query.API.struct/2`) or map (`Ecto.Query.API.map/2`) where the source\nto struct or map may be a `nil` value (as in an outer join), the source will\nbe returned unmodified.\n\n    query =\n      Post\n      |> join(:left, [p], t in Post.Translation,\n        on: t.post_id == p.id and t.locale == ^\"en\"\n      )\n      |> select_merge([_p, t], map(t, ^~w(title summary)a))\n\nIf there is no English translation for the post, the untranslated post\n`title` will be returned and `summary` will be `nil`. If there is, both\n`title` and `summary` will be the value from `Post.Translation`.\n\n`select_merge` cannot be used to set fields in associations, as\nassociations are always loaded later, overriding any previous value.\n\nDynamics can be part of a `select_merge` as values in a map that must be\ninterpolated at the root level. The rules for merging detailed above apply.\nThis allows merging dynamic values into previously selected maps and structs.","ref":"Ecto.Query.html#select_merge/3"},{"type":"function","title":"Ecto.Query.subquery/2","doc":"Converts a query into a subquery.\n\nIf a subquery is given, returns the subquery itself.\nIf any other value is given, it is converted to a query via\n`Ecto.Queryable` and wrapped in the `Ecto.SubQuery` struct.\n\n`subquery` is supported in:\n\n  * `from`,\n  * `join`,\n  * `where`, in the form `p.x in subquery(q)`,\n  * `select` and `select_merge`, in the form of `%{field: subquery(...)}`.","ref":"Ecto.Query.html#subquery/2"},{"type":"function","title":"Examples - Ecto.Query.subquery/2","doc":"# Get the average salary of the top 10 highest salaries\n    query = from Employee, order_by: [desc: :salary], limit: 10\n    from e in subquery(query), select: avg(e.salary)\n\nA prefix can be specified for a subquery, similar to standard repo operations:\n\n    query = from Employee, order_by: [desc: :salary], limit: 10\n    from e in subquery(query, prefix: \"my_prefix\"), select: avg(e.salary)\n\n\nSubquery can also be used in a `join` expression.\n\n    UPDATE posts\n      SET sync_started_at = $1\n      WHERE id IN (\n        SELECT id FROM posts\n          WHERE synced = false AND (sync_started_at IS NULL OR sync_started_at < $1)\n          LIMIT $2\n      )\n\nWe can write it as a join expression:\n\n    subset = from(p in Post,\n      where: p.synced == false and\n               (is_nil(p.sync_started_at) or p.sync_started_at < ^min_sync_started_at),\n      limit: ^batch_size\n    )\n\n    Repo.update_all(\n      from(p in Post, join: s in subquery(subset), on: s.id == p.id),\n      set: [sync_started_at: NaiveDateTime.utc_now()]\n    )\n\nOr as a `where` condition:\n\n    subset_ids = from(p in subset, select: p.id)\n    Repo.update_all(\n      from(p in Post, where: p.id in subquery(subset_ids)),\n      set: [sync_started_at: NaiveDateTime.utc_now()]\n    )\n\nIf you need to refer to a parent binding which is not known when writing the subquery,\nyou can use `parent_as` as shown in the examples under \"Named bindings\" in this module doc.\n\nYou can also use subquery directly in `select` and `select_merge`:\n\n    comments_count = from(c in Comment, where: c.post_id == parent_as(:post).id, select: count())\n    from(p in Post, as: :post, select: %{id: p.id, comments: subquery(comments_count)})","ref":"Ecto.Query.html#subquery/2-examples"},{"type":"macro","title":"Ecto.Query.union/2","doc":"A union query expression.\n\nCombines result sets of multiple queries. The `select` of each query\nmust be exactly the same, with the same types in the same order.\n\n> #","ref":"Ecto.Query.html#union/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.union/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nUnion expression returns only unique rows as if each query returned\ndistinct results. This may cause a performance penalty. If you need\nto combine multiple result sets without removing duplicate rows\nconsider using `union_all/2`.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the union. `order_by` must\nbe specified in one of the following ways, since the union of two\nor more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the union fields.\n  - Wrap the union in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#union/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.union/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, union: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    union_query = from c in Customer, select: c.city, union: ^supplier_query\n    from s in subquery(union_query), order_by: s.city","ref":"Ecto.Query.html#union/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.union/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> union(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    union(customer_query, ^supplier_query)","ref":"Ecto.Query.html#union/2-expressions-examples"},{"type":"macro","title":"Ecto.Query.union_all/2","doc":"A union all query expression.\n\nCombines result sets of multiple queries. The `select` of each query\nmust be exactly the same, with the same types in the same order.\n\n> #","ref":"Ecto.Query.html#union_all/2"},{"type":"macro","title":"Selecting literal atoms {: .warning} - Ecto.Query.union_all/2","doc":">\n> When selecting a literal atom, its value must be the same across\n> all queries. Otherwise, the value from the parent query will be\n> applied to all other queries.\n\nNote that the operations `order_by`, `limit` and `offset` of the\ncurrent `query` apply to the result of the union. `order_by` must\nbe specified in one of the following ways, since the union of two\nor more queries is not automatically aliased:\n\n  - Use `Ecto.Query.API.fragment/1` to pass an `order_by` statement that directly access the union fields.\n  - Wrap the union in a subquery and refer to the binding of the subquery.","ref":"Ecto.Query.html#union_all/2-selecting-literal-atoms-warning"},{"type":"macro","title":"Keywords examples - Ecto.Query.union_all/2","doc":"# Unordered result\n    supplier_query = from s in Supplier, select: s.city\n    from c in Customer, select: c.city, union_all: ^supplier_query\n\n    # Ordered result\n    supplier_query = from s in Supplier, select: s.city\n    union_all_query = from c in Customer, select: c.city, union_all: ^supplier_query\n    from s in subquery(union_all_query), order_by: s.city","ref":"Ecto.Query.html#union_all/2-keywords-examples"},{"type":"macro","title":"Expressions examples - Ecto.Query.union_all/2","doc":"# Unordered result\n    supplier_query = Supplier |> select([s], s.city)\n    Customer |> select([c], c.city) |> union_all(^supplier_query)\n\n    # Ordered result\n    customer_query = Customer |> select([c], c.city) |> order_by(fragment(\"city\"))\n    supplier_query = Supplier |> select([s], s.city)\n    union_all(customer_query, ^supplier_query)","ref":"Ecto.Query.html#union_all/2-expressions-examples"},{"type":"macro","title":"Ecto.Query.update/3","doc":"An update query expression.\n\nUpdates are used to update the filtered entries. In order for\nupdates to be applied, `c:Ecto.Repo.update_all/3` must be invoked.","ref":"Ecto.Query.html#update/3"},{"type":"macro","title":"Keywords example - Ecto.Query.update/3","doc":"from(u in User, update: [set: [name: \"new name\"]])","ref":"Ecto.Query.html#update/3-keywords-example"},{"type":"macro","title":"Expressions examples - Ecto.Query.update/3","doc":"User |> update([u], set: [name: \"new name\"])\n    User |> update(set: [name: \"new name\"])","ref":"Ecto.Query.html#update/3-expressions-examples"},{"type":"macro","title":"Interpolation - Ecto.Query.update/3","doc":"new_name = \"new name\"\n    from(u in User, update: [set: [name: ^new_name]])\n\n    new_name = \"new name\"\n    from(u in User, update: [set: [name: fragment(\"upper(?)\", ^new_name)]])","ref":"Ecto.Query.html#update/3-interpolation"},{"type":"macro","title":"Operators - Ecto.Query.update/3","doc":"The update expression in Ecto supports the following operators:\n\n  * `set` - sets the given field in the table to the given value\n\n        from(u in User, update: [set: [name: \"new name\"]])\n\n  * `inc` - increments (or decrements if the value is negative) the given field in the table by the given value\n\n        from(u in User, update: [inc: [accesses: 1]])\n\n  * `push` - pushes (appends) the given value to the end of the array field\n\n        from(u in User, update: [push: [tags: \"cool\"]])\n\n  * `pull` - pulls (removes) the given value from the array field\n\n        from(u in User, update: [pull: [tags: \"not cool\"]])","ref":"Ecto.Query.html#update/3-operators"},{"type":"macro","title":"Ecto.Query.where/3","doc":"An AND where query expression.\n\n`where` expressions are used to filter the result set. If there is more\nthan one where expression, they are combined with an `and` operator. All\nwhere expressions have to evaluate to a boolean value.\n\n`where` also accepts a keyword list where the field given as key is going to\nbe compared with the given value. The fields will always refer to the source\ngiven in `from`.","ref":"Ecto.Query.html#where/3"},{"type":"macro","title":"Keywords example - Ecto.Query.where/3","doc":"from(c in City, where: c.country == \"Sweden\")\n    from(c in City, where: [country: \"Sweden\"])\n\nIt is also possible to interpolate the whole keyword list, allowing you to\ndynamically filter the source:\n\n    filters = [country: \"Sweden\"]\n    from(c in City, where: ^filters)","ref":"Ecto.Query.html#where/3-keywords-example"},{"type":"macro","title":"Expressions examples - Ecto.Query.where/3","doc":"City |> where([c], c.country == \"Sweden\")\n    City |> where(country: \"Sweden\")","ref":"Ecto.Query.html#where/3-expressions-examples"},{"type":"macro","title":"Ecto.Query.windows/3","doc":"Defines windows which can be used with `Ecto.Query.WindowAPI`.\n\nReceives a keyword list where keys are names of the windows\nand values are a keyword list with window expressions.","ref":"Ecto.Query.html#windows/3"},{"type":"macro","title":"Examples - Ecto.Query.windows/3","doc":"# Compare each employee's salary with the average salary in his or her department\n    from e in Employee,\n      select: {e.depname, e.empno, e.salary, over(avg(e.salary), :department)},\n      windows: [department: [partition_by: e.depname]]\n\nIn the example above, we get the average salary per department.\n`:department` is the window name, partitioned by `e.depname`\nand `avg/1` is the window function. For more information\non windows functions, see `Ecto.Query.WindowAPI`.","ref":"Ecto.Query.html#windows/3-examples"},{"type":"macro","title":"Window expressions - Ecto.Query.windows/3","doc":"The following keys are allowed when specifying a window.\n\n### :partition_by\n\nA list of fields to partition the window by, for example:\n\n    windows: [department: [partition_by: e.depname]]\n\nA list of atoms can also be interpolated for dynamic partitioning:\n\n    fields = [:depname, :year]\n    windows: [dynamic_window: [partition_by: ^fields]]\n\n### :order_by\n\nA list of fields to order the window by, for example:\n\n    windows: [ordered_names: [order_by: e.name]]\n\nIt works exactly as the keyword query version of `order_by/3`.\n\n### :frame\n\nA fragment which defines the frame for window functions.","ref":"Ecto.Query.html#windows/3-window-expressions"},{"type":"macro","title":"Examples - Ecto.Query.windows/3","doc":"# Compare each employee's salary for each month with his average salary for previous 3 months\n    from p in Payroll,\n      select: {p.empno, p.date, p.salary, over(avg(p.salary), :prev_months)},\n      windows: [prev_months: [partition_by: p.empno, order_by: p.date, frame: fragment(\"ROWS 3 PRECEDING EXCLUDE CURRENT ROW\")]]","ref":"Ecto.Query.html#windows/3-examples"},{"type":"macro","title":"Ecto.Query.with_cte/3","doc":"A common table expression (CTE) also known as WITH expression.\n\n`name` must be a compile-time literal string that is being used\nas the table name to join the CTE in the main query or in the\nrecursive CTE.\n\n**IMPORTANT!** Beware of using CTEs. In raw SQL, CTEs can be\nused as a mechanism to organize queries, but said mechanism\nhas no purpose in Ecto since Ecto queries are composable by\ndefinition. In other words, if you need to break a large query\ninto parts, use all of the functionality in Elixir and in this\nmodule to structure your code. Furthermore, breaking a query\ninto CTEs can negatively impact performance, as the database\nmay not optimize efficiently across CTEs. The main use case\nfor CTEs in Ecto is to provide recursive definitions, which\nwe outline in the following section. Non-recursive CTEs can\noften be written as joins or subqueries, which provide better\nperformance.","ref":"Ecto.Query.html#with_cte/3"},{"type":"macro","title":"Options - Ecto.Query.with_cte/3","doc":"* `:as` - the CTE query itself or a fragment\n  * `:materialized` - a boolean indicating whether the CTE should\n  be materialized. If blank, the database's default behaviour\n  will be used (only supported by Postgrex, for the built-in adapters)\n  * `:operation` - one of `:all`, `:update_all`, or `:delete_all`\n  indicating the operation type of the CTE query. If blank, it defaults to `:all`,\n  making the CTE query a SELECT query. (only supported by Postgres built-in adapter)","ref":"Ecto.Query.html#with_cte/3-options"},{"type":"macro","title":"Recursive CTEs - Ecto.Query.with_cte/3","doc":"Use `recursive_ctes/2` to enable recursive mode for CTEs.\n\nIn the CTE query itself use the same table name to leverage\nrecursion that has been passed to the `name` argument. Make sure\nto write a stop condition to avoid an infinite recursion loop.\nGenerally speaking, you should only use CTEs in Ecto for\nwriting recursive queries.","ref":"Ecto.Query.html#with_cte/3-recursive-ctes"},{"type":"macro","title":"Expression examples - Ecto.Query.with_cte/3","doc":"Products and their category names for breadcrumbs:\n\n    category_tree_initial_query =\n      Category\n      |> where([c], is_nil(c.parent_id))\n\n    category_tree_recursion_query =\n      Category\n      |> join(:inner, [c], ct in \"category_tree\", on: c.parent_id == ct.id)\n\n    category_tree_query =\n      category_tree_initial_query\n      |> union_all(^category_tree_recursion_query)\n\n    Product\n    |> recursive_ctes(true)\n    |> with_cte(\"category_tree\", as: ^category_tree_query)\n    |> join(:left, [p], c in \"category_tree\", on: c.id == p.category_id)\n    |> group_by([p], p.id)\n    |> select([p, c], %{p | category_names: fragment(\"ARRAY_AGG(?)\", c.name)})\n\nIt's also possible to pass a raw SQL fragment:\n\n    @raw_sql_category_tree \"\"\"\n    SELECT * FROM categories WHERE c.parent_id IS NULL\n    UNION ALL\n    SELECT * FROM categories AS c, category_tree AS ct WHERE ct.id = c.parent_id\n    \"\"\"\n\n    Product\n    |> recursive_ctes(true)\n    |> with_cte(\"category_tree\", as: fragment(@raw_sql_category_tree))\n    |> join(:inner, [p], c in \"category_tree\", on: c.id == p.category_id)\n\nYou can also query over the CTE table itself. In such cases, you can pass\ntuple with the CTE table name as the first element and an Ecto schema as the second\nelement. This will cast the result rows to Ecto structs as long as the Ecto\nschema maps over the same fields in the CTE table:\n\n    {\"category_tree\", Category}\n    |> recursive_ctes(true)\n    |> with_cte(\"category_tree\", as: ^category_tree_query)\n    |> join(:left, [c], p in assoc(c, :products))\n    |> group_by([c], c.id)\n    |> select([c, p], %{c | products_count: count(p.id)})\n\nKeep in mind that the query above will inherit all properties from the `Category` schema,\ninclude a `@schema_prefix` if any is set. In such cases, you can disable those properties\nby setting them as option:\n\n    from(cte in {\"category_tree\", Category}, prefix: nil)\n    |> recursive_ctes(true)\n    |> with_cte(\"category_tree\", as: ^category_tree_query)\n\nFor Postgres built-in adapter, it is possible to define data-modifying CTE queries:\n\n    update_categories_query =\n      Category\n      |> where([c], is_nil(c.parent_id))\n      |> update([c], set: [name: \"Root category\"])\n      |> select([c], c)\n\n    {\"update_categories\", Category}\n    |> with_cte(\"update_categories\", as: ^update_categories_query, operation: :update_all)\n    |> select([c], c)\n\nNote: In order to retrieve the updates rows from a CTE query, the parent query\nmust select rows from the CTE table instead of the table referenced by the CTE query.\nFor example, `\"update_categories\"` will return updates rows for `\"category\"` table, but\nselecting from `\"category\"` table directly will return unaffected rows.\nFor more details see Postgres documentation on data-modifying CTEs and how these work\nwith snapshots.\n\nKeyword syntax is not supported for this feature.","ref":"Ecto.Query.html#with_cte/3-expression-examples"},{"type":"macro","title":"Limitation: CTEs on schemas with source fields - Ecto.Query.with_cte/3","doc":"Ecto allows developers to say that a table in their Ecto schema\nmaps to a different column in their database:\n\n    field :group_id, :integer, source: :iGroupId\n\nAt the moment, using a schema with source fields in CTE may emit\ninvalid queries. If you are running into such scenarios, your best\noption is to use a fragment as your CTE.","ref":"Ecto.Query.html#with_cte/3-limitation-ctes-on-schemas-with-source-fields"},{"type":"function","title":"Ecto.Query.with_named_binding/3","doc":"Applies a callback function to a query if it doesn't contain the given named binding.\nOtherwise, returns the original query.\n\nThe callback function must accept a queryable and return an `Ecto.Query` struct\nthat contains the provided named binding, otherwise an error is raised. It can also\naccept second argument which is the atom representing the name of a binding.\n\nFor example, one might use this function as a convenience to conditionally add a new\nnamed join to a query:\n\n    if has_named_binding?(query, :comments) do\n      query\n    else\n      join(query, :left, [p], c in assoc(p, :comments), as: :comments)\n    end\n\nWith this function it can be simplified to:\n\n    with_named_binding(query, :comments, fn  query, binding ->\n      join(query, :left, [p], a in assoc(p, ^binding), as: ^binding)\n    end)\n\nFor more information on named bindings see \"Named bindings\" in this module doc or `has_named_binding?/2`.","ref":"Ecto.Query.html#with_named_binding/3"},{"type":"macro","title":"Ecto.Query.with_ties/3","doc":"Enables or disables ties for limit expressions.\n\nIf there are multiple records tied for the last position in an ordered\nlimit result, setting this value to `true` will return all of the tied\nrecords, even if the final result exceeds the specified limit.\n\nMust be a boolean or evaluate to a boolean at runtime. Can only be applied\nto queries with a `limit` expression or an error is raised. If `limit`\nis redefined then `with_ties` must be reapplied.\n\nNot all databases support this option and the ones that do might list it\nunder the `FETCH` command. Databases may require a corresponding `order_by`\nstatement to evaluate ties.","ref":"Ecto.Query.html#with_ties/3"},{"type":"macro","title":"Keywords example - Ecto.Query.with_ties/3","doc":"from(p in Post, where: p.author_id == ^current_user, order_by: [desc: p.visits], limit: 10, with_ties: true)","ref":"Ecto.Query.html#with_ties/3-keywords-example"},{"type":"macro","title":"Expressions example - Ecto.Query.with_ties/3","doc":"Post |> where([p], p.author_id == ^current_user) |> order_by([p], desc: p.visits) |> limit(10) |> with_ties(true)","ref":"Ecto.Query.html#with_ties/3-expressions-example"},{"type":"type","title":"Ecto.Query.dynamic_expr/0","doc":"","ref":"Ecto.Query.html#t:dynamic_expr/0"},{"type":"type","title":"Ecto.Query.t/0","doc":"","ref":"Ecto.Query.html#t:t/0"},{"type":"behaviour","title":"Ecto.Repo","doc":"Defines a repository.\n\nA repository maps to an underlying data store, controlled by the\nadapter. For example, Ecto ships with a Postgres adapter that\nstores data into a PostgreSQL database.\n\nWhen used, the repository expects the `:otp_app` and `:adapter` as\noption. The `:otp_app` should point to an OTP application that has\nthe repository configuration. For example, the repository:\n\n    defmodule Repo do\n      use Ecto.Repo,\n        otp_app: :my_app,\n        adapter: Ecto.Adapters.Postgres\n    end\n\nCould be configured with:\n\n    config :my_app, Repo,\n      database: \"ecto_simple\",\n      username: \"postgres\",\n      password: \"postgres\",\n      hostname: \"localhost\"\n\nMost of the configuration that goes into the `config` is specific\nto the adapter. For this particular example, you can check\n[`Ecto.Adapters.Postgres`](https://hexdocs.pm/ecto_sql/Ecto.Adapters.Postgres.html)\nfor more information. In spite of this, the following configuration values\nare shared across all adapters:\n\n  * `:name`- The name of the Repo supervisor process\n\n  * `:priv` - the directory where to keep repository data, like\n    migrations, schema and more. Defaults to \"priv/YOUR_REPO\".\n    It must always point to a subdirectory inside the priv directory\n\n  * `:url` - an URL that specifies storage information. Read below\n    for more information\n\n  * `:log` - the log level used when logging the query with Elixir's\n    Logger. Can be any of `Logger.level/0` values or `false`. If false,\n    disables logging for that repository. Defaults to `:debug`\n\n  * `:pool_size` - the size of the pool used by the connection module.\n    Defaults to `10`\n\n  * `:telemetry_prefix` - we recommend adapters to publish events\n    using the [Telemetry](`:telemetry`) library. By default, the telemetry prefix\n    is based on the module name, so if your module is called\n    `MyApp.Repo`, the prefix will be `[:my_app, :repo]`. See the\n    \"Telemetry Events\" section to see which events we recommend\n    adapters to publish. Note that if you have multiple databases, you\n    should keep the `:telemetry_prefix` consistent for each repo and\n    use the `:repo` property in the event metadata for distinguishing\n    between repos.\n\n  * `:stacktrace`- when true, publishes the stacktrace in telemetry events\n    and allows more advanced logging.","ref":"Ecto.Repo.html"},{"type":"behaviour","title":"URLs - Ecto.Repo","doc":"Repositories by default support URLs. For example, the configuration\nabove could be rewritten to:\n\n    config :my_app, Repo,\n      url: \"ecto://postgres:postgres@localhost/ecto_simple\"\n\nThe schema can be of any value. The path represents the database name\nwhile options are simply merged in.\n\nURL can include query parameters to override shared and adapter-specific\noptions, like `ssl`, `timeout` and `pool_size`. The following example\nshows how to pass these configuration values:\n\n    config :my_app, Repo,\n      url: \"ecto://postgres:postgres@localhost/ecto_simple?ssl=true&pool_size=10\"\n\nIn case the URL needs to be dynamically configured, for example by\nreading a system environment variable, such can be done via the\n`c:init/2` repository callback:\n\n    def init(_type, config) do\n      {:ok, Keyword.put(config, :url, System.get_env(\"DATABASE_URL\"))}\n    end","ref":"Ecto.Repo.html#module-urls"},{"type":"behaviour","title":"Shared options - Ecto.Repo","doc":"Almost all of the repository functions outlined in this module accept the following\noptions:\n\n  * `:timeout` - The time in milliseconds (as an integer) to wait for the query call to\n    finish. `:infinity` will wait indefinitely (default: `15_000`)\n  * `:log` - Can be any of the `Logger.level/0` values or `false`. If `false`,\n    logging is disabled. Defaults to the configured Repo logger level\n  * `:telemetry_event` - The telemetry event name to dispatch the event under.\n    See the next section for more information\n  * `:telemetry_options` - Extra options to attach to telemetry event name.\n    See the next section for more information","ref":"Ecto.Repo.html#module-shared-options"},{"type":"behaviour","title":"Adapter-Specific Errors - Ecto.Repo","doc":"Many of the functions in this module may raise adapter-specific errors, such as `PostgrexError`.\nThis can happen, for example, when the underlying database cannot execute the specified query.","ref":"Ecto.Repo.html#module-adapter-specific-errors"},{"type":"behaviour","title":"Telemetry events - Ecto.Repo","doc":"There are two types of telemetry events. The ones emitted by Ecto and the\nones that are adapter specific.\n\n#","ref":"Ecto.Repo.html#module-telemetry-events"},{"type":"behaviour","title":"Ecto telemetry events - Ecto.Repo","doc":"The following events are emitted by all Ecto repositories:\n\n  * `[:ecto, :repo, :init]` - it is invoked whenever a repository starts.\n    The measurement is a single `system_time` entry in native unit. The\n    metadata is the `:repo` and all initialization options under `:opts`.\n\n#","ref":"Ecto.Repo.html#module-ecto-telemetry-events"},{"type":"behaviour","title":"Adapter-specific events - Ecto.Repo","doc":"We recommend adapters to publish certain `Telemetry` events listed below.\nThose events will use the `:telemetry_prefix` outlined above which defaults\nto `[:my_app, :repo]`.\n\nFor instance, to receive all query events published by a repository called\n`MyApp.Repo`, one would define a module:\n\n    defmodule MyApp.Telemetry do\n      def handle_event([:my_app, :repo, :query], measurements, metadata, config) do\n        IO.inspect binding()\n      end\n    end\n\nThen, in the `Application.start/2` callback, attach the handler to this event using\na unique handler id:\n\n    :ok = :telemetry.attach(\"my-app-handler-id\", [:my_app, :repo, :query], &MyApp.Telemetry.handle_event/4, %{})\n\nFor details, see [the telemetry documentation](https://hexdocs.pm/telemetry/).\n\nBelow we list all events developers should expect from Ecto. All examples\nbelow consider a repository named `MyApp.Repo`:\n\n#### `[:my_app, :repo, :query]`\n\nThis event should be invoked on every query sent to the adapter, including\nqueries that are related to the transaction management.\n\nThe `:measurements` map may include the following, all given in the\n`:native` time unit:\n\n  * `:idle_time` - the time the connection spent waiting before being checked out for the query\n  * `:queue_time` - the time spent waiting to check out a database connection\n  * `:query_time` - the time spent executing the query\n  * `:decode_time` - the time spent decoding the data received from the database\n  * `:total_time` - the sum of (`queue_time`, `query_time`, and `decode_time`)️\n\nAll measurements are given in the `:native` time unit. You can read more\nabout it in the docs for `System.convert_time_unit/3`.\n\nA telemetry `:metadata` map including the following fields. Each database\nadapter may emit different information here. For Ecto.SQL databases, it\nwill look like this:\n\n  * `:type` - the type of the Ecto query. For example, for Ecto.SQL\n    databases, it would be `:ecto_sql_query`\n  * `:repo` - the Ecto repository\n  * `:result` - the query result\n  * `:params` - the dumped query parameters (formatted for database drivers like Postgrex)\n  * `:cast_params` - the casted query parameters (normalized before dumping)\n  * `:query` - the query sent to the database as a string\n  * `:source` - the source the query was made on (may be `nil`)\n  * `:stacktrace` - the stacktrace information, if enabled, or `nil`\n  * `:options` - extra options given to the repo operation under\n    `:telemetry_options`","ref":"Ecto.Repo.html#module-adapter-specific-events"},{"type":"behaviour","title":"Read-only repositories - Ecto.Repo","doc":"You can mark a repository as read-only by passing the `:read_only`\nflag on `use`:\n\n    use Ecto.Repo, otp_app: ..., adapter: ..., read_only: true\n\nBy passing the `:read_only` option, none of the functions that perform\nwrite operations, such as `c:insert/2`, `c:insert_all/3`, `c:update_all/3`,\nand friends will be defined.","ref":"Ecto.Repo.html#module-read-only-repositories"},{"type":"callback","title":"Ecto.Repo.__adapter__/0","doc":"Returns the adapter tied to the repository.","ref":"Ecto.Repo.html#c:__adapter__/0"},{"type":"callback","title":"Ecto.Repo.aggregate/3","doc":"Calculate the given `aggregate`.\n\nIf the query has a limit, offset, distinct or combination set, it will be\nautomatically wrapped in a subquery in order to return the\nproper result.\n\nAny preload or select in the query will be ignored in favor of\nthe column being aggregated.\n\nThe aggregation will fail if any `group_by` field is set.","ref":"Ecto.Repo.html#c:aggregate/3"},{"type":"callback","title":"Options - Ecto.Repo.aggregate/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:aggregate/3-options"},{"type":"callback","title":"Examples - Ecto.Repo.aggregate/3","doc":"# Returns the number of blog posts\n    Repo.aggregate(Post, :count)\n\n    # Returns the number of blog posts in the \"private\" schema path\n    # (in Postgres) or database (in MySQL)\n    Repo.aggregate(Post, :count, prefix: \"private\")","ref":"Ecto.Repo.html#c:aggregate/3-examples"},{"type":"callback","title":"Ecto.Repo.aggregate/4","doc":"Calculate the given `aggregate` over the given `field`.\n\nSee `c:aggregate/3` for general considerations and options.","ref":"Ecto.Repo.html#c:aggregate/4"},{"type":"callback","title":"Examples - Ecto.Repo.aggregate/4","doc":"# Returns the sum of the number of visits for every blog post\n    Repo.aggregate(Post, :sum, :visits)\n\n    # Returns the sum of the number of visits for every blog post in the\n    # \"private\" schema path (in Postgres) or database (in MySQL)\n    Repo.aggregate(Post, :sum, :visits, prefix: \"private\")\n\n    # Returns the average number of visits for the first 10 blog posts\n    query = from Post, limit: 10\n    Repo.aggregate(query, :avg, :visits)","ref":"Ecto.Repo.html#c:aggregate/4-examples"},{"type":"callback","title":"Ecto.Repo.all/2","doc":"Fetches all entries from the data store matching the given query.\n\nMay raise `Ecto.QueryError` if query validation fails.","ref":"Ecto.Repo.html#c:all/2"},{"type":"callback","title":"Options - Ecto.Repo.all/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:all/2-options"},{"type":"callback","title":"Example - Ecto.Repo.all/2","doc":"# Fetch all post titles\n    query = from p in Post,\n         select: p.title\n    MyRepo.all(query)","ref":"Ecto.Repo.html#c:all/2-example"},{"type":"function","title":"Ecto.Repo.all_running/0","doc":"Returns all running Ecto repositories.\n\nThe list is returned in no particular order. The list\ncontains either atoms, for named Ecto repositories, or\nPIDs.","ref":"Ecto.Repo.html#all_running/0"},{"type":"callback","title":"Ecto.Repo.checked_out?/0","doc":"Returns true if a connection has been checked out.\n\nThis is true if inside a `c:Ecto.Repo.checkout/2` or\n`c:Ecto.Repo.transaction/2`.","ref":"Ecto.Repo.html#c:checked_out?/0"},{"type":"callback","title":"Examples - Ecto.Repo.checked_out?/0","doc":"MyRepo.checked_out?\n    #=> false\n\n    MyRepo.transaction(fn ->\n      MyRepo.checked_out? #=> true\n    end)\n\n    MyRepo.checkout(fn ->\n      MyRepo.checked_out? #=> true\n    end)","ref":"Ecto.Repo.html#c:checked_out?/0-examples"},{"type":"callback","title":"Ecto.Repo.checkout/2","doc":"Checks out a connection for the duration of the function.\n\nIt returns the result of the function. This is useful when\nyou need to perform multiple operations against the repository\nin a row and you want to avoid checking out the connection\nmultiple times.\n\n`checkout/2` and `transaction/2` can be combined and nested\nmultiple times. If `checkout/2` is called inside the function\nof another `checkout/2` call, the function is simply executed,\nwithout checking out a new connection.","ref":"Ecto.Repo.html#c:checkout/2"},{"type":"callback","title":"Options - Ecto.Repo.checkout/2","doc":"See the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:checkout/2-options"},{"type":"callback","title":"Ecto.Repo.config/0","doc":"Returns the adapter configuration stored in the `:otp_app` environment.\n\nIf the `c:init/2` callback is implemented in the repository,\nit will be invoked with the first argument set to `:runtime`.","ref":"Ecto.Repo.html#c:config/0"},{"type":"callback","title":"Ecto.Repo.default_options/1","doc":"A user customizable callback invoked to retrieve default options\nfor operations.\n\nThis can be used to provide default values per operation that\nhave higher precedence than the values given on configuration\nor when starting the repository. It can also be used to set\nquery specific options, such as `:prefix`.\n\nThis callback is invoked as the entry point for all repository\noperations. For example, if you are executing a query with preloads,\nthis callback will be invoked once at the beginning, but the\noptions returned here will be passed to all following operations.","ref":"Ecto.Repo.html#c:default_options/1"},{"type":"callback","title":"Ecto.Repo.delete/2","doc":"Deletes a struct using its primary key.\n\nIf the struct has no primary key, `Ecto.NoPrimaryKeyFieldError`\nwill be raised. If the struct has been removed prior to the call,\n`Ecto.StaleEntryError` will be raised. If more than one database\noperation is required, they're automatically wrapped in a transaction.\n\nIt returns `{:ok, struct}` if the struct has been successfully\ndeleted or `{:error, changeset}` if there was a validation\nor a known constraint error. By default, constraint errors will\nraise the `Ecto.ConstraintError` exception, unless a changeset is\ngiven as the first argument with the relevant constraints declared\nin it (see `Ecto.Changeset`).","ref":"Ecto.Repo.html#c:delete/2"},{"type":"callback","title":"Options - Ecto.Repo.delete/2","doc":"* `:returning` - selects which fields to return. It accepts a list\n    of fields to be returned from the database. When `true`, returns\n    all fields, including those marked as `load_in_query: false`. When\n    `false`, no extra fields are returned. It will always include all\n    fields in `read_after_writes`. Be aware that the fields returned\n    from the database overwrite what was supplied by the user. Any field\n    not returned by the database will be present with the original value\n    supplied by the user. Not all databases support this option.\n\n  * `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This overrides the prefix set\n    in the query and any `@schema_prefix` set in the schema.\n\n  * `:stale_error_field` - The field where stale errors will be added in\n    the returning changeset. This option can be used to avoid raising\n    `Ecto.StaleEntryError`.\n\n  * `:stale_error_message` - The message to add to the configured\n    `:stale_error_field` when stale errors happen, defaults to \"is stale\".\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:delete/2-options"},{"type":"callback","title":"Example - Ecto.Repo.delete/2","doc":"post = MyRepo.get!(Post, 42)\n    case MyRepo.delete post do\n      {:ok, struct}       -> # Deleted with success\n      {:error, changeset} -> # Something went wrong\n    end","ref":"Ecto.Repo.html#c:delete/2-example"},{"type":"callback","title":"Ecto.Repo.delete!/2","doc":"Same as `c:delete/2` but returns the struct or raises if the changeset is invalid.","ref":"Ecto.Repo.html#c:delete!/2"},{"type":"callback","title":"Ecto.Repo.delete_all/2","doc":"Deletes all entries matching the given query.\n\nIt returns a tuple containing the number of entries and any returned\nresult as second element. The second element is `nil` by default\nunless a `select` is supplied in the delete query. Note, however,\nnot all databases support returning data from DELETEs.","ref":"Ecto.Repo.html#c:delete_all/2"},{"type":"callback","title":"Options - Ecto.Repo.delete_all/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for remaining options.","ref":"Ecto.Repo.html#c:delete_all/2-options"},{"type":"callback","title":"Examples - Ecto.Repo.delete_all/2","doc":"MyRepo.delete_all(Post)\n\n    from(p in Post, where: p.id   MyRepo.delete_all()\n\n    # With returning results, if supported by the database.\n    {_count, posts} = from(p in Post, where: p.id   MyRepo.delete_all()","ref":"Ecto.Repo.html#c:delete_all/2-examples"},{"type":"callback","title":"Ecto.Repo.exists?/2","doc":"Checks if there exists an entry that matches the given query.\n\nReturns a boolean.","ref":"Ecto.Repo.html#c:exists?/2"},{"type":"callback","title":"Options - Ecto.Repo.exists?/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:exists?/2-options"},{"type":"callback","title":"Examples - Ecto.Repo.exists?/2","doc":"# checks if any posts exist\n    Repo.exists?(Post)\n\n    # checks if any posts exist in the \"private\" schema path (in Postgres) or\n    # database (in MySQL)\n    Repo.exists?(Post, prefix: \"private\")\n\n    # checks if any post with a like count greater than 10 exists\n    query = from p in Post, where: p.like_count > 10\n    Repo.exists?(query)","ref":"Ecto.Repo.html#c:exists?/2-examples"},{"type":"callback","title":"Ecto.Repo.get/3","doc":"Fetches a single struct from the data store where the primary key matches the\ngiven id.\n\nReturns `nil` if no result was found. If the struct in the queryable\nhas no or more than one primary key, it will raise an argument error.","ref":"Ecto.Repo.html#c:get/3"},{"type":"callback","title":"Options - Ecto.Repo.get/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:get/3-options"},{"type":"callback","title":"Example - Ecto.Repo.get/3","doc":"MyRepo.get(Post, 42)\n\n    MyRepo.get(Post, 42, prefix: \"public\")","ref":"Ecto.Repo.html#c:get/3-example"},{"type":"callback","title":"Ecto.Repo.get!/3","doc":"Similar to `c:get/3` but raises `Ecto.NoResultsError` if no record was found.","ref":"Ecto.Repo.html#c:get!/3"},{"type":"callback","title":"Options - Ecto.Repo.get!/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:get!/3-options"},{"type":"callback","title":"Example - Ecto.Repo.get!/3","doc":"MyRepo.get!(Post, 42)\n\n    MyRepo.get!(Post, 42, prefix: \"public\")","ref":"Ecto.Repo.html#c:get!/3-example"},{"type":"callback","title":"Ecto.Repo.get_by/3","doc":"Fetches a single result from the query.\n\nReturns `nil` if no result was found. Raises if more than one entry.","ref":"Ecto.Repo.html#c:get_by/3"},{"type":"callback","title":"Options - Ecto.Repo.get_by/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:get_by/3-options"},{"type":"callback","title":"Example - Ecto.Repo.get_by/3","doc":"MyRepo.get_by(Post, title: \"My post\")\n\n    MyRepo.get_by(Post, [title: \"My post\"], prefix: \"public\")","ref":"Ecto.Repo.html#c:get_by/3-example"},{"type":"callback","title":"Ecto.Repo.get_by!/3","doc":"Similar to `c:get_by/3` but raises `Ecto.NoResultsError` if no record was found.\n\nRaises if more than one entry.","ref":"Ecto.Repo.html#c:get_by!/3"},{"type":"callback","title":"Options - Ecto.Repo.get_by!/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:get_by!/3-options"},{"type":"callback","title":"Example - Ecto.Repo.get_by!/3","doc":"MyRepo.get_by!(Post, title: \"My post\")\n\n    MyRepo.get_by!(Post, [title: \"My post\"], prefix: \"public\")","ref":"Ecto.Repo.html#c:get_by!/3-example"},{"type":"callback","title":"Ecto.Repo.get_dynamic_repo/0","doc":"Returns the atom name or pid of the current repository.\n\nSee `c:put_dynamic_repo/1` for more information.","ref":"Ecto.Repo.html#c:get_dynamic_repo/0"},{"type":"callback","title":"Ecto.Repo.in_transaction?/0","doc":"Returns true if the current process is inside a transaction.\n\nIf you are using the `Ecto.Adapters.SQL.Sandbox` in tests, note that even\nthough each test is inside a transaction, `in_transaction?/0` will only\nreturn true inside transactions explicitly created with `transaction/2`. This\nis done so the test environment mimics dev and prod.","ref":"Ecto.Repo.html#c:in_transaction?/0"},{"type":"callback","title":"Examples - Ecto.Repo.in_transaction?/0","doc":"MyRepo.in_transaction?\n    #=> false\n\n    MyRepo.transaction(fn ->\n      MyRepo.in_transaction? #=> true\n    end)","ref":"Ecto.Repo.html#c:in_transaction?/0-examples"},{"type":"callback","title":"Ecto.Repo.init/2","doc":"A callback executed when the repo starts or when configuration is read.\n\nThe first argument is the context the callback is being invoked. If it\nis called because the Repo supervisor is starting, it will be `:supervisor`.\nIt will be `:runtime` if it is called for reading configuration without\nactually starting a process.\n\nThe second argument is the repository configuration as stored in the\napplication environment. It must return `{:ok, keyword}` with the updated\nlist of configuration or `:ignore` (only in the `:supervisor` case).","ref":"Ecto.Repo.html#c:init/2"},{"type":"callback","title":"Ecto.Repo.insert/2","doc":"Inserts a struct defined via `Ecto.Schema` or a changeset.\n\nIn case a struct is given, the struct is converted into a changeset\nwith all non-nil fields as part of the changeset.\n\nIn case a changeset is given, the changes in the changeset are\nmerged with the struct fields, and all of them are sent to the\ndatabase. If more than one database operation is required, they're\nautomatically wrapped in a transaction.\n\nIt returns `{:ok, struct}` if the struct has been successfully\ninserted or `{:error, changeset}` if there was a validation\nor a known constraint error.","ref":"Ecto.Repo.html#c:insert/2"},{"type":"callback","title":"Options - Ecto.Repo.insert/2","doc":"* `:returning` - selects which fields to return. It accepts a list\n    of fields to be returned from the database. When `true`, returns\n    all fields, including those marked as `load_in_query: false`. When\n    `false`, no extra fields are returned. It will always include all\n    fields in `read_after_writes` as well as any autogenerated id. Be\n    aware that the fields returned from the database overwrite what was\n    supplied by the user. Any field not returned by the database will be\n    present with the original value supplied by the user. Not all databases\n    support this option and it may not be available during upserts. See\n    the \"Upserts\" section for more information.\n\n  * `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This overrides the prefix set\n    in the query and any `@schema_prefix` set on any schemas. Also, the\n    `@schema_prefix` for the parent record will override all default\n    `@schema_prefix`s set in any child schemas for associations.\n\n  * `:on_conflict` - It may be one of `:raise` (the default), `:nothing`,\n    `:replace_all`, `{:replace_all_except, fields}`, `{:replace, fields}`,\n    a keyword list of update instructions or an `Ecto.Query` query for updates.\n    See the \"Upserts\" section for more information.\n\n  * `:conflict_target` - A list of column names to verify for conflicts.\n    It is expected those columns to have unique indexes on them that may conflict.\n    If none is specified, the conflict target is left up to the database.\n    It may also be `{:unsafe_fragment, binary_fragment}` to pass any\n    expression to the database without any sanitization, this is useful\n    for partial index or index with expressions, such as\n    `{:unsafe_fragment, \"(coalesce(firstname, \"\"), coalesce(lastname, \"\")) WHERE middlename IS NULL\"}` for\n    `ON CONFLICT (coalesce(firstname, \"\"), coalesce(lastname, \"\")) WHERE middlename IS NULL` SQL query.\n\n  * `:stale_error_field` - The field where stale errors will be added in\n    the returning changeset. This option can be used to avoid raising\n    `Ecto.StaleEntryError`.\n\n  * `:stale_error_message` - The message to add to the configured\n    `:stale_error_field` when stale errors happen, defaults to \"is stale\".\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:insert/2-options"},{"type":"callback","title":"Examples - Ecto.Repo.insert/2","doc":"A typical example is calling `MyRepo.insert/1` with a struct\nand acting on the return value:\n\n    case MyRepo.insert %Post{title: \"Ecto is great\"} do\n      {:ok, struct}       -> # Inserted with success\n      {:error, changeset} -> # Something went wrong\n    end","ref":"Ecto.Repo.html#c:insert/2-examples"},{"type":"callback","title":"Upserts - Ecto.Repo.insert/2","doc":"`c:insert/2` provides upserts (update or inserts) via the `:on_conflict`\noption. The `:on_conflict` option supports the following values:\n\n  * `:raise` - raises if there is a conflicting primary key or unique index\n\n  * `:nothing` - ignores the error in case of conflicts\n\n  * `:replace_all` - replace **all** values on the existing row with the values\n    in the schema/changeset, including fields not explicitly set in the changeset,\n    such as IDs and autogenerated timestamps (`inserted_at` and `updated_at`).\n    Do not use this option if you have auto-incrementing primary keys, as they\n    will also be replaced. You most likely want to use `{:replace_all_except, [:id]}`\n    or `{:replace, fields}` explicitly instead. This option requires a schema\n\n  * `{:replace_all_except, fields}` - same as above except the given fields are\n    not replaced. This option requires a schema\n\n  * `{:replace, fields}` - replace only specific columns. This option requires\n    `:conflict_target`\n\n  * a keyword list of update instructions - such as the one given to\n    `c:update_all/3`, for example: `[set: [title: \"new title\"]]`\n\n  * an `Ecto.Query` that will act as an `UPDATE` statement, such as the\n    one given to `c:update_all/3`. Similarly to `c:update_all/3`, auto\n    generated values, such as timestamps are not automatically updated.\n    If the struct cannot be found, `Ecto.StaleEntryError` will be raised.\n\nUpserts map to \"ON CONFLICT\" on databases like Postgres and \"ON DUPLICATE KEY\"\non databases such as MySQL.\n\nAs an example, imagine `:title` is marked as a unique column in\nthe database:\n\n    {:ok, inserted} = MyRepo.insert(%Post{title: \"this is unique\"})\n\nNow we can insert with the same title but do nothing on conflicts:\n\n    {:ok, ignored} = MyRepo.insert(%Post{title: \"this is unique\"}, on_conflict: :nothing)\n\nBecause we used `on_conflict: :nothing`, instead of getting an error,\nwe got `{:ok, struct}`. However the returned struct does not reflect\nthe data in the database. If the primary key is auto-generated by the\ndatabase, the primary key in the `ignored` record will be nil if there\nwas no insertion. For example, if you use the default primary key\n(which has name `:id` and a type of `:id`), then `ignored.id` above\nwill be nil if there was no insertion.\n\nIf your id is generated by your application (typically the case for\n`:binary_id`) or if you pass another value for `:on_conflict`, detecting\nif an insert or update happened is slightly more complex, as the database\ndoes not actually inform us what happened. Let's insert a post with the\nsame title but use a query to update the body column in case of conflicts:\n\n    # In Postgres (it requires the conflict target for updates):\n    on_conflict = [set: [body: \"updated\"]]\n    {:ok, updated} = MyRepo.insert(%Post{title: \"this is unique\"},\n                                   on_conflict: on_conflict, conflict_target: :title)\n\n    # In MySQL (conflict target is not supported):\n    on_conflict = [set: [title: \"updated\"]]\n    {:ok, updated} = MyRepo.insert(%Post{id: inserted.id, title: \"updated\"},\n                                   on_conflict: on_conflict)\n\nIn the examples above, even though it returned `:ok`, we do not know\nif we inserted new data or if we updated only the `:on_conflict` fields.\nIn case an update happened, the data in the struct most likely does\nnot match the data in the database. For example, autogenerated fields\nsuch as `inserted_at` will point to now rather than the time the\nstruct was actually inserted.\n\nIf you need to guarantee the data in the returned struct mirrors the\ndatabase, you have three options:\n\n  * Use `on_conflict: :replace_all`, although that will replace all\n    fields in the database with the ones in the struct/changeset,\n    including autogenerated fields such as `inserted_at` and `updated_at`:\n\n        MyRepo.insert(%Post{title: \"this is unique\"},\n                      on_conflict: :replace_all, conflict_target: :title)\n\n  * Specify `read_after_writes: true` in your schema for choosing\n    fields that are read from the database after every operation.\n    Or pass `returning: true` to `insert` to read all fields back.\n    (Note that it will only read from the database if at least one\n    field is updated).\n\n        MyRepo.insert(%Post{title: \"this is unique\"}, returning: true,\n                      on_conflict: on_conflict, conflict_target: :title)\n\n  * Alternatively, read the data again from the database in a separate\n    query. This option requires the primary key to be generated by the\n    database:\n\n        {:ok, updated} = MyRepo.insert(%Post{title: \"this is unique\"}, on_conflict: on_conflict)\n        Repo.get(Post, updated.id)\n\nBecause of the inability to know if the struct is up to date or not,\ninserting a struct with associations and using the `:on_conflict` option\nat the same time is not recommended, as Ecto will be unable to actually\ntrack the proper status of the association.","ref":"Ecto.Repo.html#c:insert/2-upserts"},{"type":"callback","title":"Ecto.Repo.insert!/2","doc":"Same as `c:insert/2` but returns the struct or raises if the changeset is invalid.","ref":"Ecto.Repo.html#c:insert!/2"},{"type":"callback","title":"Ecto.Repo.insert_all/3","doc":"Inserts all entries into the repository.\n\nIt expects a schema module (`MyApp.User`) or a source (`\"users\"`) or\nboth (`{\"users\", MyApp.User}`) as the first argument. The second\nargument is a list of entries to be inserted, either as keyword\nlists or as maps. The keys of the entries are the field names as\natoms and the value should be the respective value for the field\ntype or, optionally, an `Ecto.Query` that returns a single entry\nwith a single value.\n\nIt returns a tuple containing the number of entries\nand any returned result as second element. If the database\ndoes not support RETURNING in INSERT statements or no\nreturn result was selected, the second element will be `nil`.\n\nWhen a schema module is given, the entries given will be properly dumped\nbefore being sent to the database. If the schema primary key has type\n`:id` or `:binary_id`, it will be handled either at the adapter\nor the storage layer. However any other primary key type or autogenerated\nvalue, like `Ecto.UUID` and timestamps, won't be autogenerated when\nusing `c:insert_all/3`. You must set those fields explicitly. This is by\ndesign as this function aims to be a more direct way to insert data into\nthe database without the conveniences of `c:insert/2`. This is also\nconsistent with `c:update_all/3` that does not handle auto generated\nvalues as well.\n\nIt is also not possible to use `insert_all` to insert across multiple\ntables, therefore associations are not supported.\n\nIf a source is given, without a schema module, the given fields are passed\nas is to the adapter.","ref":"Ecto.Repo.html#c:insert_all/3"},{"type":"callback","title":"Options - Ecto.Repo.insert_all/3","doc":"* `:returning` - selects which fields to return. When `true`,\n    returns all fields in the given schema. May be a list of\n    fields, where a struct is still returned but only with the\n    given fields. Or `false`, where nothing is returned (the default).\n    This option is not supported by all databases.\n\n  * `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This overrides the prefix set\n    in the query and any `@schema_prefix` set in the schema.\n\n  * `:on_conflict` - It may be one of `:raise` (the default), `:nothing`,\n    `:replace_all`, `{:replace_all_except, fields}`, `{:replace, fields}`,\n    a keyword list of update instructions or an `Ecto.Query`\n    query for updates. See the \"Upserts\" section for more information.\n\n  * `:conflict_target` - A list of column names to verify for conflicts.\n    It is expected those columns to have unique indexes on them that may conflict.\n    If none is specified, the conflict target is left up to the database.\n    It may also be `{:unsafe_fragment, binary_fragment}` to pass any\n    expression to the database without any sanitization, this is useful\n    for partial index or index with expressions, such as\n    `{:unsafe_fragment, \"(coalesce(firstname, \"\"), coalesce(lastname, \"\")) WHERE middlename IS NULL\"}` for\n    `ON CONFLICT (coalesce(firstname, \"\"), coalesce(lastname, \"\")) WHERE middlename IS NULL` SQL query.\n\n  * `:placeholders` - A map with placeholders. This feature is not supported\n    by all databases. See the \"Placeholders\" section for more information.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for remaining options.","ref":"Ecto.Repo.html#c:insert_all/3-options"},{"type":"callback","title":"Source query - Ecto.Repo.insert_all/3","doc":"A query can be given instead of a list with entries. This query needs to select\ninto a map containing only keys that are available as writeable columns in the\nschema.","ref":"Ecto.Repo.html#c:insert_all/3-source-query"},{"type":"callback","title":"Examples - Ecto.Repo.insert_all/3","doc":"MyRepo.insert_all(Post, [[title: \"My first post\"], [title: \"My second post\"]])\n\n    MyRepo.insert_all(Post, [%{title: \"My first post\"}, %{title: \"My second post\"}])\n\n    query = from p in Post,\n      join: c in assoc(p, :comments),\n      select: %{\n        author_id: p.author_id,\n        posts: count(p.id, :distinct),\n        interactions: sum(p.likes) + count(c.id)\n      },\n      group_by: p.author_id\n    MyRepo.insert_all(AuthorStats, query)","ref":"Ecto.Repo.html#c:insert_all/3-examples"},{"type":"callback","title":"Upserts - Ecto.Repo.insert_all/3","doc":"`c:insert_all/3` provides upserts (update or inserts) via the `:on_conflict`\noption. The `:on_conflict` option supports the following values:\n\n  * `:raise` - raises if there is a conflicting primary key or unique index\n\n  * `:nothing` - ignores the error in case of conflicts\n\n  * `:replace_all` - replace **all** values on the existing row with the values\n    in the schema/changeset, including fields not explicitly set in the changeset,\n    such as IDs and autogenerated timestamps (`inserted_at` and `updated_at`).\n    Do not use this option if you have auto-incrementing primary keys, as they\n    will also be replaced. You most likely want to use `{:replace_all_except, [:id]}`\n    or `{:replace, fields}` explicitly instead. This option requires a schema\n\n  * `{:replace_all_except, fields}` - same as above except the given fields\n    are not replaced. This option requires a schema\n\n  * `{:replace, fields}` - replace only specific columns. This option requires\n    `:conflict_target`\n\n  * a keyword list of update instructions - such as the one given to\n    `c:update_all/3`, for example: `[set: [title: \"new title\"]]`\n\n  * an `Ecto.Query` that will act as an `UPDATE` statement, such as the\n    one given to `c:update_all/3`\n\nUpserts map to \"ON CONFLICT\" on databases like Postgres and \"ON DUPLICATE KEY\"\non databases such as MySQL.","ref":"Ecto.Repo.html#c:insert_all/3-upserts"},{"type":"callback","title":"Return values - Ecto.Repo.insert_all/3","doc":"By default, both Postgres and MySQL will return the number of entries\ninserted on `c:insert_all/3`. However, when the `:on_conflict` option\nis specified, Postgres and MySQL will return different results.\n\nPostgres will only count a row if it was affected and will\nreturn 0 if no new entry was added.\n\nMySQL will return, at a minimum, the number of entries attempted. For example,\nif `:on_conflict` is set to `:nothing`, MySQL will return\nthe number of entries attempted to be inserted, even when no entry\nwas added.\n\nAlso note that if `:on_conflict` is a query, MySQL will return\nthe number of attempted entries plus the number of entries modified\nby the UPDATE query.","ref":"Ecto.Repo.html#c:insert_all/3-return-values"},{"type":"callback","title":"Placeholders - Ecto.Repo.insert_all/3","doc":"Passing in a map for the `:placeholders` allows you to send less\ndata over the wire when you have many entries with the same value\nfor a field. To use a placeholder, replace its value in each of your\nentries with `{:placeholder, key}`,  where `key` is the key you\nare using in the `:placeholders` option map. For example:\n\n    placeholders = %{blob: large_blob_of_text(...)}\n\n    entries = [\n      %{title: \"v1\", body: {:placeholder, :blob}},\n      %{title: \"v2\", body: {:placeholder, :blob}}\n    ]\n\n    Repo.insert_all(Post, entries, placeholders: placeholders)\n\nKeep in mind that:\n\n  * placeholders cannot be nested in other values. For example, you\n    cannot put a placeholder inside an array. Instead, the whole\n    array has to be the placeholder\n\n  * a placeholder key can only be used with columns of the same type\n\n  * placeholders require a database that supports index parameters,\n    so they are not currently compatible with MySQL","ref":"Ecto.Repo.html#c:insert_all/3-placeholders"},{"type":"callback","title":"Ecto.Repo.insert_or_update/2","doc":"Inserts or updates a changeset depending on whether the struct is persisted\nor not.\n\nThe distinction whether to insert or update will be made on the\n`Ecto.Schema.Metadata` field `:state`. The `:state` is automatically set by\nEcto when loading or building a schema.\n\nPlease note that for this to work, you will have to load existing structs from\nthe database. So even if the struct exists, this won't work:\n\n    struct = %Post{id: \"existing_id\", ...}\n    MyRepo.insert_or_update changeset\n    # => {:error, changeset} # id already exists","ref":"Ecto.Repo.html#c:insert_or_update/2"},{"type":"callback","title":"Options - Ecto.Repo.insert_or_update/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This overrides the prefix set\n    in the query and any `@schema_prefix` set any schemas. Also, the\n    `@schema_prefix` for the parent record will override all default\n    `@schema_prefix`s set in any child schemas for associations.\n  * `:stale_error_field` - The field where stale errors will be added in\n    the returning changeset. This option can be used to avoid raising\n    `Ecto.StaleEntryError`. Only applies to updates.\n  * `:stale_error_message` - The message to add to the configured\n    `:stale_error_field` when stale errors happen, defaults to \"is stale\".\n    Only applies to updates.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:insert_or_update/2-options"},{"type":"callback","title":"Example - Ecto.Repo.insert_or_update/2","doc":"result =\n      case MyRepo.get(Post, id) do\n        nil  -> %Post{id: id} # Post not found, we build one\n        post -> post          # Post exists, let's use it\n      end\n      |> Post.changeset(changes)\n      |> MyRepo.insert_or_update\n\n    case result do\n      {:ok, struct}       -> # Inserted or updated with success\n      {:error, changeset} -> # Something went wrong\n    end","ref":"Ecto.Repo.html#c:insert_or_update/2-example"},{"type":"callback","title":"Ecto.Repo.insert_or_update!/2","doc":"Same as `c:insert_or_update/2` but returns the struct or raises if the changeset\nis invalid.","ref":"Ecto.Repo.html#c:insert_or_update!/2"},{"type":"callback","title":"Ecto.Repo.load/2","doc":"Loads `data` into a schema or a map.\n\nThe first argument can be a a schema module or a map (of types).\nThe first argument determines the return value: a struct or a map,\nrespectively.\n\nThe second argument `data` specifies fields and values that are to be loaded.\nIt can be a map, a keyword list, or a `{fields, values}` tuple.\nFields can be atoms or strings.\n\nFields that are not present in the schema (or `types` map) are ignored.\nIf any of the values has invalid type, an error is raised.\n\nTo load data from non-database sources, use `Ecto.embedded_load/3`.","ref":"Ecto.Repo.html#c:load/2"},{"type":"callback","title":"Examples - Ecto.Repo.load/2","doc":"iex> MyRepo.load(User, %{name: \"Alice\", age: 25})\n    %User{name: \"Alice\", age: 25}\n\n    iex> MyRepo.load(User, [name: \"Alice\", age: 25])\n    %User{name: \"Alice\", age: 25}\n\n`data` can also take form of `{fields, values}`:\n\n    iex> MyRepo.load(User, {[:name, :age], [\"Alice\", 25]})\n    %User{name: \"Alice\", age: 25, ...}\n\nThe first argument can also be a `types` map:\n\n    iex> types = %{name: :string, age: :integer}\n    iex> MyRepo.load(types, %{name: \"Alice\", age: 25})\n    %{name: \"Alice\", age: 25}\n\nThis function is especially useful when parsing raw query results:\n\n    iex> result = Ecto.Adapters.SQL.query!(MyRepo, \"SELECT * FROM users\", [])\n    iex> Enum.map(result.rows, &MyRepo.load(User, {result.columns, &1}))\n    [%User{...}, ...]","ref":"Ecto.Repo.html#c:load/2-examples"},{"type":"callback","title":"Ecto.Repo.one/2","doc":"Fetches a single result from the query.\n\nReturns `nil` if no result was found. Raises if more than one entry.","ref":"Ecto.Repo.html#c:one/2"},{"type":"callback","title":"Options - Ecto.Repo.one/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:one/2-options"},{"type":"callback","title":"Examples - Ecto.Repo.one/2","doc":"Repo.one(from p in Post, join: c in assoc(p, :comments), where: p.id == ^post_id)\n\n    query = from p in Post, join: c in assoc(p, :comments), where: p.id == ^post_id\n    Repo.one(query, prefix: \"private\")","ref":"Ecto.Repo.html#c:one/2-examples"},{"type":"callback","title":"Ecto.Repo.one!/2","doc":"Similar to `c:one/2` but raises `Ecto.NoResultsError` if no record was found.\n\nRaises if more than one entry.","ref":"Ecto.Repo.html#c:one!/2"},{"type":"callback","title":"Options - Ecto.Repo.one!/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:one!/2-options"},{"type":"callback","title":"Ecto.Repo.preload/3","doc":"Preloads all associations on the given struct or structs.\n\nThis is similar to `Ecto.Query.preload/3` except it allows\nyou to preload structs after they have been fetched from the\ndatabase.\n\nIn case the association was already loaded, preload won't attempt\nto reload it.\n\nIf you want to reset the loaded fields, see `Ecto.reset_fields/2`.","ref":"Ecto.Repo.html#c:preload/3"},{"type":"callback","title":"Options - Ecto.Repo.preload/3","doc":"* `:force` - By default, Ecto won't preload associations that\n    are already loaded. By setting this option to true, any existing\n    association will be discarded and reloaded.\n  * `:in_parallel` - If the preloads must be done in parallel. It can\n    only be performed when we have more than one preload and the\n    repository is not in a transaction. Defaults to `true`.\n  * `:prefix` - the prefix to fetch preloads from. By default, queries\n    will use the same prefix as the first struct in the given collection.\n    This option allows the prefix to be changed.\n  * `:on_preloader_spawn` - when preloads are done in parallel, this function\n    will be called in the processes that perform the preloads. This can be useful\n    for context propagation for traces.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:preload/3-options"},{"type":"callback","title":"Examples - Ecto.Repo.preload/3","doc":"# Use a single atom to preload an association\n    posts = Repo.preload posts, :comments\n\n    # Use a list of atoms to preload multiple associations\n    posts = Repo.preload posts, [:comments, :authors]\n\n    # Use a keyword list to preload nested associations as well\n    posts = Repo.preload posts, [comments: [:replies, :likes], authors: []]\n\n    # You can mix atoms and keywords, but the atoms must come first\n    posts = Repo.preload posts, [:authors, comments: [:likes, replies: [:reactions]]]\n\n    # Use a keyword list to customize how associations are queried\n    posts = Repo.preload posts, [comments: from(c in Comment, order_by: c.published_at)]\n\n    # Use a two-element tuple for a custom query and nested association definition\n    query = from c in Comment, order_by: c.published_at\n    posts = Repo.preload posts, [comments: {query, [:replies, :likes]}]\n\nThe query given to preload may also preload its own associations.","ref":"Ecto.Repo.html#c:preload/3-examples"},{"type":"callback","title":"Ecto.Repo.prepare_query/3","doc":"A user customizable callback invoked for query-based operations.\n\nThis callback can be used to further modify the query and options\nbefore it is transformed and sent to the database.\n\nThis callback is invoked for all query APIs, including the `stream`\nfunctions. It is also invoked for `insert_all` if a source query is\ngiven. It is not invoked for any of the other schema functions.","ref":"Ecto.Repo.html#c:prepare_query/3"},{"type":"callback","title":"Examples - Ecto.Repo.prepare_query/3","doc":"Let's say you want to filter out records that were \"soft-deleted\"\n(have `deleted_at` column set) from all operations unless an admin\nis running the query; you can define the callback like this:\n\n    @impl true\n    def prepare_query(_operation, query, opts) do\n      if opts[:admin] do\n        {query, opts}\n      else\n        query = from(x in query, where: is_nil(x.deleted_at))\n        {query, opts}\n      end\n    end\n\nAnd then execute the query:\n\n    Repo.all(query)              # only non-deleted records are returned\n    Repo.all(query, admin: true) # all records are returned\n\nThe callback will be invoked for all queries, including queries\nmade from associations and preloads. It is not invoked for each\nindividual join inside a query.","ref":"Ecto.Repo.html#c:prepare_query/3-examples"},{"type":"callback","title":"Ecto.Repo.put_dynamic_repo/1","doc":"Sets the dynamic repository to be used in further interactions.\n\nSometimes you may want a single Ecto repository to talk to\nmany different database instances. By default, when you call\n`MyApp.Repo.start_link/1`, it will start a repository with\nname `MyApp.Repo`. But if you want to start multiple repositories,\nyou can give each of them a different name:\n\n    MyApp.Repo.start_link(name: :tenant_foo, hostname: \"foo.example.com\")\n    MyApp.Repo.start_link(name: :tenant_bar, hostname: \"bar.example.com\")\n\nYou can also start repositories without names by explicitly\nsetting the name to nil:\n\n    MyApp.Repo.start_link(name: nil, hostname: \"temp.example.com\")\n\nHowever, once the repository is started, you can't directly interact with\nit, since all operations in `MyApp.Repo` are sent by default to the repository\nnamed `MyApp.Repo`. You can change the default repo at compile time with:\n\n    use Ecto.Repo, default_dynamic_repo: :name_of_repo\n\nOr you can change it anytime at runtime by calling `put_dynamic_repo/1`:\n\n    MyApp.Repo.put_dynamic_repo(:tenant_foo)\n\nFrom this moment on, all future queries done by the current process will\nrun on `:tenant_foo`.","ref":"Ecto.Repo.html#c:put_dynamic_repo/1"},{"type":"callback","title":"Ecto.Repo.reload/2","doc":"Reloads a given schema or schema list from the database.\n\nWhen using with lists, it is expected that all of the structs in the list belong\nto the same schema. Ordering is guaranteed to be kept. Results not found in\nthe database will be returned as `nil`.","ref":"Ecto.Repo.html#c:reload/2"},{"type":"callback","title":"Example - Ecto.Repo.reload/2","doc":"MyRepo.reload(post)\n    %Post{}\n\n    MyRepo.reload([post1, post2])\n    [%Post{}, %Post{}]\n\n    MyRepo.reload([deleted_post, post1])\n    [nil, %Post{}]","ref":"Ecto.Repo.html#c:reload/2-example"},{"type":"callback","title":"Ecto.Repo.reload!/2","doc":"Similar to `c:reload/2`, but raises when something is not found.\n\nWhen using with lists, ordering is guaranteed to be kept.","ref":"Ecto.Repo.html#c:reload!/2"},{"type":"callback","title":"Example - Ecto.Repo.reload!/2","doc":"MyRepo.reload!(post)\n    %Post{}\n\n    MyRepo.reload!([post1, post2])\n    [%Post{}, %Post{}]","ref":"Ecto.Repo.html#c:reload!/2-example"},{"type":"callback","title":"Ecto.Repo.rollback/1","doc":"Rolls back the current transaction.\n\nThe transaction will return the value given as `{:error, value}`.\n\nNote that calling `rollback` causes the code in the transaction to stop executing.","ref":"Ecto.Repo.html#c:rollback/1"},{"type":"callback","title":"Ecto.Repo.start_link/1","doc":"Starts any connection pooling or supervision and return `{:ok, pid}`\nor just `:ok` if nothing needs to be done.\n\nReturns `{:error, {:already_started, pid}}` if the repo is already\nstarted or `{:error, term}` in case anything else goes wrong.","ref":"Ecto.Repo.html#c:start_link/1"},{"type":"callback","title":"Options - Ecto.Repo.start_link/1","doc":"See the configuration in the moduledoc for options shared between adapters,\nfor adapter-specific configuration see the adapter's documentation.","ref":"Ecto.Repo.html#c:start_link/1-options"},{"type":"callback","title":"Ecto.Repo.stop/1","doc":"Shuts down the repository.","ref":"Ecto.Repo.html#c:stop/1"},{"type":"callback","title":"Ecto.Repo.stream/2","doc":"Returns a lazy enumerable that emits all entries from the data store\nmatching the given query.\n\nSQL adapters, such as Postgres and MySQL, can only enumerate a stream\ninside a transaction.\n\nMay raise `Ecto.QueryError` if query validation fails.","ref":"Ecto.Repo.html#c:stream/2"},{"type":"callback","title":"Options - Ecto.Repo.stream/2","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\n  * `:max_rows` - The number of rows to load from the database as we stream.\n    It is supported at least by Postgres and MySQL and defaults to 500.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:stream/2-options"},{"type":"callback","title":"Example - Ecto.Repo.stream/2","doc":"# Fetch all post titles\n    query = from p in Post,\n         select: p.title\n    stream = MyRepo.stream(query)\n    MyRepo.transaction(fn ->\n      Enum.to_list(stream)\n    end)","ref":"Ecto.Repo.html#c:stream/2-example"},{"type":"callback","title":"Ecto.Repo.transaction/2","doc":"Runs the given function or `Ecto.Multi` inside a transaction.","ref":"Ecto.Repo.html#c:transaction/2"},{"type":"callback","title":"Use with function - Ecto.Repo.transaction/2","doc":"`c:transaction/2` can be called with both a function of arity\nzero or one. The arity zero function will just be executed as is:\n\n    import Ecto.Changeset, only: [change: 2]\n\n    MyRepo.transaction(fn ->\n      MyRepo.update!(change(alice, balance: alice.balance - 10))\n      MyRepo.update!(change(bob, balance: bob.balance + 10))\n    end)\n\nWhile the arity one function will receive the repo of the transaction\nas its first argument:\n\n    MyRepo.transaction(fn repo ->\n      repo.insert!(%Post{})\n    end)\n\nIf an Elixir exception occurs the transaction will be rolled back\nand the exception will bubble up from the transaction function.\nIf no exception occurs, the transaction is committed when the\nfunction returns. A transaction can be explicitly rolled back\nby calling `c:rollback/1`, this will immediately leave the function\nand return the value given to `rollback` as `{:error, value}`.\n\nA successful transaction returns the value returned by the function\nwrapped in a tuple as `{:ok, value}`.\n\n#","ref":"Ecto.Repo.html#c:transaction/2-use-with-function"},{"type":"callback","title":"Nested transactions - Ecto.Repo.transaction/2","doc":"If `c:transaction/2` is called inside another transaction, the function\nis simply executed, without wrapping the new transaction call in any\nway. If there is an error in the inner transaction and the error is\nrescued, or the inner transaction is rolled back, the whole outer\ntransaction is aborted, guaranteeing nothing will be committed.\n\nBelow is an example of how rollbacks work with nested transactions:\n\n    {:error, :rollback} =\n      MyRepo.transaction(fn ->\n        {:error, :posting_not_allowed} =\n          MyRepo.transaction(fn ->\n            # This function call causes the following to happen:\n            #\n            #   * the transaction is rolled back in the database,\n            #   * code execution is stopped within the current function,\n            #   * and the value, passed to `rollback/1` is returned from\n            #     `MyRepo.transaction/1` as the second element in the error\n            #     tuple.\n            #\n            MyRepo.rollback(:posting_not_allowed)\n\n            # `rollback/1` stops execution, so code here won't be run\n          end)\n\n        # The transaction here is now aborted and any further\n        # operation will raise an exception.\n      end)\n\nSee the \"Aborted transactions\" section for more examples of aborted\ntransactions and how to handle them.\n\nIn practice, managing nested transactions can become complex quickly.\nFor this reason, Ecto provides `Ecto.Multi` for composing transactions.","ref":"Ecto.Repo.html#c:transaction/2-nested-transactions"},{"type":"callback","title":"Use with Ecto.Multi - Ecto.Repo.transaction/2","doc":"`c:transaction/2` also accepts the `Ecto.Multi` struct as first argument.\n`Ecto.Multi` allows you to compose transactions operations, step by step,\nand manage what happens in case of success or failure.\n\nWhen an `Ecto.Multi` is given to this function, a transaction will be started,\nall operations applied and in case of success committed returning `{:ok, changes}`:\n\n    # With Ecto.Multi\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:post, %Post{})\n    |> MyRepo.transaction\n\nIn case of any errors the transaction will be rolled back and\n`{:error, failed_operation, failed_value, changes_so_far}` will be returned.\n\nExplore the `Ecto.Multi` documentation to learn more and find detailed examples.","ref":"Ecto.Repo.html#c:transaction/2-use-with-ecto-multi"},{"type":"callback","title":"Aborted transactions - Ecto.Repo.transaction/2","doc":"When an operation inside a transaction fails, the transaction is aborted in the database.\nFor instance, if you attempt an insert that violates a unique constraint, the insert fails\nand the transaction is aborted. In such cases, any further operation inside the transaction\nwill raise exceptions.\n\nTake the following transaction as an example:\n\n    Repo.transaction(fn repo ->\n      case repo.insert(changeset) do\n        {:ok, post} ->\n          repo.insert(%Status{value: \"success\"})\n\n        {:error, changeset} ->\n          repo.insert(%Status{value: \"failure\"})\n      end\n    end)\n\nIf the changeset is valid, but the insert operation fails due to a database constraint,\nthe subsequent `repo.insert(%Failure{})` operation will raise an exception because the\ndatabase has already aborted the transaction and thus making the operation invalid.\nIn Postgres, the exception would look like this:\n\n    ** (Postgrex.Error) ERROR 25P02 (in_failed_sql_transaction) current transaction is aborted, commands ignored until end of transaction block\n\nIf the changeset is invalid before it reaches the database due to a validation error,\nno statement is sent to the database, an `:error` tuple is returned, and `repo.insert(%Failure{})`\noperation will execute as usual.\n\nWe have two options to deal with such scenarios:\n\nIf you don't want to change the semantics of your code,  you can also use the savepoints\nfeature by passing the `:mode` option like this: `repo.insert(changeset, mode: :savepoint)`.\nIn case of an exception, the transaction will rollback to the savepoint and prevent\nthe transaction from failing.\n\nAnother alternative is to handle this operation outside of the transaction.\nFor example, you can choose to perform an explicit `repo.rollback` call in the\n`{:error, changeset}` clause and then perform the `repo.insert(%Failure{})` outside\nof the transaction. You might also consider using `Ecto.Multi`, as they automatically\nrollback whenever an operation fails.","ref":"Ecto.Repo.html#c:transaction/2-aborted-transactions"},{"type":"callback","title":"Working with processes - Ecto.Repo.transaction/2","doc":"The transaction is per process. A separate process started inside a\ntransaction won't be part of the same transaction and will use a separate\nconnection altogether.\n\nWhen using the `Ecto.Adapters.SQL.Sandbox` in tests, while it may be\npossible to share the connection between processes, the parent process\nwill typically hold the connection until the transaction completes. This\nmay lead to a deadlock if the child process attempts to use the same connection.\nSee the docs for\n[`Ecto.Adapters.SQL.Sandbox`](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html)\nfor more information.","ref":"Ecto.Repo.html#c:transaction/2-working-with-processes"},{"type":"callback","title":"Options - Ecto.Repo.transaction/2","doc":"See the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:transaction/2-options"},{"type":"callback","title":"Ecto.Repo.update/2","doc":"Updates a changeset using its primary key.\n\nA changeset is required as it is the only mechanism for\ntracking dirty changes. Only the fields present in the `changes` part\nof the changeset are sent to the database. Any other, in-memory\nchanges done to the schema are ignored. If more than one database\noperation is required, they're automatically wrapped in a transaction.\n\nIf the struct has no primary key, `Ecto.NoPrimaryKeyFieldError`\nwill be raised.\n\nIf the struct cannot be found, `Ecto.StaleEntryError` will be raised.\n\nIt returns `{:ok, struct}` if the struct has been successfully\nupdated or `{:error, changeset}` if there was a validation\nor a known constraint error.","ref":"Ecto.Repo.html#c:update/2"},{"type":"callback","title":"Options - Ecto.Repo.update/2","doc":"* `:returning` - selects which fields to return. It accepts a list\n    of fields to be returned from the database. When `true`, returns\n    all fields, including those marked as `load_in_query: false`. When\n    `false`, no extra fields are returned. It will always include all\n    fields in `read_after_writes`. Be aware that the fields returned\n    from the database overwrite what was supplied by the user. Any field\n    not returned by the database will be present with the original value\n    supplied by the user. Not all databases support this option.\n\n  * `:force` - By default, if there are no changes in the changeset,\n    `c:update/2` is a no-op. By setting this option to true, update\n    callbacks will always be executed, even if there are no changes\n    (including timestamps).\n\n  * `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This overrides the prefix set\n    in the query and any `@schema_prefix` set on any schemas. Also, the\n    `@schema_prefix` for the parent record will override all default\n    `@schema_prefix`s set in any child schemas for associations.\n\n  * `:stale_error_field` - The field where stale errors will be added in\n    the returning changeset. This option can be used to avoid raising\n    `Ecto.StaleEntryError`.\n\n  * `:stale_error_message` - The message to add to the configured\n    `:stale_error_field` when stale errors happen, defaults to \"is stale\".\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for more options.","ref":"Ecto.Repo.html#c:update/2-options"},{"type":"callback","title":"Example - Ecto.Repo.update/2","doc":"post = MyRepo.get!(Post, 42)\n    post = Ecto.Changeset.change post, title: \"New title\"\n    case MyRepo.update post do\n      {:ok, struct}       -> # Updated with success\n      {:error, changeset} -> # Something went wrong\n    end","ref":"Ecto.Repo.html#c:update/2-example"},{"type":"callback","title":"Ecto.Repo.update!/2","doc":"Same as `c:update/2` but returns the struct or raises if the changeset is invalid.","ref":"Ecto.Repo.html#c:update!/2"},{"type":"callback","title":"Ecto.Repo.update_all/3","doc":"Updates all entries matching the given query with the given values.\n\nIt returns a tuple containing the number of entries and any returned\nresult as second element. The second element is `nil` by default\nunless a `select` is supplied in the update query. Note, however,\nnot all databases support returning data from UPDATEs.\n\nKeep in mind this `update_all` will not update autogenerated\nfields like the `updated_at` columns.\n\nSee `Ecto.Query.update/3` for update operations that can be\nperformed on fields.","ref":"Ecto.Repo.html#c:update_all/3"},{"type":"callback","title":"Options - Ecto.Repo.update_all/3","doc":"* `:prefix` - The prefix to run the query on (such as the schema path\n    in Postgres or the database in MySQL). This will be applied to all `from`\n    and `join`s in the query that did not have a prefix previously given\n    either via the `:prefix` option on `join`/`from` or via `@schema_prefix`\n    in the schema. For more information see the \"Query Prefix\" section of the\n    `Ecto.Query` documentation.\n\nSee the [\"Shared options\"](#module-shared-options) section at the module\ndocumentation for remaining options.","ref":"Ecto.Repo.html#c:update_all/3-options"},{"type":"callback","title":"Examples - Ecto.Repo.update_all/3","doc":"MyRepo.update_all(Post, set: [title: \"New title\"])\n\n    MyRepo.update_all(Post, inc: [visits: 1])\n\n    from(p in Post, where: p.id   MyRepo.update_all(set: [title: \"New title\"])\n\n    from(p in Post, where: p.id   MyRepo.update_all([])\n\n    from(p in Post, where: p.id   MyRepo.update_all([])\n\n    from(p in Post, where: p.id   MyRepo.update_all([])\n\n    from(p in Post, where: p.id   MyRepo.update_all([])","ref":"Ecto.Repo.html#c:update_all/3-examples"},{"type":"type","title":"Ecto.Repo.t/0","doc":"","ref":"Ecto.Repo.html#t:t/0"},{"type":"module","title":"Ecto.Schema","doc":"An Ecto schema maps external data into Elixir structs.\n\nThe definition of the schema is possible through two main APIs:\n`schema/2` and `embedded_schema/1`.\n\n`schema/2` is typically used to map data from a persisted source,\nusually a database table, into Elixir structs and vice-versa. For\nthis reason, the first argument of `schema/2` is the source (table)\nname. Structs defined with `schema/2` also contain a `__meta__` field\nwith metadata holding the status of the struct, for example, if it\nhas been built, loaded or deleted.\n\nOn the other hand, `embedded_schema/1` is used for defining schemas\nthat are embedded in other schemas or only exist in-memory. For example,\nyou can use such schemas to receive data from a command line interface\nand validate it, without ever persisting it elsewhere. Such structs\ndo not contain a `__meta__` field, as they are never persisted.\n\nBesides working as data mappers, `embedded_schema/1` and `schema/2` can\nalso be used together to decouple how the data is represented in your\napplications from the database. Let's see some examples.","ref":"Ecto.Schema.html"},{"type":"module","title":"Example - Ecto.Schema","doc":"defmodule User do\n      use Ecto.Schema\n\n      schema \"users\" do\n        field :name, :string\n        field :age, :integer, default: 0\n        field :password, :string, redact: true\n        has_many :posts, Post\n      end\n    end\n\nBy default, a schema will automatically generate a primary key which is named\n`id` and of type `:integer`. The `field` macro defines a field in the schema\nwith given name and type. `has_many` associates many posts with the user\nschema. Schemas are regular structs and can be created and manipulated directly\nusing Elixir's struct API:\n\n    iex> user = %User{name: \"jane\"}\n    iex> %{user | age: 30}\n\nHowever, most commonly, structs are cast, validated and manipulated with the\n`Ecto.Changeset` module.\n\nNote that the name of the database table does not need to correlate to your\nmodule name.  For example, if you are working with a legacy database, you can\nreference the table name when you define your schema:\n\n    defmodule User do\n      use Ecto.Schema\n\n      schema \"legacy_users\" do\n        # ... fields ...\n      end\n    end\n\nSource-based schemas are queryable by default, which means we can pass them\nto `Ecto.Repo` modules and also build queries:\n\n    MyRepo.all(User)\n    MyRepo.all(from u in User, where: u.id == 13)\n\nThe repository will then run the query against the source/table.\n\nEmbedded schemas are defined similarly to source-based schemas. For example,\nyou can use an embedded schema to represent your UI, mapping and validating\nits inputs, and then you convert such embedded schema to other schemas that\nare persisted to the database:\n\n    defmodule SignUp do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :name, :string\n        field :age, :integer\n        field :email, :string\n        field :accepts_conditions, :boolean\n      end\n    end\n\n    defmodule Profile do\n      use Ecto.Schema\n\n      schema \"profiles\" do\n        field :name\n        field :age\n        belongs_to :account, Account\n      end\n    end\n\n    defmodule Account do\n      use Ecto.Schema\n\n      schema \"accounts\" do\n        field :email\n      end\n    end\n\nThe `SignUp` schema can be cast and validated with the help of the\n`Ecto.Changeset` module, and afterwards, you can copy its data to\nthe `Profile` and `Account` structs that will be persisted to the\ndatabase with the help of `Ecto.Repo`. On the other hand, embedded\nschemas cannot be queried directly (they are not queryable).\n\n> #### `use Ecto.Schema` {: .info}\n>\n> When you `use Ecto.Schema`, it will:\n>\n> - import `Ecto.Schema` macros `schema/2` and `embedded_schema/1`\n> - register default values for module attributes that can be overridden, such as\n> `@primary_key` and `@timestamps_opts`\n> - define reflection functions such as `__schema__/1` and `__changeset__/1`\n>\n> We detail those throughout the module documentation.","ref":"Ecto.Schema.html#module-example"},{"type":"module","title":"Redacting fields - Ecto.Schema","doc":"A field marked with `redact: true` will display a value of `**redacted**`\nwhen inspected in changes inside a `Ecto.Changeset` and be excluded from\ninspect on the schema unless the schema module is tagged with\nthe option `@ecto_derive_inspect_for_redacted_fields false`.","ref":"Ecto.Schema.html#module-redacting-fields"},{"type":"module","title":"Schema attributes - Ecto.Schema","doc":"Supported attributes for configuring the defined schema. They must\nbe set after the `use Ecto.Schema` call and before the `schema/2`\ndefinition.\n\nThese attributes are:\n\n  * `@primary_key` - configures the schema primary key. It expects\n    a tuple `{field_name, type, options}` with the primary key field\n    name, type (typically `:id` or `:binary_id`, but can be any type) and\n    options. It also accepts `false` to disable the generation of a primary\n    key field. Defaults to `{:id, :id, autogenerate: true}`.\n\n  * `@schema_prefix` - configures the schema prefix. Defaults to `nil`,\n    which generates structs and queries without prefix. When set, the\n    prefix will be used by every built struct and on queries whenever\n    the schema is used in a `from` or a `join`. In PostgreSQL, the prefix\n    is called \"SCHEMA\" (typically set via Postgres' `search_path`).\n    In MySQL the prefix points to databases.\n\n  * `@schema_context` - configures the schema context. Defaults to `nil`,\n    which generates structs and queries without context. Context are not used\n    by the built-in SQL adapters.\n\n  * `@foreign_key_type` - configures the default foreign key type\n    used by `belongs_to` associations. It must be set in the same\n    module that defines the `belongs_to`. Defaults to `:id`;\n\n  * `@timestamps_opts` - configures the default timestamps type\n    used by `timestamps`. Defaults to `[type: :naive_datetime]`;\n\n  * `@derive` - the same as `@derive` available in `Kernel.defstruct/1`\n    as the schema defines a struct behind the scenes;\n\n  * `@field_source_mapper` - a function that receives the current field name\n    and returns the mapping of this field name in the underlying source.\n    In other words, it is a mechanism to automatically generate the `:source`\n    option for the `field` macro. It defaults to `fn x -> x end`, where no\n    field transformation is done;\n\nThe advantage of configuring the schema via those attributes is\nthat they can be set with a macro to configure application wide\ndefaults.\n\nFor example, if your database does not support autoincrementing\nprimary keys and requires something like UUID or a RecordID, you\ncan configure and use `:binary_id` as your primary key type as follows:\n\n    # Define a module to be used as base\n    defmodule MyApp.Schema do\n      defmacro __using__(_) do\n        quote do\n          use Ecto.Schema\n          @primary_key {:id, :binary_id, autogenerate: true}\n          @foreign_key_type :binary_id\n        end\n      end\n    end\n\n    # Now use MyApp.Schema to define new schemas\n    defmodule MyApp.Comment do\n      use MyApp.Schema\n\n      schema \"comments\" do\n        belongs_to :post, MyApp.Post\n      end\n    end\n\nAny schemas using `MyApp.Schema` will get the `:id` field with type\n`:binary_id` as the primary key. We explain what the `:binary_id` type\nentails in the next section.\n\nThe `belongs_to` association on `MyApp.Comment` will also define\na `:post_id` field with `:binary_id` type that references the `:id`\nfield of the `MyApp.Post` schema.","ref":"Ecto.Schema.html#module-schema-attributes"},{"type":"module","title":"Primary keys - Ecto.Schema","doc":"Ecto supports two ID types, called `:id` and `:binary_id`, which are\noften used as the type for primary keys and associations.\n\nThe `:id` type is used when the primary key is an integer while the\n`:binary_id` is used for primary keys in particular binary formats,\nwhich may be `Ecto.UUID` for databases like PostgreSQL and MySQL,\nor some specific ObjectID or RecordID often imposed by NoSQL databases.\n\nIn both cases, both types have their semantics specified by the\nunderlying adapter/database. If you use the `:id` type with\n`:autogenerate`, it means the database will be responsible for\nauto-generation of the id. This is often the case for primary keys\nin relational databases which are auto-incremented.\n\nThere are two ways to define primary keys in Ecto: using the `@primary_key`\nmodule attribute and using `primary_key: true` as option for `field/3` in\nyour schema definition. They are not mutually exclusive and can be used\ntogether.\n\nUsing `@primary_key` should be preferred for single field primary keys and\nsharing primary key definitions between multiple schemas using macros.\nSetting `@primary_key` also automatically configures the reference types\nfor `has_one` and `has_many` associations.\n\nEcto also supports composite primary keys, which is where you need to use\n`primary_key: true` for the fields in your schema. This usually goes along\nwith setting `@primary_key false` to disable generation of additional\nprimary key fields.\n\nBesides `:id` and `:binary_id`, which are often used by primary\nand foreign keys, Ecto provides a huge variety of types to be used\nby any field.","ref":"Ecto.Schema.html#module-primary-keys"},{"type":"module","title":"Types and casting - Ecto.Schema","doc":"When defining the schema, types need to be given. Types are split\ninto two categories, primitive types and custom types.\n\n#","ref":"Ecto.Schema.html#module-types-and-casting"},{"type":"module","title":"Primitive types - Ecto.Schema","doc":"The primitive types are:\n\nEcto type               | Elixir type             | Literal syntax in query\n:---------------------- | :---------------------- | :---------------------\n`:id`                   | `integer`               | 1, 2, 3\n`:binary_id`            | `binary`                | ` >`\n`:integer`              | `integer`               | 1, 2, 3\n`:float`                | `float`                 | 1.0, 2.0, 3.0\n`:boolean`              | `boolean`               | true, false\n`:string`               | UTF-8 encoded `string`  | \"hello\"\n`:binary`               | `binary`                | ` >`\n`{:array, inner_type}`  | `list`                  | `[value, value, value, ...]`\n`:map`                  | `map` |\n`{:map, inner_type}`    | `map` |\n`:decimal`              | [`Decimal`](https://github.com/ericmj/decimal) |\n`:date`                 | `Date` |\n`:time`                 | `Time` |\n`:time_usec`            | `Time` |\n`:naive_datetime`       | `NaiveDateTime` |\n`:naive_datetime_usec`  | `NaiveDateTime` |\n`:utc_datetime`         | `DateTime` |\n`:utc_datetime_usec`    | `DateTime` |\n\n**Notes:**\n\n  * When using database migrations provided by \"Ecto SQL\", you can pass\n    your Ecto type as the column type. However, note the same Ecto type\n    may support multiple database types. For example, all of `:varchar`,\n    `:text`, `:bytea`, etc. translate to Ecto's `:string`. Similarly,\n    Ecto's `:decimal` can be used for `:numeric` and other database\n    types. For more information, see [all migration types](https://hexdocs.pm/ecto_sql/Ecto.Migration.html#module-field-types).\n\n  * For the `{:array, inner_type}` and `{:map, inner_type}` type,\n    replace `inner_type` with one of the valid types, such as `:string`.\n\n  * For the `:decimal` type, `+Infinity`, `-Infinity`, and `NaN` values\n    are not supported, even though the `Decimal` library handles them.\n    To support them, you can create a custom type.\n\n  * For calendar types with and without microseconds, the precision is\n    enforced when persisting to the DB. For example, casting `~T[09:00:00]`\n    as `:time_usec` will succeed and result in `~T[09:00:00.000000]`, but\n    persisting a type without microseconds as `:time_usec` will fail.\n    Similarly, casting `~T[09:00:00.000000]` as `:time` will succeed, but\n    persisting will not. This is the same behaviour as seen in other types,\n    where casting has to be done explicitly and is never performed\n    implicitly when loading from or dumping to the database.\n\n#","ref":"Ecto.Schema.html#module-primitive-types"},{"type":"module","title":"Custom types - Ecto.Schema","doc":"Besides providing primitive types, Ecto allows custom types to be\nimplemented by developers, allowing Ecto behaviour to be extended.\n\nA custom type is a module that implements one of the `Ecto.Type`\nor `Ecto.ParameterizedType` behaviours. By default, Ecto provides\nthe following custom types:\n\nCustom type             | Database type           | Elixir type\n:---------------------- | :---------------------- | :---------------------\n`Ecto.UUID`             | `:uuid` (as a binary)   | `string()` (as a UUID)\n`Ecto.Enum`             | `:string`               | `atom()`\n\nFinally, schemas can also have virtual fields by passing the\n`virtual: true` option. These fields are not persisted to the database\nand can optionally not be type checked by declaring type `:any`.\n\n#","ref":"Ecto.Schema.html#module-custom-types"},{"type":"module","title":"The datetime types - Ecto.Schema","doc":"Four different datetime primitive types are available:\n\n  * `naive_datetime` - has a precision of seconds and casts values\n    to Elixir's `NaiveDateTime` struct which has no timezone information.\n\n  * `naive_datetime_usec` - has a default precision of microseconds and\n    also casts values to `NaiveDateTime` with no timezone information.\n\n  * `utc_datetime` - has a precision of seconds and casts values to\n    Elixir's `DateTime` struct and expects the time zone to be set to UTC.\n\n  * `utc_datetime_usec` has a default precision of microseconds and also\n    casts values to `DateTime` expecting the time zone be set to UTC.\n\nAll of those types are represented by the same timestamp/datetime in the\nunderlying data storage, the difference are in their precision and how the\ndata is loaded into Elixir.\n\nHaving different precisions allows developers to choose a type that will\nbe compatible with the database and your project's precision requirements.\nFor example, some older versions of MySQL do not support microseconds in\ndatetime fields.\n\nWhen choosing what datetime type to work with, keep in mind that Elixir\nfunctions like `NaiveDateTime.utc_now/0` have a default precision of 6.\nCasting a value with a precision greater than 0 to a non-`usec` type will\ntruncate all microseconds and set the precision to 0.\n\n#","ref":"Ecto.Schema.html#module-the-datetime-types"},{"type":"module","title":"The map type - Ecto.Schema","doc":"The map type allows developers to store an Elixir map directly\nin the database:\n\n    # In your migration\n    create table(:users) do\n      add :data, :map\n    end\n\n    # In your schema\n    field :data, :map\n\n    # Now in your code\n    user = Repo.insert! %User{data: %{\"foo\" => \"bar\"}}\n\nKeep in mind that we advise the map keys to be strings or integers\ninstead of atoms. Atoms may be accepted depending on how maps are\nserialized but the database will always convert atom keys to strings\ndue to security reasons.\n\nIn order to support maps, different databases may employ different\ntechniques. For example, PostgreSQL will store those values in jsonb\nfields, allowing you to just query parts of it. MSSQL, on\nthe other hand, does not yet provide a JSON type, so the value will be\nstored in a text field.\n\nFor maps to work in such databases, Ecto will need a JSON library.\nBy default Ecto will use [Jason](https://github.com/michalmuskala/jason)\nwhich needs to be added to your deps in `mix.exs`:\n\n    {:jason, \"~> 1.0\"}\n\nYou can however configure the adapter to use another library. For example,\nif using Postgres:\n\n    config :postgrex, :json_library, YourLibraryOfChoice\n\nOr if using MySQL:\n\n    config :mariaex, :json_library, YourLibraryOfChoice\n\nIf changing the JSON library, remember to recompile the adapter afterwards\nby cleaning the current build:\n\n    mix deps.clean --build postgrex\n\n#","ref":"Ecto.Schema.html#module-the-map-type"},{"type":"module","title":"Casting - Ecto.Schema","doc":"When directly manipulating the struct, it is the responsibility of\nthe developer to ensure the field values have the proper type. For\nexample, you can create a user struct with an invalid value\nfor `age`:\n\n    iex> user = %User{age: \"0\"}\n    iex> user.age\n    \"0\"\n\nHowever, if you attempt to persist the struct above, an error will\nbe raised since Ecto validates the types when sending them to the\nadapter/database.\n\nTherefore, when working with and manipulating external data, it is\nrecommended to use `Ecto.Changeset`'s that are able to filter\nand properly cast external data:\n\n    changeset = Ecto.Changeset.cast(%User{}, %{\"age\" => \"0\"}, [:age])\n    user = Repo.insert!(changeset)\n\n**You can use Ecto schemas and changesets to cast and validate any kind\nof data, regardless if the data will be persisted to an Ecto repository\nor not**.","ref":"Ecto.Schema.html#module-casting"},{"type":"module","title":"Reflection - Ecto.Schema","doc":"Any schema module will generate the `__schema__` function that can be\nused for runtime introspection of the schema:\n\n* `__schema__(:source)` - Returns the source as given to `schema/2`;\n* `__schema__(:prefix)` - Returns optional prefix for source provided by\n  `@schema_prefix` schema attribute;\n* `__schema__(:primary_key)` - Returns a list of primary key fields (empty if there is none);\n\n* `__schema__(:fields)` - Returns a list of all non-virtual field names;\n* `__schema__(:virtual_fields)` - Returns a list of all virtual field names;\n* `__schema__(:field_source, field)` - Returns the alias of the given field;\n\n* `__schema__(:type, field)` - Returns the type of the given non-virtual field;\n* `__schema__(:virtual_type, field)` - Returns the type of the given virtual field;\n\n* `__schema__(:associations)` - Returns a list of all association field names;\n* `__schema__(:association, assoc)` - Returns the association reflection of the given assoc;\n\n* `__schema__(:embeds)` - Returns a list of all embedded field names;\n* `__schema__(:embed, embed)` - Returns the embedding reflection of the given embed;\n\n* `__schema__(:read_after_writes)` - Non-virtual fields that must be read back\n  from the database after every write (insert or update);\n\n* `__schema__(:autogenerate_id)` - Primary key that is auto generated on insert;\n* `__schema__(:autogenerate_fields)` - Returns a list of fields names that are auto\n  generated on insert, except for the primary key;\n\n* `__schema__(:redact_fields)` - Returns a list of redacted field names;\n\nFurthermore, both `__struct__` and `__changeset__` functions are\ndefined so structs and changeset functionalities are available.","ref":"Ecto.Schema.html#module-reflection"},{"type":"module","title":"Working with typespecs - Ecto.Schema","doc":"Generating typespecs for schemas is out of the scope of `Ecto.Schema`.\n\nIn order to be able to use types such as `User.t()`, `t/0` has to be defined manually:\n\n    defmodule User do\n      use Ecto.Schema\n\n      @type t :: %__MODULE__{\n        name: String.t(),\n        age: non_neg_integer()\n      }\n\n      # ... schema ...\n    end\n\nDefining the type of each field is not mandatory, but it is preferable.","ref":"Ecto.Schema.html#module-working-with-typespecs"},{"type":"macro","title":"Ecto.Schema.belongs_to/3","doc":"Indicates a one-to-one or many-to-one association with another schema.\n\nThe current schema belongs to zero or one records of the other schema. The other\nschema often has a `has_one` or a `has_many` field with the reverse association.\n\nYou should use `belongs_to` in the table that contains the foreign key. Imagine\na company   employee relationship. If the employee contains the `company_id` in\nthe underlying database table, we say the employee belongs to company.\n\nIn fact, when you invoke this macro, a field with the name of foreign key is\nautomatically defined in the schema for you.","ref":"Ecto.Schema.html#belongs_to/3"},{"type":"macro","title":"Options - Ecto.Schema.belongs_to/3","doc":"* `:foreign_key` - Sets the foreign key field name, defaults to the name\n    of the association suffixed by `_id`. For example, `belongs_to :company`\n    will define foreign key of `:company_id`. The associated `has_one` or `has_many`\n    field in the other schema should also have its `:foreign_key` option set\n    with the same value.\n\n  * `:references` - Sets the key on the other schema to be used for the\n    association, defaults to: `:id`\n\n  * `:define_field` - When false, does not automatically define a `:foreign_key`\n    field, implying the user is defining the field manually elsewhere\n\n  * `:type` - Sets the type of automatically defined `:foreign_key`.\n    Defaults to: `:integer` and can be set per schema via `@foreign_key_type`\n\n  * `:on_replace` - The action taken on associations when the record is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, `:nilify`, `:update`, or `:delete`.\n    See `Ecto.Changeset`'s section on related data for more info.\n\n  * `:defaults` - Default values to use when building the association.\n    It may be a keyword list of options that override the association schema\n    or an `atom`/`{module, function, args}` that receives the association struct\n    and the owner struct as arguments. For example, if you set\n    `Comment.belongs_to :post, defaults: [public: true]`,\n    then when using `Ecto.build_assoc(comment, :post)`, the post will have\n    `post.public == true`. Alternatively, you can set it to\n    `Comment.belongs_to :post, defaults: :update_post`, which will invoke\n    `Comment.update_post(post, comment)`, or set it to a MFA tuple such as\n    `{Mod, fun, [arg3, arg4]}`, which will invoke `Mod.fun(post, comment, arg3, arg4)`\n\n  * `:primary_key` - If the underlying belongs_to field is a primary key\n\n  * `:source` - Defines the name that is to be used in database for this field\n\n  * `:where` - A filter for the association. See \"Filtering associations\"\n    in `has_many/3`.","ref":"Ecto.Schema.html#belongs_to/3-options"},{"type":"macro","title":"Examples - Ecto.Schema.belongs_to/3","doc":"defmodule Comment do\n      use Ecto.Schema\n\n      schema \"comments\" do\n        belongs_to :post, Post\n      end\n    end\n\n    # The post can come preloaded on the comment record\n    [comment] = Repo.all(from(c in Comment, where: c.id == 42, preload: :post))\n    comment.post #=> %Post{...}\n\nIf you need custom options on the underlying field, you can define the\nfield explicitly and then pass `define_field: false` to `belongs_to`:\n\n    defmodule Comment do\n      use Ecto.Schema\n\n      schema \"comments\" do\n        field :post_id, :integer, ... # custom options\n        belongs_to :post, Post, define_field: false\n      end\n    end","ref":"Ecto.Schema.html#belongs_to/3-examples"},{"type":"macro","title":"Polymorphic associations - Ecto.Schema.belongs_to/3","doc":"One common use case for belongs to associations is to handle\npolymorphism. For example, imagine you have defined a Comment\nschema and you wish to use it for commenting on both tasks and\nposts.\n\nSome abstractions would force you to define some sort of\npolymorphic association with two fields in your database:\n\n    * commentable_type\n    * commentable_id\n\nThe problem with this approach is that it breaks references in\nthe database. You can't use foreign keys and it is very inefficient,\nboth in terms of query time and storage.\n\nIn Ecto, we have three ways to solve this issue. The simplest\nis to define multiple fields in the Comment schema, one for each\nassociation:\n\n    * task_id\n    * post_id\n\nUnless you have dozens of columns, this is simpler for the developer,\nmore DB friendly and more efficient in all aspects.\n\nAlternatively, because Ecto does not tie a schema to a given table,\nwe can use separate tables for each association. Let's start over\nand define a new Comment schema:\n\n    defmodule Comment do\n      use Ecto.Schema\n\n      schema \"abstract table: comments\" do\n        # This will be used by associations on each \"concrete\" table\n        field :assoc_id, :integer\n      end\n    end\n\nNotice we have changed the table name to \"abstract table: comments\".\nYou can choose whatever name you want, the point here is that this\nparticular table will never exist.\n\nNow in your Post and Task schemas:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        has_many :comments, {\"posts_comments\", Comment}, foreign_key: :assoc_id\n      end\n    end\n\n    defmodule Task do\n      use Ecto.Schema\n\n      schema \"tasks\" do\n        has_many :comments, {\"tasks_comments\", Comment}, foreign_key: :assoc_id\n      end\n    end\n\nNow each association uses its own specific table, \"posts_comments\"\nand \"tasks_comments\", which must be created on migrations. The\nadvantage of this approach is that we never store unrelated data\ntogether, also ensuring we keep database references fast and correct.\n\nWhen using this technique, the only limitation is that you cannot\nbuild comments directly. For example, the command below\n\n    Repo.insert!(%Comment{})\n\nwill attempt to use the abstract table. Instead, one should use\n\n    Repo.insert!(build_assoc(post, :comments))\n\nleveraging the `Ecto.build_assoc/3` function. You can also\nuse `Ecto.assoc/2` or pass a tuple in the query syntax\nto easily retrieve associated comments to a given post or\ntask:\n\n    # Fetch all comments associated with the given task\n    Repo.all(Ecto.assoc(task, :comments))\n\nOr all comments in a given table:\n\n    Repo.all from(c in {\"posts_comments\", Comment}), ...)\n\nThe third and final option is to use `many_to_many/3` to\ndefine the relationships between the resources. In this case,\nthe comments table won't have the foreign key, instead there\nis an intermediary table responsible for associating the entries:\n\n    defmodule Comment do\n      use Ecto.Schema\n      schema \"comments\" do\n        # ...\n      end\n    end\n\nIn your posts and tasks:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        many_to_many :comments, Comment, join_through: \"posts_comments\"\n      end\n    end\n\n    defmodule Task do\n      use Ecto.Schema\n\n      schema \"tasks\" do\n        many_to_many :comments, Comment, join_through: \"tasks_comments\"\n      end\n    end\n\nSee `many_to_many/3` for more information on this particular approach.","ref":"Ecto.Schema.html#belongs_to/3-polymorphic-associations"},{"type":"macro","title":"Ecto.Schema.embedded_schema/1","doc":"Defines an embedded schema with the given field definitions.\n\nAn embedded schema is either embedded into another\nschema or kept exclusively in memory. For this reason,\nan embedded schema does not require a source name and\nit does not include a metadata field.\n\nEmbedded schemas by default set the primary key type\nto `:binary_id` but such can be configured with the\n`@primary_key` attribute.","ref":"Ecto.Schema.html#embedded_schema/1"},{"type":"macro","title":"Ecto.Schema.embeds_many/3","doc":"Indicates an embedding of many schemas.\n\nThe current schema has zero or more records of the other schema embedded\ninside of it. Embeds have all the things regular schemas have.\n\nIt is recommended to declare your `embeds_many/3` field with type `:map`\nin your migrations, instead of using `{:array, :map}`. Ecto can work with\nboth maps and arrays as the container for embeds (and in most databases\nmaps are represented as JSON which allows Ecto to choose what works best).\n\nThe embedded may or may not have a primary key. Ecto uses the primary keys\nto detect if an embed is being updated or not. If a primary is not present\nand you still want the list of embeds to be updated, `:on_replace` must be\nset to `:delete`, forcing all current embeds to be deleted and replaced by\nnew ones whenever a new list of embeds is set.\n\nFor encoding and decoding of embeds, please read the docs for\n`embeds_one/3`.","ref":"Ecto.Schema.html#embeds_many/3"},{"type":"macro","title":"Options - Ecto.Schema.embeds_many/3","doc":"* `:on_replace` - The action taken on associations when the embed is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, or `:delete`.\n    See `Ecto.Changeset`'s section on related data for more info.\n\n  * `:source` - Defines the name that is to be used in database for this field.\n    This is useful when attaching to an existing database. The value should be\n    an atom.\n\n  * `:load_in_query` - When false, the field will not be loaded when\n    selecting the whole struct in a query, such as `from p in Post, select: p`.\n    Defaults to `true`.","ref":"Ecto.Schema.html#embeds_many/3-options"},{"type":"macro","title":"Examples - Ecto.Schema.embeds_many/3","doc":"defmodule Order do\n      use Ecto.Schema\n\n      schema \"orders\" do\n        embeds_many :items, Item\n      end\n    end\n\n    defmodule Item do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :title\n      end\n    end\n\n    # The items are loaded with the order\n    order = Repo.get!(Order, 42)\n    order.items #=> [%Item{...}, ...]\n\nAdding and removal of embeds can only be done via the `Ecto.Changeset`\nAPI so Ecto can properly track the embed life-cycle:\n\n    # Order has no items\n    order = Repo.get!(Order, 42)\n    order.items\n    # => []\n\n    items  = [%Item{title: \"Soap\"}]\n\n    # Generate a changeset\n    changeset = Ecto.Changeset.change(order)\n\n    # Put a one or more new items\n    changeset = Ecto.Changeset.put_embed(changeset, :items, items)\n\n    # Update the order and fetch items\n    items = Repo.update!(changeset).items\n\n    # Items are generated with a unique identification\n    items\n    # => [%Item{id: \"20a97d94-f79b-4e63-a875-85deed7719b7\", title: \"Soap\"}]\n\nUpdating of embeds must be done using a changeset for each changed embed.\n\n    # Order has an existing items\n    order = Repo.get!(Order, 42)\n    order.items\n    # => [%Item{id: \"20a97d94-f79b-4e63-a875-85deed7719b7\", title: \"Soap\"}]\n\n    # Generate a changeset\n    changeset = Ecto.Changeset.change(order)\n\n    # Put the updated item as a changeset\n    current_item = List.first(order.items)\n    item_changeset = Ecto.Changeset.change(current_item, title: \"Mujju's Soap\")\n    order_changeset = Ecto.Changeset.put_embed(changeset, :items, [item_changeset])\n\n    # Update the order and fetch items\n    items = Repo.update!(order_changeset).items\n\n    # Item has the updated title\n    items\n    # => [%Item{id: \"20a97d94-f79b-4e63-a875-85deed7719b7\", title: \"Mujju's Soap\"}]","ref":"Ecto.Schema.html#embeds_many/3-examples"},{"type":"macro","title":"Inline embedded schema - Ecto.Schema.embeds_many/3","doc":"The schema module can be defined inline in the parent schema in simple\ncases:\n\n    defmodule Parent do\n      use Ecto.Schema\n\n      schema \"parents\" do\n        field :name, :string\n\n        embeds_many :children, Child do\n          field :name, :string\n          field :age,  :integer\n        end\n      end\n    end\n\nPrimary keys are automatically set up for  embedded  schemas as well,\ndefaulting  to  `{:id,  :binary_id, autogenerate:   true}`. You can\ncustomize it by passing a `:primary_key` option with the same arguments\nas `@primary_key` (see the [Schema attributes](https://hexdocs.pm/ecto/Ecto.Schema.html#module-schema-attributes)\nsection for more info).\n\nDefining embedded schema in such a way will define a `Parent.Child` module\nwith the appropriate struct. In order to properly cast the embedded schema.\nWhen casting the inline-defined embedded schemas you need to use the `:with`\noption of `cast_embed/3` to provide the proper function to do the casting.\nFor example:\n\n    def changeset(schema, params) do\n      schema\n      |> cast(params, [:name])\n      |> cast_embed(:children, with: &child_changeset/2)\n    end\n\n    defp child_changeset(schema, params) do\n      schema\n      |> cast(params, [:name, :age])\n    end","ref":"Ecto.Schema.html#embeds_many/3-inline-embedded-schema"},{"type":"macro","title":"Ecto.Schema.embeds_many/4","doc":"Indicates an embedding of many schemas.\n\nFor options and examples see documentation of `embeds_many/3`.","ref":"Ecto.Schema.html#embeds_many/4"},{"type":"macro","title":"Ecto.Schema.embeds_one/3","doc":"Indicates an embedding of a schema.\n\nThe current schema has zero or one records of the other schema embedded\ninside of it. It uses a field similar to the `:map` type for storage,\nbut allows embeds to have all the things regular schema can.\n\nYou must declare your `embeds_one/3` field with type `:map` at the\ndatabase level.\n\nThe embedded may or may not have a primary key. Ecto uses the primary keys\nto detect if an embed is being updated or not. If a primary key is not present,\n`:on_replace` should be set to either `:update` or `:delete` if there is a\ndesire to either update or delete the current embed when a new one is set.","ref":"Ecto.Schema.html#embeds_one/3"},{"type":"macro","title":"Options - Ecto.Schema.embeds_one/3","doc":"* `:primary_key` - The `:primary_key` option can be used with the same arguments\n    as `@primary_key` (see the [Schema attributes](https://hexdocs.pm/ecto/Ecto.Schema.html#module-schema-attributes)\n    section for more info). Primary keys are automatically set up for  embedded  schemas as well,\n    defaulting  to  `{:id,  :binary_id, autogenerate:   true}`.\n\n  * `:on_replace` - The action taken on associations when the embed is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, `:update`, or `:delete`.\n    See `Ecto.Changeset`'s section on related data for more info.\n\n  * `:source` - Defines the name that is to be used in database for this field.\n    This is useful when attaching to an existing database. The value should be\n    an atom.\n\n  * `:load_in_query` - When false, the field will not be loaded when\n    selecting the whole struct in a query, such as `from p in Post, select: p`.\n    Defaults to `true`.","ref":"Ecto.Schema.html#embeds_one/3-options"},{"type":"macro","title":"Examples - Ecto.Schema.embeds_one/3","doc":"defmodule Order do\n      use Ecto.Schema\n\n      schema \"orders\" do\n        embeds_one :item, Item\n      end\n    end\n\n    defmodule Item do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :title\n      end\n    end\n\n    # The item is loaded with the order\n    order = Repo.get!(Order, 42)\n    order.item #=> %Item{...}\n\nAdding and removal of embeds can only be done via the `Ecto.Changeset`\nAPI so Ecto can properly track the embed life-cycle:\n\n    order = Repo.get!(Order, 42)\n    item  = %Item{title: \"Soap\"}\n\n    # Generate a changeset\n    changeset = Ecto.Changeset.change(order)\n\n    # Put a new embed to the changeset\n    changeset = Ecto.Changeset.put_embed(changeset, :item, item)\n\n    # Update the order, and fetch the item\n    item = Repo.update!(changeset).item\n\n    # Item is generated with a unique identification\n    item\n    # => %Item{id: \"20a97d94-f79b-4e63-a875-85deed7719b7\", title: \"Soap\"}","ref":"Ecto.Schema.html#embeds_one/3-examples"},{"type":"macro","title":"Inline embedded schema - Ecto.Schema.embeds_one/3","doc":"The schema module can be defined inline in the parent schema in simple\ncases:\n\n    defmodule Parent do\n      use Ecto.Schema\n\n      schema \"parents\" do\n        field :name, :string\n\n        embeds_one :child, Child do\n          field :name, :string\n          field :age,  :integer\n        end\n      end\n    end\n\nOptions should be passed before the `do` block like this:\n\n    embeds_one :child, Child, on_replace: :delete, primary_key: false do\n      field :name, :string\n      field :age,  :integer\n    end\n\nDefining embedded schema in such a way will define a `Parent.Child` module\nwith the appropriate struct. In order to properly cast the embedded schema.\nWhen casting the inline-defined embedded schemas you need to use the `:with`\noption of `Ecto.Changeset.cast_embed/3` to provide the proper function to do the casting.\nFor example:\n\n    def changeset(schema, params) do\n      schema\n      |> cast(params, [:name])\n      |> cast_embed(:child, with: &child_changeset/2)\n    end\n\n    defp child_changeset(schema, params) do\n      schema\n      |> cast(params, [:name, :age])\n    end","ref":"Ecto.Schema.html#embeds_one/3-inline-embedded-schema"},{"type":"macro","title":"Encoding and decoding - Ecto.Schema.embeds_one/3","doc":"Because many databases do not support direct encoding and decoding\nof embeds, it is often emulated by Ecto by using specific encoding\nand decoding rules.\n\nFor example, PostgreSQL will store embeds on top of JSONB columns,\nwhich means types in embedded schemas won't go through the usual\ndump->DB->load cycle but rather encode->DB->decode->cast. This means\nthat, when using embedded schemas with databases like PG or MySQL,\nmake sure all of your types can be JSON encoded/decoded correctly.\nEcto provides this guarantee for all built-in types.\n\nWhen decoding, if a key exists in the database not defined in the\nschema, it'll be ignored. If a field exists in the schema thats not\nin the database, it's value will be `nil`.","ref":"Ecto.Schema.html#embeds_one/3-encoding-and-decoding"},{"type":"macro","title":"Ecto.Schema.embeds_one/4","doc":"Indicates an embedding of a schema.\n\nFor options and examples see documentation of `embeds_one/3`.","ref":"Ecto.Schema.html#embeds_one/4"},{"type":"macro","title":"Ecto.Schema.field/3","doc":"Defines a field on the schema with given name and type.\n\nThe field name will be used as is to read and write to the database\nby all of the built-in adapters unless overridden with the `:source`\noption.","ref":"Ecto.Schema.html#field/3"},{"type":"macro","title":"Options - Ecto.Schema.field/3","doc":"* `:default` - Sets the default value on the schema and the struct.\n\n    The default value is calculated at compilation time, so don't use\n    expressions like `DateTime.utc_now` or `Ecto.UUID.generate` as\n    they would then be the same for all records: in this scenario you can use\n    the `:autogenerate` option to generate at insertion time.\n\n    The default value is validated against the field's type at compilation time\n    and it will raise an ArgumentError if there is a type mismatch. If you cannot\n    infer  the field's type at compilation time, you can use the\n    `:skip_default_validation` option on the field to skip validations.\n\n    Once a default value is set, if you send changes to the changeset that\n    contains the same value defined as default, validations will not be performed\n    since there are no changes after all.\n\n  * `:source` - Defines the name that is to be used in the database for this field.\n    This is useful when attaching to an existing database. The value should be\n    an atom. This is a last minute translation before the query goes to the database.\n    All references within your Elixir code must still be to the field name,\n    such as in association foreign keys.\n\n  * `:autogenerate` - a `{module, function, args}` tuple for a function\n    to call to generate the field value before insertion if value is not set.\n    A shorthand value of `true` is equivalent to `{type, :autogenerate, []}`.\n\n  * `:read_after_writes` - When true, the field is always read back\n    from the database after insert and updates.\n\n    For relational databases, this means the RETURNING option of those\n    statements is used. For this reason, MySQL does not support this\n    option and will raise an error if a schema is inserted/updated with\n    read after writes fields.\n\n  * `:virtual` - When true, the field is not persisted to the database.\n    Notice virtual fields do not support `:autogenerate` nor\n    `:read_after_writes`.\n\n  * `:primary_key` - When true, the field is used as part of the\n    composite primary key.\n\n  * `:load_in_query` - When false, the field will not be loaded when\n    selecting the whole struct in a query, such as `from p in Post, select: p`.\n    Defaults to `true`.\n\n  * `:redact` - When true, it will display a value of `**redacted**`\n    when inspected in changes inside a `Ecto.Changeset` and be excluded\n    from inspect on the schema. Defaults to `false`.\n\n  * `:skip_default_validation` - When true, it will skip the type validation\n    step at compile time.","ref":"Ecto.Schema.html#field/3-options"},{"type":"macro","title":"Ecto.Schema.has_many/3","doc":"Indicates a one-to-many association with another schema.\n\nThe current schema has zero or more records of the other schema. The other\nschema often has a `belongs_to` field with the reverse association.","ref":"Ecto.Schema.html#has_many/3"},{"type":"macro","title":"Options - Ecto.Schema.has_many/3","doc":"* `:foreign_key` - Sets the foreign key, this should map to a field on the\n    other schema, defaults to the underscored name of the current schema\n    suffixed by `_id`\n\n  * `:references` - Sets the key on the current schema to be used for the\n    association, defaults to the primary key on the schema\n\n  * `:through` - Allow this association to be defined in terms of existing\n    associations. Read the section on `:through` associations for more info\n\n  * `:on_delete` - The action taken on associations when parent record\n    is deleted. May be `:nothing` (default), `:nilify_all` and `:delete_all`.\n    Using this option is DISCOURAGED for most relational databases. Instead,\n    in your migration, set `references(:parent_id, on_delete: :delete_all)`.\n    Opposite to the migration option, this option cannot guarantee integrity\n    and it is only triggered for `c:Ecto.Repo.delete/2` (and not on\n    `c:Ecto.Repo.delete_all/2`) and it never cascades. If posts has many comments,\n    which has many tags, and you delete a post, only comments will be deleted.\n    If your database does not support references, cascading can be manually\n    implemented by using `Ecto.Multi` or `Ecto.Changeset.prepare_changes/2`.\n\n  * `:on_replace` - The action taken on associations when the record is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, `:nilify`, `:delete` or\n    `:delete_if_exists`. See `Ecto.Changeset`'s section about `:on_replace` for\n    more info.\n\n  * `:defaults` - Default values to use when building the association.\n    It may be a keyword list of options that override the association schema\n    or an `atom`/`{module, function, args}` that receives the association struct\n    and the owner struct as arguments. For example, if you set\n    `Post.has_many :comments, defaults: [public: true]`,\n    then when using `Ecto.build_assoc(post, :comments)`, the comment will have\n    `comment.public == true`. Alternatively, you can set it to\n    `Post.has_many :comments, defaults: :update_comment`, which will invoke\n    `Post.update_comment(comment, post)`, or set it to a MFA tuple such as\n    `{Mod, fun, [arg3, arg4]}`, which will invoke `Mod.fun(comment, post, arg3, arg4)`\n\n  * `:where` - A filter for the association. See \"Filtering associations\" below.\n    It does not apply to `:through` associations.\n\n  * `:preload_order` - Sets the default `order_by` of the association.\n    It is used when the association is preloaded.\n    For example, if you set `Post.has_many :comments, preload_order: [asc: :content]`,\n    whenever the `:comments` associations is preloaded,\n    the comments will be order by the `:content` field.\n    See `Ecto.Query.order_by/3` for more examples.","ref":"Ecto.Schema.html#has_many/3-options"},{"type":"macro","title":"Examples - Ecto.Schema.has_many/3","doc":"defmodule Post do\n      use Ecto.Schema\n      schema \"posts\" do\n        has_many :comments, Comment\n      end\n    end\n\n    # Get all comments for a given post\n    post = Repo.get(Post, 42)\n    comments = Repo.all assoc(post, :comments)\n\n    # The comments can come preloaded on the post struct\n    [post] = Repo.all(from(p in Post, where: p.id == 42, preload: :comments))\n    post.comments #=> [%Comment{...}, ...]\n\n`has_many` can be used to define hierarchical relationships within a single\nschema, for example threaded comments.\n\n    defmodule Comment do\n      use Ecto.Schema\n      schema \"comments\" do\n        field :content, :string\n        field :parent_id, :integer\n        belongs_to :parent, Comment, foreign_key: :parent_id, references: :id, define_field: false\n        has_many :children, Comment, foreign_key: :parent_id, references: :id\n      end\n    end","ref":"Ecto.Schema.html#has_many/3-examples"},{"type":"macro","title":"Filtering associations - Ecto.Schema.has_many/3","doc":"It is possible to specify a `:where` option that will filter the records\nreturned by the association. Querying, joining or preloading the association\nwill use the given conditions as shown next:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        has_many :public_comments, Comment,\n          where: [public: true]\n      end\n    end\n\nThe `:where` option expects a keyword list where the key is an atom\nrepresenting the field and the value is either:\n\n  * `nil` - which specifies the field must be nil\n  * `{:not, nil}` - which specifies the field must not be nil\n  * `{:in, list}` - which specifies the field must be one of the values in a list\n  * `{:fragment, expr}` - which specifies a fragment string as the filter\n    (see `Ecto.Query.API.fragment/1`) with the field's value given to it\n    as the only argument\n  * or any other value which the field is compared directly against\n\nNote the values above are distinctly different from the values you\nwould pass to `where` when building a query. For example, if you\nattempt to build a query such as\n\n    from Post, where: [id: nil]\n\nit will emit an error. This is because queries can be built dynamically,\nand therefore passing `nil` can lead to security errors. However, the\n`:where` values for an association are given at compile-time, which is\nless dynamic and cannot leverage the full power of Ecto queries, which\nexplains why they have different APIs.\n\n**Important!** Please use this feature only when strictly necessary,\notherwise it is very easy to end-up with large schemas with dozens of\ndifferent associations polluting your schema and affecting your\napplication performance. For instance, if you are using associations\nonly for different querying purposes, then it is preferable to build\nand compose queries. For instance, instead of having two associations,\none for comments and another for deleted comments, you might have\na single comments association and filter it instead:\n\n    posts\n    |> Ecto.assoc(:comments)\n    |> Comment.deleted()\n\nOr when preloading:\n\n    from posts, preload: [comments: ^Comment.deleted()]","ref":"Ecto.Schema.html#has_many/3-filtering-associations"},{"type":"macro","title":"has_many/has_one :through - Ecto.Schema.has_many/3","doc":"Ecto also supports defining associations in terms of other associations\nvia the `:through` option. Let's see an example:\n\n    defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        has_many :comments, Comment\n        has_one :permalink, Permalink\n\n        # In the has_many :through example below, the `:comments`\n        # in the list [:comments, :author] refers to the\n        # `has_many :comments` in the Post own schema and the\n        # `:author` refers to the `belongs_to :author` of the\n        # Comment's schema (the module below).\n        # (see the description below for more details)\n        has_many :comments_authors, through: [:comments, :author]\n\n        # Specify the association with custom source\n        has_many :tags, {\"posts_tags\", Tag}\n      end\n    end\n\n    defmodule Comment do\n      use Ecto.Schema\n\n      schema \"comments\" do\n        belongs_to :author, Author\n        belongs_to :post, Post\n        has_one :post_permalink, through: [:post, :permalink]\n      end\n    end\n\nIn the example above, we have defined a `has_many :through` association\nnamed `:comments_authors`. A `:through` association always expects a list\nand the first element of the list must be a previously defined association\nin the current module. For example, `:comments_authors` first points to\n`:comments` in the same module (Post), which then points to `:author` in\nthe next schema, `Comment`.\n\nThis `:through` association will return all authors for all comments\nthat belongs to that post:\n\n    # Get all comments authors for a given post\n    post = Repo.get(Post, 42)\n    authors = Repo.all assoc(post, :comments_authors)\n\n`:through` associations can also be preloaded. In such cases, not only\nthe `:through` association is preloaded but all intermediate steps are\npreloaded too:\n\n    [post] = Repo.all(from(p in Post, where: p.id == 42, preload: :comments_authors))\n    post.comments_authors #=> [%Author{...}, ...]\n\n    # The comments for each post will be preloaded too\n    post.comments #=> [%Comment{...}, ...]\n\n    # And the author for each comment too\n    hd(post.comments).author #=> %Author{...}\n\nWhen the `:through` association is expected to return one or zero items,\n`has_one :through` should be used instead, as in the example at the beginning\nof this section:\n\n    # How we defined the association above in Comments\n    has_one :post_permalink, through: [:post, :permalink]\n\n    # Get a preloaded comment\n    [comment] = Repo.all(Comment) |> Repo.preload(:post_permalink)\n    comment.post_permalink #=> %Permalink{...}\n\nIf possible, Ecto will avoid traversing intermediate associations in\nqueries. For example, in the example above, `Comment` has a `post_id`\ncolumn (defined by `belongs_to :post`) and it is expected for\n`Permalink` to have the same. Therefore, when preloading the permalinks,\nEcto may avoid traversing the \"posts\" table altogether. Of course, this\nassumes your database guarantees those references are valid, which can\nbe done by defining foreign key constraints and references your database\n(often done via `EctoSQL` migrations).\n\nNote `:through` associations are read-only. For example, you cannot use\n`Ecto.Changeset.cast_assoc/3` to modify through associations.","ref":"Ecto.Schema.html#has_many/3-has_many-has_one-through"},{"type":"macro","title":"Ecto.Schema.has_one/3","doc":"Indicates a one-to-one association with another schema.\n\nThe current schema has zero or one records of the other schema. The other\nschema often has a `belongs_to` field with the reverse association.","ref":"Ecto.Schema.html#has_one/3"},{"type":"macro","title":"Options - Ecto.Schema.has_one/3","doc":"* `:foreign_key` - Sets the foreign key, this should map to a field on the\n    other schema, defaults to the underscored name of the current module\n    suffixed by `_id`\n\n  * `:references`  - Sets the key on the current schema to be used for the\n    association, defaults to the primary key on the schema\n\n  * `:through` - If this association must be defined in terms of existing\n    associations. Read the section in `has_many/3` for more information\n\n  * `:on_delete` - The action taken on associations when parent record\n    is deleted. May be `:nothing` (default), `:nilify_all` and `:delete_all`.\n    Using this option is DISCOURAGED for most relational databases. Instead,\n    in your migration, set `references(:parent_id, on_delete: :delete_all)`.\n    Opposite to the migration option, this option cannot guarantee integrity\n    and it is only triggered for `c:Ecto.Repo.delete/2` (and not on\n    `c:Ecto.Repo.delete_all/2`) and it never cascades. If posts has many comments,\n    which has many tags, and you delete a post, only comments will be deleted.\n    If your database does not support references, cascading can be manually\n    implemented by using `Ecto.Multi` or `Ecto.Changeset.prepare_changes/2`\n\n  * `:on_replace` - The action taken on associations when the record is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, `:nilify`, `:update`, or\n    `:delete`. See `Ecto.Changeset`'s section on related data for more info.\n\n  * `:defaults` - Default values to use when building the association.\n    It may be a keyword list of options that override the association schema\n    or an `atom`/`{module, function, args}` that receives the association struct\n    and the owner struct as arguments. For example, if you set\n    `Post.has_one :banner, defaults: [public: true]`,\n    then when using `Ecto.build_assoc(post, :banner)`, the banner will have\n    `banner.public == true`. Alternatively, you can set it to\n    `Post.has_one :banner, defaults: :update_banner`, which will invoke\n    `Post.update_banner(banner, post)`, or set it to a MFA tuple such as\n    `{Mod, fun, [arg3, arg4]}`, which will invoke `Mod.fun(banner, post, arg3, arg4)`\n\n  * `:where` - A filter for the association. See \"Filtering associations\"\n    in `has_many/3`. It does not apply to `:through` associations.","ref":"Ecto.Schema.html#has_one/3-options"},{"type":"macro","title":"Examples - Ecto.Schema.has_one/3","doc":"defmodule Post do\n      use Ecto.Schema\n\n      schema \"posts\" do\n        has_one :permalink, Permalink\n\n        # Specify the association with custom source\n        has_one :category, {\"posts_categories\", Category}\n      end\n    end\n\n    # The permalink can come preloaded on the post struct\n    [post] = Repo.all(from(p in Post, where: p.id == 42, preload: :permalink))\n    post.permalink #=> %Permalink{...}","ref":"Ecto.Schema.html#has_one/3-examples"},{"type":"macro","title":"Ecto.Schema.many_to_many/3","doc":"Indicates a many-to-many association with another schema.\n\nThe association happens through a join schema or source, containing\nforeign keys to the associated schemas. For example, the association\nbelow:\n\n    # from MyApp.Post\n    many_to_many :tags, MyApp.Tag, join_through: \"posts_tags\"\n\nis backed by relational databases through a join table as follows:\n\n    [Post]   [posts_tags]   [Tag]\n      id      id\n\nMore information on the migration for creating such a schema is shown\nbelow.","ref":"Ecto.Schema.html#many_to_many/3"},{"type":"macro","title":"Options - Ecto.Schema.many_to_many/3","doc":"* `:join_through` - Specifies the source of the associated data.\n    It may be a string, like \"posts_tags\", representing the\n    underlying storage table or an atom, like `MyApp.PostTag`,\n    representing a schema. This option is required.\n\n  * `:join_keys` - Specifies how the schemas are associated. It\n    expects a keyword list with two entries, the first being how\n    the join table should reach the current schema and the second\n    how the join table should reach the associated schema. In the\n    example above, it defaults to: `[post_id: :id, tag_id: :id]`.\n    The keys are inflected from the schema names.\n\n  * `:on_delete` - The action taken on associations when the parent record\n    is deleted. May be `:nothing` (default) or `:delete_all`.\n    Using this option is DISCOURAGED for most relational databases. Instead,\n    in your migration, set `references(:parent_id, on_delete: :delete_all)`.\n    Opposite to the migration option, this option cannot guarantee integrity\n    and it is only triggered for `c:Ecto.Repo.delete/2` (and not on\n    `c:Ecto.Repo.delete_all/2`). This option can only remove data from the\n    join source, never the associated records, and it never cascades.\n\n  * `:on_replace` - The action taken on associations when the record is\n    replaced when casting or manipulating parent changeset. May be\n    `:raise` (default), `:mark_as_invalid`, or `:delete`.\n    `:delete` will only remove data from the join source, never the\n    associated records. See `Ecto.Changeset`'s section on related data\n    for more info.\n\n  * `:defaults` - Default values to use when building the association.\n    It may be a keyword list of options that override the association schema\n    or an `atom`/`{module, function, args}` that receives the association struct\n    and the owner struct as arguments. For example, if you set\n    `Post.many_to_many :tags, defaults: [public: true]`,\n    then when using `Ecto.build_assoc(post, :tags)`, the tag will have\n    `tag.public == true`. Alternatively, you can set it to\n    `Post.many_to_many :tags, defaults: :update_tag`, which will invoke\n    `Post.update_tag(tag, post)`, or set it to a MFA tuple such as\n    `{Mod, fun, [arg3, arg4]}`, which will invoke `Mod.fun(tag, post, arg3, arg4)`\n\n  * `:join_defaults` - The same as `:defaults` but it applies to the join schema\n    instead. This option will raise if it is given and the `:join_through` value\n    is not a schema.\n\n  * `:unique` - When true, checks if the associated entries are unique\n    whenever the association is cast or changed via the parent record.\n    For instance, it would verify that a given tag cannot be attached to\n    the same post more than once. This exists mostly as a quick check\n    for user feedback, as it does not guarantee uniqueness at the database\n    level. Therefore, you should also set a unique index in the database\n    join table, such as: `create unique_index(:posts_tags, [:post_id, :tag_id])`\n\n  * `:where` - A filter for the association. See \"Filtering associations\"\n    in `has_many/3`\n\n  * `:join_where` - A filter for the join table. See \"Filtering associations\"\n    in `has_many/3`\n\n  * `:preload_order` - Sets the default `order_by` when preloading the association.\n    It may be a keyword list/list of fields or an MFA tuple, such as `{Mod, fun, []}`.\n    Both cases must resolve to a valid `order_by` expression. See `Ecto.Query.order_by/3`\n    to learn more about about ordering expressions.\n    See the [preload order](#many_to_many/3-preload-order) section below to learn how\n    this option can be utilized","ref":"Ecto.Schema.html#many_to_many/3-options"},{"type":"macro","title":"Using Ecto.assoc/2 - Ecto.Schema.many_to_many/3","doc":"One of the benefits of using `many_to_many` is that Ecto will avoid\nloading the intermediate whenever possible, making your queries more\nefficient. For this reason, developers should not refer to the join\ntable of `many_to_many` in queries. The join table is accessible in\nfew occasions, such as in `Ecto.assoc/2`. For example, if you do this:\n\n    post\n    |> Ecto.assoc(:tags)\n    |> where([t, _pt, p], p.public == t.public)\n\nIt may not work as expected because the `posts_tags` table may not be\nincluded in the query. You can address this problem in multiple ways.\nOne option is to use `...`:\n\n    post\n    |> Ecto.assoc(:tags)\n    |> where([t, ..., p], p.public == t.public)\n\nAnother and preferred option is to rewrite to an explicit `join`, which\nleaves out the intermediate bindings as they are resolved only later on:\n\n    # keyword syntax\n    from t in Tag,\n      join: p in assoc(t, :post), on: p.id == ^post.id\n\n    # pipe syntax\n    Tag\n    |> join(:inner, [t], p in assoc(t, :post), on: p.id == ^post.id)\n\nIf you need to access the join table, then you likely want to use\n`has_many/3` with the `:through` option instead.","ref":"Ecto.Schema.html#many_to_many/3-using-ecto-assoc-2"},{"type":"macro","title":"Removing data - Ecto.Schema.many_to_many/3","doc":"If you attempt to remove associated `many_to_many` data, **Ecto will\nalways remove data from the join schema and never from the target\nassociations** be it by setting `:on_replace` to `:delete`, `:on_delete`\nto `:delete_all` or by using changeset functions such as\n`Ecto.Changeset.put_assoc/3`. For example, if a `Post` has a many to many\nrelationship with `Tag`, setting `:on_delete` to `:delete_all` will\nonly delete entries from the \"posts_tags\" table in case `Post` is\ndeleted.","ref":"Ecto.Schema.html#many_to_many/3-removing-data"},{"type":"macro","title":"Migration - Ecto.Schema.many_to_many/3","doc":"How your migration should be structured depends on the value you pass\nin `:join_through`. If `:join_through` is simply a string, representing\na table, you may define a table without primary keys and you must not\ninclude any further columns, as those values won't be set by Ecto:\n\n    create table(:posts_tags, primary_key: false) do\n      add :post_id, references(:posts)\n      add :tag_id, references(:tags)\n    end\n\nHowever, if your `:join_through` is a schema, like `MyApp.PostTag`, your\njoin table may be structured as any other table in your codebase,\nincluding timestamps:\n\n    create table(:posts_tags) do\n      add :post_id, references(:posts)\n      add :tag_id, references(:tags)\n      timestamps()\n    end\n\nBecause `:join_through` contains a schema, in such cases, autogenerated\nvalues and primary keys will be automatically handled by Ecto.","ref":"Ecto.Schema.html#many_to_many/3-migration"},{"type":"macro","title":"Preload Order - Ecto.Schema.many_to_many/3","doc":"The `:preload_order` option may be used to return the preloaded structs\nin a deterministic order. It accepts either a compile-time keyword list/list\nor an MFA tuple, such as `{Mod, fun, []}`. The MFA tuple will be used to\ngenerate the `order_by` expression at runtime.\n\nWhen specifying a compile-time keyword list/list, the ordering applies to the\nassociation's table and not the join table. Ordering by the join table can be\nachieved by specifying an MFA tuple that utilizes `Ecto.Query.dynamic/2`.\n\nFor example, say we have an association `Assoc` being joined through the table\n`join_through`. The default preload query generated by Ecto is roughly:\n\n    from a in Assoc, join: jt in \"join_through\", on: ...\n\nIf `:preload_order` is given as `[asc: :field]` then the preload query will be\nchanged to the following:\n\n    from a in Assoc, join: jt in \"join_through\", on: ..., order_by: [asc: a.field]\n\nSimilarly, any compile-time keyword list/list will have its fields interpreted\nas belonging to the association's table. To order by a field from the join table,\nan MFA tuple can be specified that utilizes `Ecto.Query.dynamic/2`.\n\nFor example, if `:preload_order` is given as `{Mod, fun, []}`, corresponding to\nthe following function:\n\n    defmodule Mod do\n      def fun() do\n        [desc: dynamic([assoc, join], join.field)]\n      end\n    end\n\nthen the preload query will be changed to the following:\n\n    from a in Assoc, join: jt in \"join_through\", on: ..., order_by: [desc: jt.field]\n\nNote the ordering of the bindings. The join table always comes last.","ref":"Ecto.Schema.html#many_to_many/3-preload-order"},{"type":"macro","title":"Examples - Ecto.Schema.many_to_many/3","doc":"defmodule Post do\n      use Ecto.Schema\n      schema \"posts\" do\n        many_to_many :tags, Tag, join_through: \"posts_tags\"\n      end\n    end\n\n    # Let's create a post and a tag\n    post = Repo.insert!(%Post{})\n    tag = Repo.insert!(%Tag{name: \"introduction\"})\n\n    # We can associate at any time post and tags together using changesets\n    post\n    |> Repo.preload(:tags) # Load existing data\n    |> Ecto.Changeset.change() # Build the changeset\n    |> Ecto.Changeset.put_assoc(:tags, [tag]) # Set the association\n    |> Repo.update!\n\n    # In a later moment, we may get all tags for a given post\n    post = Repo.get(Post, 42)\n    tags = Repo.all(assoc(post, :tags))\n\n    # The tags may also be preloaded on the post struct for reading\n    [post] = Repo.all(from(p in Post, where: p.id == 42, preload: :tags))\n    post.tags #=> [%Tag{...}, ...]","ref":"Ecto.Schema.html#many_to_many/3-examples"},{"type":"macro","title":"Join Schema Example - Ecto.Schema.many_to_many/3","doc":"You may prefer to use a join schema to handle many_to_many associations. The\ndecoupled nature of Ecto allows us to create a \"join\" struct which\n`belongs_to` both sides of the many to many association.\n\nIn our example, a `User` has and belongs to many `Organization`s:\n\n    defmodule MyApp.Repo.Migrations.CreateUserOrganization do\n      use Ecto.Migration\n\n      def change do\n        create table(:users_organizations) do\n          add :user_id, references(:users)\n          add :organization_id, references(:organizations)\n\n          timestamps()\n        end\n      end\n    end\n\n    defmodule UserOrganization do\n      use Ecto.Schema\n\n      @primary_key false\n      schema \"users_organizations\" do\n        belongs_to :user, User\n        belongs_to :organization, Organization\n        timestamps() # Added bonus, a join schema will also allow you to set timestamps\n      end\n\n      def changeset(struct, params \\\\ %{}) do\n        struct\n        |> Ecto.Changeset.cast(params, [:user_id, :organization_id])\n        |> Ecto.Changeset.validate_required([:user_id, :organization_id])\n        # Maybe do some counter caching here!\n      end\n    end\n\n    defmodule User do\n      use Ecto.Schema\n\n      schema \"users\" do\n        many_to_many :organizations, Organization, join_through: UserOrganization\n      end\n    end\n\n    defmodule Organization do\n      use Ecto.Schema\n\n      schema \"organizations\" do\n        many_to_many :users, User, join_through: UserOrganization\n      end\n    end\n\n    # Then to create the association, pass in the ID's of an existing\n    # User and Organization to UserOrganization.changeset\n    changeset = UserOrganization.changeset(%UserOrganization{}, %{user_id: id, organization_id: id})\n\n    case Repo.insert(changeset) do\n      {:ok, assoc} -> # Assoc was created!\n      {:error, changeset} -> # Handle the error\n    end","ref":"Ecto.Schema.html#many_to_many/3-join-schema-example"},{"type":"macro","title":"Ecto.Schema.schema/2","doc":"Defines a schema struct with a source name and field definitions.\n\nAn additional field called `__meta__` is added to the struct for storing\ninternal Ecto state. This field always has a `Ecto.Schema.Metadata` struct\nas value and can be manipulated with the `Ecto.put_meta/2` function.","ref":"Ecto.Schema.html#schema/2"},{"type":"macro","title":"Ecto.Schema.timestamps/1","doc":"Generates `:inserted_at` and `:updated_at` timestamp fields.\n\nThe fields generated by this macro will automatically be set to\nthe current time when inserting and updating values in a repository.","ref":"Ecto.Schema.html#timestamps/1"},{"type":"macro","title":"Options - Ecto.Schema.timestamps/1","doc":"* `:inserted_at` - the Ecto schema name of the field for insertion times or `false`\n  * `:updated_at` - the Ecto schema name of the field for update times or `false`\n  * `:inserted_at_source` - the name of the database column for insertion times or `false`\n  * `:updated_at_source` - the name of the database column for update times or `false`\n  * `:type` - the timestamps type, defaults to `:naive_datetime`.\n  * `:autogenerate` - a module-function-args tuple used for generating\n    both `inserted_at` and `updated_at` timestamps\n\nAll options can be pre-configured by setting `@timestamps_opts`.","ref":"Ecto.Schema.html#timestamps/1-options"},{"type":"type","title":"Ecto.Schema.belongs_to/1","doc":"","ref":"Ecto.Schema.html#t:belongs_to/1"},{"type":"type","title":"Ecto.Schema.embedded_schema/0","doc":"","ref":"Ecto.Schema.html#t:embedded_schema/0"},{"type":"type","title":"Ecto.Schema.embeds_many/1","doc":"","ref":"Ecto.Schema.html#t:embeds_many/1"},{"type":"type","title":"Ecto.Schema.embeds_one/1","doc":"","ref":"Ecto.Schema.html#t:embeds_one/1"},{"type":"type","title":"Ecto.Schema.has_many/1","doc":"","ref":"Ecto.Schema.html#t:has_many/1"},{"type":"type","title":"Ecto.Schema.has_one/1","doc":"","ref":"Ecto.Schema.html#t:has_one/1"},{"type":"type","title":"Ecto.Schema.many_to_many/1","doc":"","ref":"Ecto.Schema.html#t:many_to_many/1"},{"type":"type","title":"Ecto.Schema.prefix/0","doc":"","ref":"Ecto.Schema.html#t:prefix/0"},{"type":"type","title":"Ecto.Schema.schema/0","doc":"","ref":"Ecto.Schema.html#t:schema/0"},{"type":"type","title":"Ecto.Schema.source/0","doc":"","ref":"Ecto.Schema.html#t:source/0"},{"type":"type","title":"Ecto.Schema.t/0","doc":"","ref":"Ecto.Schema.html#t:t/0"},{"type":"module","title":"Ecto.Schema.Metadata","doc":"Stores metadata of a struct.","ref":"Ecto.Schema.Metadata.html"},{"type":"module","title":"State - Ecto.Schema.Metadata","doc":"The state of the schema is stored in the `:state` field and allows\nfollowing values:\n\n  * `:built` - the struct was constructed in memory and is not persisted\n    to database yet;\n  * `:loaded` - the struct was loaded from database and represents\n    persisted data;\n  * `:deleted` - the struct was deleted and no longer represents persisted\n    data.","ref":"Ecto.Schema.Metadata.html#module-state"},{"type":"module","title":"Source - Ecto.Schema.Metadata","doc":"The `:source` tracks the (table or collection) where the struct is or should\nbe persisted to.","ref":"Ecto.Schema.Metadata.html#module-source"},{"type":"module","title":"Prefix - Ecto.Schema.Metadata","doc":"Tracks the source prefix in the data storage.","ref":"Ecto.Schema.Metadata.html#module-prefix"},{"type":"module","title":"Context - Ecto.Schema.Metadata","doc":"The `:context` field represents additional state some databases require\nfor proper updates of data. It is not used by the built-in adapters of\n`Ecto.Adapters.Postgres` and `Ecto.Adapters.MySQL`.","ref":"Ecto.Schema.Metadata.html#module-context"},{"type":"module","title":"Schema - Ecto.Schema.Metadata","doc":"The `:schema` field refers the module name for the schema this metadata belongs to.","ref":"Ecto.Schema.Metadata.html#module-schema"},{"type":"type","title":"Ecto.Schema.Metadata.context/0","doc":"","ref":"Ecto.Schema.Metadata.html#t:context/0"},{"type":"type","title":"Ecto.Schema.Metadata.state/0","doc":"","ref":"Ecto.Schema.Metadata.html#t:state/0"},{"type":"type","title":"Ecto.Schema.Metadata.t/0","doc":"","ref":"Ecto.Schema.Metadata.html#t:t/0"},{"type":"type","title":"Ecto.Schema.Metadata.t/1","doc":"","ref":"Ecto.Schema.Metadata.html#t:t/1"},{"type":"module","title":"Mix.Ecto","doc":"Conveniences for writing Ecto related Mix tasks.","ref":"Mix.Ecto.html"},{"type":"function","title":"Mix.Ecto.ensure_implements/3","doc":"Returns `true` if module implements behaviour.","ref":"Mix.Ecto.html#ensure_implements/3"},{"type":"function","title":"Mix.Ecto.ensure_repo/2","doc":"Ensures the given module is an Ecto.Repo.","ref":"Mix.Ecto.html#ensure_repo/2"},{"type":"function","title":"Mix.Ecto.no_umbrella!/1","doc":"Gets a path relative to the application path.\n\nRaises on umbrella application.","ref":"Mix.Ecto.html#no_umbrella!/1"},{"type":"function","title":"Mix.Ecto.open?/2","doc":"Asks if the user wants to open a file based on ECTO_EDITOR.\n\nBy default, it attempts to open the file and line using the\n`file:line` notation. For example, if your editor is called\n`subl`, it will open the file as:\n\n    subl path/to/file:line\n\nIt is important that you choose an editor command that does\nnot block nor that attempts to run an editor directly in the\nterminal. Command-line based editors likely need extra\nconfiguration so they open up the given file and line in a\nseparate window.\n\nCustom editors are supported by using the `__FILE__` and\n`__LINE__` notations, for example:\n\n    ECTO_EDITOR=\"my_editor +__LINE__ __FILE__\"\n\nand Elixir will properly interpolate values.","ref":"Mix.Ecto.html#open?/2"},{"type":"function","title":"Mix.Ecto.parse_repo/1","doc":"Parses the repository option from the given command line args list.\n\nIf no repo option is given, it is retrieved from the application environment.","ref":"Mix.Ecto.html#parse_repo/1"},{"type":"task","title":"mix ecto","doc":"Prints Ecto tasks and their information.\n\n    $ mix ecto","ref":"Mix.Tasks.Ecto.html"},{"type":"task","title":"mix ecto.create","doc":"Create the storage for the given repository.\n\nThe repositories to create are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nSince Ecto tasks can only be executed once, if you need to create\nmultiple repositories, set `:ecto_repos` accordingly or pass the `-r`\nflag multiple times.","ref":"Mix.Tasks.Ecto.Create.html"},{"type":"task","title":"Examples - mix ecto.create","doc":"$ mix ecto.create\n    $ mix ecto.create -r Custom.Repo","ref":"Mix.Tasks.Ecto.Create.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.create","doc":"* `-r`, `--repo` - the repo to create\n  * `--quiet` - do not log output\n  * `--no-compile` - do not compile before creating\n  * `--no-deps-check` - do not compile before creating","ref":"Mix.Tasks.Ecto.Create.html#module-command-line-options"},{"type":"task","title":"mix ecto.drop","doc":"Drop the storage for the given repository.\n\nThe repositories to drop are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nSince Ecto tasks can only be executed once, if you need to drop\nmultiple repositories, set `:ecto_repos` accordingly or pass the `-r`\nflag multiple times.","ref":"Mix.Tasks.Ecto.Drop.html"},{"type":"task","title":"Examples - mix ecto.drop","doc":"$ mix ecto.drop\n    $ mix ecto.drop -r Custom.Repo","ref":"Mix.Tasks.Ecto.Drop.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.drop","doc":"* `-r`, `--repo` - the repo to drop\n  * `-q`, `--quiet` - run the command quietly\n  * `-f`, `--force` - do not ask for confirmation when dropping the database.\n    Configuration is asked only when `:start_permanent` is set to true\n    (typically in production)\n  * `--force-drop` - force the database to be dropped even\n    if it has connections to it (requires PostgreSQL 13+)\n  * `--no-compile` - do not compile before dropping\n  * `--no-deps-check` - do not compile before dropping","ref":"Mix.Tasks.Ecto.Drop.html#module-command-line-options"},{"type":"task","title":"mix ecto.gen.repo","doc":"Generates a new repository.\n\nThe repository will be placed in the `lib` directory.","ref":"Mix.Tasks.Ecto.Gen.Repo.html"},{"type":"task","title":"Examples - mix ecto.gen.repo","doc":"$ mix ecto.gen.repo -r Custom.Repo\n\nThis generator will automatically open the config/config.exs\nafter generation if you have `ECTO_EDITOR` set in your environment\nvariable.","ref":"Mix.Tasks.Ecto.Gen.Repo.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.gen.repo","doc":"* `-r`, `--repo` - the repo to generate","ref":"Mix.Tasks.Ecto.Gen.Repo.html#module-command-line-options"},{"type":"module","title":"Ecto.Enum","doc":"A custom type that maps atoms to strings or integers.\n\n`Ecto.Enum` must be used whenever you want to keep atom values in a field.\nSince atoms cannot be persisted to the database, `Ecto.Enum` converts them\nto a string or an integer when writing to the database and converts them back\nto atoms when loading data. It can be used in your schemas as follows:\n\n    # Stored as strings\n    field :status, Ecto.Enum, values: [:foo, :bar, :baz]\n\nor\n\n    # Stored as integers\n    field :status, Ecto.Enum, values: [foo: 1, bar: 2, baz: 5]\n\nTherefore, the type to be used in your migrations for enum fields depend\non the choice above. For the cases above, one would do, respectively:\n\n    add :status, :string\n\nor\n\n    add :status, :integer\n\nSome databases also support enum types, which you could use in combination\nwith the above.\n\nComposite types, such as `:array`, are also supported which allow selecting\nmultiple values per record:\n\n    field :roles, {:array, Ecto.Enum}, values: [:author, :editor, :admin]\n\nOverall, `:values` must be a list of atoms or a keyword list. Values will be\ncast to atoms safely and only if the atom exists in the list (otherwise an\nerror will be raised). Attempting to load any string/integer not represented\nby an atom in the list will be invalid.\n\nThe helper function `mappings/2` returns the mappings for a given schema and\nfield, which can be used in places like form drop-downs. For example, given\nthe following schema:\n\n    defmodule EnumSchema do\n      use Ecto.Schema\n\n      schema \"my_schema\" do\n        field :my_enum, Ecto.Enum, values: [:foo, :bar, :baz]\n      end\n    end\n\nYou can call `mappings/2` like this:\n\n    Ecto.Enum.mappings(EnumSchema, :my_enum)\n    #=> [foo: \"foo\", bar: \"bar\", baz: \"baz\"]\n\nIf you want the values only, you can use `Ecto.Enum.values/2`, and if you want\nthe dump values only, you can use `Ecto.Enum.dump_values/2`.","ref":"Ecto.Enum.html"},{"type":"module","title":"Embeds - Ecto.Enum","doc":"`Ecto.Enum` allows to customize how fields are dumped within embeds through the\n`:embed_as` option. Two alternatives are supported: `:values`, which will save\nthe enum keys (and not their respective mapping), and `:dumped`, which will save\nthe dumped value. The default is `:values`. For example, assuming the following\nschema:\n\n    defmodule EnumSchema do\n      use Ecto.Schema\n\n      schema \"my_schema\" do\n        embeds_one :embed, Embed do\n          field :embed_as_values, Ecto.Enum, values: [foo: 1, bar: 2], embed_as: :values\n          field :embed_as_dump, Ecto.Enum, values: [foo: 1, bar: 2], embed_as: :dumped\n        end\n      end\n    end\n\nThe `:embed_as_values` field value will save `:foo` or `:bar`, while the\n`:embed_as_dump` field value will save `1` or `2`.","ref":"Ecto.Enum.html#module-embeds"},{"type":"function","title":"Ecto.Enum.dump_values/2","doc":"Returns the possible dump values for a given schema or types map and field","ref":"Ecto.Enum.html#dump_values/2"},{"type":"function","title":"Ecto.Enum.mappings/2","doc":"Returns the mappings between values and dumped values","ref":"Ecto.Enum.html#mappings/2"},{"type":"function","title":"Ecto.Enum.values/2","doc":"Returns the possible values for a given schema or types map and field","ref":"Ecto.Enum.html#values/2"},{"type":"behaviour","title":"Ecto.ParameterizedType","doc":"Parameterized types are Ecto types that can be customized per field.\n\nParameterized types allow a set of options to be specified in the schema\nwhich are initialized on compilation and passed to the callback functions\nas the last argument.\n\nFor example, `field :foo, :string` behaves the same for every field.\nOn the other hand, `field :foo, Ecto.Enum, values: [:foo, :bar, :baz]`\nwill likely have a different set of values per field.\n\nNote that options are specified as a keyword, but it is idiomatic to\nconvert them to maps inside `c:init/1` for easier pattern matching in\nother callbacks.\n\nParameterized types are a superset of regular types. In other words,\nwith parameterized types you can do everything a regular type does,\nand more. For example, parameterized types can handle `nil` values\nin both `load` and `dump` callbacks, they can customize `cast` behavior\nper query and per changeset, and also control how values are embedded.\n\nHowever, parameterized types are also more complex. Therefore, if\neverything you need to achieve can be done with basic types, they\nshould be preferred to parameterized ones.","ref":"Ecto.ParameterizedType.html"},{"type":"behaviour","title":"Examples - Ecto.ParameterizedType","doc":"To create a parameterized type, create a module as shown below:\n\n    defmodule MyApp.MyType do\n      use Ecto.ParameterizedType\n\n      def type(_params), do: :string\n\n      def init(opts) do\n        validate_opts(opts)\n        Enum.into(opts, %{})\n      end\n\n      def cast(data, params) do\n        ...\n        {:ok, cast_data}\n      end\n\n      def load(data, _loader, params) do\n        ...\n        {:ok, loaded_data}\n      end\n\n      def dump(data, dumper, params) do\n        ...\n        {:ok, dumped_data}\n      end\n\n      def equal?(a, b, _params) do\n        a == b\n      end\n    end\n\nTo use this type in a schema field, specify the type and parameters like this:\n\n    schema \"foo\" do\n      field :bar, MyApp.MyType, opt1: :baz, opt2: :boo\n    end\n\nTo use this type in places where you need it to be initialized (for example,\nschemaless changesets), you can use `init/2`.\n\n> #### `use Ecto.ParameterizedType` {: .info}\n>\n> When you `use Ecto.ParameterizedType`, it will set\n> `@behaviour Ecto.ParameterizedType` and define default, overridable\n> implementations for `c:embed_as/2` and `c:equal?/3`.","ref":"Ecto.ParameterizedType.html#module-examples"},{"type":"callback","title":"Ecto.ParameterizedType.autogenerate/1","doc":"Generates a loaded version of the data.\n\nThis callback is invoked when a parameterized type is given\nto `field` with the `:autogenerate` flag.","ref":"Ecto.ParameterizedType.html#c:autogenerate/1"},{"type":"callback","title":"Ecto.ParameterizedType.cast/2","doc":"Casts the given input to the ParameterizedType with the given parameters.\n\nIf the parameterized type is also a composite type,\nthe inner type can be cast by calling `Ecto.Type.cast/2`\ndirectly.\n\nFor more information on casting, see `c:Ecto.Type.cast/1`.","ref":"Ecto.ParameterizedType.html#c:cast/2"},{"type":"callback","title":"Ecto.ParameterizedType.dump/3","doc":"Dumps the given term into an Ecto native type.\n\nIt receives a `dumper` function in case the parameterized\ntype is also a composite type. In order to dump the inner\ntype, the `dumper` must be called with the inner type and\nthe inner value as argument.\n\nFor more information on dumping, see `c:Ecto.Type.dump/1`.\nNote that this callback *will* be called when dumping a `nil`\nvalue, unlike `c:Ecto.Type.dump/1`.","ref":"Ecto.ParameterizedType.html#c:dump/3"},{"type":"callback","title":"Ecto.ParameterizedType.embed_as/2","doc":"Dictates how the type should be treated inside embeds.\n\nFor more information on embedding, see `c:Ecto.Type.embed_as/1`","ref":"Ecto.ParameterizedType.html#c:embed_as/2"},{"type":"callback","title":"Ecto.ParameterizedType.equal?/3","doc":"Checks if two terms are semantically equal.","ref":"Ecto.ParameterizedType.html#c:equal?/3"},{"type":"callback","title":"Ecto.ParameterizedType.format/1","doc":"Formats output when a ParameterizedType is printed in exceptions and\nother logs.\n\nNote this callback is not used when constructing `Ecto.Changeset` validation\nerrors. See the `:message` option of most `Ecto.Changeset` validation\nfunctions for how to customize error messaging on a per `Ecto.Changeset` basis.","ref":"Ecto.ParameterizedType.html#c:format/1"},{"type":"callback","title":"Ecto.ParameterizedType.init/1","doc":"Callback to convert the options specified in  the field macro into parameters\nto be used in other callbacks.\n\nThis function is called at compile time, and should raise if invalid values are\nspecified. It is idiomatic that the parameters returned from this are a map.\n`field` and `schema` will be injected into the options automatically.\n\nFor example, this schema specification\n\n    schema \"my_table\" do\n      field :my_field, MyParameterizedType, opt1: :foo, opt2: nil\n    end\n\nwill result in the call:\n\n    MyParameterizedType.init([schema: \"my_table\", field: :my_field, opt1: :foo, opt2: nil])","ref":"Ecto.ParameterizedType.html#c:init/1"},{"type":"function","title":"Ecto.ParameterizedType.init/2","doc":"Inits a parameterized type given by `type` with `opts`.\n\nUseful when manually initializing a type for schemaless changesets.","ref":"Ecto.ParameterizedType.html#init/2"},{"type":"callback","title":"Ecto.ParameterizedType.load/3","doc":"Loads the given term into a ParameterizedType.\n\nIt receives a `loader` function in case the parameterized\ntype is also a composite type. In order to load the inner\ntype, the `loader` must be called with the inner type and\nthe inner value as argument.\n\nFor more information on loading, see `c:Ecto.Type.load/1`.\nNote that this callback *will* be called when loading a `nil`\nvalue, unlike `c:Ecto.Type.load/1`.","ref":"Ecto.ParameterizedType.html#c:load/3"},{"type":"callback","title":"Ecto.ParameterizedType.type/1","doc":"Returns the underlying schema type for the ParameterizedType.\n\nFor more information on schema types, see `c:Ecto.Type.type/0`","ref":"Ecto.ParameterizedType.html#c:type/1"},{"type":"type","title":"Ecto.ParameterizedType.opts/0","doc":"The keyword options passed from the Schema's field macro into `c:init/1`","ref":"Ecto.ParameterizedType.html#t:opts/0"},{"type":"type","title":"Ecto.ParameterizedType.params/0","doc":"The parameters for the ParameterizedType\n\nThis is the value passed back from `c:init/1` and subsequently passed\nas the last argument to all callbacks. Idiomatically it is a map.","ref":"Ecto.ParameterizedType.html#t:params/0"},{"type":"behaviour","title":"Ecto.Type","doc":"Defines functions and the `Ecto.Type` behaviour for implementing\nbasic custom types.\n\nEcto provides two types of custom types: basic types and\nparameterized types. Basic types are simple, requiring only four\ncallbacks to be implemented, and are enough for most occasions.\nParameterized types can be customized on the field definition and\nprovide a wide variety of callbacks.\n\nThe definition of basic custom types and all of their callbacks are\navailable in this module. You can learn more about parameterized\ntypes in `Ecto.ParameterizedType`. If in doubt, prefer to use\nbasic custom types and rely on parameterized types if you need\nthe extra functionality.","ref":"Ecto.Type.html"},{"type":"behaviour","title":"Example - Ecto.Type","doc":"Imagine you want to store a URI struct as part of a schema in a\nurl-shortening service. There isn't an Ecto field type to support\nthat value at runtime therefore a custom one is needed.\n\nYou also want to query not only by the full url, but for example\nby specific ports used. This is possible by putting the URI data\ninto a map field instead of just storing the plain\nstring representation.\n\n    from s in ShortUrl,\n      where: fragment(\"?->>? ILIKE ?\", s.original_url, \"port\", \"443\")\n\nSo the custom type does need to handle the conversion from\nexternal data to runtime data (`c:cast/1`) as well as\ntransforming that runtime data into the `:map` Ecto native type and\nback (`c:dump/1` and `c:load/1`).\n\n    defmodule EctoURI do\n      use Ecto.Type\n      def type, do: :map\n\n      # Provide custom casting rules.\n      # Cast strings into the URI struct to be used at runtime\n      def cast(uri) when is_binary(uri) do\n        {:ok, URI.parse(uri)}\n      end\n\n      # Accept casting of URI structs as well\n      def cast(%URI{} = uri), do: {:ok, uri}\n\n      # Everything else is a failure though\n      def cast(_), do: :error\n\n      # When loading data from the database, as long as it's a map,\n      # we just put the data back into a URI struct to be stored in\n      # the loaded schema struct.\n      def load(data) when is_map(data) do\n        data =\n          for {key, val}   #### `use Ecto.Type` {: .info}\n>\n> When you `use Ecto.Type`, it will set `@behaviour Ecto.Type` and define\n> default, overridable implementations for `c:embed_as/1` and `c:equal?/2`.","ref":"Ecto.Type.html#module-example"},{"type":"behaviour","title":"Custom types and primary keys - Ecto.Type","doc":"Remember that, if you change the type of your primary keys,\nyou will also need to change the type of all associations that\npoint to said primary key.\n\nImagine you want to encode the ID so they cannot enumerate the\ncontent in your application. An Ecto type could handle the conversion\nbetween the encoded version of the id and its representation in the\ndatabase. For the sake of simplicity, we'll use base64 encoding in\nthis example:\n\n    defmodule EncodedId do\n      use Ecto.Type\n\n      def type, do: :id\n\n      def cast(id) when is_integer(id) do\n        {:ok, encode_id(id)}\n      end\n      def cast(_), do: :error\n\n      def dump(id) when is_binary(id) do\n        {:ok, id_decoded} = Base.decode64(id)\n        {:ok, String.to_integer(id_decoded)}\n      end\n\n      def load(id) when is_integer(id) do\n        {:ok, encode_id(id)}\n      end\n\n      defp encode_id(id) do\n        id\n        |> Integer.to_string()\n        |> Base.encode64()\n      end\n    end\n\nTo use it as the type for the id in our schema, we can use the\n`@primary_key` module attribute:\n\n    defmodule BlogPost do\n      use Ecto.Schema\n\n      @primary_key {:id, EncodedId, autogenerate: true}\n      schema \"posts\" do\n        belongs_to :author, Author, type: EncodedId\n        field :content, :string\n      end\n    end\n\n    defmodule Author do\n      use Ecto.Schema\n\n      @primary_key {:id, EncodedId, autogenerate: true}\n      schema \"authors\" do\n        field :name, :string\n        has_many :posts, BlogPost\n      end\n    end\n\nThe `@primary_key` attribute will tell ecto which type to\nuse for the id.\n\nNote the `type: EncodedId` option given to `belongs_to` in\nthe `BlogPost` schema. By default, Ecto will treat\nassociations as if their keys were `:integer`s. Our primary\nkeys are a custom type, so when Ecto tries to cast those\nids, it will fail.\n\nAlternatively, you can set `@foreign_key_type EncodedId`\nafter `@primary_key` to automatically configure the type\nof all `belongs_to` fields.","ref":"Ecto.Type.html#module-custom-types-and-primary-keys"},{"type":"callback","title":"Ecto.Type.autogenerate/0","doc":"Generates a loaded version of the data.\n\nThis is callback is invoked when a custom type is given\nto `field` with the `:autogenerate` flag.","ref":"Ecto.Type.html#c:autogenerate/0"},{"type":"function","title":"Ecto.Type.base?/1","doc":"Checks if the given atom can be used as base type.\n\n    iex> base?(:string)\n    true\n    iex> base?(:array)\n    false\n    iex> base?(Custom)\n    false","ref":"Ecto.Type.html#base?/1"},{"type":"callback","title":"Ecto.Type.cast/1","doc":"Casts the given input to the custom type.\n\nThis callback is called on external input and can return any type,\nas long as the `dump/1` function is able to convert the returned\nvalue into an Ecto native type. There are two situations where\nthis callback is called:\n\n  1. When casting values by `Ecto.Changeset`\n  2. When passing arguments to `Ecto.Query`\n\nYou can return `:error` if the given term cannot be cast.\nA default error message of \"is invalid\" will be added to the\nchangeset.\n\nYou may also return `{:error, keyword()}` to customize the\nchangeset error message and its metadata. Passing a `:message`\nkey, will override the default message. It is not possible to\noverride the `:type` key.\n\nFor `{:array, CustomType}` or `{:map, CustomType}` the returned\nkeyword list will be erased and the default error will be shown.","ref":"Ecto.Type.html#c:cast/1"},{"type":"function","title":"Ecto.Type.cast/2","doc":"Casts a value to the given type.\n\n`cast/2` is used by the finder queries and changesets to cast outside values to\nspecific types.\n\nNote that nil can be cast to all primitive types as data stores allow nil to be\nset on any column.\n\nNaN and infinite decimals are not supported, use custom types instead.\n\n    iex> cast(:any, \"whatever\")\n    {:ok, \"whatever\"}\n\n    iex> cast(:any, nil)\n    {:ok, nil}\n    iex> cast(:string, nil)\n    {:ok, nil}\n\n    iex> cast(:integer, 1)\n    {:ok, 1}\n    iex> cast(:integer, \"1\")\n    {:ok, 1}\n    iex> cast(:integer, \"1.0\")\n    :error\n\n    iex> cast(:id, 1)\n    {:ok, 1}\n    iex> cast(:id, \"1\")\n    {:ok, 1}\n    iex> cast(:id, \"1.0\")\n    :error\n\n    iex> cast(:float, 1.0)\n    {:ok, 1.0}\n    iex> cast(:float, 1)\n    {:ok, 1.0}\n    iex> cast(:float, \"1\")\n    {:ok, 1.0}\n    iex> cast(:float, \"1.0\")\n    {:ok, 1.0}\n    iex> cast(:float, \"1-foo\")\n    :error\n\n    iex> cast(:boolean, true)\n    {:ok, true}\n    iex> cast(:boolean, false)\n    {:ok, false}\n    iex> cast(:boolean, \"1\")\n    {:ok, true}\n    iex> cast(:boolean, \"0\")\n    {:ok, false}\n    iex> cast(:boolean, \"whatever\")\n    :error\n\n    iex> cast(:string, \"beef\")\n    {:ok, \"beef\"}\n    iex> cast(:binary, \"beef\")\n    {:ok, \"beef\"}\n\n    iex> cast(:decimal, Decimal.new(\"1.0\"))\n    {:ok, Decimal.new(\"1.0\")}\n    iex> cast(:decimal, \"1.0bad\")\n    :error\n\n    iex> cast({:array, :integer}, [1, 2, 3])\n    {:ok, [1, 2, 3]}\n    iex> cast({:array, :integer}, [\"1\", \"2\", \"3\"])\n    {:ok, [1, 2, 3]}\n    iex> cast({:array, :string}, [1, 2, 3])\n    :error\n    iex> cast(:string, [1, 2, 3])\n    :error\n\n    iex> cast(:utc_datetime, \"2014-04-17T14:00:00Z\")\n    {:ok, ~U[2014-04-17 14:00:00Z]}\n    iex> cast(:utc_datetime, \"2014-04-17T14:00:00.030Z\")\n    {:ok, ~U[2014-04-17 14:00:00Z]}\n    iex> cast(:utc_datetime, \"2014-04-17T12:00:00-02:00\")\n    {:ok, ~U[2014-04-17 14:00:00Z]}","ref":"Ecto.Type.html#cast/2"},{"type":"function","title":"Ecto.Type.composite?/1","doc":"Checks if the given atom can be used as composite type.\n\n    iex> composite?(:array)\n    true\n    iex> composite?(:string)\n    false","ref":"Ecto.Type.html#composite?/1"},{"type":"callback","title":"Ecto.Type.dump/1","doc":"Dumps the given term into an Ecto native type.\n\nThis callback is called with any term that was stored in the struct\nand it needs to validate them and convert it to an Ecto native type.","ref":"Ecto.Type.html#c:dump/1"},{"type":"function","title":"Ecto.Type.dump/3","doc":"Dumps a value to the given type.\n\nOpposite to casting, dumping requires the returned value\nto be a valid Ecto type, as it will be sent to the\nunderlying data store.\n\n    iex> dump(:string, nil)\n    {:ok, nil}\n    iex> dump(:string, \"foo\")\n    {:ok, \"foo\"}\n\n    iex> dump(:integer, 1)\n    {:ok, 1}\n    iex> dump(:integer, \"10\")\n    :error\n\n    iex> dump(:binary, \"foo\")\n    {:ok, \"foo\"}\n    iex> dump(:binary, 1)\n    :error\n\n    iex> dump({:array, :integer}, [1, 2, 3])\n    {:ok, [1, 2, 3]}\n    iex> dump({:array, :integer}, [1, \"2\", 3])\n    :error\n    iex> dump({:array, :binary}, [\"1\", \"2\", \"3\"])\n    {:ok, [\"1\", \"2\", \"3\"]}","ref":"Ecto.Type.html#dump/3"},{"type":"callback","title":"Ecto.Type.embed_as/1","doc":"Dictates how the type should be treated inside embeds.\n\nBy default, the type is sent as itself, without calling\ndumping to keep the higher level representation. But\nit can be set to `:dump` so that it is dumped before\nbeing encoded.","ref":"Ecto.Type.html#c:embed_as/1"},{"type":"function","title":"Ecto.Type.embed_as/2","doc":"Gets how the type is treated inside embeds for the given format.\n\nSee `c:embed_as/1`.","ref":"Ecto.Type.html#embed_as/2"},{"type":"function","title":"Ecto.Type.embedded_dump/3","doc":"Dumps the `value` for `type` considering it will be embedded in `format`.","ref":"Ecto.Type.html#embedded_dump/3"},{"type":"function","title":"Examples - Ecto.Type.embedded_dump/3","doc":"iex> Ecto.Type.embedded_dump(:decimal, Decimal.new(\"1\"), :json)\n    {:ok, Decimal.new(\"1\")}","ref":"Ecto.Type.html#embedded_dump/3-examples"},{"type":"function","title":"Ecto.Type.embedded_load/3","doc":"Loads the `value` for `type` considering it was embedded in `format`.","ref":"Ecto.Type.html#embedded_load/3"},{"type":"function","title":"Examples - Ecto.Type.embedded_load/3","doc":"iex> Ecto.Type.embedded_load(:decimal, \"1\", :json)\n    {:ok, Decimal.new(\"1\")}","ref":"Ecto.Type.html#embedded_load/3-examples"},{"type":"callback","title":"Ecto.Type.equal?/2","doc":"Checks if two terms are semantically equal.\n\nThis callback is used for determining equality of types in\n`Ecto.Changeset`.\n\nBy default the terms are compared with the equal operator `==/2`.","ref":"Ecto.Type.html#c:equal?/2"},{"type":"function","title":"Ecto.Type.equal?/3","doc":"Checks if two terms are equal.\n\nDepending on the given `type` performs a structural or semantical comparison.","ref":"Ecto.Type.html#equal?/3"},{"type":"function","title":"Examples - Ecto.Type.equal?/3","doc":"iex> equal?(:integer, 1, 1)\n    true\n    iex> equal?(:decimal, Decimal.new(\"1\"), Decimal.new(\"1.00\"))\n    true","ref":"Ecto.Type.html#equal?/3-examples"},{"type":"function","title":"Ecto.Type.format/1","doc":"Format type for error messaging and logs.","ref":"Ecto.Type.html#format/1"},{"type":"function","title":"Ecto.Type.include?/3","doc":"Checks if `collection` includes a `term`.\n\nDepending on the given `type` performs a structural or semantical comparison.","ref":"Ecto.Type.html#include?/3"},{"type":"function","title":"Examples - Ecto.Type.include?/3","doc":"iex> include?(:integer, 1, 1..3)\n    true\n    iex> include?(:decimal, Decimal.new(\"1\"), [Decimal.new(\"1.00\"), Decimal.new(\"2.00\")])\n    true","ref":"Ecto.Type.html#include?/3-examples"},{"type":"callback","title":"Ecto.Type.load/1","doc":"Loads the given term into a custom type.\n\nThis callback is called when loading data from the database and\nreceives an Ecto native type. It can return any type, as long as\nthe `dump/1` function is able to convert the returned value back\ninto an Ecto native type.","ref":"Ecto.Type.html#c:load/1"},{"type":"function","title":"Ecto.Type.load/3","doc":"Loads a value with the given type.\n\n    iex> load(:string, nil)\n    {:ok, nil}\n    iex> load(:string, \"foo\")\n    {:ok, \"foo\"}\n\n    iex> load(:integer, 1)\n    {:ok, 1}\n    iex> load(:integer, \"10\")\n    :error","ref":"Ecto.Type.html#load/3"},{"type":"function","title":"Ecto.Type.match?/2","doc":"Checks if a given type matches with a primitive type\nthat can be found in queries.\n\n    iex> match?(:string, :any)\n    true\n    iex> match?(:any, :string)\n    true\n    iex> match?(:string, :string)\n    true\n\n    iex> match?({:array, :string}, {:array, :any})\n    true\n\n    iex> match?(Ecto.UUID, :uuid)\n    true\n    iex> match?(Ecto.UUID, :string)\n    false","ref":"Ecto.Type.html#match?/2"},{"type":"function","title":"Ecto.Type.primitive?/1","doc":"Checks if we have a primitive type.\n\n    iex> primitive?(:string)\n    true\n    iex> primitive?(Another)\n    false\n\n    iex> primitive?({:array, :string})\n    true\n    iex> primitive?({:array, Another})\n    true","ref":"Ecto.Type.html#primitive?/1"},{"type":"callback","title":"Ecto.Type.type/0","doc":"Returns the underlying schema type for the custom type.\n\nFor example, if you want to provide your own date\nstructures, the type function should return `:date`.\n\nNote this function is not required to return Ecto primitive\ntypes, the type is only required to be known by the adapter.","ref":"Ecto.Type.html#c:type/0"},{"type":"function","title":"Ecto.Type.type/1","doc":"Retrieves the underlying schema type for the given, possibly custom, type.\n\n    iex> type(:string)\n    :string\n    iex> type(Ecto.UUID)\n    :uuid\n\n    iex> type({:array, :string})\n    {:array, :string}\n    iex> type({:array, Ecto.UUID})\n    {:array, :uuid}\n\n    iex> type({:map, Ecto.UUID})\n    {:map, :uuid}","ref":"Ecto.Type.html#type/1"},{"type":"type","title":"Ecto.Type.base/0","doc":"","ref":"Ecto.Type.html#t:base/0"},{"type":"type","title":"Ecto.Type.composite/0","doc":"","ref":"Ecto.Type.html#t:composite/0"},{"type":"type","title":"Ecto.Type.custom/0","doc":"Custom types are represented by user-defined modules.","ref":"Ecto.Type.html#t:custom/0"},{"type":"type","title":"Ecto.Type.primitive/0","doc":"Primitive Ecto types (handled by Ecto).","ref":"Ecto.Type.html#t:primitive/0"},{"type":"type","title":"Ecto.Type.t/0","doc":"An Ecto type, primitive or custom.","ref":"Ecto.Type.html#t:t/0"},{"type":"module","title":"Ecto.UUID","doc":"An Ecto type for UUID strings.","ref":"Ecto.UUID.html"},{"type":"function","title":"Ecto.UUID.bingenerate/0","doc":"Generates a random, version 4 UUID in the binary format.","ref":"Ecto.UUID.html#bingenerate/0"},{"type":"function","title":"Ecto.UUID.cast/1","doc":"Casts either a string in the canonical, human-readable UUID format or a\n16-byte binary to a UUID in its canonical, human-readable UUID format.\n\nIf `uuid` is neither of these, `:error` will be returned.\n\nSince both binaries and strings are represent as binaries, this means some\nstrings you may not expect are actually also valid UUIDs in their binary form\nand so will be casted into their string form.\n\nIf you need further-restricted behavior or validation, you should define your\nown custom `Ecto.Type`. There is also `Ecto.UUID.load/1` if you only want to\nprocess `raw` UUIDs, which may be a more suitable reverse operation to\n`Ecto.UUID.dump/1`.","ref":"Ecto.UUID.html#cast/1"},{"type":"function","title":"Examples - Ecto.UUID.cast/1","doc":"iex> Ecto.UUID.cast(                   0x83, 0x65, 0xED, 0xDB, 0x4C, 0x89, 0x33, 0x27>>)\n    {:ok, \"601d74e4-a8d3-4b6e-8365-eddb4c893327\"}\n\n    iex> Ecto.UUID.cast(\"601d74e4-a8d3-4b6e-8365-eddb4c893327\")\n    {:ok, \"601d74e4-a8d3-4b6e-8365-eddb4c893327\"}\n\n    iex> Ecto.UUID.cast(\"warehouse worker\")\n    {:ok, \"77617265-686f-7573-6520-776f726b6572\"}","ref":"Ecto.UUID.html#cast/1-examples"},{"type":"function","title":"Ecto.UUID.cast!/1","doc":"Same as `cast/1` but raises `Ecto.CastError` on invalid arguments.","ref":"Ecto.UUID.html#cast!/1"},{"type":"function","title":"Ecto.UUID.dump/1","doc":"Converts a string representing a UUID into a raw binary.","ref":"Ecto.UUID.html#dump/1"},{"type":"function","title":"Ecto.UUID.dump!/1","doc":"Same as `dump/1` but raises `Ecto.ArgumentError` on invalid arguments.","ref":"Ecto.UUID.html#dump!/1"},{"type":"function","title":"Ecto.UUID.embed_as/1","doc":"","ref":"Ecto.UUID.html#embed_as/1"},{"type":"function","title":"Ecto.UUID.equal?/2","doc":"","ref":"Ecto.UUID.html#equal?/2"},{"type":"function","title":"Ecto.UUID.generate/0","doc":"Generates a random, version 4 UUID.","ref":"Ecto.UUID.html#generate/0"},{"type":"function","title":"Ecto.UUID.load/1","doc":"Converts a binary UUID into a string.","ref":"Ecto.UUID.html#load/1"},{"type":"function","title":"Ecto.UUID.load!/1","doc":"Same as `load/1` but raises `Ecto.ArgumentError` on invalid arguments.","ref":"Ecto.UUID.html#load!/1"},{"type":"type","title":"Ecto.UUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Ecto.UUID.html#t:raw/0"},{"type":"type","title":"Ecto.UUID.t/0","doc":"A hex-encoded UUID string.","ref":"Ecto.UUID.html#t:t/0"},{"type":"module","title":"Ecto.Query.API","doc":"Lists all functions allowed in the query API.\n\n  * Comparison operators: `==`, `!=`, ` =`, ` `\n  * Arithmetic operators: `+`, `-`, `*`, `/`\n  * Boolean operators: `and`, `or`, `not`\n  * Inclusion operator: `in/2`\n  * Subquery operators: `any`, `all` and `exists`\n  * Search functions: `like/2` and `ilike/2`\n  * Null check functions: `is_nil/1`\n  * Aggregates: `count/0`, `count/1`, `avg/1`, `sum/1`, `min/1`, `max/1`\n  * Date/time intervals: `datetime_add/3`, `date_add/3`, `from_now/2`, `ago/2`\n  * Inside select: `struct/2`, `map/2`, `merge/2`, `selected_as/2` and literals (map, tuples, lists, etc)\n  * General: `fragment/1`, `field/2`, `type/2`, `as/1`, `parent_as/1`, `selected_as/1`\n\nNote the functions in this module exist for documentation\npurposes and one should never need to invoke them directly.\nFurthermore, it is possible to define your own macros and\nuse them in Ecto queries (see docs for `fragment/1`).","ref":"Ecto.Query.API.html"},{"type":"module","title":"Intervals - Ecto.Query.API","doc":"Ecto supports following values for `interval` option: `\"year\"`, `\"month\"`,\n`\"week\"`, `\"day\"`, `\"hour\"`, `\"minute\"`, `\"second\"`, `\"millisecond\"`, and\n`\"microsecond\"`.\n\n`Date`/`Time` functions like `datetime_add/3`, `date_add/3`, `from_now/2`,\n`ago/2` take `interval` as an argument.","ref":"Ecto.Query.API.html#module-intervals"},{"type":"module","title":"Window API - Ecto.Query.API","doc":"Ecto also supports many of the windows functions found\nin SQL databases. See `Ecto.Query.WindowAPI` for more\ninformation.","ref":"Ecto.Query.API.html#module-window-api"},{"type":"module","title":"About the arithmetic operators - Ecto.Query.API","doc":"The Ecto implementation of these operators provide only\na thin layer above the adapters. So if your adapter allows you\nto use them in a certain way (like adding a date and an\ninterval in PostgreSQL), it should work just fine in Ecto\nqueries.","ref":"Ecto.Query.API.html#module-about-the-arithmetic-operators"},{"type":"function","title":"Ecto.Query.API.*/2","doc":"Binary `*` operation.","ref":"Ecto.Query.API.html#*/2"},{"type":"function","title":"Ecto.Query.API.+/2","doc":"Binary `+` operation.","ref":"Ecto.Query.API.html#+/2"},{"type":"function","title":"Ecto.Query.API.-/2","doc":"Binary `-` operation.","ref":"Ecto.Query.API.html#-/2"},{"type":"function","title":"Ecto.Query.API.//2","doc":"Binary `/` operation.","ref":"Ecto.Query.API.html#//2"},{"type":"function","title":"Ecto.Query.API.!=/2","doc":"Binary `!=` operation.","ref":"Ecto.Query.API.html#!=/2"},{"type":"function","title":"Ecto.Query.API.</2","doc":"Binary `<` operation.","ref":"Ecto.Query.API.html#%3C/2"},{"type":"function","title":"Ecto.Query.API.<=/2","doc":"Binary `<=` operation.","ref":"Ecto.Query.API.html#%3C=/2"},{"type":"function","title":"Ecto.Query.API.==/2","doc":"Binary `==` operation.","ref":"Ecto.Query.API.html#==/2"},{"type":"function","title":"Ecto.Query.API.>/2","doc":"Binary `>` operation.","ref":"Ecto.Query.API.html#%3E/2"},{"type":"function","title":"Ecto.Query.API.>=/2","doc":"Binary `>=` operation.","ref":"Ecto.Query.API.html#%3E=/2"},{"type":"function","title":"Ecto.Query.API.ago/2","doc":"Subtracts the given interval from the current time in UTC.\n\nThe current time in UTC is retrieved from Elixir and\nnot from the database.\n\nSee [Intervals](#module-intervals) for supported `interval` values.","ref":"Ecto.Query.API.html#ago/2"},{"type":"function","title":"Examples - Ecto.Query.API.ago/2","doc":"from p in Post, where: p.published_at > ago(3, \"month\")","ref":"Ecto.Query.API.html#ago/2-examples"},{"type":"function","title":"Ecto.Query.API.all/1","doc":"Evaluates whether all values returned from the provided subquery match in a comparison operation.\n\n    from p in Post, where: p.visits >= all(\n      from(p in Post, select: avg(p.visits), group_by: [p.category_id])\n    )\n\nFor a post to match in the above example it must be visited at least as much as the average post in all categories.\n\n    from p in Post, where: p.visits == all(\n      from(p in Post, select: max(p.visits))\n    )\n\nThe above example matches all the posts which are tied for being the most visited.\n\nBoth `any` and `all` must be given a subquery as an argument, and they must be used on the right hand side of a comparison.\nBoth can be used with every comparison operator: `==`, `!=`, `>`, `>=`, `<`, `<=`.","ref":"Ecto.Query.API.html#all/1"},{"type":"function","title":"Ecto.Query.API.and/2","doc":"Binary `and` operation.","ref":"Ecto.Query.API.html#and/2"},{"type":"function","title":"Ecto.Query.API.any/1","doc":"Tests whether one or more values returned from the provided subquery match in a comparison operation.\n\n    from p in Product, where: p.id == any(\n      from(li in LineItem, select: [li.product_id], where: li.created_at > ^since and li.qty >= 10)\n    )\n\nA product matches in the above example if a line item was created since the provided date where the customer purchased\nat least 10 units.\n\nBoth `any` and `all` must be given a subquery as an argument, and they must be used on the right hand side of a comparison.\nBoth can be used with every comparison operator: `==`, `!=`, `>`, `>=`, `<`, `<=`.","ref":"Ecto.Query.API.html#any/1"},{"type":"function","title":"Ecto.Query.API.as/1","doc":"Refer to a named atom binding.\n\nSee the \"Named binding\" section in `Ecto.Query` for more information.","ref":"Ecto.Query.API.html#as/1"},{"type":"function","title":"Ecto.Query.API.avg/1","doc":"Calculates the average for the given entry.\n\n    from p in Payment, select: avg(p.value)","ref":"Ecto.Query.API.html#avg/1"},{"type":"function","title":"Ecto.Query.API.coalesce/2","doc":"Takes whichever value is not null, or null if they both are.\n\nIn SQL, COALESCE takes any number of arguments, but in ecto\nit only takes two, so it must be chained to achieve the same\neffect.\n\n    from p in Payment, select: p.value |> coalesce(p.backup_value) |> coalesce(0)","ref":"Ecto.Query.API.html#coalesce/2"},{"type":"function","title":"Ecto.Query.API.count/0","doc":"Counts the entries in the table.\n\n    from p in Post, select: count()","ref":"Ecto.Query.API.html#count/0"},{"type":"function","title":"Ecto.Query.API.count/1","doc":"Counts the given entry.\n\n    from p in Post, select: count(p.id)","ref":"Ecto.Query.API.html#count/1"},{"type":"function","title":"Ecto.Query.API.count/2","doc":"Counts the distinct values in given entry.\n\n    from p in Post, select: count(p.id, :distinct)","ref":"Ecto.Query.API.html#count/2"},{"type":"function","title":"Ecto.Query.API.date_add/3","doc":"Adds a given interval to a date.\n\nSee `datetime_add/3` for more information.\n\nSee [Intervals](#module-intervals) for supported `interval` values.","ref":"Ecto.Query.API.html#date_add/3"},{"type":"function","title":"Ecto.Query.API.datetime_add/3","doc":"Adds a given interval to a datetime.\n\nThe first argument is a `datetime`, the second one is the count\nfor the interval, which may be either positive or negative and\nthe interval value:\n\n    # Get all items published since the last month\n    from p in Post, where: p.published_at >\n                           datetime_add(^NaiveDateTime.utc_now(), -1, \"month\")\n\nIn the example above, we used `datetime_add/3` to subtract one month\nfrom the current datetime and compared it with the `p.published_at`.\nIf you want to perform operations on date, `date_add/3` could be used.\n\nSee [Intervals](#module-intervals) for supported `interval` values.","ref":"Ecto.Query.API.html#datetime_add/3"},{"type":"function","title":"Ecto.Query.API.exists/1","doc":"Evaluates to true if the provided subquery returns 1 or more rows.\n\n    from p in Post,\n      as: :post,\n      where:\n        exists(\n          from(\n            c in Comment,\n            where: parent_as(:post).id == c.post_id and c.replies_count > 5,\n            select: 1\n          )\n        )\n\nThis is best used in conjunction with `parent_as` to correlate the subquery\nwith the parent query to test some condition on related rows in a different table.\nIn the above example the query returns posts which have at least one comment that\nhas more than 5 replies.","ref":"Ecto.Query.API.html#exists/1"},{"type":"function","title":"Ecto.Query.API.field/2","doc":"Allows a field to be dynamically accessed.\n\n    def at_least_four(doors_or_tires) do\n      from c in Car,\n        where: field(c, ^doors_or_tires) >= 4\n    end\n\nIn the example above, both `at_least_four(:doors)` and `at_least_four(:tires)`\nwould be valid calls as the field is dynamically generated.","ref":"Ecto.Query.API.html#field/2"},{"type":"function","title":"Ecto.Query.API.filter/2","doc":"Applies the given expression as a FILTER clause against an\naggregate. This is currently only supported by Postgres.\n\n    from p in Payment, select: filter(avg(p.value), p.value > 0 and p.value   filter(p.value < 0)","ref":"Ecto.Query.API.html#filter/2"},{"type":"function","title":"Ecto.Query.API.fragment/1","doc":"Send fragments directly to the database.\n\nIt is not possible to represent all possible database queries using\nEcto's query syntax. When such is required, it is possible to use\nfragments to send any expression to the database:\n\n    def unpublished_by_title(title) do\n      from p in Post,\n        where: is_nil(p.published_at) and\n               fragment(\"lower(?)\", p.title) == ^title\n    end\n\nEvery occurrence of the `?` character will be interpreted as a place\nfor parameters, which must be given as additional arguments to\n`fragment`. If the literal character `?` is required as part of the\nfragment, it can be escaped with `\\\\?` (one escape for strings,\nanother for fragment).\n\nIn the example above, we are using the lower procedure in the\ndatabase to downcase the title column.\n\nIt is very important to keep in mind that Ecto is unable to do any\ntype casting when fragments are used. Therefore it may be necessary\nto explicitly cast parameters via `type/2`:\n\n    fragment(\"lower(?)\", p.title) == type(^title, :string)","ref":"Ecto.Query.API.html#fragment/1"},{"type":"function","title":"Literals - Ecto.Query.API.fragment/1","doc":"Sometimes you need to interpolate a literal value into a fragment,\ninstead of a parameter. For example, you may need to pass a table\nname or a collation, such as:\n\n    collation = \"es_ES\"\n    fragment(\"? COLLATE ?\", ^name, ^collation)\n\nThe example above won't work because `collation` will be passed\nas a parameter, while it has to be a literal part of the query.\n\nYou can address this by telling Ecto that variable is a literal:\n\n    fragment(\"? COLLATE ?\", ^name, literal(^collation))\n\nEcto will then escape it and make it part of the query.\n\n> ##","ref":"Ecto.Query.API.html#fragment/1-literals"},{"type":"function","title":"Literals and query caching {: .warning} - Ecto.Query.API.fragment/1","doc":">\n> Because literals are made part of the query, each interpolated\n> literal will generate a separate query, with its own cache.","ref":"Ecto.Query.API.html#fragment/1-literals-and-query-caching-warning"},{"type":"function","title":"Splicing - Ecto.Query.API.fragment/1","doc":"Sometimes you may need to interpolate a variable number of arguments\ninto the same fragment. For example, when overriding Ecto's default\n`where` behaviour for Postgres:\n\n    from p in Post, where: fragment(\"? in (?, ?)\", p.id, val1, val2)\n\nThe example above will only work if you know the number of arguments\nupfront. If it can vary, the above will not work.\n\nYou can address this by telling Ecto to splice a list argument into\nthe fragment:\n\n    from p in Post, where: fragment(\"? in (?)\", p.id, splice(^val_list))\n\nThis will let Ecto know it should expand the values of the list into\nseparate fragment arguments. For example:\n\n    from p in Post, where: fragment(\"? in (?)\", p.id, splice(^[1, 2, 3]))\n\nwould be expanded into\n\n    from p in Post, where: fragment(\"? in (?,?,?)\", p.id, ^1, ^2, ^3)","ref":"Ecto.Query.API.html#fragment/1-splicing"},{"type":"function","title":"Defining custom functions using macros and fragment - Ecto.Query.API.fragment/1","doc":"You can add a custom Ecto query function using macros.  For example\nto expose SQL's coalesce function you can define this macro:\n\n    defmodule CustomFunctions do\n      defmacro coalesce(left, right) do\n        quote do\n          fragment(\"coalesce(?, ?)\", unquote(left), unquote(right))\n        end\n      end\n    end\n\nTo have coalesce/2 available, just import the module that defines it.\n\n    import CustomFunctions\n\nThe only downside is that it will show up as a fragment when\ninspecting the Elixir query.  Other than that, it should be\nequivalent to a built-in Ecto query function.","ref":"Ecto.Query.API.html#fragment/1-defining-custom-functions-using-macros-and-fragment"},{"type":"function","title":"Keyword fragments - Ecto.Query.API.fragment/1","doc":"In order to support databases that do not have string-based\nqueries, like MongoDB, fragments also allow keywords to be given:\n\n    from p in Post,\n        where: fragment(title: [\"$eq\": ^some_value])","ref":"Ecto.Query.API.html#fragment/1-keyword-fragments"},{"type":"function","title":"Ecto.Query.API.from_now/2","doc":"Adds the given interval to the current time in UTC.\n\nThe current time in UTC is retrieved from Elixir and\nnot from the database.\n\nSee [Intervals](#module-intervals) for supported `interval` values.","ref":"Ecto.Query.API.html#from_now/2"},{"type":"function","title":"Examples - Ecto.Query.API.from_now/2","doc":"from a in Account, where: a.expires_at < from_now(3, \"month\")","ref":"Ecto.Query.API.html#from_now/2-examples"},{"type":"function","title":"Ecto.Query.API.ilike/2","doc":"Searches for `search` in `string` in a case insensitive fashion.\n\n    from p in Post, where: ilike(p.body, \"Chapter%\")\n\nTranslates to the underlying SQL ILIKE query. This operation is\nonly available on PostgreSQL.","ref":"Ecto.Query.API.html#ilike/2"},{"type":"function","title":"Ecto.Query.API.in/2","doc":"Checks if the left-value is included in the right one.\n\n    from p in Post, where: p.id in [1, 2, 3]\n\nThe right side may either be a list, a literal list\nor even a column in the database with array type:\n\n    from p in Post, where: \"elixir\" in p.tags\n\nAdditionally, the right side may also be a subquery, which should return\na single column:\n\n    from c in Comment, where: c.post_id in subquery(\n      from(p in Post, where: p.created_at > ^since, select: p.id)\n    )","ref":"Ecto.Query.API.html#in/2"},{"type":"function","title":"Ecto.Query.API.is_nil/1","doc":"Checks if the given value is nil.\n\n    from p in Post, where: is_nil(p.published_at)\n\nTo check if a given value is not nil use:\n\n    from p in Post, where: not is_nil(p.published_at)","ref":"Ecto.Query.API.html#is_nil/1"},{"type":"function","title":"Ecto.Query.API.json_extract_path/2","doc":"Returns value from the `json_field` pointed to by `path`.\n\n    from(post in Post, select: json_extract_path(post.meta, [\"author\", \"name\"]))\n\nThe path can be dynamic:\n\n    path = [\"author\", \"name\"]\n    from(post in Post, select: json_extract_path(post.meta, ^path))\n\nAnd the field can also be dynamic in combination with it:\n\n    path = [\"author\", \"name\"]\n    from(post in Post, select: json_extract_path(field(post, :meta), ^path))\n\nThe query can be also rewritten as:\n\n    from(post in Post, select: post.meta[\"author\"][\"name\"])\n\nPath elements can be integers to access values in JSON arrays:\n\n    from(post in Post, select: post.meta[\"tags\"][0][\"name\"])\n\nAny element of the path can be dynamic:\n\n    field = \"name\"\n    from(post in Post, select: post.meta[\"author\"][^field])","ref":"Ecto.Query.API.html#json_extract_path/2"},{"type":"function","title":"Warning: indexes on PostgreSQL - Ecto.Query.API.json_extract_path/2","doc":"PostgreSQL supports indexing on jsonb columns via GIN indexes.\nWhenever comparing the value of a jsonb field against a string\nor integer, Ecto will use the containment operator @> which\nis optimized. You can even use the more efficient `jsonb_path_ops`\nGIN index variant. For more information, consult PostgreSQL's docs\non [JSON indexing](https://www.postgresql.org/docs/current/datatype-json.html#JSON-INDEXING).","ref":"Ecto.Query.API.html#json_extract_path/2-warning-indexes-on-postgresql"},{"type":"function","title":"Warning: return types - Ecto.Query.API.json_extract_path/2","doc":"The underlying data in the JSON column is returned without any\nadditional decoding. This means \"null\" JSON values are not the\nsame as SQL's \"null\". For example, the `Repo.all` operation below\nreturns an empty list because `p.meta[\"author\"]` returns JSON's\nnull and therefore `is_nil` does not succeed:\n\n    Repo.insert!(%Post{meta: %{author: nil}})\n    Repo.all(from(post in Post, where: is_nil(p.meta[\"author\"])))\n\nSimilarly, other types, such as datetimes, are returned as strings.\nThis means conditions like `post.meta[\"published_at\"] > from_now(-1, \"day\")`\nmay return incorrect results or fail as the underlying database\ntries to compare incompatible types. You can, however, use `type/2`\nto force the types on the database level.","ref":"Ecto.Query.API.html#json_extract_path/2-warning-return-types"},{"type":"function","title":"Ecto.Query.API.like/2","doc":"Searches for `search` in `string`.\n\n    from p in Post, where: like(p.body, \"Chapter%\")\n\nTranslates to the underlying SQL LIKE query, therefore\nits behaviour is dependent on the database. In particular,\nPostgreSQL will do a case-sensitive operation, while the\nmajority of other databases will be case-insensitive. For\nperforming a case-insensitive `like` in PostgreSQL, see `ilike/2`.\n\nYou should be very careful when allowing user sent data to be used\nas part of LIKE query, since they allow to perform\n[LIKE-injections](https://githubengineering.com/like-injection/).","ref":"Ecto.Query.API.html#like/2"},{"type":"function","title":"Ecto.Query.API.literal/1","doc":"Allows a literal identifier to be injected into a fragment:\n\n    collation = \"es_ES\"\n    fragment(\"? COLLATE ?\", ^name, literal(^collation))\n\nThe example above will inject `collation` into the query as\na literal identifier instead of a query parameter. Note that\neach different value of `collation` will emit a different query,\nwhich will be independently prepared and cached.","ref":"Ecto.Query.API.html#literal/1"},{"type":"function","title":"Ecto.Query.API.map/2","doc":"Used in `select` to specify which fields should be returned as a map.\n\nFor example, if you don't need all fields to be returned or\nneither need a struct, you can use `map/2` to achieve both:\n\n    from p in Post,\n      select: map(p, [:title, :body])\n\n`map/2` can also be used to dynamically select fields:\n\n    fields = [:title, :body]\n    from p in Post, select: map(p, ^fields)\n\nIf the same source is selected multiple times with a `map`,\nthe fields are merged in order to avoid fetching multiple copies\nfrom the database. In other words, the expression below:\n\n    from(city in City, preload: :country,\n         select: {map(city, [:country_id]), map(city, [:name])})\n\nis expanded to:\n\n    from(city in City, preload: :country,\n         select: {map(city, [:country_id, :name]), map(city, [:country_id, :name])})\n\nFor preloads, the selected fields may be specified from the parent:\n\n    from(city in City, preload: :country,\n         select: map(city, [:country_id, :name, country: [:id, :population]]))\n\n It's also possible to select a struct from one source but only a subset of\n fields from one of its associations:\n\n    from(city in City, preload: :country,\n         select: %{city | country: map(country: [:id, :population])})\n\n**IMPORTANT**: When filtering fields for associations, you\nMUST include the foreign keys used in the relationship,\notherwise Ecto will be unable to find associated records.","ref":"Ecto.Query.API.html#map/2"},{"type":"function","title":"Ecto.Query.API.max/1","doc":"Calculates the maximum for the given entry.\n\n    from p in Payment, select: max(p.value)","ref":"Ecto.Query.API.html#max/1"},{"type":"function","title":"Ecto.Query.API.merge/2","doc":"Merges the map on the right over the map on the left.\n\nIf the map on the left side is a struct, Ecto will check\nall of the field on the right previously exist on the left\nbefore merging.\n\n    from(city in City, select: merge(city, %{virtual_field: \"some_value\"}))\n\nThis function is primarily used by `Ecto.Query.select_merge/3`\nto merge different select clauses.","ref":"Ecto.Query.API.html#merge/2"},{"type":"function","title":"Ecto.Query.API.min/1","doc":"Calculates the minimum for the given entry.\n\n    from p in Payment, select: min(p.value)","ref":"Ecto.Query.API.html#min/1"},{"type":"function","title":"Ecto.Query.API.not/1","doc":"Unary `not` operation.\n\nIt is used to negate values in `:where`. It is also used to match\nthe assert the opposite of `in/2`, `is_nil/1`, and `exists/1`.\nFor example:\n\n    from p in Post, where: p.id not in [1, 2, 3]\n\n    from p in Post, where: not is_nil(p.title)\n\n    # Retrieve all the posts that doesn't have comments.\n    from p in Post,\n      as: :post,\n      where:\n        not exists(\n          from(\n            c in Comment,\n            where: parent_as(:post).id == c.post_id\n          )\n        )","ref":"Ecto.Query.API.html#not/1"},{"type":"function","title":"Ecto.Query.API.or/2","doc":"Binary `or` operation.","ref":"Ecto.Query.API.html#or/2"},{"type":"function","title":"Ecto.Query.API.parent_as/1","doc":"Refer to a named atom binding in the parent query.\n\nThis is available only inside subqueries.\n\nSee the \"Named binding\" section in `Ecto.Query` for more information.","ref":"Ecto.Query.API.html#parent_as/1"},{"type":"function","title":"Ecto.Query.API.selected_as/1","doc":"Refer to an alias of a selected value.\n\nThis can be used to refer to aliases created using `selected_as/2`. If\nthe alias hasn't been created using `selected_as/2`, an error will be raised.\n\nEach database has its own rules governing which clauses can reference these aliases.\nIf an error is raised mentioning an unknown column, most likely the alias is being\nreferenced somewhere that is not allowed. Consult the documentation for the database\nto ensure the alias is being referenced correctly.","ref":"Ecto.Query.API.html#selected_as/1"},{"type":"function","title":"Ecto.Query.API.selected_as/2","doc":"Creates an alias for the given selected value.\n\nWhen working with calculated values, an alias can be used to simplify\nthe query. Otherwise, the entire expression would need to be copied when\nreferencing it outside of select statements.\n\nThis comes in handy when, for instance, you would like to use the calculated\nvalue in `Ecto.Query.group_by/3` or `Ecto.Query.order_by/3`:\n\n    from p in Post,\n      select: %{\n        posted: selected_as(p.posted, :date),\n        sum_visits: p.visits |> coalesce(0) |> sum() |> selected_as(:sum_visits)\n      },\n      group_by: selected_as(:date),\n      order_by: selected_as(:sum_visits)\n\nThe name of the alias must be an atom and it can only be used in the outer most\nselect expression, otherwise an error is raised. Please note that the alias name\ndoes not have to match the key when `select` returns a map, struct or keyword list.\n\nUsing this in conjunction with `selected_as/1` is recommended to ensure only defined aliases\nare referenced.","ref":"Ecto.Query.API.html#selected_as/2"},{"type":"function","title":"Subqueries and CTEs - Ecto.Query.API.selected_as/2","doc":"Subqueries and CTEs automatically alias the selected fields, for example, one can write:\n\n    # Subquery\n    s = from p in Post, select: %{visits: coalesce(p.visits, 0)}\n    from(s in subquery(s), select: s.visits)\n\n    # CTE\n    cte_query = from p in Post, select: %{visits: coalesce(p.visits, 0)}\n    Post |> with_cte(\"cte\", as: ^cte_query) |> join(:inner, [p], c in \"cte\") |> select([p, c], c.visits)\n\nHowever, one can also use `selected_as` to override the default naming:\n\n    # Subquery\n    s = from p in Post, select: %{visits: coalesce(p.visits, 0) |> selected_as(:num_visits)}\n    from(s in subquery(s), select: s.num_visits)\n\n    # CTE\n    cte_query = from p in Post, select: %{visits: coalesce(p.visits, 0) |> selected_as(:num_visits)}\n    Post |> with_cte(\"cte\", as: ^cte_query) |> join(:inner, [p], c in \"cte\") |> select([p, c], c.num_visits)\n\nThe name given to `selected_as/2` can also be referenced in `selected_as/1`,\nas in regular queries.","ref":"Ecto.Query.API.html#selected_as/2-subqueries-and-ctes"},{"type":"function","title":"Ecto.Query.API.splice/1","doc":"Allows a list argument to be spliced into a fragment.\n\n    from p in Post, where: fragment(\"? in (?)\", p.id, splice(^[1, 2, 3]))\n\nThe example above will be transformed at runtime into the following:\n\n    from p in Post, where: fragment(\"? in (?,?,?)\", p.id, ^1, ^2, ^3)","ref":"Ecto.Query.API.html#splice/1"},{"type":"function","title":"Ecto.Query.API.struct/2","doc":"Used in `select` to specify which struct fields should be returned.\n\nFor example, if you don't need all fields to be returned\nas part of a struct, you can filter it to include only certain\nfields by using `struct/2`:\n\n    from p in Post,\n      select: struct(p, [:title, :body])\n\n`struct/2` can also be used to dynamically select fields:\n\n    fields = [:title, :body]\n    from p in Post, select: struct(p, ^fields)\n\nAs a convenience, `select` allows developers to take fields\nwithout an explicit call to `struct/2`:\n\n    from p in Post, select: [:title, :body]\n\nOr even dynamically:\n\n    fields = [:title, :body]\n    from p in Post, select: ^fields\n\nFor preloads, the selected fields may be specified from the parent:\n\n    from(city in City, preload: :country,\n         select: struct(city, [:country_id, :name, country: [:id, :population]]))\n\nIf the same source is selected multiple times with a `struct`,\nthe fields are merged in order to avoid fetching multiple copies\nfrom the database. In other words, the expression below:\n\n    from(city in City, preload: :country,\n         select: {struct(city, [:country_id]), struct(city, [:name])})\n\nis expanded to:\n\n    from(city in City, preload: :country,\n         select: {struct(city, [:country_id, :name]), struct(city, [:country_id, :name])})\n\n**IMPORTANT**: When filtering fields for associations, you\nMUST include the foreign keys used in the relationship,\notherwise Ecto will be unable to find associated records.","ref":"Ecto.Query.API.html#struct/2"},{"type":"function","title":"Ecto.Query.API.sum/1","doc":"Calculates the sum for the given entry.\n\n    from p in Payment, select: sum(p.value)","ref":"Ecto.Query.API.html#sum/1"},{"type":"function","title":"Ecto.Query.API.type/2","doc":"Casts the given value to the given type at the database level.\n\nMost of the times, Ecto is able to proper cast interpolated\nvalues due to its type checking mechanism. In some situations\nthough, you may want to tell Ecto that a parameter has some\nparticular type:\n\n    type(^title, :string)\n\nIt is also possible to say the type must match the same of a column:\n\n    type(^title, p.title)\n\nOr a parameterized type, which must be previously initialized\nwith `Ecto.ParameterizedType.init/2`:\n\n    @my_enum Ecto.ParameterizedType.init(Ecto.Enum, values: [:foo, :bar, :baz])\n    type(^title, ^@my_enum)\n\nEcto will ensure `^title` is cast to the given type and enforce such\ntype at the database level. If the value is returned in a `select`,\nEcto will also enforce the proper type throughout.\n\nWhen performing arithmetic operations, `type/2` can be used to cast\nall the parameters in the operation to the same type:\n\n    from p in Post,\n      select: type(p.visits + ^a_float + ^a_integer, :decimal)\n\nInside `select`, `type/2` can also be used to cast fragments:\n\n    type(fragment(\"NOW\"), :naive_datetime)\n\nOr to type fields from schemaless queries:\n\n    from p in \"posts\", select: type(p.cost, :decimal)\n\nOr to type aggregation results:\n\n    from p in Post, select: type(avg(p.cost), :integer)\n    from p in Post, select: type(filter(avg(p.cost), p.cost > 0), :integer)\n\nOr to type comparison expression results:\n\n    from p in Post, select: type(coalesce(p.cost, 0), :integer)\n\nOr to type fields from a parent query using `parent_as/1`:\n\n    child = from c in Comment, where: type(parent_as(:posts).id, :string) == c.text\n    from Post, as: :posts, inner_lateral_join: c in subquery(child), select: c.text","ref":"Ecto.Query.API.html#type/2"},{"type":"function","title":"Ecto.Query.API.values/2","doc":"Creates a values list/constant table.\n\nA values list can be used as a source in a query, both in `Ecto.Query.from/2`\nand `Ecto.Query.join/5`.\n\nThe first argument is a list of maps representing the values of the constant table.\nEach entry in the list must have exactly the same fields or an error is raised.\n\nThe second argument is a map of types corresponding to the fields in the first argument.\nEach field must be given a type or an error is raised. Any type that can be specified in\na schema may be used.","ref":"Ecto.Query.API.html#values/2"},{"type":"function","title":"Select example - Ecto.Query.API.values/2","doc":"values = [%{id: 1, text: \"abc\"}, %{id: 2, text: \"xyz\"}]\n    types = %{id: :integer, text: :string}\n\n    query =\n      from v1 in values(values, types),\n        join: v2 in values(values, types),\n        on: v1.id == v2.id\n\n    Repo.all(query)","ref":"Ecto.Query.API.html#values/2-select-example"},{"type":"function","title":"Delete example - Ecto.Query.API.values/2","doc":"values = [%{id: 1, text: \"abc\"}, %{id: 2, text: \"xyz\"}]\n    types = %{id: :integer, text: :string}\n\n    query =\n      from p in Post,\n        join: v in values(values, types),\n        on: p.id == v.id,\n        where: p.counter == ^0\n\n    Repo.delete_all(query)","ref":"Ecto.Query.API.html#values/2-delete-example"},{"type":"function","title":"Update example - Ecto.Query.API.values/2","doc":"values = [%{id: 1, text: \"abc\"}, %{id: 2, text: \"xyz\"}]\n    types = %{id: :integer, text: :string}\n\n    query =\n      from p in Post,\n        join: v in values(values, types),\n        on: p.id == v.id,\n        update: [set: [text: v.text]]\n\n    Repo.update_all(query, [])","ref":"Ecto.Query.API.html#values/2-update-example"},{"type":"module","title":"Ecto.Query.WindowAPI","doc":"Lists all windows functions.\n\nWindows functions must always be used as the first argument\nof `over/2` where the second argument is the name of a window:\n\n    from e in Employee,\n      select: {e.depname, e.empno, e.salary, over(avg(e.salary), :department)},\n      windows: [department: [partition_by: e.depname]]\n\nIn the example above, we get the average salary per department.\n`:department` is the window name, partitioned by `e.depname`\nand `avg/1` is the window function.\n\nHowever, note that defining a window is not necessary, as the\nwindow definition can be given as the second argument to `over`:\n\n    from e in Employee,\n      select: {e.depname, e.empno, e.salary, over(avg(e.salary), partition_by: e.depname)}\n\nBoth queries are equivalent. However, if you are using the same\npartitioning over and over again, defining a window will reduce\nthe query size. See `Ecto.Query.windows/3` for all possible window\nexpressions, such as `:partition_by` and `:order_by`.","ref":"Ecto.Query.WindowAPI.html"},{"type":"function","title":"Ecto.Query.WindowAPI.avg/1","doc":"Calculates the average for the given entry.\n\n    from p in Payment, select: avg(p.value)","ref":"Ecto.Query.WindowAPI.html#avg/1"},{"type":"function","title":"Ecto.Query.WindowAPI.count/0","doc":"Counts the entries in the table.\n\n    from p in Post, select: count()","ref":"Ecto.Query.WindowAPI.html#count/0"},{"type":"function","title":"Ecto.Query.WindowAPI.count/1","doc":"Counts the given entry.\n\n    from p in Post, select: count(p.id)","ref":"Ecto.Query.WindowAPI.html#count/1"},{"type":"function","title":"Ecto.Query.WindowAPI.cume_dist/0","doc":"Returns relative rank of the current row:\n(number of rows preceding or peer with current row) / (total rows).\n\n    from p in Post,\n         select: cume_dist() |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#cume_dist/0"},{"type":"function","title":"Ecto.Query.WindowAPI.dense_rank/0","doc":"Returns rank of the current row without gaps; this function counts peer groups.\n\n    from p in Post,\n         select: dense_rank() |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#dense_rank/0"},{"type":"function","title":"Ecto.Query.WindowAPI.filter/2","doc":"Applies the given expression as a FILTER clause against an\naggregate. This is currently only supported by Postgres.\n\n    from p in Post,\n         select: avg(p.value)\n                 |> filter(p.value > 0 and p.value   over(partition_by: p.category_id, order_by: p.date)","ref":"Ecto.Query.WindowAPI.html#filter/2"},{"type":"function","title":"Ecto.Query.WindowAPI.first_value/1","doc":"Returns value evaluated at the row that is the first row of the window frame.\n\n    from p in Post,\n         select: first_value(p.id) |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#first_value/1"},{"type":"function","title":"Ecto.Query.WindowAPI.lag/3","doc":"Returns value evaluated at the row that is offset rows before\nthe current row within the partition.\n\nIf there is no such row, instead return default (which must be of the\nsame type as value). Both offset and default are evaluated with respect\nto the current row. If omitted, offset defaults to 1 and default to `nil`.\n\n    from e in Events,\n         windows: [w: [partition_by: e.name, order_by: e.tick]],\n         select: {\n           e.tick,\n           e.action,\n           e.name,\n           lag(e.action) |> over(:w), # previous_action\n           lead(e.action) |> over(:w) # next_action\n         }\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#lag/3"},{"type":"function","title":"Ecto.Query.WindowAPI.last_value/1","doc":"Returns value evaluated at the row that is the last row of the window frame.\n\n    from p in Post,\n         select: last_value(p.id) |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#last_value/1"},{"type":"function","title":"Ecto.Query.WindowAPI.lead/3","doc":"Returns value evaluated at the row that is offset rows after\nthe current row within the partition.\n\nIf there is no such row, instead return default (which must be of the\nsame type as value). Both offset and default are evaluated with respect\nto the current row. If omitted, offset defaults to 1 and default to `nil`.\n\n    from e in Events,\n         windows: [w: [partition_by: e.name, order_by: e.tick]],\n         select: {\n           e.tick,\n           e.action,\n           e.name,\n           lag(e.action) |> over(:w), # previous_action\n           lead(e.action) |> over(:w) # next_action\n         }\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#lead/3"},{"type":"function","title":"Ecto.Query.WindowAPI.max/1","doc":"Calculates the maximum for the given entry.\n\n    from p in Payment, select: max(p.value)","ref":"Ecto.Query.WindowAPI.html#max/1"},{"type":"function","title":"Ecto.Query.WindowAPI.min/1","doc":"Calculates the minimum for the given entry.\n\n    from p in Payment, select: min(p.value)","ref":"Ecto.Query.WindowAPI.html#min/1"},{"type":"function","title":"Ecto.Query.WindowAPI.nth_value/2","doc":"Returns value evaluated at the row that is the nth row of the window\nframe (counting from 1); `nil` if no such row.\n\n    from p in Post,\n         select: nth_value(p.id, 4) |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#nth_value/2"},{"type":"function","title":"Ecto.Query.WindowAPI.ntile/1","doc":"Returns integer ranging from 1 to the argument value, dividing the partition as equally as possible.\n\n    from p in Post,\n         select: ntile(10) |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#ntile/1"},{"type":"function","title":"Ecto.Query.WindowAPI.over/2","doc":"Defines a value based on the function and the window. See moduledoc for more information.\n\n    from e in Employee, select: over(avg(e.salary), partition_by: e.depname)","ref":"Ecto.Query.WindowAPI.html#over/2"},{"type":"function","title":"Ecto.Query.WindowAPI.percent_rank/0","doc":"Returns relative rank of the current row: (rank - 1) / (total rows - 1).\n\n    from p in Post,\n         select: percent_rank() |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#percent_rank/0"},{"type":"function","title":"Ecto.Query.WindowAPI.rank/0","doc":"Returns rank of the current row with gaps; same as `row_number/0` of its first peer.\n\n    from p in Post,\n         select: rank() |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#rank/0"},{"type":"function","title":"Ecto.Query.WindowAPI.row_number/0","doc":"Returns number of the current row within its partition, counting from 1.\n\n    from p in Post,\n         select: row_number() |> over(partition_by: p.category_id, order_by: p.date)\n\nNote that this function must be invoked using window function syntax.","ref":"Ecto.Query.WindowAPI.html#row_number/0"},{"type":"function","title":"Ecto.Query.WindowAPI.sum/1","doc":"Calculates the sum for the given entry.\n\n    from p in Payment, select: sum(p.value)","ref":"Ecto.Query.WindowAPI.html#sum/1"},{"type":"protocol","title":"Ecto.Queryable","doc":"Converts a data structure into an `Ecto.Query`.","ref":"Ecto.Queryable.html"},{"type":"function","title":"Ecto.Queryable.to_query/1","doc":"Converts the given `data` into an `Ecto.Query`.","ref":"Ecto.Queryable.html#to_query/1"},{"type":"type","title":"Ecto.Queryable.t/0","doc":"All the types that implement this protocol.","ref":"Ecto.Queryable.html#t:t/0"},{"type":"module","title":"Ecto.SubQuery","doc":"A struct representing subqueries.\n\nUsers of Ecto must consider this struct as opaque\nand not access its field. Authors of adapters may\nread its contents, but never modify them.\n\nSee `Ecto.Query.subquery/2` for more information.","ref":"Ecto.SubQuery.html"},{"type":"type","title":"Ecto.SubQuery.t/0","doc":"","ref":"Ecto.SubQuery.html#t:t/0"},{"type":"behaviour","title":"Ecto.Adapter","doc":"Specifies the minimal API required from adapters.","ref":"Ecto.Adapter.html"},{"type":"macrocallback","title":"Ecto.Adapter.__before_compile__/1","doc":"The callback invoked in case the adapter needs to inject code.","ref":"Ecto.Adapter.html#c:__before_compile__/1"},{"type":"callback","title":"Ecto.Adapter.checked_out?/1","doc":"Returns true if a connection has been checked out.","ref":"Ecto.Adapter.html#c:checked_out?/1"},{"type":"callback","title":"Ecto.Adapter.checkout/3","doc":"Checks out a connection for the duration of the given function.\n\nIn case the adapter provides a pool, this guarantees all of the code\ninside the given `fun` runs against the same connection, which\nmight improve performance by for instance allowing multiple related\ncalls to the datastore to share cache information:\n\n    Repo.checkout(fn ->\n      for _ <- 100 do\n        Repo.insert!(%Post{})\n      end\n    end)\n\nIf the adapter does not provide a pool, just calling the passed function\nand returning its result are enough.\n\nIf the adapter provides a pool, it is supposed to \"check out\" one of the\npool connections for the duration of the function call. Which connection\nis checked out is not passed to the calling function, so it should be done\nusing a stateful method like using the current process' dictionary, process\ntracking, or some kind of other lookup method. Make sure that this stored\nconnection is then used in the other callbacks implementations, such as\n`Ecto.Adapter.Queryable` and `Ecto.Adapter.Schema`.","ref":"Ecto.Adapter.html#c:checkout/3"},{"type":"callback","title":"Ecto.Adapter.dumpers/2","doc":"Returns the dumpers for a given type.\n\nIt receives the primitive type and the Ecto type (which may be\nprimitive as well). It returns a list of dumpers with the given\ntype usually at the beginning.\n\nThis allows developers to properly translate values coming from\nthe Ecto into adapter ones. For example, if the database does not\nsupport booleans but instead returns 0 and 1 for them, you could\nadd:\n\n    def dumpers(:boolean, type), do: [type, &bool_encode/1]\n    def dumpers(_primitive, type), do: [type]\n\n    defp bool_encode(false), do: {:ok, 0}\n    defp bool_encode(true), do: {:ok, 1}\n\nAll adapters are required to implement a clause for :binary_id types,\nsince they are adapter specific. If your adapter does not provide\nbinary ids, you may simply use `Ecto.UUID`:\n\n    def dumpers(:binary_id, type), do: [type, Ecto.UUID]\n    def dumpers(_primitive, type), do: [type]","ref":"Ecto.Adapter.html#c:dumpers/2"},{"type":"callback","title":"Ecto.Adapter.ensure_all_started/2","doc":"Ensure all applications necessary to run the adapter are started.","ref":"Ecto.Adapter.html#c:ensure_all_started/2"},{"type":"callback","title":"Ecto.Adapter.init/1","doc":"Initializes the adapter supervision tree by returning the children and adapter metadata.","ref":"Ecto.Adapter.html#c:init/1"},{"type":"callback","title":"Ecto.Adapter.loaders/2","doc":"Returns the loaders for a given type.\n\nIt receives the primitive type and the Ecto type (which may be\nprimitive as well). It returns a list of loaders with the given\ntype usually at the end.\n\nThis allows developers to properly translate values coming from\nthe adapters into Ecto ones. For example, if the database does not\nsupport booleans but instead returns 0 and 1 for them, you could\nadd:\n\n    def loaders(:boolean, type), do: [&bool_decode/1, type]\n    def loaders(_primitive, type), do: [type]\n\n    defp bool_decode(0), do: {:ok, false}\n    defp bool_decode(1), do: {:ok, true}\n\nAll adapters are required to implement a clause for `:binary_id` types,\nsince they are adapter specific. If your adapter does not provide binary\nids, you may simply use `Ecto.UUID`:\n\n    def loaders(:binary_id, type), do: [Ecto.UUID, type]\n    def loaders(_primitive, type), do: [type]","ref":"Ecto.Adapter.html#c:loaders/2"},{"type":"function","title":"Ecto.Adapter.lookup_meta/1","doc":"Returns the adapter metadata from its `c:init/1` callback.\n\nIt expects a process name of a repository. The name is either\nan atom or a PID. For a given repository, you often want to\ncall this function based on the repository dynamic repo:\n\n    Ecto.Adapter.lookup_meta(repo.get_dynamic_repo())","ref":"Ecto.Adapter.html#lookup_meta/1"},{"type":"type","title":"Ecto.Adapter.adapter_meta/0","doc":"The metadata returned by the adapter `c:init/1`.\n\nIt must be a map and Ecto itself will always inject\ntwo keys into the meta:\n\n  * the `:cache` key, which as ETS table that can be used as a cache (if available)\n  * the `:pid` key, which is the PID returned by the child spec returned in `c:init/1`","ref":"Ecto.Adapter.html#t:adapter_meta/0"},{"type":"type","title":"Ecto.Adapter.t/0","doc":"","ref":"Ecto.Adapter.html#t:t/0"},{"type":"behaviour","title":"Ecto.Adapter.Queryable","doc":"Specifies the query API required from adapters.\n\nIf your adapter is only able to respond to one or a couple of the query functions,\nadd custom implementations of those functions directly to the Repo\nby using `c:Ecto.Adapter.__before_compile__/1` instead.","ref":"Ecto.Adapter.Queryable.html"},{"type":"callback","title":"Ecto.Adapter.Queryable.execute/5","doc":"Executes a previously prepared query.\n\nThe `query_meta` field is a map containing some of the fields\nfound in the `Ecto.Query` struct, after they have been normalized.\nFor example, the values `selected` by the query, which then have\nto be returned, can be found in `query_meta`.\n\nThe `query_cache` and its state is documented in `t:query_cache/0`.\n\nThe `params` is the list of query parameters. For example, for\na query such as `from Post, where: [id: ^123]`, `params` will be\n`[123]`.\n\nFinally, `options` is a keyword list of options given to the\n`Repo` operation that triggered the adapter call. Any option is\nallowed, as this is a mechanism to allow users of Ecto to customize\nhow the adapter behaves per operation.\n\nIt must return a tuple containing the number of entries and\nthe result set as a list of lists. The entries in the actual\nlist will depend on what has been selected by the query. The\nresult set may also be `nil`, if no value is being selected.","ref":"Ecto.Adapter.Queryable.html#c:execute/5"},{"type":"function","title":"Ecto.Adapter.Queryable.plan_query/3","doc":"Plans a query using the given adapter.\n\nThis does not expect the repository and therefore does not leverage the cache.","ref":"Ecto.Adapter.Queryable.html#plan_query/3"},{"type":"callback","title":"Ecto.Adapter.Queryable.prepare/2","doc":"Commands invoked to prepare a query.\n\nIt is used on `c:Ecto.Repo.all/2`, `c:Ecto.Repo.update_all/3`,\nand `c:Ecto.Repo.delete_all/2`. If returns a tuple, saying if\nthis query can be cached or not, and the `prepared` query.\nThe `prepared` query is any term that will be passed to the\nadapter's `c:execute/5`.","ref":"Ecto.Adapter.Queryable.html#c:prepare/2"},{"type":"function","title":"Ecto.Adapter.Queryable.prepare_query/3","doc":"Plans and prepares a query for the given repo, leveraging its query cache.\n\nThis operation uses the query cache if one is available.","ref":"Ecto.Adapter.Queryable.html#prepare_query/3"},{"type":"callback","title":"Ecto.Adapter.Queryable.stream/5","doc":"Streams a previously prepared query.\n\nSee `c:execute/5` for a description of arguments.\n\nIt returns a stream of values.","ref":"Ecto.Adapter.Queryable.html#c:stream/5"},{"type":"type","title":"Ecto.Adapter.Queryable.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Ecto.Adapter.Queryable.html#t:adapter_meta/0"},{"type":"type","title":"Ecto.Adapter.Queryable.cached/0","doc":"","ref":"Ecto.Adapter.Queryable.html#t:cached/0"},{"type":"type","title":"Ecto.Adapter.Queryable.options/0","doc":"","ref":"Ecto.Adapter.Queryable.html#t:options/0"},{"type":"type","title":"Ecto.Adapter.Queryable.prepared/0","doc":"","ref":"Ecto.Adapter.Queryable.html#t:prepared/0"},{"type":"type","title":"Ecto.Adapter.Queryable.query_cache/0","doc":"Cache query metadata that is passed to `c:execute/5`.\n\nThe cache can be in 3 states, documented below.\n\nIf `{:nocache, prepared}` is given, it means the query was\nnot and cannot be cached. The `prepared` value is the value\nreturned by `c:prepare/2`.\n\nIf `{:cache, cache_function, prepared}` is given, it means\nthe query can be cached and it must be cached by calling\nthe `cache_function` function with the cache entry of your\nchoice. Once `cache_function` is called, the next time the\nsame query is given to `c:execute/5`, it will receive the\n`:cached` tuple.\n\nIf `{:cached, update_function, reset_function, cached}` is\ngiven, it means the query has been cached. You may call\n`update_function/1` if you want to update the cached result.\nOr you may call `reset_function/1`, with a new prepared query,\nto force the query to be cached again. If `reset_function/1`\nis called, the next time the same query is given to\n`c:execute/5`, it will receive the `:cache` tuple.","ref":"Ecto.Adapter.Queryable.html#t:query_cache/0"},{"type":"type","title":"Ecto.Adapter.Queryable.query_meta/0","doc":"Ecto.Query metadata fields (stored in cache)","ref":"Ecto.Adapter.Queryable.html#t:query_meta/0"},{"type":"type","title":"Ecto.Adapter.Queryable.selected/0","doc":"","ref":"Ecto.Adapter.Queryable.html#t:selected/0"},{"type":"behaviour","title":"Ecto.Adapter.Schema","doc":"Specifies the schema API required from adapters.","ref":"Ecto.Adapter.Schema.html"},{"type":"callback","title":"Ecto.Adapter.Schema.autogenerate/1","doc":"Called to autogenerate a value for id/embed_id/binary_id.\n\nReturns the autogenerated value, or nil if it must be\nautogenerated inside the storage or raise if not supported.","ref":"Ecto.Adapter.Schema.html#c:autogenerate/1"},{"type":"callback","title":"Ecto.Adapter.Schema.delete/5","doc":"Deletes a single struct with the given filters.\n\nWhile `filters` can be any record column, it is expected that\nat least the primary key (or any other key that uniquely\nidentifies an existing record) be given as a filter. Therefore,\nin case there is no record matching the given filters,\n`{:error, :stale}` is returned.","ref":"Ecto.Adapter.Schema.html#c:delete/5"},{"type":"callback","title":"Ecto.Adapter.Schema.insert/6","doc":"Inserts a single new struct in the data store.","ref":"Ecto.Adapter.Schema.html#c:insert/6"},{"type":"callback","title":"Autogenerate - Ecto.Adapter.Schema.insert/6","doc":"The primary key will be automatically included in `returning` if the\nfield has type `:id` or `:binary_id` and no value was set by the\ndeveloper or none was autogenerated by the adapter.","ref":"Ecto.Adapter.Schema.html#c:insert/6-autogenerate"},{"type":"callback","title":"Ecto.Adapter.Schema.insert_all/8","doc":"Inserts multiple entries into the data store.\n\nIn case an `Ecto.Query` given as any of the field values by the user,\nit will be sent to the adapter as a tuple with in the shape of\n`{query, params}`.","ref":"Ecto.Adapter.Schema.html#c:insert_all/8"},{"type":"callback","title":"Ecto.Adapter.Schema.update/6","doc":"Updates a single struct with the given filters.\n\nWhile `filters` can be any record column, it is expected that\nat least the primary key (or any other key that uniquely\nidentifies an existing record) be given as a filter. Therefore,\nin case there is no record matching the given filters,\n`{:error, :stale}` is returned.","ref":"Ecto.Adapter.Schema.html#c:update/6"},{"type":"type","title":"Ecto.Adapter.Schema.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Ecto.Adapter.Schema.html#t:adapter_meta/0"},{"type":"type","title":"Ecto.Adapter.Schema.constraints/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:constraints/0"},{"type":"type","title":"Ecto.Adapter.Schema.fields/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:fields/0"},{"type":"type","title":"Ecto.Adapter.Schema.filters/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:filters/0"},{"type":"type","title":"Ecto.Adapter.Schema.on_conflict/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:on_conflict/0"},{"type":"type","title":"Ecto.Adapter.Schema.options/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:options/0"},{"type":"type","title":"Ecto.Adapter.Schema.placeholders/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:placeholders/0"},{"type":"type","title":"Ecto.Adapter.Schema.returning/0","doc":"","ref":"Ecto.Adapter.Schema.html#t:returning/0"},{"type":"type","title":"Ecto.Adapter.Schema.schema_meta/0","doc":"Ecto.Schema metadata fields","ref":"Ecto.Adapter.Schema.html#t:schema_meta/0"},{"type":"behaviour","title":"Ecto.Adapter.Storage","doc":"Specifies the adapter storage API.","ref":"Ecto.Adapter.Storage.html"},{"type":"callback","title":"Ecto.Adapter.Storage.storage_down/1","doc":"Drops the storage given by options.\n\nReturns `:ok` if it was dropped successfully.\n\nReturns `{:error, :already_down}` if the storage has already been dropped or\n`{:error, term}` in case anything else goes wrong.","ref":"Ecto.Adapter.Storage.html#c:storage_down/1"},{"type":"callback","title":"Examples - Ecto.Adapter.Storage.storage_down/1","doc":"storage_down(username: \"postgres\",\n                 database: \"ecto_test\",\n                 hostname: \"localhost\")","ref":"Ecto.Adapter.Storage.html#c:storage_down/1-examples"},{"type":"callback","title":"Ecto.Adapter.Storage.storage_status/1","doc":"Returns the status of a storage given by options.\n\nCan return `:up`, `:down` or `{:error, term}` in case anything goes wrong.","ref":"Ecto.Adapter.Storage.html#c:storage_status/1"},{"type":"callback","title":"Examples - Ecto.Adapter.Storage.storage_status/1","doc":"storage_status(username: \"postgres\",\n                   database: \"ecto_test\",\n                   hostname: \"localhost\")","ref":"Ecto.Adapter.Storage.html#c:storage_status/1-examples"},{"type":"callback","title":"Ecto.Adapter.Storage.storage_up/1","doc":"Creates the storage given by options.\n\nReturns `:ok` if it was created successfully.\n\nReturns `{:error, :already_up}` if the storage has already been created or\n`{:error, term}` in case anything else goes wrong.","ref":"Ecto.Adapter.Storage.html#c:storage_up/1"},{"type":"callback","title":"Examples - Ecto.Adapter.Storage.storage_up/1","doc":"storage_up(username: \"postgres\",\n               database: \"ecto_test\",\n               hostname: \"localhost\")","ref":"Ecto.Adapter.Storage.html#c:storage_up/1-examples"},{"type":"behaviour","title":"Ecto.Adapter.Transaction","doc":"Specifies the adapter transactions API.","ref":"Ecto.Adapter.Transaction.html"},{"type":"callback","title":"Ecto.Adapter.Transaction.in_transaction?/1","doc":"Returns true if the given process is inside a transaction.","ref":"Ecto.Adapter.Transaction.html#c:in_transaction?/1"},{"type":"callback","title":"Ecto.Adapter.Transaction.rollback/2","doc":"Rolls back the current transaction.\n\nThe transaction will return the value given as `{:error, value}`.\n\nSee `c:Ecto.Repo.rollback/1`.","ref":"Ecto.Adapter.Transaction.html#c:rollback/2"},{"type":"callback","title":"Ecto.Adapter.Transaction.transaction/3","doc":"Runs the given function inside a transaction.\n\nReturns `{:ok, value}` if the transaction was successful where `value`\nis the value return by the function or `{:error, value}` if the transaction\nwas rolled back where `value` is the value given to `rollback/1`.","ref":"Ecto.Adapter.Transaction.html#c:transaction/3"},{"type":"type","title":"Ecto.Adapter.Transaction.adapter_meta/0","doc":"","ref":"Ecto.Adapter.Transaction.html#t:adapter_meta/0"},{"type":"module","title":"Ecto.Association.BelongsTo","doc":"The association struct for a `belongs_to` association.\n\nIts fields are:\n\n  * `cardinality` - The association cardinality\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `related` - The schema that is associated\n  * `related_key` - The key on the `related` schema used for the association\n  * `queryable` - The real query to use for querying association\n  * `defaults` - Default fields used when building the association\n  * `relationship` - The relationship to the specified schema, default `:parent`\n  * `on_replace` - The action taken on associations when schema is replaced","ref":"Ecto.Association.BelongsTo.html"},{"type":"module","title":"Ecto.Association.Has","doc":"The association struct for `has_one` and `has_many` associations.\n\nIts fields are:\n\n  * `cardinality` - The association cardinality\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `related` - The schema that is associated\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `related_key` - The key on the `related` schema used for the association\n  * `queryable` - The real query to use for querying association\n  * `on_delete` - The action taken on associations when schema is deleted\n  * `on_replace` - The action taken on associations when schema is replaced\n  * `defaults` - Default fields used when building the association\n  * `relationship` - The relationship to the specified schema, default is `:child`\n  * `preload_order` - Default `order_by` of the association, used only by preload","ref":"Ecto.Association.Has.html"},{"type":"module","title":"Ecto.Association.HasThrough","doc":"The association struct for `has_one` and `has_many` through associations.\n\nIts fields are:\n\n  * `cardinality` - The association cardinality\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `through` - The through associations\n  * `relationship` - The relationship to the specified schema, default `:child`","ref":"Ecto.Association.HasThrough.html"},{"type":"module","title":"Ecto.Association.ManyToMany","doc":"The association struct for `many_to_many` associations.\n\nIts fields are:\n\n  * `cardinality` - The association cardinality\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `related` - The schema that is associated\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `queryable` - The real query to use for querying association\n  * `on_delete` - The action taken on associations when schema is deleted\n  * `on_replace` - The action taken on associations when schema is replaced\n  * `defaults` - Default fields used when building the association\n  * `relationship` - The relationship to the specified schema, default `:child`\n  * `join_keys` - The keyword list with many to many join keys\n  * `join_through` - Atom (representing a schema) or a string (representing a table)\n    for many to many associations\n  * `join_defaults` - A list of defaults for join associations\n  * `preload_order` - Default `order_by` of the association, used only by preload","ref":"Ecto.Association.ManyToMany.html"},{"type":"function","title":"Ecto.Association.ManyToMany.assoc_query/2","doc":"","ref":"Ecto.Association.ManyToMany.html#assoc_query/2"},{"type":"module","title":"Ecto.Association.NotLoaded","doc":"Struct returned by associations when they are not loaded.\n\nThe fields are:\n\n  * `__field__` - the association field in `owner`\n  * `__owner__` - the schema that owns the association\n  * `__cardinality__` - the cardinality of the association","ref":"Ecto.Association.NotLoaded.html"},{"type":"type","title":"Ecto.Association.NotLoaded.t/0","doc":"","ref":"Ecto.Association.NotLoaded.html#t:t/0"},{"type":"module","title":"Ecto.Embedded","doc":"The embedding struct for `embeds_one` and `embeds_many`.\n\nIts fields are:\n\n  * `cardinality` - The association cardinality\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `related` - The schema that is embedded\n  * `on_cast` - Function name to call by default when casting embeds\n  * `on_replace` - The action taken on associations when schema is replaced","ref":"Ecto.Embedded.html"},{"type":"function","title":"Ecto.Embedded.preload_info/1","doc":"","ref":"Ecto.Embedded.html#preload_info/1"},{"type":"exception","title":"Ecto.CastError","doc":"Raised when a changeset can't cast a value.","ref":"Ecto.CastError.html"},{"type":"exception","title":"Ecto.ChangeError","doc":"","ref":"Ecto.ChangeError.html"},{"type":"exception","title":"Ecto.ConstraintError","doc":"","ref":"Ecto.ConstraintError.html"},{"type":"exception","title":"Ecto.InvalidChangesetError","doc":"Raised when we cannot perform an action because the\nchangeset is invalid.","ref":"Ecto.InvalidChangesetError.html"},{"type":"function","title":"Ecto.InvalidChangesetError.message/1","doc":"","ref":"Ecto.InvalidChangesetError.html#message/1"},{"type":"exception","title":"Ecto.InvalidURLError","doc":"","ref":"Ecto.InvalidURLError.html"},{"type":"exception","title":"Ecto.MigrationError","doc":"","ref":"Ecto.MigrationError.html"},{"type":"exception","title":"Ecto.MultiplePrimaryKeyError","doc":"","ref":"Ecto.MultiplePrimaryKeyError.html"},{"type":"exception","title":"Ecto.MultipleResultsError","doc":"","ref":"Ecto.MultipleResultsError.html"},{"type":"exception","title":"Ecto.NoPrimaryKeyFieldError","doc":"Raised at runtime when an operation that requires a primary key is invoked\nwith a schema that does not define a primary key by using `@primary_key false`","ref":"Ecto.NoPrimaryKeyFieldError.html"},{"type":"exception","title":"Ecto.NoPrimaryKeyValueError","doc":"Raised at runtime when an operation that requires a primary key is invoked\nwith a schema missing value for its primary key","ref":"Ecto.NoPrimaryKeyValueError.html"},{"type":"exception","title":"Ecto.NoResultsError","doc":"","ref":"Ecto.NoResultsError.html"},{"type":"exception","title":"Ecto.Query.CastError","doc":"Raised at runtime when a value cannot be cast.","ref":"Ecto.Query.CastError.html"},{"type":"exception","title":"Ecto.Query.CompileError","doc":"Raised at compilation time when the query cannot be compiled.","ref":"Ecto.Query.CompileError.html"},{"type":"exception","title":"Ecto.QueryError","doc":"Raised at runtime when the query is invalid.","ref":"Ecto.QueryError.html"},{"type":"exception","title":"Ecto.StaleEntryError","doc":"","ref":"Ecto.StaleEntryError.html"},{"type":"exception","title":"Ecto.SubQueryError","doc":"Raised at runtime when a subquery is invalid.","ref":"Ecto.SubQueryError.html"},{"type":"extras","title":"Changelog for v3.x","doc":"# Changelog for v3.x","ref":"changelog.html"},{"type":"extras","title":"v3.11.0 (2023-11-14) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-11-0-2023-11-14"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Association] Allow `preload_order` to take MFAs for `many_to_many` associations. This allows ordering by the join table\n  * [Ecto.Query] Add `:operation` option to `with_cte/3`. This allows CTEs to perform updates and deletes\n  * [Ecto.Query] Support `splice(^...)` in `fragment`\n  * [Ecto.Query] Add `prepend_order_by/3`\n  * [Ecto.Query] Allow `selected_as/1` and `selected_as/2` to take interpolated names\n  * [Ecto.Query] Allow map update syntax to work with `nil` values in `select`\n  * [Ecto.Query] Allow hints to inject SQL using `unsafe_fragment`\n  * [Ecto.Query] Support `values/2` lists\n  * [Ecto.Repo] Add `:on_preload_spawn` option to `preload/3`\n  * [Ecto.Schema] Support `:load_in_query` option for embeds\n  * [Ecto.Schema] Support `:returning` option for delete\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Association] Ensure parent prefix is passed to `on_delete` queries\n  * [Ecto.Changeset] Ensure duplicate primary keys are always detected for embeds\n  * [Ecto.Embedded] Raise `ArgumentError` when specifying an autogenerated `:id` primary key\n  * [Ecto.Query] Ensure subquery selects generate unique cache keys\n  * [Ecto.Query] Raise on literal non-base binary/uuids in query\n  * [Ecto.Repo] Reset `belongs_to` association if foreign key update results in a mismatch\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Adapter changes - Changelog for v3.x","doc":"* Adapters now receive `nil` for encoding/decoding\n  * Adapters now receive `type` instead of `{:maybe, type}` as the first argument to `loaders/2`\n\n#","ref":"changelog.html#adapter-changes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [Ecto.Query] Keyword hints are no longer supported. Please use `unsafe_fragment` inside of hints instead","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.10.3 (2023-07-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-10-3-2023-07-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow dynamic `field/2` in `type/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changesets] Limit the largest integer to less than 32 digits\n  * [Ecto.Type] Limit the largest integer to less than 32 digits","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.10.2 (2023-06-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-10-2-2023-06-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Support a three-arity function with position on `cast_assoc` and `cast_embed`\n  * [Ecto.Changeset] Add support for maps in `validate_length/3`\n  * [Ecto.Changeset] Add `:nulls_distinct` option to `unsafe_validate_unique`\n  * [Ecto.Query] Support `array_join` type for ClickHouse adapter\n  * [Ecto.Query.API] Support parameterized and custom map types in json path validation\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Repo] Respect parent prefix in `Repo.aggregate`\n  * [Ecto.Query.API] Fix late binding in `json_extract_path`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* Deprecate MFAs on `:with`","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.10.1 (2023-04-12) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-10-1-2023-04-12"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Consider `sort_param` even if the relation param was not given\n  * [Ecto.Query] Correct typespec to avoid Dialyzer warnings","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.10.0 (2023-04-10) - Changelog for v3.x","doc":"This release contains many improvements to Ecto.Changeset, functions like `Ecto.Changeset.changed?/2` and `field_missing?/2` will help make your code more expressive. Improvements to association and embed handling will also make it easier to manage more complex forms, especially those embedded within Phoenix.LiveView applications.\n\nOn the changeset front, note this release unifies the handling of empty values between `cast/4` and `validate_required/3`. **If you were setting `:empty_values` in the past and you want to preserve this new behaviour throughout, you may want to update your code** from this:\n\n    Ecto.Changeset.cast(changeset, params, [:field1, :field2], empty_values: [\"\", []])\n\nto:\n\n    empty_values = [[]] ++ Ecto.Changeset.empty_values()\n    Ecto.Changeset.cast(changeset, params, [:field1, :field2], empty_values: empty_values)\n\nQueries have also been improved to support LIMIT WITH TIES as well as materialized CTEs.\n\n#","ref":"changelog.html#v3-10-0-2023-04-10"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add `get_assoc`/`get_embed`\n  * [Ecto.Changeset] Add `field_missing?/2`\n  * [Ecto.Changeset] Add `changed?/2` and `changed?/3` with predicates support\n  * [Ecto.Changeset] Allow `Regex` to be used in constraint names for exact matches\n  * [Ecto.Changeset] Allow `:empty_values` option in `cast/4` to include a function which must return true if the value is empty\n  * [Ecto.Changeset] `cast/4` will by default consider strings made only of whitespace characters to be empty\n  * [Ecto.Changeset] Add support for `:sort_param` and `:drop_param` on `cast_assoc` and `cast_embed`\n  * [Ecto.Query] Support materialized option in CTEs\n  * [Ecto.Query] Support dynamic field inside `json_extract_path`\n  * [Ecto.Query] Support interpolated values for from/join prefixes\n  * [Ecto.Query] Support ties in limit expressions through `with_ties/3`\n  * [Ecto.Schema] Add `:autogenerate_fields` to the schema reflection API\n  * [Ecto.ParameterizedType] Add optional callback `format/1`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Make unsafe validate unique exclude primary key only for loaded schemas\n  * [Ecto.Changeset] Raise when change provided to `validate_format/4` is not a string\n  * [Ecto.Query] Fix bug in `json_extract_path` where maps were not allowed to be nested inside of embeds\n  * [Ecto.Schema] Allow inline embeds to overwrite conflicting aliases","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.6 (2023-07-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-6-2023-07-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow dynamic `field/2` in `type/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changesets] Limit the largest integer to less than 32 digits\n  * [Ecto.Type] Limit the largest integer to less than 32 digits","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.5 (2023-03-22) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-5-2023-03-22"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Rename `@opaque dynamic` type to `@opaque dynamic_expr` to avoid conflicts with Erlang/OTP 26","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.4 (2022-12-21) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-4-2022-12-21"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Fix regression with interpolated preloads introduced in v3.9.3","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.3 (2022-12-20) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-3-2022-12-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto] Add `reset_fields/2`\n  * [Ecto.Multi] Add `exists?/4` function\n  * [Ecto.Repo] Keep url scheme in the repo configuration\n  * [Ecto.Query] Add support for cross lateral joins\n  * [Ecto.Query] Allow preloads to use `dynamic/2`\n  * [Ecto.Query.API] Allow the entire path to be interpolated in `json_extract_path/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.9.2 (2022-11-18) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-2-2022-11-18"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow `selected_as` inside CTE\n * [Ecto.Query] Allow `selected_as` to be used in subquery\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Repo] Fix preloading through associations on `nil`\n  * [Ecto.Query] Fix select merging a `selected_as` field into a source","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.1 (2022-10-06) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-1-2022-10-06"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow `selected_as` at the root of `dynamic/2`\n  * [Ecto.Query] Allow `selected_as` to be used with `type/2`\n  * [Ecto.Query] Allow `selected_as` to be used with `select_merge`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Reenable support for embedded schemas in `unsafe_validate_unique/4`\n  * [Ecto.Query] Ensure `join_where` conditions preload correctly in `many_to_many` or with queries with one or many joins","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.0 (2022-09-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-9-0-2022-09-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add `:force_changes` option to `cast/4`\n  * [Ecto.Enum] Allow enum fields to be embed either as their values or their dumped versions\n  * [Ecto.Query] Support `^%{field: dynamic(...)}` in `select` and `select_merge`\n  * [Ecto.Query] Support `%{field: subquery(...)}` in `select` and `select_merge`\n  * [Ecto.Query] Support select aliases through `selected_as/1` and `selected_as/2`\n  * [Ecto.Query] Allow `parent_as/1` in `type/2`\n  * [Ecto.Query] Add `with_named_binding/3`\n  * [Ecto.Query] Allow fragment sources in keyword queries\n  * [Ecto.Repo] Support `idle_interval` query parameter in connection URL\n  * [Ecto.Repo] Log human-readable UUIDs by using pre-dumped query parameters\n  * [Ecto.Schema] Support preloading associations in embedded schemas\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [Ecto.Changeset] Raise when schemaless changeset or embedded schema is used in `unsafe_validate_unique/4`\n  * [Ecto.Query] Respect virtual field type in subqueries\n  * [Ecto.Query] Don't select struct fields overridden with `nil`\n  * [Ecto.Query] Fix `select_merge` not tracking `load_in_query: false` field\n  * [Ecto.Query] Fix field source when used in `json_extract_path`\n  * [Ecto.Query] Properly build CTEs at compile time\n  * [Ecto.Query] Properly order subqueries in `dynamic`\n  * [Ecto.Repo] Fix `insert_all` query parameter count when using value queries alongside `placeholder`\n  * [Ecto.Repo] Raise if combination query is used in a `many` preload\n  * [Ecto.Schema] Ignore associations that aren't loaded on insert","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.8.4 (2022-06-04) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-8-4-2022-06-04"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Multi] Add `one/2` and `all/2` functions\n  * [Ecto.Query] Support `literal(...)` in `fragment`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [Ecto.Schema] Make sure fields are inspected in the correct order in Elixir v1.14+","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.8.3 (2022-05-11) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-8-3-2022-05-11"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [Ecto.Query] Allow source aliases to be used in `type/2`\n  * [Ecto.Schema] Avoid \"undefined behaviour/struct\" warnings and errors during compilation","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.8.2 (2022-05-05) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-8-2-2022-05-05"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [Ecto.Adapter] Do not require adapter metadata to be raw maps\n  * [Ecto.Association] Respect `join_where` in many to many `on_replace` deletes\n  * [Ecto.Changeset] Check if list is in `empty_values` before nested validations","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.8.1 (2022-04-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-8-1-2022-04-27"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [Ecto.Query] Fix regression where a join's on parameter on `update_all` was out of order","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.8.0 (2022-04-26) - Changelog for v3.x","doc":"Ecto v3.8 requires Elixir v1.10+.\n\n#","ref":"changelog.html#v3-8-0-2022-04-26"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto] Add new Embedded chapter to Introductory guides\n  * [Ecto.Changeset] Allow custom `:error_key` in unique_constraint\n  * [Ecto.Changeset] Add `:match` option to all constraint functions\n  * [Ecto.Query] Support dynamic aliases\n  * [Ecto.Query] Allow using `type/2` with virtual fields\n  * [Ecto.Query] Suggest alternatives to inexistent fields in queries\n  * [Ecto.Query] Support passing queries using subqueries to `insert_all`\n  * [Ecto.Repo] Allow `stacktrace: true` so stacktraces are included in telemetry events and logs\n  * [Ecto.Schema] Validate options given to schema fields\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Address regression on `validate_subset` no longer working with custom array types\n  * [Ecto.Changeset] **Potentially breaking change**: Detect `empty_values` inside lists when casting. This may cause issues if you were relying on the casting of empty values (by default, only `\"\"`).\n  * [Ecto.Query] Handle atom list sigils in `select`\n  * [Ecto.Query] Improve tracking of `select_merge` inside subqueries\n  * [Ecto.Repo] Properly handle literals in queries given to `insert_all`\n  * [Ecto.Repo] Don't surface persisted data as changes on embed updates\n  * [Ecto.Repo] **Potentially breaking change**: Raise if an association doesn't have a primary key and is preloaded in a join query. Previously, this would silently produce the wrong the result in certain circumstances.\n  * [Ecto.Schema] Preserve parent prefix on join tables","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.7.2 (2022-03-13) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-7-2-2022-03-13"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Schema] Add option to skip validations for default values\n  * [Ecto.Query] Allow coalesce in `type/2`\n  * [Ecto.Query] Support parameterized types in type/2\n  * [Ecto.Query] Allow arbitrary parentheses in query expressions","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.7.1 (2021-08-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-7-1-2021-08-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Embedded] Make `Ecto.Embedded` public and describe struct fields\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Repo] Make sure parent changeset is included in changes for `insert`/`update`/`delete` when there are errors processing the parent itself","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.7.0 (2021-08-19) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-7-0-2021-08-19"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add `Ecto.Changeset.traverse_validations/2`\n  * [Ecto.Enum] Add `Ecto.Enum.mappings/2` and `Ecto.Enum.dump_values/2`\n  * [Ecto.Query] Add support for dynamic `as(^as)` and `parent_as(^as)`\n  * [Ecto.Repo] Add stale changeset to `Ecto.StaleEntryError` fields\n  * [Ecto.Schema] Add support for `@schema_context` to set context metadata on schema definition\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Fix changeset inspection not redacting when embedded\n  * [Ecto.Changeset] Use semantic comparison on `validate_inclusion`, `validate_exclusion`, and `validate_subset`\n  * [Ecto.Enum] Raise on duplicate values in `Ecto.Enum`\n  * [Ecto.Query] Make sure `hints` are included in the query cache\n  * [Ecto.Repo] Support placeholders in `insert_all` without schemas\n  * [Ecto.Repo] Wrap in a subquery when query given to `Repo.aggregate` has combination\n  * [Ecto.Repo] Fix CTE subqueries not finding parent bindings\n  * [Ecto.Repo] Return changeset with assocs if any of the assocs are invalid","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.2 (2021-05-28) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-6-2-2021-05-28"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Support macros in `with_cte`\n  * [Ecto.Repo] Add `Ecto.Repo.all_running/0` to list all running repos\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Do not omit nil fields in a subquery select\n  * [Ecto.Query] Allow `parent_as` to look for an alias all the way up across subqueries\n  * [Ecto.Query] Raise if a nil value is given to a query from a nested map parameter\n  * [Ecto.Query] Fix `insert_all` when using both `:on_conflict` and `:placeholders`\n  * [mix ecto.load] Do not pass `--force` to underlying compile task","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.1 (2021-04-12) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-6-1-2021-04-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Allow the `:query` option in `unsafe_validate_unique`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Add the relation id in `apply_changes` if the relation key exists (instead of hardcoding it to `id`)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.0 (2021-04-03) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-6-0-2021-04-03"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Support `:repo_opts` in `unsafe_validate_unique`\n  * [Ecto.Changeset] Add a validation error if trying to cast a cardinality one embed/assoc with anything other than a map or keyword list\n  * [Ecto.Enum] Allow enums to map to custom values\n  * [Ecto.Multi] Add `Ecto.Multi.put/3` for directly storing values\n  * [Ecto.Query] **Potentially breaking change**: optimize `many_to_many` queries so it no longer load intermediary tables in more occasions. This may cause issues if you are using `Ecto.assoc/2` to load `many_to_many` associations and then trying to access intermediate bindings (which is discouraged but it was possible)\n  * [Ecto.Repo] Allow `insert_all` to be called with a query instead of rows\n  * [Ecto.Repo] Add `:placeholders` support to `insert_all` to avoid sending the same value multiple times\n  * [Ecto.Schema] Support `:preload_order` on `has_many` and `many_to_many` associations\n  * [Ecto.UUID] Add bang UUID conversion methods\n  * [Ecto.Query] The `:hints` option now accepts dynamic values when supplied as tuples\n  * [Ecto.Query] Support `select: map(source, fields)` where `source` is a fragment\n  * [Ecto.Query] Allow referring to the parent query in a join's subquery select via `parent_as`\n  * [mix ecto] Support file and line interpolation on `ECTO_EDITOR`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Change `apply_changes/1` to add the relation to the `struct.relation_id` if relation struct is persisted\n  * [Ecto.Query] Remove unnecessary INNER JOIN in many to many association query\n  * [Ecto.Query] Allow parametric types to be interpolated in queries\n  * [Ecto.Schema] Raise `ArgumentError` when default has invalid type","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.8 (2021-02-21) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-8-2021-02-21"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Support map/2 on fragments and subqueries","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.5.7 (2021-02-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-7-2021-02-07"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Fixes param ordering issue on dynamic queries with subqueries","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.6 (2021-01-20) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-6-2021-01-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Schema] Support `on_replace: :delete_if_exists` on associations\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Allow unary minus operator in query expressions\n  * [Ecto.Schema] Allow nil values on typed maps","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.5 (2020-11-12) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-5-2020-11-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Add support for subqueries operators: `all`, `any`, and `exists`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Use association source on `put_assoc` with maps/keywords\n  * [Ecto.Enum] Add `cast` clause for nil values on `Ecto.Enum`\n  * [Ecto.Schema] Allow nested type `:any` for non-virtual fields","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.4 (2020-10-28) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-4-2020-10-28"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mix ecto.drop] Provide `--force-drop` for databases that may support it\n  * [guides] Add new \"Multi tenancy with foreign keys\" guide\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Make keys optional in specs\n  * [Ecto.Enum] Make sure `values/2` works for virtual fields\n  * [Ecto.Query] Fix missing type on CTE queries that select a single field","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.3 (2020-10-21) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-3-2020-10-21"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Do not reset parameter counter for nested CTEs\n  * [Ecto.Type] Fix regression where array type with nils could no longer be cast/load/dump\n  * [Ecto.Type] Fix CaseClauseError when casting a decimal with a binary remainder","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.2 (2020-10-12) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-2-2020-10-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Repo] Add Repo.reload/2 and Repo.reload!/2\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Fix \"__schema__/1 is undefined or private\" error while inspecting a schemaless changeset\n  * [Ecto.Repo] Invoke `c:Ecto.Repo.default_options/1` per entry-point operation","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.1 (2020-10-08) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-5-1-2020-10-08"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Warn if there are duplicate IDs in the parent schema for `cast_assoc/3`/`cast_embed/3`\n  * [Ecto.Schema] Allow `belongs_to` to accept options for parameterized types\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Keep field types when using a subquery with source","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.5.0 (2020-10-03) - Changelog for v3.x","doc":"v3.5 requires Elixir v1.8+.\n\n#","ref":"changelog.html#v3-5-0-2020-10-03"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Ensure `:empty_values` in `cast/4` does not automatically propagate to following cast calls. If you want a given set of `:empty_values` to apply to all `cast/4` calls, change the value stored in `changeset.empty_values` instead\n  * [Ecto.Changeset] **Potentially breaking change**: Do not force repository updates to happen when using `optimistic_lock`. The lock field will only be incremented if the record has other changes. If no changes, nothing happens.\n  * [Ecto.Changeset] Do not automatically share empty values across `cast/3` calls\n  * [Ecto.Query] Consider query prefix in cte/combination query cache\n  * [Ecto.Query] Allow the entry to be marked as nil when using left join with subqueries\n  * [Ecto.Query] Support subqueries inside dynamic expressions\n  * [Ecto.Repo] Fix preloading when using dynamic repos and the sandbox in automatic mode\n  * [Ecto.Repo] Do not duplicate collections when associations are preloaded for repeated elements\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Enum] Add `Ecto.Enum` as a custom parameterized type\n  * [Ecto.Query] Allow `:prefix` in `from` to be set to nil\n  * [Ecto.Query] Do not restrict subqueries in `where` to map/struct types\n  * [Ecto.Query] Allow atoms in query without interpolation in order to support Ecto.Enum\n  * [Ecto.Schema] Do not validate uniqueness if there is a prior error on the field\n  * [Ecto.Schema] Allow `redact: true` in `field`\n  * [Ecto.Schema] Support parameterized types via `Ecto.ParameterizedType`\n  * [Ecto.Schema] Rewrite embeds and assocs as parameterized types. This means `__schema__(:type, assoc_or_embed)` now returns a parameterized type. To check if something is an association, use `__schema__(:assocs)` or `__schema__(:embeds)` instead","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.4.6 (2020-08-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-6-2020-08-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow `count/0` on `type/2`\n  * [Ecto.Multi] Support anonymous functions in multiple functions\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Consider booleans as literals in unions, subqueries, ctes, etc\n  * [Ecto.Schema] Generate IDs for nested embeds","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.5 (2020-06-14) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-5-2020-06-14"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Allow custom error key in `unsafe_validate_unique`\n  * [Ecto.Changeset] Improve performance when casting large params maps\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Improve error message for invalid `cast_assoc`\n  * [Ecto.Query] Fix inspecting query with fragment CTE\n  * [Ecto.Query] Fix inspecting dynamics with aliased bindings\n  * [Ecto.Query] Improve error message when selecting a single atom\n  * [Ecto.Repo] Reduce data-copying when preloading multiple associations\n  * [Ecto.Schema] Do not define a compile-time dependency for schema in `:join_through`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.4 (2020-05-11) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-4-2020-05-11"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Schema] Add `join_where` support to `many_to_many`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.4.3 (2020-04-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-3-2020-04-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Support `as/1` and `parent_as/1` for lazy named bindings and to allow parent references from subqueries\n  * [Ecto.Query] Support `x in subquery(query)`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Do not raise for missing assocs if :force is given to preload\n  * [Ecto.Repo] Return error from `Repo.delete` on invalid changeset from `prepare_changeset`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.2 (2020-04-10) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-2-2020-04-10"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Support multiple fields in `unique_constraint/3`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.4.1 (2020-04-08) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-4-1-2020-04-08"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto] Add `Ecto.embedded_load/3` and `Ecto.embedded_dump/2`\n  * [Ecto.Query] Improve error message on invalid JSON expressions\n  * [Ecto.Repo] Emit `[:ecto, :repo, :init]` telemetry event upon Repo init\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Do not support JSON selectors on `type/2`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [Ecto.Repo] Deprecate `conflict_target: {:constraint, _}`. It is a discouraged approach and `{:unsafe_fragment, _}` is still available if someone definitely needs it","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.4.0 (2020-03-24) - Changelog for v3.x","doc":"v3.4 requires Elixir v1.7+.\n\n#","ref":"changelog.html#v3-4-0-2020-03-24"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Allow dynamic queries in CTE and improve error message\n  * [Ecto.Query] Add `Ecto.Query.API.json_extract_path/2` and JSON path support to query syntax. For example, `posts.metadata[\"tags\"][0][\"name\"]` will return the name of the first tag stored in the `:map` metadata field\n  * [Ecto.Repo] Add new `default_options/1` callback to repository\n  * [Ecto.Repo] Support passing `:telemetry_options` to repository operations\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Properly add validation annotation to `validate_acceptance`\n  * [Ecto.Query] Raise if there is loaded non-empty association data without related key when preloading. This typically means not all fields have been loaded in a query\n  * [Ecto.Schema] Show meaningful error in case `schema` is invoked twice in an `Ecto.Schema`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.4 (2020-02-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-3-4-2020-02-27"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mix ecto] Do not rely on map ordering when parsing repos\n  * [mix ecto.gen.repo] Improve error message when a repo is not given","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.3 (2020-02-14) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-3-3-2020-02-14"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Support fragments in `lock`\n  * [Ecto.Query] Handle `nil` in `select_merge` with similar semantics to SQL databases (i.e. it simply returns `nil` itself)","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.3.2 (2020-01-28) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-3-2-2020-01-28"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Only bump optimistic lock in case of success\n  * [Ecto.Query] Allow macros in Ecto window expressions\n  * [Ecto.Schema] Support `:join_defaults` on `many_to_many` associations\n  * [Ecto.Schema] Allow MFargs to be given to association `:defaults`\n  * [Ecto.Type] Add `Ecto.Type.embedded_load` and `Ecto.Type.embedded_dump`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Repo] Ignore empty hostname when parsing database url (Elixir v1.10 support)\n  * [Ecto.Repo] Rewrite combinations on Repo.exists? queries\n  * [Ecto.Schema] Respect child `@schema_prefix` in `cast_assoc`\n  * [mix ecto.gen.repo] Use `config_path` when writing new config in `mix ecto.gen.repo`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.1 (2019-12-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-3-1-2019-12-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query.WindowAPI] Support `filter/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query.API] Fix `coalesce/2` usage with mixed types","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.0 (2019-12-11) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-3-0-2019-12-11"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Adapter] Add `storage_status/1` callback to `Ecto.Adapters.Storage` behaviour\n  * [Ecto.Changeset] Add `Ecto.Changeset.apply_action!/2`\n  * [Ecto.Changeset] Remove actions restriction in `Ecto.Changeset.apply_action/2`\n  * [Ecto.Repo] Introduce `c:Ecto.Repo.aggregate/2`\n  * [Ecto.Repo] Support `{:replace_all_except, fields}` in `:on_conflict`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Make sure the `:prefix` option in `:from`/`:join` also cascades to subqueries\n  * [Ecto.Query] Make sure the `:prefix` option in `:join` also cascades to queries\n  * [Ecto.Query] Use database returned values for literals. Previous Ecto versions knew literals from queries should not be discarded for combinations but, even if they were not discarded, we would ignore the values returned by the database\n  * [Ecto.Repo] Do not wrap schema operations in a transaction if already inside a transaction. We have also removed the **private** option called `:skip_transaction`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [Ecto.Repo] `:replace_all_except_primary_keys` is deprecated in favor of `{:replace_all_except, fields}` in `:on_conflict`","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.2.5 (2019-11-03) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-2-5-2019-11-03"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Fix a bug where executing some queries would leak the `{:maybe, ...}` type","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.4 (2019-11-02) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-2-4-2019-11-02"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Improve error message on invalid join binding\n  * [Ecto.Query] Make sure the `:prefix` option in `:join` also applies to through associations\n  * [Ecto.Query] Invoke custom type when loading aggregations from the database (but fallback to database value if it can't be cast)\n  * [mix ecto.gen.repo] Support Elixir v1.9 style configs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.3 (2019-10-17) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-2-3-2019-10-17"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Do not convert enums given to `validate_inclusion` to a list\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Improve error message on non-atom keys to change/put_change\n  * [Ecto.Changeset] Allow :with to be given as a `{module, function, args}` tuple on `cast_association/cast_embed`\n  * [Ecto.Changeset] Add `fetch_change!/2` and `fetch_field!/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.2.2 (2019-10-01) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-2-2-2019-10-01"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Fix keyword arguments given to `:on` when a bind is not given to join\n  * [Ecto.Repo] Make sure a preload given to an already preloaded has_many :through is loaded","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.1 (2019-09-17) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-2-1-2019-09-17"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add rollover logic for default incrementer in `optimistic_lock`\n  * [Ecto.Query] Also expand macros when used inside `type/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Ensure queries with non-cacheable queries in CTEs/combinations are also not-cacheable","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.0 (2019-09-07) - Changelog for v3.x","doc":"v3.2 requires Elixir v1.6+.\n\n#","ref":"changelog.html#v3-2-0-2019-09-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Add common table expressions support `with_cte/3` and `recursive_ctes/2`\n  * [Ecto.Query] Allow `dynamic/3` to be used in `order_by`, `distinct`, `group_by`, as well as in `partition_by`, `order_by`, and `frame` inside `windows`\n  * [Ecto.Query] Allow filters in `type/2` expressions\n  * [Ecto.Repo] Merge options given to the repository into the changeset `repo_opts` and assign it back to make it available down the chain\n  * [Ecto.Repo] Add `prepare_query/3` callback that is invoked before query operations\n  * [Ecto.Repo] Support `:returning` option in `Ecto.Repo.update/2`\n  * [Ecto.Repo] Support passing a one arity function to `Ecto.Repo.transaction/2`, where the argument is the current repo\n  * [Ecto.Type] Add a new `embed_as/1` callback to `Ecto.Type` that allows adapters to control embedding behaviour\n  * [Ecto.Type] Add `use Ecto.Type` for convenience that implements the new required callbacks\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Association] Ensure we delete an association before inserting when replacing on `has_one`\n  * [Ecto.Query] Do not allow interpolated `nil` in literal keyword list when building query\n  * [Ecto.Query] Do not remove literals from combinations, otherwise UNION/INTERSECTION queries may not match the number of values in `select`\n  * [Ecto.Query] Do not attempt to merge at compile-time non-keyword lists given to `select_merge`\n  * [Ecto.Repo] Do not override `:through` associations on preload unless forcing\n  * [Ecto.Repo] Make sure prefix option cascades to combinations and recursive queries\n  * [Ecto.Schema] Use OS time without drift when generating timestamps\n  * [Ecto.Type] Allow any datetime in `datetime_add`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.7 (2019-06-27) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-7-2019-06-27"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Make sure `put_assoc` with empty changeset propagates on insert","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.6 (2019-06-19) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-6-2019-06-19"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Repo] Add `:read_only` repositories\n  * [Ecto.Schema] Also validate options given to `:through` associations\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Do not mark `put_assoc` from `[]` to `[]` or from `nil` to `nil` as change\n  * [Ecto.Query] Remove named binding when excluding joins\n  * [mix ecto.gen.repo] Use `:config_path` instead of hardcoding to `config/config.exs`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.5 (2019-06-06) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-5-2019-06-06"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Repo] Allow `:default_dynamic_repo` option on `use Ecto.Repo`\n  * [Ecto.Schema] Support `{:fragment, ...}` in the `:where` option for associations\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Fix handling of literals in combinators (union, except, intersection)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.4 (2019-05-07) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-4-2019-05-07"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Convert validation enums to lists before adding them as validation metadata\n  * [Ecto.Schema] Properly propagate prefix to join_through source in many_to_many associations","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.3 (2019-04-30) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-3-2019-04-30"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Expose the enum that was validated against in errors from enum-based validations","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.1.2 (2019-04-24) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-2-2019-04-24"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Add support for `type+over`\n  * [Ecto.Schema] Allow schema fields to be excluded from queries\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Do not list a field as changed if it is updated to its original value\n  * [Ecto.Query] Keep literal numbers and bitstring in subqueries and unions\n  * [Ecto.Query] Improve error message for invalid `type/2` expression\n  * [Ecto.Query] Properly count interpolations in `select_merge/2`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.1 (2019-04-04) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-1-1-2019-04-04"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto] Do not require Jason (i.e. it should continue to be an optional dependency)\n  * [Ecto.Repo] Make sure `many_to_many` and `Ecto.Multi` work with dynamic repos","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.0 (2019-04-02) - Changelog for v3.x","doc":"v3.1 requires Elixir v1.5+.\n\n#","ref":"changelog.html#v3-1-0-2019-04-02"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add `not_equal_to` option for `validate_number`\n  * [Ecto.Query] Improve error message for missing `fragment` arguments\n  * [Ecto.Query] Improve error message on missing struct key for structs built in `select`\n  * [Ecto.Query] Allow dynamic named bindings\n  * [Ecto.Repo] Add dynamic repository support with `Ecto.Repo.put_dynamic_repo/1` and `Ecto.Repo.get_dynamic_repo/0` (experimental)\n  * [Ecto.Type] Cast naive_datetime/utc_datetime strings without seconds\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Changeset] Do not run `unsafe_validate_unique` query unless relevant fields were changed\n  * [Ecto.Changeset] Raise if an unknown field is given on `Ecto.Changeset.change/2`\n  * [Ecto.Changeset] Expose the type that was validated in errors generated by `validate_length/3`\n  * [Ecto.Query] Add support for `field/2` as first element of `type/2` and alias as second element of `type/2`\n  * [Ecto.Query] Do not attempt to assert types of named bindings that are not known at compile time\n  * [Ecto.Query] Properly cast boolean expressions in select\n  * [Mix.Ecto] Load applications during repo lookup so their app environment is available\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [Ecto.LogEntry] Fully deprecate previously soft deprecated API","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.0.7 (2019-02-06) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-7-2019-02-06"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] `reverse_order` reverses by primary key if no order is given","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.6 (2018-12-31) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-6-2018-12-31"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Query] Add `reverse_order/1`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Multi] Raise better error message on accidental rollback inside `Ecto.Multi`\n  * [Ecto.Query] Properly merge deeply nested preloaded joins\n  * [Ecto.Query] Raise better error message on missing select on schemaless queries\n  * [Ecto.Schema] Fix parameter ordering in assoc `:where`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.5 (2018-12-08) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-5-2018-12-08"},{"type":"extras","title":"Backwards incompatible changes - Changelog for v3.x","doc":"* [Ecto.Schema] The `:where` option added in Ecto 3.0.0 had a major flaw and it has been reworked in this version. This means a tuple of three elements can no longer be passed to `:where`, instead a keyword list must be given. Check the \"Filtering associations\" section in `has_many/3` docs for more information\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Do not raise on lists of tuples that are not keywords. Instead, let custom Ecto.Type handle them\n  * [Ecto.Query] Allow `prefix: nil` to be given to subqueries\n  * [Ecto.Query] Use different cache keys for unions/intersections/excepts\n  * [Ecto.Repo] Fix support for upserts with `:replace` without a schema\n  * [Ecto.Type] Do not lose precision when casting `utc_datetime_usec` with a time zone different than Etc/UTC","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.4 (2018-11-29) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-4-2018-11-29"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Decimal] Bump decimal dependency\n  * [Ecto.Repo] Remove unused `:pool_timeout`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.0.3 (2018-11-20) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-3-2018-11-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Changeset] Add `count: :bytes` option in `validate_length/3`\n  * [Ecto.Query] Support passing `Ecto.Query` in `Ecto.Repo.insert_all`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Type] Respect adapter types when loading/dumping arrays and maps\n  * [Ecto.Query] Ensure no bindings in order_by when using combinations in `Ecto.Query`\n  * [Ecto.Repo] Ensure adapter is compiled (instead of only loaded) before invoking it\n  * [Ecto.Repo] Support new style child spec from adapters","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.2 (2018-11-17) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-2-2018-11-17"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.LogEntry] Bring old Ecto.LogEntry APIs back for compatibility\n  * [Ecto.Repo] Consider non-joined fields when merging preloaded assocs only at root\n  * [Ecto.Repo] Take field sources into account in :replace_all_fields upsert option\n  * [Ecto.Type] Convert `:utc_datetime` to `DateTime` when sending it to adapters","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.1 (2018-11-03) - Changelog for v3.x","doc":"#","ref":"changelog.html#v3-0-1-2018-11-03"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Query] Ensure parameter order is preserved when using more than 32 parameters\n  * [Ecto.Query] Consider query prefix when planning association joins\n  * [Ecto.Repo] Consider non-joined fields as unique parameters when merging preloaded query assocs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.0 (2018-10-29) - Changelog for v3.x","doc":"Note this version includes changes from `ecto` and `ecto_sql` but in future releases all `ecto_sql` entries will be listed in their own CHANGELOG.\n\n#","ref":"changelog.html#v3-0-0-2018-10-29"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Adapters.MySQL] Add ability to specify cli_protocol for `ecto.create` and `ecto.drop` commands\n  * [Ecto.Adapters.PostgreSQL] Add ability to specify maintenance database name for PostgreSQL adapter for `ecto.create` and `ecto.drop` commands\n  * [Ecto.Changeset] Store constraint name in error metadata for constraints\n  * [Ecto.Changeset] Add `validations/1` and `constraints/1` instead of allowing direct access on the struct fields\n  * [Ecto.Changeset] Add `:force_update` option when casting relations, to force an update even if there are no changes\n  * [Ecto.Migration] Migrations now lock the migrations table in order to avoid concurrent migrations in a cluster. The type of lock can be configured via the `:migration_lock` repository configuration and defaults to \"FOR UPDATE\" or disabled if set to nil\n  * [Ecto.Migration] Add `:migration_default_prefix` repository configuration\n  * [Ecto.Migration] Add reversible version of `remove/2` subcommand\n  * [Ecto.Migration] Add support for non-empty arrays as defaults in migrations\n  * [Ecto.Migration] Add support for logging notices/alerts/warnings when running migrations (only supported by Postgres currently)\n  * [Ecto.Migrator] Warn when migrating and there is a higher version already migrated in the database\n  * [Ecto.Multi] Add support for anonymous functions in `insert/4`, `update/4`, `insert_or_update/4`, and `delete/4`\n  * [Ecto.Query] Support tuples in `where` and `having`, allowing queries such as `where: {p.foo, p.bar} > {^foo, ^bar}`\n  * [Ecto.Query] Support arithmetic operators in queries as a thin layer around the DB functionality\n  * [Ecto.Query] Allow joins in queries to be named via `:as` and allow named bindings\n  * [Ecto.Query] Support excluding specific join types in `exclude/2`\n  * [Ecto.Query] Allow virtual field update in subqueries\n  * [Ecto.Query] Support `coalesce/2` in queries, such as `select: coalesce(p.title, p.old_title)`\n  * [Ecto.Query] Support `filter/2` in queries, such as `select: filter(count(p.id), p.public == true)`\n  * [Ecto.Query] The `:prefix` and `:hints` options are now supported on both `from` and `join` expressions\n  * [Ecto.Query] Support `:asc_nulls_last`, `:asc_nulls_first`, `:desc_nulls_last`, and `:desc_nulls_first` in `order_by`\n  * [Ecto.Query] Allow variables (sources) to be given in queries, for example, useful for invoking functions, such as `fragment(\"some_function(?)\", p)`\n  * [Ecto.Query] Add support for `union`, `union_all`, `intersection`, `intersection_all`, `except` and `except_all`\n  * [Ecto.Query] Add support for `windows` and `over`\n  * [Ecto.Query] Raise when comparing a string with a charlist during planning\n  * [Ecto.Repo] Only start transactions if an association or embed has changed, this reduces the overhead during repository operations\n  * [Ecto.Repo] Support `:replace_all_except_primary_key` as `:on_conflict` strategy\n  * [Ecto.Repo] Support `{:replace, fields}` as `:on_conflict` strategy\n  * [Ecto.Repo] Support `:unsafe_fragment` as `:conflict_target`\n  * [Ecto.Repo] Support `select` in queries given to `update_all` and `delete_all`\n  * [Ecto.Repo] Add `Repo.exists?/2`\n  * [Ecto.Repo] Add `Repo.checkout/2` - useful when performing multiple operations in short-time to interval, allowing the pool to be bypassed\n  * [Ecto.Repo] Add `:stale_error_field` to `Repo.insert/update/delete` that converts `Ecto.StaleEntryError` into a changeset error. The message can also be set with `:stale_error_message`\n  * [Ecto.Repo] Preloading now only sorts results by the relationship key instead of sorting by the whole struct\n  * [Ecto.Schema] Allow `:where` option to be given to `has_many`/`has_one`/`belongs_to`/`many_to_many`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Inspect] Do not fail when inspecting query expressions which have a number of bindings more than bindings available\n  * [Ecto.Migration] Keep double underscores on autogenerated index names to be consistent with changesets\n  * [Ecto.Query] Fix `Ecto.Query.API.map/2` for single nil column with join\n  * [Ecto.Migration] Ensure `create_if_not_exists` is properly reversible\n  * [Ecto.Repo] Allow many_to_many associations to be preloaded via a function (before the behaviour was erratic)\n  * [Ecto.Schema] Make autogen ID loading work with custom type\n  * [Ecto.Schema] Make `updated_at` have the same value as `inserted_at`\n  * [Ecto.Schema] Ensure all fields are replaced with `on_conflict: :replace_all/:replace_all_except_primary_key` and not only the fields sent as changes\n  * [Ecto.Type] Return `:error` when casting NaN or infinite decimals\n  * [mix ecto.migrate] Properly run migrations after ECTO_EDITOR changes\n  * [mix ecto.migrations] List migrated versions even if the migration file is deleted\n  * [mix ecto.load] The task now fails on SQL errors on Postgres\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"Although Ecto 3.0 is a major bump version, the functionality below emits deprecation warnings to ease the migration process. The functionality below will be removed in future Ecto 3.1+ releases.\n\n  * [Ecto.Changeset] Passing a list of binaries to `cast/3` is deprecated, please pass a list of atoms instead\n  * [Ecto.Multi] `Ecto.Multi.run/3` now receives the repo in which the transaction is executing as the first argument to functions, and the changes so far as the second argument\n  * [Ecto.Query] `join/5` now expects `on: expr` as last argument instead of simply `expr`. This was done in order to properly support the `:as`, `:hints` and `:prefix` options\n  * [Ecto.Repo] The `:returning` option for `update_all` and `delete_all` has been deprecated as those statements now support `select` clauses\n  * [Ecto.Repo] Passing `:adapter` via config is deprecated in favor of passing it on `use Ecto.Repo`\n  * [Ecto.Repo] The `:loggers` configuration is deprecated in favor of \"Telemetry Events\"\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Backwards incompatible changes - Changelog for v3.x","doc":"* [Ecto.DateTime] `Ecto.Date`, `Ecto.Time` and `Ecto.DateTime` were previously deprecated and have now been removed\n  * [Ecto.DataType] `Ecto.DataType` protocol has been removed\n  * [Ecto.Migration] Automatically inferred index names may differ in Ecto v3.0 for indexes on complex column names\n  * [Ecto.Multi] `Ecto.Multi.run/5` now receives the repo in which the transaction is executing as the first argument to functions, and the changes so far as the second argument\n  * [Ecto.Query] A `join` no longer wraps `fragment` in parentheses. In some cases, such as common table expressions, you will have to explicitly wrap the fragment in parens.\n  * [Ecto.Repo] The `on_conflict: :replace_all` option now will also send fields with default values to the database. If you prefer the old behaviour that only sends the changes in the changeset, you can set it to `on_conflict: {:replace, Map.keys(changeset.changes)}` (this change is also listed as a bug fix)\n  * [Ecto.Repo] The repository operations are no longer called from association callbacks - this behaviour was not guaranteed in previous versions but we are listing as backwards incompatible changes to help with users relying on this behaviour\n  * [Ecto.Repo] `:pool_timeout` is no longer supported in favor of a new queue system described in `DBConnection.start_link/2` under \"Queue config\". For most users, configuring `:timeout` is enough, as it now includes both queue and query time\n  * [Ecto.Schema] `:time`, `:naive_datetime` and `:utc_datetime` no longer keep microseconds information. If you want to keep microseconds, use `:time_usec`, `:naive_datetime_usec`, `:utc_datetime_usec`\n  * [Ecto.Schema] The `@schema_prefix` option now only affects the `from`/`join` of where the schema is used and no longer the whole query\n  * [Ecto.Schema.Metadata] The `source` key no longer returns a tuple of the schema_prefix and the table/collection name. It now returns just the table/collection string. You can now access the schema_prefix via the `prefix` key.\n  * [Mix.Ecto] `Mix.Ecto.ensure_started/2` has been removed. However, in Ecto 2.2 the  `Mix.Ecto` module was not considered part of the public API and should not have been used but we are listing this for guidance.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Adapter changes - Changelog for v3.x","doc":"* [Ecto.Adapter] Split `Ecto.Adapter` into `Ecto.Adapter.Queryable` and `Ecto.Adapter.Schema` to provide more granular repository APIs\n  * [Ecto.Adapter] The `:sources` field in `query_meta` now contains three elements tuples with `{source, schema, prefix}` in order to support `from`/`join` prefixes (#2572)\n  * [Ecto.Adapter] The database types `time`, `utc_datetime` and `naive_datetime` should translate to types with seconds precision while the database types `time_usec`, `utc_datetime_usec` and `naive_datetime_usec` should have microseconds precision (#2291)\n  * [Ecto.Adapter] The `on_conflict` argument for `insert` and `insert_all` no longer receives a `{:replace_all, list(), atom()}` tuple. Instead, it receives a `{fields :: [atom()], list(), atom()}` where `fields` is a list of atoms of the fields to be replaced (#2181)\n  * [Ecto.Adapter] `insert`/`update`/`delete` now receive both `:source` and `:prefix` fields instead of a single `:source` field with both `source` and `prefix` in it (#2490)\n  * [Ecto.Adapter.Migration] A new `lock_for_migration/4` callback has been added. It is implemented by default by `Ecto.Adapters.SQL` (#2215)\n  * [Ecto.Adapter.Migration] The `execute_ddl` should now return `{:ok, []}` to make space for returning notices/hints/warnings in the future (adapters leveraging `Ecto.Adapters.SQL` do not have to perform any change)\n  * [Ecto.Query] The `from` field in `Ecto.Query` now returns a `Ecto.Query.FromExpr` with the `:source` field, unifying the behaviour in `from` and `join` expressions (#2497)\n  * [Ecto.Query] Tuple expressions are now supported in queries. For example, `where: {p.foo, p.bar} > {p.bar, p.baz}` should translate to `WHERE (p.foo, p.bar) > (p.bar, p.baz)` in SQL databases. Adapters should be changed to handle `{:{}, meta, exprs}` in the query AST (#2344)\n  * [Ecto.Query] Adapters should support the following arithmetic operators in queries `+`, `-`, `*` and `/` (#2400)\n  * [Ecto.Query] Adapters should support `filter/2` in queries, as in `select: filter(count(p.id), p.public == true)` (#2487)","ref":"changelog.html#adapter-changes"},{"type":"extras","title":"Previous versions - Changelog for v3.x","doc":"* See the CHANGELOG.md [in the v2.2 branch](https://github.com/elixir-ecto/ecto/blob/v2.2/CHANGELOG.md)","ref":"changelog.html#previous-versions"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThis guide is an introduction to [Ecto](https://github.com/elixir-lang/ecto),\nthe database wrapper and query generator for Elixir. Ecto provides a\nstandardized API and a set of abstractions for talking to all the different\nkinds of databases, so that Elixir developers can query whatever database\nthey're using by employing similar constructs.\n\nIn this guide, we're going to learn some basics about Ecto, such as creating,\nreading, updating and destroying records from a PostgreSQL database. If you want\nto see the code from this guide, you can view it [at ecto/examples/friends on GitHub](https://github.com/elixir-lang/ecto/tree/master/examples/friends).\n\n**This guide will require you to have setup PostgreSQL beforehand.**","ref":"getting-started.html"},{"type":"extras","title":"Adding Ecto to an application - Getting Started","doc":"To start off with, we'll generate a new Elixir application by running this command:\n\n```\nmix new friends --sup\n```\n\nThe `--sup` option ensures that this application has [a supervision tree](https://elixir-lang.org/getting-started/mix-otp/supervisor-and-application.html), which we'll need for Ecto a little later on.\n\nTo add Ecto to this application, there are a few steps that we need to take. The first step will be adding Ecto and a driver called Postgrex to our `mix.exs` file, which we'll do by changing the `deps` definition in that file to this:\n\n```elixir\ndefp deps do\n  [\n    {:ecto_sql, \"~> 3.0\"},\n    {:postgrex, \">= 0.0.0\"}\n  ]\nend\n```\n\nEcto provides the common querying API, but we need the Postgrex driver installed too, as that is what Ecto uses to speak in terms a PostgreSQL database can understand. Ecto talks to its own `Ecto.Adapters.Postgres` module, which then in turn talks to the `postgrex` package to talk to PostgreSQL.\n\nTo install these dependencies, we will run this command:\n\n```\nmix deps.get\n```\n\nThe Postgrex application will receive queries from Ecto and execute them\nagainst our database. If we didn't do this step, we wouldn't be able to do any\nquerying at all.\n\nThat's the first two steps taken now. We have installed Ecto and Postgrex as\ndependencies of our application. We now need to setup some configuration for\nEcto so that we can perform actions on a database from within the\napplication's code.\n\nWe can set up this configuration by running this command:\n\n```\nmix ecto.gen.repo -r Friends.Repo\n```\n\nThis command will generate the configuration required to connect to a database. The first bit of configuration is in `config/config.exs`:\n\n```elixir\nconfig :friends, Friends.Repo,\n  database: \"friends\",\n  username: \"user\",\n  password: \"pass\",\n  hostname: \"localhost\"\n```\n\n**NOTE**: Your PostgreSQL database may be setup to\n\n - not require a username and password. If the above configuration doesn't work, try removing the username and password fields, or setting them both to \"postgres\".\n - be running on a non-standard port. The default port is `5432`. You can specify your specific port by adding it to the config: e.g. `port: 15432`.\n\nThis configures how Ecto will connect to our database, called \"friends\". Specifically, it configures a \"repo\". More information about [Ecto.Repo can be found in its documentation](https://hexdocs.pm/ecto/Ecto.Repo.html).\n\nThe `Friends.Repo` module is defined in `lib/friends/repo.ex` by our `mix ecto.gen.repo` command:\n\n```elixir\ndefmodule Friends.Repo do\n  use Ecto.Repo,\n    otp_app: :friends,\n    adapter: Ecto.Adapters.Postgres\nend\n```\n\nThis module is what we'll be using to query our database shortly. It uses the `Ecto.Repo` module, and the `otp_app` tells Ecto which Elixir application it can look for database configuration in. In this case, we've specified that it is the `:friends` application where Ecto can find that configuration and so Ecto will use the configuration that was set up in `config/config.exs`. Finally, we configure the database `:adapter` to Postgres.\n\nThe final piece of configuration is to setup the `Friends.Repo` as a supervisor within the application's supervision tree, which we can do in `lib/friends/application.ex`, inside the `start/2` function:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Friends.Repo,\n  ]\n\n  ...\n```\n\nThis piece of configuration will start the Ecto process which receives and executes our application's queries. Without it, we wouldn't be able to query the database at all!\n\nThere's one final bit of configuration that we'll need to add ourselves, since the generator does not add it. Underneath the configuration in `config/config.exs`, add this line:\n\n```elixir\nconfig :friends, ecto_repos: [Friends.Repo]\n```\n\nThis tells our application about the repo, which will allow us to run commands such as `mix ecto.create` very soon.\n\nWe've now configured our application so that it's able to make queries to our database. Let's now create our database, add a table to it, and then perform some queries.","ref":"getting-started.html#adding-ecto-to-an-application"},{"type":"extras","title":"Setting up the database - Getting Started","doc":"To be able to query a database, it first needs to exist. We can create the database with this command:\n\n```\nmix ecto.create\n```\n\nIf the database has been created successfully, then you will see this message:\n\n```\nThe database for Friends.Repo has been created.\n```\n\n**NOTE:** If you get an error, you should try changing your configuration in `config/config.exs`, as it may be an authentication error.\n\nA database by itself isn't very queryable, so we will need to create a table within that database. To do that, we'll use what's referred to as a _migration_. If you've come from Active Record (or similar), you will have seen these before. A migration is a single step in the process of constructing your database.\n\nLet's create a migration now with this command:\n\n```\nmix ecto.gen.migration create_people\n```\n\nThis command will generate a brand new migration file in `priv/repo/migrations`, which is empty by default:\n\n```elixir\ndefmodule Friends.Repo.Migrations.CreatePeople do\n  use Ecto.Migration\n\n  def change do\n\n  end\nend\n```\n\nLet's add some code to this migration to create a new table called \"people\", with a few columns in it:\n\n```elixir\ndefmodule Friends.Repo.Migrations.CreatePeople do\n  use Ecto.Migration\n\n  def change do\n    create table(:people) do\n      add :first_name, :string\n      add :last_name, :string\n      add :age, :integer\n    end\n  end\nend\n```\n\nThis new code will tell Ecto to create a new table called `people`, and add three new fields: `first_name`, `last_name` and `age` to that table. The types of these fields are `string` and `integer`. (The different types that Ecto supports are covered in the [Ecto.Schema](https://hexdocs.pm/ecto/Ecto.Schema.html) documentation.)\n\n**NOTE**: The naming convention for tables in Ecto databases is to use a pluralized name.\n\nTo run this migration and create the `people` table in our database, we will run this command:\n\n```\nmix ecto.migrate\n```\n\nIf we found out that we made a mistake in this migration, we could run `mix ecto.rollback` to undo the changes in the migration. We could then fix the changes in the migration and run `mix ecto.migrate` again. If we ran `mix ecto.rollback` now, it would delete the table that we just created.\n\nWe now have a table created in our database. The next step that we'll need to do is to create the schema.","ref":"getting-started.html#setting-up-the-database"},{"type":"extras","title":"Creating the schema - Getting Started","doc":"The schema is an Elixir representation of data from our database. Schemas are commonly associated with a database table, however they can be associated with a database view as well.\n\nLet's create the schema within our application at `lib/friends/person.ex`:\n\n```elixir\ndefmodule Friends.Person do\n  use Ecto.Schema\n\n  schema \"people\" do\n    field :first_name, :string\n    field :last_name, :string\n    field :age, :integer\n  end\nend\n```\n\nThis defines the schema from the database that this schema maps to. In this case, we're telling Ecto that the `Friends.Person` schema maps to the `people` table in the database, and the `first_name`, `last_name` and `age` fields in that table. The second argument passed to `field` tells Ecto how we want the information from the database to be represented in our schema.\n\nWe've called this schema `Person` because the naming convention in Ecto for schemas is a singularized name.\n\nWe can play around with this schema in an IEx session by starting one up with `iex -S mix` and then running this code in it:\n\n```elixir\nperson = %Friends.Person{}\n```\n\nThis code will give us a new `Friends.Person` struct, which will have `nil` values for all the fields. We can set values on these fields by generating a new struct:\n\n```elixir\nperson = %Friends.Person{age: 28}\n```\n\nOr with syntax like this:\n\n```elixir\nperson = %{person | age: 28}\n```\n\nWe can retrieve values using this syntax:\n\n```elixir\nperson.age # => 28\n```\n\nLet's take a look at how we can insert data into the database.","ref":"getting-started.html#creating-the-schema"},{"type":"extras","title":"Inserting data - Getting Started","doc":"We can insert a new record into our `people` table with this code:\n\n```elixir\nperson = %Friends.Person{}\nFriends.Repo.insert(person)\n```\n\nTo insert the data into our database, we call `insert` on `Friends.Repo`, which is the module that uses Ecto to talk to our database. This function tells Ecto that we want to insert a new `Friends.Person` record into the database corresponding with `Friends.Repo`. The `person` struct here represents the data that we want to insert into the database.\n\nA successful insertion will return a tuple, like so:\n\n```elixir\n{:ok,\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: nil,\n  first_name: nil, id: 1, last_name: nil}}\n```\n\nThe `:ok` atom can be used for pattern matching purposes to ensure that the insertion succeeds. A situation where the insertion may not succeed is if you have a constraint on the database itself. For instance, if the database had a unique constraint on a field called `email` so that an email can only be used for one person record, then the insertion would fail.\n\nYou may wish to pattern match on the tuple in order to refer to the record inserted into the database:\n\n```elixir\n{:ok, person} = Friends.Repo.insert person\n```","ref":"getting-started.html#inserting-data"},{"type":"extras","title":"Validating changes - Getting Started","doc":"In Ecto, you may wish to validate changes before they go to the database. For instance, you may wish that a person has both a first name and a last name before a record can be entered into the database. For this, Ecto has [_changesets_](https://hexdocs.pm/ecto/Ecto.Changeset.html).\n\nLet's add a changeset to our `Friends.Person` module inside `lib/friends/person.ex` now:\n\n```elixir\ndef changeset(person, params \\\\ %{}) do\n  person\n  |> Ecto.Changeset.cast(params, [:first_name, :last_name, :age])\n  |> Ecto.Changeset.validate_required([:first_name, :last_name])\nend\n```\n\nThis changeset takes a `person` and a set of params, which are to be the changes to apply to this person. The `changeset` function first casts the `first_name`, `last_name` and `age` keys from the parameters passed in to the changeset. Casting tells the changeset what parameters are allowed to be passed through in this changeset, and anything not in the list will be ignored.\n\nOn the next line, we call `validate_required` which says that, for this changeset, we expect `first_name` and `last_name` to have values specified. Let's use this changeset to attempt to create a new record without a `first_name` and `last_name`:\n\n```elixir\nperson = %Friends.Person{}\nchangeset = Friends.Person.changeset(person, %{})\nFriends.Repo.insert(changeset)\n```\n\nOn the first line here, we get a struct from the `Friends.Person` module. We know what that does, because we saw it not too long ago. On the second line we do something brand new: we define a changeset. This changeset says that on the specified `person` object, we're looking to make some changes. In this case, we're not looking to change anything at all.\n\nOn the final line, rather than inserting the `person`, we insert the `changeset`. The `changeset` knows about the `person`, the changes and the validation rules that must be met before the data can be entered into the database. When this third line runs, we'll see this:\n\n```elixir\n{:error,\n #Ecto.Changeset , valid?: false>}\n```\n\nJust like the last time we did an insertion, this returns a tuple. This time however, the first element in the tuple is `:error`, which indicates something bad happened. The specifics of what happened are included in the changeset which is returned. We can access these by doing some pattern matching:\n\n```elixir\n{:error, changeset} = Friends.Repo.insert(changeset)\n```\n\nThen we can get to the errors by doing `changeset.errors`:\n\n```elixir\n[first_name: {\"can't be blank\", [validation: :required]}, last_name: {\"can't be blank\", [validation: :required]}]\n```\n\nAnd we can ask the changeset itself if it is valid, even before doing an insertion:\n\n```elixir\nchangeset.valid?\n#=> false\n```\n\nSince this changeset has errors, no new record was inserted into the `people`\ntable.\n\nLet's try now with some valid data.\n\n```elixir\nperson = %Friends.Person{}\nchangeset = Friends.Person.changeset(person, %{first_name: \"Ryan\", last_name: \"Bigg\"})\n```\n\nWe start out here with a normal `Friends.Person` struct. We then create a changeset for that `person` which has a `first_name` and a `last_name` parameter specified. At this point, we can ask the changeset if it has errors:\n\n```elixir\nchangeset.errors\n#=> []\n```\n\nAnd we can ask if it's valid or not:\n\n```elixir\nchangeset.valid?\n#=> true\n```\n\nThe changeset does not have errors, and is valid. Therefore if we try to insert this changeset it will work:\n\n```elixir\nFriends.Repo.insert(changeset)\n#=> {:ok,\n     %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: nil,\n      first_name: \"Ryan\", id: 3, last_name: \"Bigg\"}}\n```\n\n\nDue to `Friends.Repo.insert` returning a tuple, we can use a `case` to determine different code paths depending on what happens:\n\n```elixir\ncase Friends.Repo.insert(changeset) do\n  {:ok, person} ->\n    # do something with person\n  {:error, changeset} ->\n    # do something with changeset\nend\n```\n\n**NOTE:** `changeset.valid?` will not check constraints (such as `uniqueness_constraint`). For that, you will need to attempt to do an insertion and check for errors from the database. It's for this reason it's best practice to try inserting data and validate the returned tuple from `Friends.Repo.insert` to get the correct errors, as prior to insertion the changeset will only contain validation errors from the application itself.\n\nIf the insertion of the changeset succeeds, then you can do whatever you wish with the `person` returned in that result. If it fails, then you have access to the changeset and its errors. In the failure case, you may wish to present these errors to the end user. The errors in the changeset are a keyword list that looks like this:\n\n\n```elixir\n[first_name: {\"can't be blank\", [validation: :required]},\n last_name: {\"can't be blank\", [validation: :required]}]\n```\n\nThe first element of the tuple is the validation message, and the second element is a keyword list of options for the validation message. Imagine that we had a field called `bio` that we were validating, and that field has to be longer than 15 characters. This is what would be returned:\n\n\n```elixir\n[first_name: {\"can't be blank\", [validation: :required]},\n last_name: {\"can't be blank\", [validation: :required]},\n bio: {\"should be at least %{count} character(s)\", [count: 15, validation: :length, kind: :min, type: :string]}]\n```\n\nTo display these error messages in a human friendly way, we can use `Ecto.Changeset.traverse_errors/2`:\n\n```elixir\ntraverse_errors(changeset, fn {msg, opts} ->\n  Enum.reduce(opts, msg, fn {key, value}, acc ->\n    String.replace(acc, \"%{#{key}}\", to_string(value))\n  end)\nend)\n```\n\nThis will return the following for the errors shown above:\n\n```elixir\n%{\n  first_name: [\"can't be blank\"],\n  last_name: [\"can't be blank\"],\n  bio: [\"should be at least 15 character(s)\"],\n}\n```\n\nOne more final thing to mention here: you can trigger an exception to be thrown by using `Friends.Repo.insert!/2`. If a changeset is invalid, you will see an `Ecto.InvalidChangesetError` exception. Here's a quick example of that:\n\n```\nFriends.Repo.insert! Friends.Person.changeset(%Friends.Person{}, %{first_name: \"Ryan\"})\n\n** (Ecto.InvalidChangesetError) could not perform insert because changeset is invalid.\n\nErrors\n\n    %{last_name: [{\"can't be blank\", [validation: :required]}]}\n\nApplied changes\n\n    %{first_name: \"Ryan\"}\n\nParams\n\n    %{\"first_name\" => \"Ryan\"}\n\nChangeset\n\n    #Ecto.Changeset ,\n      valid?: false\n    >\n\n   (ecto) lib/ecto/repo/schema.ex:257: Ecto.Repo.Schema.insert!/4\n```\n\nThis exception shows us the changes from the changeset, and how the changeset is invalid. This can be useful if you want to insert a bunch of data and then have an exception raised if that data is not inserted correctly at all.\n\nNow that we've covered inserting data into the database, let's look at how we can pull that data back out.","ref":"getting-started.html#validating-changes"},{"type":"extras","title":"Our first queries - Getting Started","doc":"Querying a database requires two steps in Ecto. First, we must construct the query and then we must execute that query against the database by passing the query to the repository. Before we do this, let's re-create the database for our app and setup some test data. To re-create the database, we'll run these commands:\n\n```\nmix ecto.drop\nmix ecto.create\nmix ecto.migrate\n```\n\nThen to create the test data, we'll run this in an `iex -S mix` session:\n\n```elixir\npeople = [\n  %Friends.Person{first_name: \"Ryan\", last_name: \"Bigg\", age: 28},\n  %Friends.Person{first_name: \"John\", last_name: \"Smith\", age: 27},\n  %Friends.Person{first_name: \"Jane\", last_name: \"Smith\", age: 26},\n]\n\nEnum.each(people, fn (person) -> Friends.Repo.insert(person) end)\n```\n\nThis code will create three new people in our database, Ryan, John and Jane. Note here that we could've used a changeset to validate the data going into the database, but the choice was made not to use one.\n\nWe'll be querying for these people in this section. Let's jump in!\n\n#","ref":"getting-started.html#our-first-queries"},{"type":"extras","title":"Fetching a single record - Getting Started","doc":"Let's start off with fetching just one record from our `people` table:\n\n```elixir\nFriends.Person |> Ecto.Query.first\n```\n\nThat code will generate an `Ecto.Query`, which will be this:\n\n```\n#Ecto.Query \n```\n\nThe code between the angle brackets ` ` here shows the Ecto query which has been constructed. We could construct this query ourselves with almost exactly the same syntax:\n\n```elixir\nrequire Ecto.Query\nEcto.Query.from p in Friends.Person, order_by: [asc: p.id], limit: 1\n```\n\nWe need to `require Ecto.Query` here to enable the macros from that module. Then it's a matter of calling the `from` function from `Ecto.Query` and passing in the code from between the angle brackets. As we can see here, `Ecto.Query.first` saves us from having to specify the `order` and `limit` for the query.\n\nTo execute the query that we've just constructed, we can call `Friends.Repo.one`:\n\n```elixir\nFriends.Person |> Ecto.Query.first |> Friends.Repo.one\n```\n\nThe `one` function retrieves just one record from our database and returns a new struct from the `Friends.Person` module:\n\n```elixir\n%Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 28,\n first_name: \"Ryan\", id: 1, last_name: \"Bigg\"}\n```\n\nSimilar to `first`, there is also `last`:\n\n```elixir\nFriends.Person |> Ecto.Query.last |> Friends.Repo.one\n#=> %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 26,\n     first_name: \"Jane\", id: 3, last_name: \"Smith\"}\n ```\n\nThe `Ecto.Repo.one` function will only return a struct if there is one record in the\nresult from the database. If there is more than one record returned, an\n`Ecto.MultipleResultsError` exception will be thrown. Some code that would\ncause that issue to happen is:\n\n```elixir\nFriends.Person |> Friends.Repo.one\n```\n\nWe've left out the `Ecto.Query.first` here, and so there is no `limit` or `order` clause applied to the executed query. We'll see the executed query in the debug log:\n\n```\n[timestamp] [debug] SELECT p0.\"id\", p0.\"first_name\", p0.\"last_name\", p0.\"age\" FROM \"people\" AS p0 [] OK query=1.8ms\n```\n\nThen immediately after that, we will see the `Ecto.MultipleResultsError` exception:\n\n```\n** (Ecto.MultipleResultsError) expected at most one result but got 3 in query:\n\nfrom p in Friends.Person\n\n    lib/ecto/repo/queryable.ex:67: Ecto.Repo.Queryable.one/4\n```\n\nThis happens because Ecto doesn't know what one record out of all the records\nreturned that we want. Ecto will only return a result if we are explicit in\nour querying about which result we want.\n\nIf there is no record which matches the query, `one` will return `nil`.\n\n#","ref":"getting-started.html#fetching-a-single-record"},{"type":"extras","title":"Fetching all records - Getting Started","doc":"To fetch all records from the schema, Ecto provides the `all` function:\n\n```elixir\nFriends.Person |> Friends.Repo.all\n```\n\nThis will return a `Friends.Person` struct representation of all the records that currently exist within our `people` table:\n\n```elixir\n[%Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 28,\n  first_name: \"Ryan\", id: 1, last_name: \"Bigg\"},\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 27,\n  first_name: \"John\", id: 2, last_name: \"Smith\"},\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 26,\n  first_name: \"Jane\", id: 3, last_name: \"Smith\"}]\n```\n\n#","ref":"getting-started.html#fetching-all-records"},{"type":"extras","title":"Fetch a single record based on ID - Getting Started","doc":"To fetch a record based on its ID, you use the `get` function:\n\n```elixir\nFriends.Person |> Friends.Repo.get(1)\n#=> %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 28,\n     first_name: \"Ryan\", id: 1, last_name: \"Bigg\"}\n```\n\n#","ref":"getting-started.html#fetch-a-single-record-based-on-id"},{"type":"extras","title":"Fetch a single record based on a specific attribute - Getting Started","doc":"If we want to get a record based on something other than the `id` attribute, we can use `get_by`:\n\n```elixir\n Friends.Person |> Friends.Repo.get_by(first_name: \"Ryan\")\n #=> %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 28,\n      first_name: \"Ryan\", id: 1, last_name: \"Bigg\"}\n```\n\n#","ref":"getting-started.html#fetch-a-single-record-based-on-a-specific-attribute"},{"type":"extras","title":"Filtering results - Getting Started","doc":"If we want to get multiple records matching a specific attribute, we can use `where`:\n\n```elixir\nFriends.Person |> Ecto.Query.where(last_name: \"Smith\") |> Friends.Repo.all\n```\n\n```elixir\n[%Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 27,\n  first_name: \"John\", id: 2, last_name: \"Smith\"},\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 26,\n  first_name: \"Jane\", id: 3, last_name: \"Smith\"}]\n```\n\nIf we leave off the `Friends.Repo.all` on the end of this, we will see the query Ecto generates:\n\n```\n#Ecto.Query \n```\n\nWe can also use this query syntax to fetch these same records:\n\n```elixir\nEcto.Query.from(p in Friends.Person, where: p.last_name == \"Smith\") |> Friends.Repo.all\n```\n\nOne important thing to note with both query syntaxes is that they require variables to be pinned, using the pin operator (`^`). Otherwise, this happens:\n\n```elixir\nlast_name = \"Smith\"\nFriends.Person |> Ecto.Query.where(last_name: last_name) |> Friends.Repo.all\n```\n\n```\n** (Ecto.Query.CompileError) unbound variable `last_name` in query. If you are attempting to interpolate a value, use ^var\n    (ecto)   expanding macro: Ecto.Query.where/2\n             iex:15: (file)\n    (elixir) expanding macro: Kernel.|>/2\n             iex:15: (file)\n```\n\nThe same will happen in the longer query syntax too:\n\n```elixir\nEcto.Query.from(p in Friends.Person, where: p.last_name == last_name) |> Friends.Repo.all\n```\n\n```\n** (Ecto.Query.CompileError) unbound variable `last_name` in query. If you are attempting to interpolate a value, use ^var\n    (ecto)   expanding macro: Ecto.Query.where/3\n             iex:15: (file)\n    (ecto)   expanding macro: Ecto.Query.from/2\n             iex:15: (file)\n    (elixir) expanding macro: Kernel.|>/2\n             iex:15: (file)\n```\n\nTo get around this, we use the pin operator (`^`):\n\n```elixir\nlast_name = \"Smith\"\nFriends.Person |> Ecto.Query.where(last_name: ^last_name) |> Friends.Repo.all\n```\n\nOr:\n\n```elixir\nlast_name = \"Smith\"\nEcto.Query.from(p in Friends.Person, where: p.last_name == ^last_name) |> Friends.Repo.all\n```\n\nThe pin operator instructs the query builder to use parameterized SQL queries protecting against SQL injection.\n\n#","ref":"getting-started.html#filtering-results"},{"type":"extras","title":"Composing Ecto queries - Getting Started","doc":"Ecto queries don't have to be built in one spot. They can be built up by calling `Ecto.Query` functions on existing queries. For instance, if we want to find all people with the last name \"Smith\", we can do:\n\n```elixir\nquery = Friends.Person |> Ecto.Query.where(last_name: \"Smith\")\n```\n\nIf we want to scope this down further to only people with the first name of \"Jane\", we can do this:\n\n```elixir\nquery = query |> Ecto.Query.where(first_name: \"Jane\")\n```\n\nOur query will now have two `where` clauses in it:\n\n```\n#Ecto.Query \n```\n\nThis can be useful if you want to do something with the first query, and then build off that query later on.","ref":"getting-started.html#composing-ecto-queries"},{"type":"extras","title":"Updating records - Getting Started","doc":"Updating records in Ecto requires us to first fetch a record from the database. We then create a changeset from that record and the changes we want to make to that record, and then call the `Ecto.Repo.update` function.\n\nLet's fetch the first person from our database and change their age. First, we'll fetch the person:\n\n```elixir\nperson = Friends.Person |> Ecto.Query.first |> Friends.Repo.one\n```\n\nNext, we'll build a changeset. We need to build a changeset because if we just create a new `Friends.Person` struct with the new age, Ecto wouldn't be able to know that the age has changed without inspecting the database. Let's build that changeset:\n\n```elixir\nchangeset = Friends.Person.changeset(person, %{age: 29})\n```\n\nThis changeset will inform the database that we want to update the record to have the `age` set to 29. To tell the database about the change we want to make, we run this command:\n\n```elixir\nFriends.Repo.update(changeset)\n```\n\nJust like `Friends.Repo.insert`, `Friends.Repo.update` will return a tuple:\n\n```elixir\n{:ok,\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 29,\n  first_name: \"Ryan\", id: 1, last_name: \"Bigg\"}}\n```\n\nIf the changeset fails for any reason, the result of `Friends.Repo.update` will be `{:error, changeset}`. We can see this in action by passing through a blank `first_name` in our changeset's parameters:\n\n```elixir\nchangeset = Friends.Person.changeset(person, %{first_name: \"\"})\nFriends.Repo.update(changeset)\n#=> {:error,\n     #Ecto.Changeset ,\n       valid?: false\n     >}\n```\n\nThis means that you can also use a `case` statement to do different things depending on the outcome of the `update` function:\n\n```elixir\ncase Friends.Repo.update(changeset) do\n  {:ok, person} ->\n    # do something with person\n  {:error, changeset} ->\n    # do something with changeset\nend\n```\n\nSimilar to `insert!`, there is also `update!` which will raise an exception if the changeset is invalid:\n\n```\nchangeset = Friends.Person.changeset(person, %{first_name: \"\"})\nFriends.Repo.update! changeset\n\n** (Ecto.InvalidChangesetError) could not perform update because changeset is invalid.\n\nErrors\n\n  %{first_name: [{\"can't be blank\", [validation: :required]}]}\n\nApplied changes\n\n  %{}\n\nParams\n\n  %{\"first_name\" => \"\"}\n\nChangeset\n\n    #Ecto.Changeset ,\n      valid?: false\n    >\n\n    (ecto) lib/ecto/repo/schema.ex:270: Ecto.Repo.Schema.update!/4\n```","ref":"getting-started.html#updating-records"},{"type":"extras","title":"Deleting records - Getting Started","doc":"We've now covered creating (`insert`), reading (`get`, `get_by`, `where`) and updating records. The last thing that we'll cover in this guide is how to delete a record using Ecto.\n\nSimilar to updating, we must first fetch a record from the database and then call `Friends.Repo.delete` to delete that record:\n\n```elixir\nperson = Friends.Repo.get(Friends.Person, 1)\nFriends.Repo.delete(person)\n#=> {:ok,\n %Friends.Person{__meta__: #Ecto.Schema.Metadata , age: 29,\n  first_name: \"Ryan\", id: 2, last_name: \"Bigg\"}}\n```\n\nSimilar to `insert` and `update`, `delete` returns a tuple. If the deletion succeeds, then the first element in the tuple will be `:ok`, but if it fails then it will be an `:error`.","ref":"getting-started.html#deleting-records"},{"type":"extras","title":"Embedded Schemas","doc":"# Embedded Schemas\n\nEmbedded schemas allow you to define and validate structured data. This data can live in memory, or can be stored in the database. Some use cases for embedded schemas include:\n\n- You are maintaining intermediate-state data, like when UI form fields map onto multiple tables in a database.\n\n- You are working within a persisted parent schema and you want to embed data that is...\n\n  - simple, like a map of user preferences inside a User schema.\n  - changes often, like a list of product images with associated structured data inside a Product schema.\n  - requires complex tracking and validation, like an Address schema inside a User schema.\n\n- You are using a document storage database and you want to interact with and manipulate embedded documents.","ref":"embedded-schemas.html"},{"type":"extras","title":"User Profile Example - Embedded Schemas","doc":"Let's explore an example where we have a User and want to store \"profile\" information about them. The data we want to store is UI-dependent information which is likely to change over time alongside changes in the UI. Also, this data is not necessarily important enough to warrant new User `field`s in the User schema, as it is not data that is fundamental to the User. An embedded schema is a good solution for this kind of data.\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :full_name, :string\n    field :email, :string\n    field :avatar_url, :string\n    field :confirmed_at, :naive_datetime\n\n    embeds_one :profile, Profile do\n      field :online, :boolean\n      field :dark_mode, :boolean\n      field :visibility, Ecto.Enum, values: [:public, :private, :friends_only]\n    end\n\n    timestamps()\n  end\nend\n```\n\n#","ref":"embedded-schemas.html#user-profile-example"},{"type":"extras","title":"Embeds - Embedded Schemas","doc":"There are two ways to represent embedded data within a schema, `Ecto.Schema.embeds_many/3`, which creates a list of embeds, and `Ecto.Schema.embeds_one/3`, which creates only a single instance of the embed. Your choice here affects the behavior of embed-specific functions like `Ecto.Changeset.put_embed/4` and `Ecto.Changeset.cast_embed/3`, so choose whichever is most appropriate to your use case. In our example we are going to use `Ecto.Schema.embeds_one/3` since users will only ever have one profile associated with them.\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :full_name, :string\n    field :email, :string\n    field :avatar_url, :string\n    field :confirmed_at, :naive_datetime\n\n    embeds_one :profile, Profile do\n      field :online, :boolean\n      field :dark_mode, :boolean\n      field :visibility, Ecto.Enum, values: [:public, :private, :friends_only]\n    end\n\n    timestamps()\n  end\nend\n```\n\nEmbedded schemas defined in such way are said to be defined inline, which means that they are:\n\n- generated as a module in the parent scope with the appropriate struct (for the example above, the module will be `User.Profile`)\n- persisted within the parent schema\n- required to provide the `with` option to `Ecto.Changeset.cast_embed/3`\n\n#","ref":"embedded-schemas.html#embeds"},{"type":"extras","title":"Extracting the embeds - Embedded Schemas","doc":"While the above `User` schema is simple and sufficient, we might want to work independently with the embedded profile struct. For example, if there was a lot of functionality devoted solely to manipulating the profile data, we'd want to consider extracting the embedded schema into its own module. This can be achieved with `Ecto.Schema.embedded_schema/1`.\n\n```elixir\n# user/user.ex\ndefmodule User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :full_name, :string\n    field :email, :string\n    field :avatar_url, :string\n    field :confirmed_at, :naive_datetime\n\n    embeds_one :profile, UserProfile\n\n    timestamps()\n  end\nend\n\n# user/user_profile.ex\ndefmodule UserProfile do\n  use Ecto.Schema\n\n  embedded_schema do\n    field :online, :boolean\n    field :dark_mode, :boolean\n    field :visibility, Ecto.Enum, values: [:public, :private, :friends_only]\n  end\nend\n```\n\nEmbedded schemas defined in such way are said to be explicit-defined, which:\n\n- are dedicated modules having own scope, changeset functions, props, documentation, etc...\n- could be embedded by multiple parent schemas\n- are persistence agnostic, which means that `embedded_schema` doesn't require to be persisted\n\nIt is important to remember that `embedded_schema` has many use cases independent of `embeds_one` and `embeds_many`. As they are persistent agnostic, they are ideal for scenarios where you want to manage structured data without necessarily persisting it. For example, if you want to build a contact form, you still want to parse and validate the data, but the data is likely not persisted anywhere. Instead, it is used to send an email. Embedded schemas would be a good fit for such a use case.\n\n#","ref":"embedded-schemas.html#extracting-the-embeds"},{"type":"extras","title":"Migrations - Embedded Schemas","doc":"If you wish to save your embedded schema to the database, you need to write a migration to include the embedded data.\n\n```elixir\nalter table(\"users\") do\n  add :profile, :map\nend\n```\n\nWhether you use `embeds_one` or `embeds_many`, it is recommended to use the `:map` data type (although `{:array, :map}` will work with `embeds_many` as well). The reason is that typical relational databases are likely to represent a `:map` as JSON (or JSONB in Postgres), allowing Ecto adapter libraries more flexibility over how to efficiently store the data.\n\n#","ref":"embedded-schemas.html#migrations"},{"type":"extras","title":"Changesets - Embedded Schemas","doc":"Changeset functionality for embeds will allow you to enforce arbitrary validations on the data. You can define a changeset function for each module. For example, the UserProfile module could require the `online` and `visibility` fields to be present when generating a changeset.\n\n```elixir\ndefmodule UserProfile do\n  # ...\n\n  def changeset(%UserProfile{} = profile, attrs \\\\ %{}) do\n    profile\n    |> cast(attrs, [:online, :dark_mode, :visibility])\n    |> validate_required([:online, :visibility])\n  end\nend\n\nprofile = %UserProfile{}\nUserProfile.changeset(profile, %{online: true, visibility: :public})\n```\n\nMeanwhile, the User changeset function can require its own validations without worrying about the details of the UserProfile changes because it can pass that responsibility to UserProfile via `cast_embed/3`. A validation failure in an embed will cause the parent changeset to be invalid, even if the parent changeset itself had no errors.\n\n```elixir\ndefmodule User do\n  # ...\n\n  def changeset(user, attrs \\\\ %{}) do\n    user\n    |> cast(attrs, [:full_name, :email, :avatar_url])\n    |> cast_embed(:profile, required: true)\n  end\nend\n\nchangeset = User.changeset(%User{}, %{profile: %{online: true}})\nchangeset.valid? # => false; \"visibility can't be blank\"\nchangeset = User.changeset(%User{}, %{profile: %{online: true, visibility: :public}})\nchangeset.valid? # => true\n```\n\nIn situations where you have kept the embedded schema within the parent module, e.g., you have not extracted a UserProfile, you can still have custom changeset functions for the embedded data within the parent schema.\n\n```elixir\ndefmodule User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :full_name, :string\n    field :email, :string\n    field :avatar_url, :string\n    field :confirmed_at, :naive_datetime\n\n    embeds_one :profile, Profile do\n      field :online, :boolean\n      field :dark_mode, :boolean\n      field :visibility, Ecto.Enum, values: [:public, :private, :friends_only]\n    end\n\n    timestamps()\n  end\n\n  def changeset(%User{} = user, attrs \\\\ %{}) do\n    user\n    |> cast(attrs, [:full_name, :email])\n    |> cast_embed(:profile, required: true, with: &profile_changeset/2)\n  end\n\n  def profile_changeset(profile, attrs \\\\ %{}) do\n    profile\n    |> cast(attrs, [:online, :dark_mode, :visibility])\n    |> validate_required([:online, :visibility])\n  end\nend\n\nchangeset = User.changeset(%User{}, %{profile: %{online: true, visibility: :public}})\nchangeset.valid? # => true\n```\n\n#","ref":"embedded-schemas.html#changesets"},{"type":"extras","title":"Querying embedded data - Embedded Schemas","doc":"Once you have written embedded data to the database, you can use it in queries on the parent schema.\n\n```elixir\nuser_changeset = User.changeset(%User{}, %{profile: %{online: true, visibility: :public}})\n{:ok, _user} = Repo.insert(user_changeset)\n\n(Ecto.Query.from u in User, select: {u.profile[\"online\"], u.profile[\"visibility\"]}) |> Repo.one\n# => {true, \"public\"}\n\n(Ecto.Query.from u in User, select: u.profile, where: u.profile[\"visibility\"] == ^:public) |> Repo.all\n# => [\n#  %UserProfile{\n#    id: \"...\",\n#    online: true,\n#    dark_mode: nil,\n#    visibility: :public\n#  }\n#]\n```\n\nIn databases where `:map`s are stored as JSONB (like Postgres), Ecto constructs the appropriate jsonpath queries for you. More examples of embedded schema queries are documented in [`json_extract_path/2`](https://hexdocs.pm/ecto/Ecto.Query.API.html#json_extract_path/2).","ref":"embedded-schemas.html#querying-embedded-data"},{"type":"extras","title":"Testing with Ecto","doc":"# Testing with Ecto\n\nAfter you have successfully set up your database connection with Ecto for your application,\nits usage for your tests requires further changes, especially if you want to leverage the\n[Ecto SQL Sandbox](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html) that allows\nyou to run tests that talk to the database concurrently.\n\nCreate the `config/test.exs` file or append the following content:\n\n```elixir\nimport Config\n\nconfig :my_app, MyApp.Repo,\n  username: \"postgres\",\n  password: \"postgres\",\n  database: \"myapp_test\",\n  hostname: \"localhost\",\n  pool: Ecto.Adapters.SQL.Sandbox\n\n ```\n\nThereby, we configure the database connection for our test setup.\nIn this case, we use a Postgres database and set it up to use the sandbox pool that will wrap each test in a transaction.\n\nMake sure we import the configuration for the test environment at the very bottom of `config/config.exs`:\n\n```elixir\nimport_config \"#{config_env()}.exs\"\n```\n\nWe also need to add an explicit statement to the end of `test/test_helper.exs` about the `sandbox` mode:\n\n```elixir\nEcto.Adapters.SQL.Sandbox.mode(MyApp.Repo, :manual)\n```\n\nLastly, you need to establish the database connection ahead of your tests.\nYou can enable it either for all of your test cases by extending the `ExUnit` template or by setting it up individually for each test. Let's start with the former and place it to the `test/support/repo_case.ex`:\n\n```elixir\ndefmodule MyApp.RepoCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      alias MyApp.Repo\n\n      import Ecto\n      import Ecto.Query\n      import MyApp.RepoCase\n\n      # and any other stuff\n    end\n  end\n\n  setup tags do\n    :ok = Ecto.Adapters.SQL.Sandbox.checkout(MyApp.Repo)\n\n    unless tags[:async] do\n      Ecto.Adapters.SQL.Sandbox.mode(MyApp.Repo, {:shared, self()})\n    end\n\n    :ok\n  end\nend\n```\n\nThe case template above brings `Ecto` and `Ecto.Query` functions into your tests and checks-out a database connection. It also enables a shared sandbox connection mode in case the test is not running asynchronously.\nSee [`Ecto.Adapters.SQL.Sandbox`](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html) for more information.\n\nTo add `test/support/` folder for compilation in test environment we need to update `mix.exs` configuration\n\n```elixir\n  def project do\n    [\n      # ...\n      elixirc_paths: elixirc_paths(Mix.env())\n    ]\n  end\n\n  # Specifies which paths to compile per environment.\n  defp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\n  defp elixirc_paths(_), do: [\"lib\"]\n```\n\nAnd then in each test that uses the repository:\n\n```elixir\ndefmodule MyApp.MyTest do\n  use MyApp.RepoCase\n\n  # Tests etc...\nend\n```\n\nIn case you don't want to define a \"case template\", you can checkout on each individual case:\n\n```elixir\ndefmodule MyApp.MyTest do\n  use ExUnit.Case\n\n  setup do\n    :ok = Ecto.Adapters.SQL.Sandbox.checkout(MyApp.Repo)\n  end\n\n  # Tests etc...\nend\n```\n\nFor convenience reasons, you can also define `aliases` to automatically set up your database at the execution of your tests.\nChange the following content in your `mix.exs`.\n\n```elixir\n\n  def project do\n    [app: :my_app,\n\n     ...\n\n     aliases: aliases()]\n  end\n\n  defp aliases do\n    [ ...\n     \"test\": [\"ecto.create --quiet\", \"ecto.migrate\", \"test\"]\n    ]\n  end\n```","ref":"testing-with-ecto.html"},{"type":"extras","title":"Basic CRUD","doc":"# Basic CRUD\n\nIn this document, \"Internal data\" represents data or logic hardcoded into your Elixir code. \"External data\" means data that comes from the user via forms, APIs, and often need to be normalized, pruned, and validated via Ecto.Changeset.","ref":"crud.html"},{"type":"extras","title":"Fetching records - Basic CRUD","doc":"{: .col-2}\n\n#","ref":"crud.html#fetching-records"},{"type":"extras","title":"Single record - Basic CRUD","doc":"##","ref":"crud.html#single-record"},{"type":"extras","title":"Fetching record by ID - Basic CRUD","doc":"```elixir\nRepo.get(Movie, 1)\n```\n\n##","ref":"crud.html#fetching-record-by-id"},{"type":"extras","title":"Fetching record by attributes - Basic CRUD","doc":"```elixir\nRepo.get_by(Movie, title: \"Ready Player One\")\n```\n\n##","ref":"crud.html#fetching-record-by-attributes"},{"type":"extras","title":"Fetching the first record - Basic CRUD","doc":"```elixir\nMovie |> Ecto.Query.first() |> Repo.one()\n```\n\n##","ref":"crud.html#fetching-the-first-record"},{"type":"extras","title":"Fetching the last record - Basic CRUD","doc":"```elixir\nMovie |> Ecto.Query.last() |> Repo.one()\n```\n\n##","ref":"crud.html#fetching-the-last-record"},{"type":"extras","title":"Use `!` to raise if none is found - Basic CRUD","doc":"```elixir\nRepo.get!(Movie, 1)\nRepo.get_by!(Movie, title: \"Ready Player One\")\nMovie |> Ecto.Query.first() |> Repo.one!()\n```\n\n#","ref":"crud.html#use-to-raise-if-none-is-found"},{"type":"extras","title":"Multiple records - Basic CRUD","doc":"##","ref":"crud.html#multiple-records"},{"type":"extras","title":"Fetch all at once - Basic CRUD","doc":"```elixir\nMovie |> Repo.all()\n```\n\n##","ref":"crud.html#fetch-all-at-once"},{"type":"extras","title":"Stream all - Basic CRUD","doc":"```elixir\nMovie |> Repo.stream() |> Enum.each(fn record -> ... end)\n```\n\n##","ref":"crud.html#stream-all"},{"type":"extras","title":"Check at least one exists? - Basic CRUD","doc":"```elixir\nMovie |> Repo.exists?()\n```","ref":"crud.html#check-at-least-one-exists"},{"type":"extras","title":"Querying records - Basic CRUD","doc":"{: .col-2}\n\n#","ref":"crud.html#querying-records"},{"type":"extras","title":"Keyword-based queries - Basic CRUD","doc":"##","ref":"crud.html#keyword-based-queries"},{"type":"extras","title":"Bindingless queries - Basic CRUD","doc":"```elixir\nquery =\n  from Movie,\n  where: [title: \"Ready Player One\"],\n  select: [:title, :tagline]\nRepo.all(query)\n```\n\n##","ref":"crud.html#bindingless-queries"},{"type":"extras","title":"Bindings in queries - Basic CRUD","doc":"```elixir\nquery =\n  from m in Movie,\n  where: m.title == \"Ready Player One\",\n  select: [m.title, m.tagline]\nRepo.all(query)\n```\n\n#","ref":"crud.html#bindings-in-queries"},{"type":"extras","title":"Interpolation with `^` - Basic CRUD","doc":"```elixir\ntitle = \"Ready Player One\"\nquery =\n  from m in Movie,\n  where: m.title == ^title,\n  select: [m.title, m.tagline]\nRepo.all(query)\n```\n\n#","ref":"crud.html#interpolation-with"},{"type":"extras","title":"Pipe-based queries - Basic CRUD","doc":"```elixir\nMovie\n|> where([m], m.title == \"Ready Player One\")\n|> select([m], {m.title, m.tagline})\n|> Repo.all\n```","ref":"crud.html#pipe-based-queries"},{"type":"extras","title":"Inserting records - Basic CRUD","doc":"{: .col-2}\n\n#","ref":"crud.html#inserting-records"},{"type":"extras","title":"Single record - Basic CRUD","doc":"##","ref":"crud.html#single-record"},{"type":"extras","title":"Using internal data - Basic CRUD","doc":"```elixir\n%Person{name: \"Bob\"}\n|> Repo.insert()\n```\n\n##","ref":"crud.html#using-internal-data"},{"type":"extras","title":"Using external data - Basic CRUD","doc":"```elixir\n# Params represent data from a form, API, CLI, etc\nparams = %{\"name\" => \"Bob\"}\n\n%Person{}\n|> Ecto.Changeset.cast(params, [:name])\n|> Repo.insert()\n```\n\n#","ref":"crud.html#using-external-data"},{"type":"extras","title":"Multiple records - Basic CRUD","doc":"```elixir\ndata = [%{name: \"Bob\"}, %{name: \"Alice\"}]\nRepo.insert_all(Person, data)\n```","ref":"crud.html#multiple-records"},{"type":"extras","title":"Updating records - Basic CRUD","doc":"{: .col-2}\n\n#","ref":"crud.html#updating-records"},{"type":"extras","title":"Single record - Basic CRUD","doc":"##","ref":"crud.html#single-record"},{"type":"extras","title":"Using internal data - Basic CRUD","doc":"```elixir\nperson =\n  Person\n  |> Ecto.Query.first()\n  |> Repo.one!()\n\nchangeset = change(person, %{age: 29})\nRepo.update(changeset)\n```\n\n##","ref":"crud.html#using-internal-data"},{"type":"extras","title":"Using external data - Basic CRUD","doc":"```elixir\n# Params represent data from a form, API, CLI, etc\nparams = %{\"age\" => \"29\"}\n\nperson =\n  Person\n  |> Ecto.Query.first()\n  |> Repo.one!()\n\nchangeset = cast(person, params, [:age])\nRepo.update(changeset)\n```\n\n#","ref":"crud.html#using-external-data"},{"type":"extras","title":"Multiple records (using queries) - Basic CRUD","doc":"```elixir\nRepo.update_all(Person, set: [age: 29])\n```","ref":"crud.html#multiple-records-using-queries"},{"type":"extras","title":"Deleting records - Basic CRUD","doc":"{: .col-2}\n\n#","ref":"crud.html#deleting-records"},{"type":"extras","title":"Single record - Basic CRUD","doc":"```elixir\nperson = Repo.get!(Person, 1)\nRepo.delete(person)\n```\n\n#","ref":"crud.html#single-record"},{"type":"extras","title":"Multiple records (using queries) - Basic CRUD","doc":"```elixir\nRepo.delete_all(Person)\n```","ref":"crud.html#multiple-records-using-queries"},{"type":"extras","title":"Associations","doc":"# Associations\n\nIn this document, \"Internal data\" represents data or logic hardcoded into your Elixir code. \"External data\" means data that comes from the user via forms, APIs, and often need to be normalized, pruned, and validated via `Ecto.Changeset`.","ref":"associations.html"},{"type":"extras","title":"Has many / belongs to - Associations","doc":"{: .col-2}\n\n#","ref":"associations.html#has-many-belongs-to"},{"type":"extras","title":"The has many association - Associations","doc":"```elixir\ndefmodule Movie do\n  use Ecto.Schema\n\n  schema \"movies\" do\n    field :title, :string\n    field :release_date, :date\n    has_many :characters, Character\n  end\nend\n```\n\n#","ref":"associations.html#the-has-many-association"},{"type":"extras","title":"The belongs to association - Associations","doc":"```elixir\ndefmodule Character do\n  use Ecto.Schema\n\n  schema \"characters\" do\n    field :name, :string\n    field :age, :integer\n    belongs_to :movie, Movie\n  end\nend\n```","ref":"associations.html#the-belongs-to-association"},{"type":"extras","title":"Has one / belongs to - Associations","doc":"{: .col-2}\n\n#","ref":"associations.html#has-one-belongs-to"},{"type":"extras","title":"The has one association - Associations","doc":"```elixir\ndefmodule Movie do\n  use Ecto.Schema\n\n  schema \"movies\" do\n    field :title, :string\n    field :release_date, :date\n    has_one :screenplay, Screenplay\n  end\nend\n```\n\n#","ref":"associations.html#the-has-one-association"},{"type":"extras","title":"The belongs association - Associations","doc":"```elixir\ndefmodule Screenplay do\n  use Ecto.Schema\n\n  schema \"screenplays\" do\n    field :lead_writer, :string\n    belongs_to :movie, Movie\n  end\nend\n```","ref":"associations.html#the-belongs-association"},{"type":"extras","title":"Many to many - Associations","doc":"{: .col-2}\n\n#","ref":"associations.html#many-to-many"},{"type":"extras","title":"Through a join table - Associations","doc":"##","ref":"associations.html#through-a-join-table"},{"type":"extras","title":"The first schema - Associations","doc":"```elixir\ndefmodule Movie do\n  use Ecto.Schema\n\n  schema \"movies\" do\n    field :title, :string\n    field :release_date, :date\n    many_to_many :actors, Actor, join_through: \"movies_actors\"\n  end\nend\n```\n{: .wrap}\n\n##","ref":"associations.html#the-first-schema"},{"type":"extras","title":"The second schema - Associations","doc":"```elixir\ndefmodule Actor do\n  use Ecto.Schema\n\n  schema \"actors\" do\n    field :name, :string\n    many_to_many :movies, Movie, join_through: \"movies_actors\"\n  end\nend\n```\n{: .wrap}\n\n#","ref":"associations.html#the-second-schema"},{"type":"extras","title":"Through a join schema - Associations","doc":"##","ref":"associations.html#through-a-join-schema"},{"type":"extras","title":"The first schema - Associations","doc":"```elixir\ndefmodule User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    many_to_many :organizations, Organization, join_through: UserOrganization\n  end\nend\n```\n{: .wrap}\n\n##","ref":"associations.html#the-first-schema"},{"type":"extras","title":"The second schema - Associations","doc":"```elixir\ndefmodule Organization do\n  use Ecto.Schema\n\n  schema \"organizations\" do\n    many_to_many :users, User, join_through: UserOrganization\n  end\nend\n```\n{: .wrap}\n\n##","ref":"associations.html#the-second-schema"},{"type":"extras","title":"The join schema - Associations","doc":"```elixir\ndefmodule UserOrganization do\n  use Ecto.Schema\n\n  @primary_key false\n  schema \"users_organizations\" do\n    belongs_to :user, User\n    belongs_to :organization, Organization\n    timestamps()\n  end\nend\n```\n{: .wrap}","ref":"associations.html#the-join-schema"},{"type":"extras","title":"Querying associated records - Associations","doc":"{: .col-2}\n\n#","ref":"associations.html#querying-associated-records"},{"type":"extras","title":"Preloading in the parent record query - Associations","doc":"```elixir\nquery = from m in Movie, preload: :characters\nRepo.all(query)\n```\n\n#","ref":"associations.html#preloading-in-the-parent-record-query"},{"type":"extras","title":"Preloading when parent records are already loaded - Associations","doc":"```elixir\nmovies = Repo.all(Movie)\nmovies = Repo.preload(movies, :characters)\n```\n\n#","ref":"associations.html#preloading-when-parent-records-are-already-loaded"},{"type":"extras","title":"Preloading with join to generate a single query - Associations","doc":"##","ref":"associations.html#preloading-with-join-to-generate-a-single-query"},{"type":"extras","title":"Regular join - Associations","doc":"```elixir\nquery =\n  from m in Movie,\n  join: c in Character,\n  on: m.id == c.movie_id,\n  preload: :characters\nRepo.all(query)\n```\n\n##","ref":"associations.html#regular-join"},{"type":"extras","title":"Join using `assoc` - Associations","doc":"```elixir\nquery =\n  from m in Movie,\n  join: c in assoc(m, :characters),\n  preload: :characters\nRepo.all(query)\n```","ref":"associations.html#join-using-assoc"},{"type":"extras","title":"Inserting associated records - Associations","doc":"#","ref":"associations.html#inserting-associated-records"},{"type":"extras","title":"Inserting a child record to an existing parent - Associations","doc":"##","ref":"associations.html#inserting-a-child-record-to-an-existing-parent"},{"type":"extras","title":"Using internal data - Associations","doc":"```elixir\nRepo.get_by!(Movie, title: \"The Shawshank Redemption\")\n|> Ecto.build_assoc(:characters, name: \"Red\", age: 60)\n|> Repo.insert()\n```\n\n##","ref":"associations.html#using-internal-data"},{"type":"extras","title":"Using external data - Associations","doc":"```elixir\n# Params represent data from a form, API, CLI, etc\nparams = %{\"name\" => \"Red\", \"age\" => 60}\n\nRepo.get_by!(Movie, title: \"The Shawshank Redemption\")\n|> Ecto.build_assoc(:characters)\n|> cast(params, [:name, :age])\n|> Repo.insert()\n```\n\n#","ref":"associations.html#using-external-data"},{"type":"extras","title":"Inserting parent and child records together - Associations","doc":"##","ref":"associations.html#inserting-parent-and-child-records-together"},{"type":"extras","title":"Using internal data - Associations","doc":"```elixir\nRepo.insert(\n  %Movie{\n    title: \"The Shawshank Redemption\",\n    release_date: ~D[1994-10-14],\n    characters: [\n      %Character{name: \"Andy Dufresne\", age: 50},\n      %Character{name: \"Red\", age: 60}\n    ]\n  }\n)\n```\n\n##","ref":"associations.html#using-internal-data"},{"type":"extras","title":"Using external data - Associations","doc":"```elixir\n# Params represent data from a form, API, CLI, etc\nparams = %{\n  \"title\" => \"Shawshank Redemption\",\n  \"release_date\" => \"1994-10-14\",\n  \"characters\" =>\n    [\n      %{\"name\" => \"Andy Dufresne\", \"age\" => \"50\"},\n      %{\"name\" => \"Red\", \"age\" => \"60\"}\n    ]\n}\n\n%Movie{}\n|> cast(params, [:title, :release_date])\n|> cast_assoc(:characters)\n|> Repo.insert()\n```","ref":"associations.html#using-external-data"},{"type":"extras","title":"Updating associated records - Associations","doc":"#","ref":"associations.html#updating-associated-records"},{"type":"extras","title":"Updating records individually - Associations","doc":"##","ref":"associations.html#updating-records-individually"},{"type":"extras","title":"For individual updates, fetch and update records directly - Associations","doc":"```elixir\nmovie =\n  Repo.get_by!(Movie, title: \"The Shawshank Redemption\")\n  |> Repo.preload(:screenplay)\n\nmovie.screenplay\n|> change(%{lead_writter: \"Frank Darabont\"})\n|> Repo.update()\n```\n\n#","ref":"associations.html#for-individual-updates-fetch-and-update-records-directly"},{"type":"extras","title":"Updating all associated records, using internal data - Associations","doc":"##","ref":"associations.html#updating-all-associated-records-using-internal-data"},{"type":"extras","title":"Using `Ecto.Changeset.put_assoc/3` - Associations","doc":"```elixir\nmovie =\n  Repo.get_by!(Movie, title: \"The Shawshank Redemption\")\n  |> Repo.preload(:characters)\n\nIO.inspect(movie.characters)\n#=> [%{name: \"Andy Dufresne\", age: 50},\n#=>  %{name: \"Red\", age: 60}]\n\ncharacters =\n  Enum.map(characters, fn character ->\n    update_in(character.age, &(&1 + 1)))\n  end)\n\n{:ok, movie} =\n  movie\n  |> change()\n  |> put_assoc(:characters, characters)\n  |> Repo.update()\n\nmovie.characters |> Enum.map(&(&1.age)) |> IO.inspect\n#=> [51, 61]\n```\n\nNote: the example above performs the same operation on all entries,\ntherefore it can be written as a query. Queries should be preferred\nwhen possible as they avoid loading all data into memory and are\nmore performant. See next example.\n\n##","ref":"associations.html#using-ecto-changeset-put_assoc-3"},{"type":"extras","title":"Using `c:Ecto.Repo.update_all/3` - Associations","doc":"```elixir\nmovie = Repo.get_by!(Movie, title: \"The Shawshank Redemption\")\n\nmovie\n# Query to load all characters associated to a given movie\n|> Ecto.assoc(:characters)\n|> Repo.update_all(inc: [age: 1])\n```\n\n#","ref":"associations.html#using-c-ecto-repo-update_all-3"},{"type":"extras","title":"Updating all associated records, using external data - Associations","doc":"##","ref":"associations.html#updating-all-associated-records-using-external-data"},{"type":"extras","title":"Using `Ecto.Changeset.cast_assoc/3` - Associations","doc":"```elixir\n# Params represent data from a form, API, CLI, etc\nparams = %{\n  \"director\" => \"Frank Darabont\",\n  \"characters\" => [\n    %{\"id\" => 1, \"name\" => \"Andy Dufresne\"},\n    %{\"name\" => \"Red\", \"age\" => 60}\n  ]\n}\n\nmovie =\n  Repo.get_by!(Movie, title: \"The Shawshank Redemption\")\n  |> Repo.preload(:characters)\n\nIO.inspect(movie.characters)\n#=> [%{id: 1, name: \"Andy\", age: 50}]\n\n{:ok, movie} =\n  movie\n  |> cast(params, [\"director\"])\n  |> cast_assoc(:characters)\n  |> Repo.update()\n\nIO.inspect(movie.characters)\n#=> [%{id: 1, name: \"Andy Dufresne\", age: 50},\n#=>  %{id: 2, name: \"Red\", age: 60}]\n```\n\nWhen using `Ecto.Changeset.cast_assoc/3`:\n\n* Entries without ID are added.\n* Existing entries with matching IDs are updated.\n* Existing entries without matching IDs will raise\n  but it can be configured using `:on_replace`.\n* Additional options are supported to customize\n  casting, sorting, and deletion","ref":"associations.html#using-ecto-changeset-cast_assoc-3"},{"type":"extras","title":"Aggregates and subqueries","doc":"# Aggregates and subqueries\n\nNow it's time to discuss aggregates and subqueries. As we will learn, one builds directly on the other.","ref":"aggregates-and-subqueries.html"},{"type":"extras","title":"Aggregates - Aggregates and subqueries","doc":"Ecto includes a convenience function in repositories to calculate aggregates.\n\nFor example, if we assume every post has an integer column named visits, we can find the average number of visits across all posts with:\n\n```elixir\nMyApp.Repo.aggregate(MyApp.Post, :avg, :visits)\n#=> Decimal.new(1743)\n```\n\nBehind the scenes, the query above translates to:\n\n```elixir\nMyApp.Repo.one(from p in MyApp.Post, select: avg(p.visits))\n```\n\nThe `c:Ecto.Repo.aggregate/4` function supports any of the aggregate operations listed in the `Ecto.Query.API` module.\n\nAt first, it looks like the implementation of `aggregate/4` is quite straight-forward. You could even start to wonder why it was added to Ecto in the first place. However, complexities start to arise on queries that rely on `limit`, `offset` or `distinct` clauses.\n\nImagine that instead of calculating the average of all posts, you want the average of only the top 10. Your first try may be:\n\n```elixir\nMyApp.Repo.one(\n  from p in MyApp.Post,\n    order_by: [desc: :visits],\n    limit: 10,\n    select: avg(p.visits)\n)\n#=> Decimal.new(1743)\n```\n\nOops. The query above returned the same value as the queries before. The option `limit: 10` has no effect here since it is limiting the aggregated result and queries with aggregates return only a single row anyway. In order to retrieve the correct result, we would need to first find the top 10 posts and only then aggregate. That's exactly what `aggregate/4` does:\n\n```elixir\nquery =\n  from MyApp.Post,\n    order_by: [desc: :visits],\n    limit: 10\n\nMyApp.Repo.aggregate(query, :avg, :visits)\n#=> Decimal.new(4682)\n```\n\nWhen `limit`, `offset` or `distinct` is specified in the query, `aggregate/4` automatically wraps the given query in a subquery. Therefore the query executed by `aggregate/4` above is rather equivalent to:\n\n```elixir\ninner_query =\n  from MyApp.Post,\n    order_by: [desc: :visits],\n    limit: 10\n\nquery =\n  from q in subquery(inner_query),\n  select: avg(q.visits)\n\nMyApp.Repo.one(query)\n```\n\nLet's take a closer look at subqueries.","ref":"aggregates-and-subqueries.html#aggregates"},{"type":"extras","title":"Subqueries - Aggregates and subqueries","doc":"In the previous section we have already learned some queries that would be hard to express without support for subqueries. That's one of many examples that caused subqueries to be added to Ecto.\n\nSubqueries in Ecto are created by calling `Ecto.Query.subquery/1`. This function receives any data structure that can be converted to a query, via the `Ecto.Queryable` protocol, and returns a subquery construct (which is also queryable).\n\nIn Ecto, it is allowed for a subquery to select a whole table (`p`) or a field (`p.field`). All fields selected in a subquery can be accessed from the parent query. Let's revisit the aggregate query we saw in the previous section:\n\n```elixir\ninner_query =\n  from MyApp.Post,\n    order_by: [desc: :visits],\n    limit: 10\n\nquery =\n  from q in subquery(inner_query),\n    select: avg(q.visits)\n\nMyApp.Repo.one(query)\n```\n\nBecause the query does not specify a `:select` clause, it will return `select: p` where `p` is controlled by `MyApp.Post` schema. Since the query will return all fields in `MyApp.Post`, when we convert it to a subquery, all of the fields from `MyApp.Post` will be available on the parent query, such as `q.visits`. In fact, Ecto will keep the schema properties across queries. For example, if you write `q.field_that_does_not_exist`, your Ecto query won't compile.\n\nEcto also allows an Elixir map to be returned from a subquery, making the map keys directly available to the parent query.\n\nLet's see one last example. Imagine you manage a library (as in an actual library in the real world) and there is a table that logs every time the library lends a book. The \"lendings\" table uses an auto-incrementing primary key and can be backed by the following schema:\n\n```elixir\ndefmodule Library.Lending do\n  use Ecto.Schema\n\n  schema \"lendings\" do\n    belongs_to :book, MyApp.Book       # defines book_id\n    belongs_to :visitor, MyApp.Visitor # defines visitor_id\n  end\nend\n```\n\nNow consider we want to retrieve the name of every book alongside the name of the last person the library has lent it to. To do so, we need to find the last lending ID of every book, and then join on the book and visitor tables. With subqueries, that's straight-forward:\n\n```elixir\nlast_lendings =\n  from l in MyApp.Lending,\n    group_by: l.book_id,\n    select: %{\n      book_id: l.book_id,\n      last_lending_id: max(l.id)\n    }\n\nfrom l in Lending,\n  join: last in subquery(last_lendings),\n  on: last.last_lending_id == l.id,\n  join: b in assoc(l, :book),\n  join: v in assoc(l, :visitor),\n  select: {b.name, v.name}\n```","ref":"aggregates-and-subqueries.html#subqueries"},{"type":"extras","title":"Composable transactions with Multi","doc":"# Composable transactions with Multi\n\nEcto relies on database transactions when multiple operations must be performed atomically. The most common example used for transactions are bank transfers between two people:\n\n```elixir\nRepo.transaction(fn ->\n  mary_update =\n    from Account,\n      where: [id: ^mary.id],\n      update: [inc: [balance: +10]]\n\n  {1, _} = Repo.update_all(mary_update, [])\n\n  john_update =\n    from Account,\n      where: [id: ^john.id],\n      update: [inc: [balance: -10]]\n\n  {1, _} = Repo.update_all(john_update, [])\nend)\n```\n\nIn Ecto, transactions can be performed via the `Repo.transaction` function. When we expect both operations to succeed, as above, transactions are quite straight-forward. However, transactions get more complicated if we need to check the status of each operation along the way:\n\n```elixir\nRepo.transaction(fn ->\n  mary_update =\n    from Account,\n      where: [id: ^mary.id],\n      update: [inc: [balance: +10]]\n\n  case Repo.update_all(mary_update, []) do\n    {1, _} ->\n      john_update =\n        from Account,\n          where: [id: ^john.id],\n          update: [inc: [balance: -10]]\n\n      case Repo.update_all(john_update, []) do\n        {1, _} -> {mary, john}\n        {_, _} -> Repo.rollback({:failed_transfer, john})\n      end\n\n    {_, _} ->\n      Repo.rollback({:failed_transfer, mary})\n  end\nend)\n```\n\nTransactions in Ecto can also be nested arbitrarily. For example, imagine the transaction above is moved into its own function that receives both accounts, defined as `transfer_money(mary, john, 10)`, and besides transferring money we also want to log the transfer:\n\n```elixir\nRepo.transaction(fn ->\n  case transfer_money(mary, john, 10) do\n    {:ok, {mary, john}} ->\n      transfer = %Transfer{\n        from: mary.id,\n        to: john.id,\n        amount: 10\n      }\n\n      Repo.insert!(transfer)\n\n    {:error, error} ->\n      Repo.rollback(error)\n  end\nend)\n```\n\nThe snippet above starts a transaction and then calls `transfer_money/3` that also runs in a transaction. In the case of multiple transactions, they are all flattened, which means a failure in an inner transaction causes the outer transaction to also fail. That's why matching and rolling back on `{:error, error}` is important.\n\nWhile nesting transactions can improve the code readability by breaking large transactions into multiple smaller transactions, there is still a lot of boilerplate involved in handling the success and failure scenarios. Furthermore, composition is quite limited, as all operations must still be performed inside transaction blocks.\n\nA more declarative approach when working with transactions would be to define all operations we want to perform in a transaction decoupled from the transaction execution. This way we would be able to compose transactions operations without worrying about its execution context or about each individual success/failure scenario. That's exactly what `Ecto.Multi` allows us to do.","ref":"composable-transactions-with-multi.html"},{"type":"extras","title":"Composing with data structures - Composable transactions with Multi","doc":"Let's rewrite the snippets above using `Ecto.Multi`. The first snippet that transfers money between Mary and John can be rewritten to:\n\n```elixir\nmary_update =\n  from Account,\n    where: [id: ^mary.id],\n    update: [inc: [balance: +10]]\n\njohn_update =\n  from Account,\n    where: [id: ^john.id],\n    update: [inc: [balance: -10]]\n\nEcto.Multi.new()\n|> Ecto.Multi.update_all(:mary, mary_update, [])\n|> Ecto.Multi.run(:check_mary, fn\n  _repo, %{mary: {1, _}} -> {:ok, nil}\n  _repo, %{mary: {_, _}} -> {:error, {:failed_transfer, mary}}\n)\n|> Ecto.Multi.update_all(:john, john_update, [])\n|> Ecto.Multi.run(:check_john, fn\n  _repo, %{john: {1, _}} -> {:ok, nil}\n  _repo, %{john: {_, _}} -> {:error, {:failed_transfer, john}}\n)\n```\n\n`Ecto.Multi` is a data structure that defines multiple operations that must be performed together, without worrying about when they will be executed. `Ecto.Multi` mirrors most of the `Ecto.Repo` API, with the difference that each operation must be explicitly named. In the example above, we have defined two update operations, named `:mary` and `:john`, and two validation operations, named `:check_mary` and `:check_john`. As we will see later, the names are important when handling the transaction results.\n\nSince `Ecto.Multi` is just a data structure, we can pass it as argument to other functions, as well as return it. Assuming the multi above is moved into its own function, defined as `transfer_money(mary, john, value)`, we can add a new operation to the multi that logs the transfer as follows:\n\n```elixir\ntransfer = %Transfer{\n  from: mary.id,\n  to: john.id,\n  amount: 10\n}\n\ntransfer_money(mary, john, 10)\n|> Ecto.Multi.insert(:transfer, transfer)\n```\n\nThis is considerably simpler than the nested transaction approach we have seen earlier. Once all operations are defined in the multi, we can finally call `Repo.transaction`, this time passing the multi:\n\n```elixir\ntransfer = %Transfer{\n  from: mary.id,\n  to: john.id,\n  amount: 10\n}\n\ntransfer_money(mary, john, 10)\n|> Ecto.Multi.insert(:transfer, transfer)\n|> Repo.transaction()\n|> case do\n  {:ok, %{transfer: transfer}} ->\n    # Handle success case\n  {:error, name, value, changes_so_far} ->\n    # Handle failure case\nend\n```\n\nIf all operations in the multi succeed, it returns `{:ok, map}` where the map contains the name of all operations as keys and their success value. If any operation in the multi fails, the transaction is rolled back and `Repo.transaction` returns `{:error, name, value, changes_so_far}`, where `name` is the name of the failed operation, `value` is the failure value and `changes_so_far` is a map of the previously successful multi operations that have been rolled back due to the failure.\n\nIn other words, `Ecto.Multi` takes care of all the flow control boilerplate while decoupling the transaction definition from its execution, allowing us to compose operations as needed.","ref":"composable-transactions-with-multi.html#composing-with-data-structures"},{"type":"extras","title":"Dependent values - Composable transactions with Multi","doc":"Besides operations such as `insert`, `update` and `delete`, `Ecto.Multi` also provides functions for handling more complex scenarios. For example, `prepend` and `append` can be used to merge multis together. And more generally, the functions `Ecto.Multi.run/3` and `Ecto.Multi.run/5` can be used to define any operation that depends on the results of a previous multi operation. In addition, `Ecto.Multi` also gives us `put` and `inspect`, which allow us to dynamically update and inspect changes.\n\nLet's study a more practical example. In [Constraints and Upserts](Constraints and Upserts.md), we want to modify a post while possibly giving it a list of tags as a string separated by commas. At the end of the guide, we present a solution that inserts any missing tag and then fetches all of them using only two queries:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  # Schema is the same\n  schema \"posts\" do\n    field :title\n    field :body\n    many_to_many :tags, MyApp.Tag,\n      join_through: \"posts_tags\",\n      on_replace: :delete\n    timestamps()\n  end\n\n  # Changeset is the same\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:title, :body])\n    |> Ecto.Changeset.put_assoc(:tags, parse_tags(params))\n  end\n\n  # Parse tags has slightly changed\n  defp parse_tags(params)  do\n    (params[\"tags\"] || \"\")\n    |> String.split(\",\")\n    |> Enum.map(&String.trim/1)\n    |> Enum.reject(& &1 == \"\")\n    |> insert_and_get_all()\n  end\n\n  defp insert_and_get_all([]) do\n    []\n  end\n\n  defp insert_and_get_all(names) do\n    timestamp =\n      NaiveDateTime.utc_now()\n      |> NaiveDateTime.truncate(:second)\n\n    maps =\n      Enum.map(names, &%{\n        name: &1,\n        inserted_at: timestamp,\n        updated_at: timestamp\n      })\n\n    Repo.insert_all(MyApp.Tag, maps, on_conflict: :nothing)\n\n    Repo.all(from t in MyApp.Tag, where: t.name in ^names)\n  end\nend\n```\n\nWhile `insert_and_get_all/1` is idempotent, allowing us to run it multiple times and get the same result back, it does not run inside a transaction, so any failure while attempting to modify the parent post struct would end-up creating tags that have no posts associated to them.\n\nLet's fix the problem above by introducing using `Ecto.Multi`. Let's start by splitting the logic into both `Post` and `Tag` modules and keeping it free from side-effects such as database operations:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  schema \"posts\" do\n    field :title\n    field :body\n    many_to_many :tags, MyApp.Tag,\n      join_through: \"posts_tags\",\n      on_replace: :delete\n    timestamps()\n  end\n\n  def changeset(struct, tags, params) do\n    struct\n    |> Ecto.Changeset.cast(params, [:title, :body])\n    |> Ecto.Changeset.put_assoc(:tags, tags)\n  end\nend\n\ndefmodule MyApp.Tag do\n  use Ecto.Schema\n\n  schema \"tags\" do\n    field :name\n    timestamps()\n  end\n\n  def parse(tags) do\n    (tags || \"\")\n    |> String.split(\",\")\n    |> Enum.map(&String.trim/1)\n    |> Enum.reject(& &1 == \"\")\n  end\nend\n```\n\nNow, whenever we need to introduce a post with tags, we can create a multi that wraps all operations and the repository access:\n\n```elixir\nalias MyApp.Tag\n\ndef insert_or_update_post_with_tags(post, params) do\n  Ecto.Multi.new()\n  |> Ecto.Multi.run(:tags, fn _repo, changes ->\n    insert_and_get_all_tags(changes, params)\n  end)\n  |> Ecto.Multi.run(:post, fn _repo, changes ->\n    insert_or_update_post(changes, post, params)\n  end)\n  |> Repo.transaction()\nend\n\ndefp insert_and_get_all_tags(_changes, params) do\n  case MyApp.Tag.parse(params[\"tags\"]) do\n    [] ->\n      {:ok, []}\n\n    names ->\n      timestamp =\n        NaiveDateTime.utc_now()\n        |> NaiveDateTime.truncate(:second)\n\n      maps =\n        Enum.map(names, &%{\n          name: &1,\n          inserted_at: timestamp,\n          updated_at: timestamp\n        })\n\n      Repo.insert_all(Tag, maps, on_conflict: :nothing)\n\n      query = from t in Tag, where: t.name in ^names\n\n      {:ok, Repo.all(query)}\n  end\nend\n\ndefp insert_or_update_post(%{tags: tags}, post, params) do\n  post\n  |> MyApp.Post.changeset(tags, params)\n  |> Repo.insert_or_update()\nend\n```\n\nIn the example above we have used `Ecto.Multi.run/3` twice, albeit for two different reasons.\n\n  1. In `Ecto.Multi.run(:tags, ...)`, we used `run/3` because we need to perform both `insert_all` and `all` operations, and while the multi exposes `Ecto.Multi.insert_all/4`, it does not have an equivalent to `Ecto.Repo.all`. Whenever we need to perform a repository operation that is not supported by `Ecto.Multi`, we can always fallback to `run/3` or `run/5`.\n\n  2. In `Ecto.Multi.run(:post, ...)`, we used `run/3` because we need to access the value of a previous multi operation. The function given to `run/3` receives, as second argument, a map with the results of the operations performed so far. To grab the tags returned in the previous step, we simply pattern match on `%{tags: tags}` on `insert_or_update_post`.\n\n> Note: The first argument received by the function given to `run/3` is the repo in which the transaction is executing.\n\nWhile `run/3` is very handy when we need to go beyond the functionalities provided natively by `Ecto.Multi`, it has the downside that operations defined with `Ecto.Multi.run/3` are opaque and therefore they cannot be inspected by functions such as `Ecto.Multi.to_list/1`. Still, `Ecto.Multi` allows us to greatly simplify control flow logic and remove boilerplate when working with transactions.","ref":"composable-transactions-with-multi.html#dependent-values"},{"type":"extras","title":"Constraints and Upserts","doc":"# Constraints and Upserts\n\nIn this guide we will learn how to use constraints and upserts. To showcase those features, we will work on a practical scenario: which is by studying a many to many relationship between posts and tags.","ref":"constraints-and-upserts.html"},{"type":"extras","title":"put_assoc vs cast_assoc - Constraints and Upserts","doc":"Imagine we are building an application that has blog posts and such posts may have many tags. Not only that, a given tag may also belong to many posts. This is a classic scenario where we would use `many_to_many` associations. Our migrations would look like:\n\n```elixir\ncreate table(:posts) do\n  add :title, :string\n  add :body, :text\n  timestamps()\nend\n\ncreate table(:tags) do\n  add :name, :string\n  timestamps()\nend\n\ncreate unique_index(:tags, [:name])\n\ncreate table(:posts_tags, primary_key: false) do\n  add :post_id, references(:posts)\n  add :tag_id, references(:tags)\nend\n```\n\nNote we added a unique index to the tag name because we don't want to have duplicated tags in our database. It is important to add an index at the database level instead of using a validation since there is always a chance two tags with the same name would be validated and inserted simultaneously, passing the validation and leading to duplicated entries.\n\nNow let's also imagine we want the user to input such tags as a list of words split by comma, such as: \"elixir, erlang, ecto\". Once this data is received in the server, we will break it apart into multiple tags and associate them to the post, creating any tag that does not yet exist in the database.\n\nWhile the constraints above sound reasonable, that's exactly what put us in trouble with `cast_assoc/3`. The `cast_assoc/3` changeset function was designed to receive external parameters and compare them with the associated data in our structs. To do so correctly, Ecto requires tags to be sent as a list of maps. We can see an example of this in [Polymorphic associations with many to many](Polymorphic associations with many to many.md). However, here we expect tags to be sent in a string separated by comma.\n\nFurthermore, `cast_assoc/3` relies on the primary key field for each tag sent in order to decide if it should be inserted, updated or deleted. Again, because the user is simply passing a string, we don't have the ID information at hand.\n\nWhen we can't cope with `cast_assoc/3`, it is time to use `put_assoc/4`. In `put_assoc/4`, we give Ecto structs or changesets instead of parameters, giving us the ability to manipulate the data as we want. Let's define the schema and the changeset function for a post which may receive tags as a string:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  schema \"posts\" do\n    field :title\n    field :body\n\n    many_to_many :tags, MyApp.Tag,\n      join_through: \"posts_tags\",\n      on_replace: :delete\n\n    timestamps()\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:title, :body])\n    |> Ecto.Changeset.put_assoc(:tags, parse_tags(params))\n  end\n\n  defp parse_tags(params)  do\n    (params[\"tags\"] || \"\")\n    |> String.split(\",\")\n    |> Enum.map(&String.trim/1)\n    |> Enum.reject(& &1 == \"\")\n    |> Enum.map(&get_or_insert_tag/1)\n  end\n\n  defp get_or_insert_tag(name) do\n    Repo.get_by(MyApp.Tag, name: name) ||\n      Repo.insert!(%Tag{name: name})\n  end\nend\n```\n\nIn the changeset function above, we moved all the handling of tags to a separate function, called `parse_tags/1`, which checks for the parameter, breaks each tag apart via `String.split/2`, then removes any left over whitespace with `String.trim/1`, rejects any empty string and finally checks if the tag exists in the database or not, creating one in case none exists.\n\nThe `parse_tags/1` function is going to return a list of `MyApp.Tag` structs which are then passed to `put_assoc/4`. By calling `put_assoc/4`, we are telling Ecto those should be the tags associated to the post from now on. In case a previous tag was associated to the post and not given in `put_assoc/4`, Ecto will invoke the behaviour defined in the `:on_replace` option, which we have set to `:delete`. The `:delete` behaviour will remove the association between the post and the removed tag from the database.\n\nAnd that's all we need to use `many_to_many` associations with `put_assoc/4`. `put_assoc/4` is very useful when we want to have more explicit control over our associations and it also works with `has_many`, `belongs_to` and all others association types.\n\nHowever, our code is not yet ready for production. Let's see why.","ref":"constraints-and-upserts.html#put_assoc-vs-cast_assoc"},{"type":"extras","title":"Constraints and race conditions - Constraints and Upserts","doc":"Remember we added a unique index to the tag `:name` column when creating the tags table. We did so to protect us from having duplicate tags in the database.\n\nBy adding the unique index and then using `get_by`  with a `insert!` to get or insert a tag, we introduced a potential error in our application. If two posts are submitted at the same time with a similar tag, there is a chance we will check if the tag exists at the same time, leading both submissions to believe there is no such tag in the database. When that happens, only one of the submissions will succeed while the other one will fail. That's a race condition: your code will error from time to time, only when certain conditions are met. And those conditions are time sensitive.\n\nLuckily Ecto gives us a mechanism to handle constraint errors from the database.","ref":"constraints-and-upserts.html#constraints-and-race-conditions"},{"type":"extras","title":"Checking for constraint errors - Constraints and Upserts","doc":"Since our `get_or_insert_tag(name)` function fails when a tag already exists in the database, we need to handle such scenarios accordingly. Let's rewrite it taking race conditions into account:\n\n```elixir\ndefp get_or_insert_tag(name) do\n  %Tag{}\n  |> Ecto.Changeset.change(name: name)\n  |> Ecto.Changeset.unique_constraint(:name)\n  |> Repo.insert()\n  |> case do\n    {:ok, tag} -> tag\n    {:error, _} -> Repo.get_by!(MyApp.Tag, name: name)\n  end\nend\n```\n\nInstead of inserting the tag directly, we now build a changeset, which allows us to use the `unique_constraint` annotation. Now if the `Repo.insert` operation fails because the unique index for `:name` is violated, Ecto won't raise, but return an `{:error, changeset}` tuple. Therefore, if `Repo.insert` succeeds, it is because the tag was saved, otherwise the tag already exists, which we then fetch with `Repo.get_by!`.\n\nWhile the mechanism above fixes the race condition, it is a quite expensive one: we need to perform two queries for every tag that already exists in the database: the (failed) insert and then the repository lookup. Given that's the most common scenario, we may want to rewrite it to the following:\n\n```elixir\ndefp get_or_insert_tag(name) do\n  Repo.get_by(MyApp.Tag, name: name) ||\n    maybe_insert_tag(name)\nend\n\ndefp maybe_insert_tag(name) do\n  %Tag{}\n  |> Ecto.Changeset.change(name: name)\n  |> Ecto.Changeset.unique_constraint(:name)\n  |> Repo.insert\n  |> case do\n    {:ok, tag} -> tag\n    {:error, _} -> Repo.get_by!(MyApp.Tag, name: name)\n  end\nend\n```\n\nThe above performs 1 query for every tag that already exists, 2 queries for every new tag and possibly 3 queries in the case of race conditions. While the above would perform slightly better on average, Ecto has a better option in stock.","ref":"constraints-and-upserts.html#checking-for-constraint-errors"},{"type":"extras","title":"Upserts - Constraints and Upserts","doc":"Ecto supports the so-called \"upsert\" command which is an abbreviation for \"update or insert\". The idea is that we try to insert a record and in case it conflicts with an existing entry, for example due to a unique index, we can choose how we want the database to act by either raising an error (the default behaviour), ignoring the insert (no error) or by updating the conflicting database entries.\n\n\"upsert\" in Ecto is done with the `:on_conflict` option. Let's rewrite `get_or_insert_tag(name)` once more but this time using the `:on_conflict` option. Remember that \"upsert\" is a new feature in PostgreSQL 9.5, so make sure you are up to date.\n\nYour first try in using `:on_conflict` may be by setting it to `:nothing`, as below:\n\n```elixir\ndefp get_or_insert_tag(name) do\n  Repo.insert!(\n    %MyApp.Tag{name: name},\n    on_conflict: :nothing\n  )\nend\n```\n\nWhile the above won't raise an error in case of conflicts, it also won't update the struct given, so it will return a tag without ID. One solution is to force an update to happen in case of conflicts, even if the update is about setting the tag name to its current name. In such cases, PostgreSQL also requires the `:conflict_target` option to be given, which is the column (or a list of columns) we are expecting the conflict to happen:\n\n```elixir\ndefp get_or_insert_tag(name) do\n  Repo.insert!(\n    %MyApp.Tag{name: name},\n    on_conflict: [set: [name: name]],\n    conflict_target: :name\n  )\nend\n```\n\nAnd that's it! We try to insert a tag with the given name and if such tag already exists, we tell Ecto to update its name to the current value, updating the tag and fetching its id. While the above is certainly a step up from all solutions so far, it still performs one query per tag. If 10 tags are sent, we will perform 10 queries. Can we further improve this?","ref":"constraints-and-upserts.html#upserts"},{"type":"extras","title":"Upserts and insert_all - Constraints and Upserts","doc":"Ecto accepts the `:on_conflict` option not only in `c:Ecto.Repo.insert/2` but also in the `c:Ecto.Repo.insert_all/3` function. This means we can build one query that attempts to insert all missing tags and then another query that fetches all of them at once. Let's see how our `Post` schema will look like after those changes:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  # We need to import Ecto.Query\n  import Ecto.Query\n\n  # Schema is the same\n  schema \"posts\" do\n    add :title\n    add :body\n\n    many_to_many :tags, MyApp.Tag,\n      join_through: \"posts_tags\",\n      on_replace: :delete\n\n    timestamps()\n  end\n\n  # Changeset is the same\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:title, :body])\n    |> Ecto.Changeset.put_assoc(:tags, parse_tags(params))\n  end\n\n  # Parse tags has slightly changed\n  defp parse_tags(params)  do\n    (params[\"tags\"] || \"\")\n    |> String.split(\",\")\n    |> Enum.map(&String.trim/1)\n    |> Enum.reject(& &1 == \"\")\n    |> insert_and_get_all()\n  end\n\n  defp insert_and_get_all([]) do\n    []\n  end\n  defp insert_and_get_all(names) do\n    timestamp =\n      NaiveDateTime.utc_now()\n      |> NaiveDateTime.truncate(:second)\n\n    placeholders = %{timestamp: timestamp}\n\n    maps =\n      Enum.map(names, &%{\n        name: &1,\n        inserted_at: {:placeholder, :timestamp},\n        updated_at: {:placeholder, :timestamp}\n      })\n\n    Repo.insert_all(\n      MyApp.Tag,\n      maps,\n      placeholders: placeholders,\n      on_conflict: :nothing\n    )\n\n    Repo.all(from t in MyApp.Tag, where: t.name in ^names)\n  end\nend\n```\n\nInstead of getting and inserting each tag individually, the code above works on all tags at once, first by building a list of maps which is given to `insert_all`. Then we look up all tags with the given names. Regardless of how many tags are sent, we will perform only 2 queries - unless no tag is sent, in which we return an empty list back promptly. This solution is only possible thanks to the `:on_conflict` option, which guarantees `insert_all` won't fail in case a unique index is violated, such as from duplicate tag names. Remember, `insert_all` won't autogenerate values like timestamps. That's why we define a timestamp placeholder and reuse it across `inserted_at` and `updated_at` fields.\n\nFinally, keep in mind that we haven't used transactions in any of the examples so far. That decision was deliberate as we relied on the fact that getting or inserting tags is an idempotent operation, i.e. we can repeat it many times for a given input and it will always give us the same result back. Therefore, even if we fail to introduce the post to the database due to a validation error, the user will be free to resubmit the form and we will just attempt to get or insert the same tags once again. The downside of this approach is that tags will be created even if creating the post fails, which means some tags may not have posts associated to them. In case that's not desired, the whole operation could be wrapped in a transaction or modeled with `Ecto.Multi`.","ref":"constraints-and-upserts.html#upserts-and-insert_all"},{"type":"extras","title":"Data mapping and validation","doc":"# Data mapping and validation\n\nWe will take a look at the role schemas play when validating and casting data through changesets. As we will see, sometimes the best solution is not to completely avoid schemas, but break a large schema into smaller ones. Maybe one for reading data, another for writing. Maybe one for your database, another for your forms.","ref":"data-mapping-and-validation.html"},{"type":"extras","title":"Schemas are mappers - Data mapping and validation","doc":"The `Ecto.Schema` moduledoc says:\n\n> An Ecto schema is used to map *any* data source into an Elixir struct.\n\nWe put emphasis on *any* because it is a common misconception to think Ecto schemas map only to your database tables.\n\nFor instance, when you write a web application using Phoenix and you use Ecto to receive external changes and apply such changes to your database, we have this mapping:\n\n    Database   Ecto schema   Forms / API\n\nAlthough there is a single Ecto schema mapping to both your database and your API, in many situations it is better to break this mapping in two. Let's see some practical examples.\n\nImagine you are working with a client that wants the \"Sign Up\" form to contain the fields \"First name\", \"Last name\" along side \"E-mail\" and other information. You know there are a couple problems with this approach.\n\nFirst of all, not everyone has a first and last name. Although your client is decided on presenting both fields, they are a UI concern, and you don't want the UI to dictate the shape of your data. Furthermore, you know it would be useful to break the \"Sign Up\" information across two tables, the \"accounts\" and \"profiles\" tables.\n\nGiven the requirements above, how would we implement the Sign Up feature in the backend?\n\nOne approach would be to have two schemas, Account and Profile, with virtual fields such as `first_name` and `last_name`, and [use associations along side nested forms](https://dashbit.co/blog/working-with-ecto-associations-and-embeds) to tie the schemas to your UI. One of such schemas would be:\n\n```elixir\ndefmodule Profile do\n  use Ecto.Schema\n\n  schema \"profiles\" do\n    field :name\n    field :first_name, :string, virtual: true\n    field :last_name, :string, virtual: true\n    ...\n  end\nend\n```\n\nIt is not hard to see how we are polluting our Profile schema with UI requirements by adding fields such `first_name` and `last_name`. If the Profile schema is used for both reading and writing data, it may end-up in an awkward place where it is not useful for any, as it contains fields that map just to one or the other operation.\n\nOne alternative solution is to break the \"Database   Ecto schema   Forms / API\" mapping in two parts. The first will cast and validate the external data with its own structure which you then transform and write to the database. For such, let's define a schema named `Registration` that will take care of casting and validating the form data exclusively, mapping directly to the UI fields:\n\n```elixir\ndefmodule Registration do\n  use Ecto.Schema\n\n  embedded_schema do\n    field :first_name\n    field :last_name\n    field :email\n  end\nend\n```\n\nWe used `embedded_schema` because it is not our intent to persist it anywhere. With the schema in hand, we can use Ecto changesets and validations to process the data:\n\n```elixir\nfields = [:first_name, :last_name, :email]\n\nchangeset =\n  %Registration{}\n  |> Ecto.Changeset.cast(params[\"sign_up\"], fields)\n  |> validate_required(...)\n  |> validate_length(...)\n```\n\nNow that the registration changes are mapped and validated, we can check if the resulting changeset is valid and act accordingly:\n\n```elixir\nif changeset.valid? do\n  # Get the modified registration struct from changeset\n  registration = Ecto.Changeset.apply_changes(changeset)\n  account = Registration.to_account(registration)\n  profile = Registration.to_profile(registration)\n\n  MyApp.Repo.transaction fn ->\n    MyApp.Repo.insert_all \"accounts\", [account]\n    MyApp.Repo.insert_all \"profiles\", [profile]\n  end\n\n  {:ok, registration}\nelse\n  # Annotate the action so the UI shows errors\n  changeset = %{changeset | action: :registration}\n  {:error, changeset}\nend\n```\n\nThe `to_account/1` and `to_profile/1` functions in `Registration` would receive the registration struct and split the attributes apart accordingly:\n\n```elixir\ndef to_account(registration) do\n  Map.take(registration, [:email])\nend\n\ndef to_profile(%{first_name: first, last_name: last}) do\n  %{name: \"#{first} #{last}\"}\nend\n```\n\nIn the example above, by breaking apart the mapping between the database and Elixir and between Elixir and the UI, our code becomes clearer and our data structures simpler.\n\nNote we have used `MyApp.Repo.insert_all/2` to add data to both \"accounts\" and \"profiles\" tables directly. We have chosen to bypass schemas altogether. However, there is nothing stopping you from also defining both `Account` and `Profile` schemas and changing `to_account/1` and `to_profile/1` to respectively return `%Account{}` and `%Profile{}` structs. Once structs are returned, they could be inserted through the usual `MyApp.Repo.insert/2` operation. Doing so can be especially useful if there are uniqueness or other constraints that you want to check during insertion.","ref":"data-mapping-and-validation.html#schemas-are-mappers"},{"type":"extras","title":"Schemaless changesets - Data mapping and validation","doc":"Although we chose to define a `Registration` schema to use in the changeset, Ecto also allows developers to use changesets without schemas. We can dynamically define the data and their types. Let's rewrite the registration changeset above to bypass schemas:\n\n```elixir\ndata = %{}\ntypes = %{name: :string, email: :string}\n\n# The data+types tuple is equivalent to %Registration{}\nchangeset =\n  {data, types}\n  |> Ecto.Changeset.cast(params[\"sign_up\"], Map.keys(types))\n  |> validate_required(...)\n  |> validate_length(...)\n```\n\nYou can use this technique to validate API endpoints, search forms, and other sources of data. The choice of using schemas depends mostly if you want to use the same mapping in different places or if you desire the compile-time guarantees Elixir structs gives you. Otherwise, you can bypass schemas altogether, be it when using changesets or interacting with the repository.\n\nHowever, the most important lesson in this guide is not when to use or not to use schemas, but rather understand when a big problem can be broken into smaller problems that can be solved independently leading to an overall cleaner solution. The choice of using schemas or not above didn't affect the solution as much as the choice of breaking the registration problem apart.","ref":"data-mapping-and-validation.html#schemaless-changesets"},{"type":"extras","title":"Dynamic queries","doc":"# Dynamic queries\n\nEcto was designed from the ground up to have an expressive query API that leverages Elixir syntax to write queries that are pre-compiled for performance and safety. When building queries, we may use the keywords syntax\n\n```elixir\nimport Ecto.Query\n\nfrom p in Post,\n  where: p.author == \"José\" and p.category == \"Elixir\",\n  where: p.published_at > ^minimum_date,\n  order_by: [desc: p.published_at]\n```\n\nor the pipe-based one\n\n```elixir\nimport Ecto.Query\n\nPost\n|> where([p], p.author == \"José\" and p.category == \"Elixir\")\n|> where([p], p.published_at > ^minimum_date)\n|> order_by([p], desc: p.published_at)\n```\n\nBoth APIs are also composable. For example, imagine you want to abstract the `published_at` filtering and sorting into a function, with the keyword syntax you could write:\n\n```elixir\ndef most_recent_from(query, minimum_date) do\n  from p in query,\n    where: p.published_at > ^minimum_date,\n    order_by: [desc: p.published_at]\nend\n```\n\nand with the pipe syntax:\n\n```elixir\ndef most_recent_from(query, minimum_date) do\n  query\n  |> where([p], p.published_at > ^minimum_date)\n  |> order_by([p], desc: p.published_at)\nend\n```\n\nThe examples above show you can build and compose queries at a high-level: by composing each call to `where`, `order_by`, and so on. However, sometimes you want the contents of the `where` or the `order_by` themselves to be defined dynamically. For example, a web application that provides search functionality on top of existing posts. The user should be able to specify multiple criteria, such as the author name, the post category, publishing interval, etc.\n\nFuthermore, while many developers prefer the pipe-based syntax, having to repeat the binding `p` made it quite verbose compared to the keyword one.\n\nTo solve those problems, Ecto also provides a data-structure centric API to build queries as well as a very powerful mechanism for dynamic queries. Let's take a look.","ref":"dynamic-queries.html"},{"type":"extras","title":"Focusing on data structures - Dynamic queries","doc":"Ecto provides a simpler API for both keyword and pipe based queries by making data structures first-class. Let's see an example:\n\n```elixir\nfrom p in Post,\n  where: [author: \"José\", category: \"Elixir\"],\n  where: p.published_at > ^minimum_date,\n  order_by: [desc: :published_at]\n```\n\nand\n\n```elixir\nPost\n|> where(author: \"José\", category: \"Elixir\")\n|> where([p], p.published_at > ^minimum_date)\n|> order_by(desc: :published_at)\n```\n\nNotice how we were able to ditch the `p` selector in most expressions. All Ecto constructs accept data structures as input. Such data structures can also be specified dynamically, shown below:\n\n```elixir\nwhere = [author: \"José\", category: \"Elixir\"]\norder_by = [desc: :published_at]\nPost\n|> where(^where)\n|> where([p], p.published_at > ^minimum_date)\n|> order_by(^order_by)\n```\n\nWhile using data-structures already brings a good amount of flexibility to Ecto queries, not all expressions can be converted to data structures. For example, `where` converts a key-value to a `key == value` comparison, and therefore order-based comparisons such as `p.published_at > ^minimum_date` need to be written as before.","ref":"dynamic-queries.html#focusing-on-data-structures"},{"type":"extras","title":"Dynamic fragments - Dynamic queries","doc":"For cases where we cannot rely on data structures but still desire to build queries dynamically, Ecto includes the `Ecto.Query.dynamic/2` macro.\n\nThe `dynamic` macro allows us to conditionally build query fragments and interpolate them in the main query. For example, imagine that in the example above you may optionally filter posts by a date of publication. You could of course write it like this:\n\n```elixir\nquery =\n  Post\n  |> where(^where)\n  |> order_by(^order_by)\n\nquery =\n  if published_at = params[\"published_at\"] do\n    where(query, [p], p.published_at   where(^where)\n|> where(^filter_published_at)\n|> order_by(^order_by)\n```\n\nThe `dynamic` macro allows us to build dynamic expressions that are later interpolated into the query. `dynamic` expressions can also be interpolated into dynamic expressions, allowing developers to build complex expressions dynamically without hassle.\n\nBy using dynamic fragments, we can decouple the processing of parameters from the query generation. Let's see a more complex example.","ref":"dynamic-queries.html#dynamic-fragments"},{"type":"extras","title":"Building dynamic queries - Dynamic queries","doc":"Let's go back to the original problem. We want to build a search functionality where the user can configure how to traverse all posts in many different ways. For example, the user may choose how to order the data, filter by author and category, as well as select posts published after a certain date.\n\nTo tackle this in Ecto, we can break our problem into a bunch of small functions, that build either data structures or dynamic fragments, and then we interpolate it into the query:\n\n```elixir\ndef filter(params) do\n  Post\n  |> order_by(^filter_order_by(params[\"order_by\"]))\n  |> where(^filter_where(params))\nend\n\ndef filter_order_by(\"published_at_desc\"),\n  do: [desc: dynamic([p], p.published_at)]\n\ndef filter_order_by(\"published_at\"),\n  do: [asc: dynamic([p], p.published_at)]\n\ndef filter_order_by(_),\n  do: []\n\ndef filter_where(params) do\n  Enum.reduce(params, dynamic(true), fn\n    {\"author\", value}, dynamic ->\n      dynamic([p], ^dynamic and p.author == ^value)\n\n    {\"category\", value}, dynamic ->\n      dynamic([p], ^dynamic and p.category == ^value)\n\n    {\"published_at\", value}, dynamic ->\n      dynamic([p], ^dynamic and p.published_at > ^value)\n\n    {_, _}, dynamic ->\n      # Not a where parameter\n      dynamic\n  end)\nend\n```\n\nBecause we were able to break our problem into smaller functions that receive regular data structures, we can use all the tools available in Elixir to work with data. For handling the `order_by` parameter, it may be best to simply pattern match on the `order_by` parameter. For building the `where` clause, we can use `reduce` to start with an empty dynamic (that always returns true) and refine it with new conditions as we traverse the parameters.\n\nTesting also becomes simpler as we can test each function in isolation, even when using dynamic queries:\n\n```elixir\ntest \"filter published at based on the given date\" do\n  assert dynamic_match?(\n           filter_where(%{}),\n           \"true\"\n         )\n\n  assert dynamic_match?(\n           filter_where(%{\"published_at\" => \"2010-04-17\"}),\n           \"true and q.published_at > ^\\\"2010-04-17\\\"\"\n         )\nend\n\ndefp dynamic_match?(dynamic, string) do\n  inspect(dynamic) == \"dynamic([q], #{string})\"\nend\n```\n\nIn the example above, we created a small helper that allows us to assert on the dynamic contents by matching on the results of `inspect(dynamic)`.","ref":"dynamic-queries.html#building-dynamic-queries"},{"type":"extras","title":"Dynamic and joins - Dynamic queries","doc":"Even query joins can be tackled dynamically. For example, let's do two modifications to the example above. Let's say we can also sort by author name (\"author_name\" and \"author_name_desc\") and at the same time let's say that authors are in a separate table, which means our authors filter in `filter_where` now need to go through the join table.\n\nOur final solution would look like this:\n\n```elixir\ndef filter(params) do\n  Post\n  # 1. Add named join binding\n  |> join(:inner, [p], assoc(p, :authors), as: :authors)\n  |> order_by(^filter_order_by(params[\"order_by\"]))\n  |> where(^filter_where(params))\nend\n\n# 2. Returned dynamic with join binding\ndef filter_order_by(\"published_at_desc\"),\n  do: [desc: dynamic([p], p.published_at)]\n\ndef filter_order_by(\"published_at\"),\n  do: dynamic([p], p.published_at)\n\ndef filter_order_by(\"author_name_desc\"),\n  do: [desc: dynamic([authors: a], a.name)]\n\ndef filter_order_by(\"author_name\"),\n  do: dynamic([authors: a], a.name)\n\ndef filter_order_by(_),\n  do: []\n\n# 3. Change the authors clause inside reduce\ndef filter_where(params) do\n  Enum.reduce(params, dynamic(true), fn\n    {\"author\", value}, dynamic ->\n      dynamic([authors: a], ^dynamic and a.name == ^value)\n\n    {\"category\", value}, dynamic ->\n      dynamic([p], ^dynamic and p.category == ^value)\n\n    {\"published_at\", value}, dynamic ->\n      dynamic([p], ^dynamic and p.published_at > ^value)\n\n    {_, _}, dynamic ->\n      # Not a where parameter\n      dynamic\n  end)\nend\n```\n\nAdding more filters in the future is simply a matter of adding more clauses to the `Enum.reduce/3` call in `filter_where`.","ref":"dynamic-queries.html#dynamic-and-joins"},{"type":"extras","title":"Multi tenancy with query prefixes","doc":"# Multi tenancy with query prefixes\n\nWith Ecto we can run queries in different prefixes using a single pool of database connections. For databases engines such as Postgres, Ecto's prefix [maps to Postgres' DDL schemas](https://www.postgresql.org/docs/current/static/ddl-schemas.html). For MySQL, each prefix is a different database on its own.\n\n> ##","ref":"multi-tenancy-with-query-prefixes.html"},{"type":"extras","title":"Multi-tenancy and migrations {: .warning} - Multi tenancy with query prefixes","doc":">\n> When working with multi-tenant databases, you need to apply database migrations (such as adding/removing tables, columns, and indexes) to each tenant. If your application grows to dozens of thousands of tenants or more, those migrations can eventually become too expensive and take a long time to complete.\n\nQuery prefixes may be useful in different scenarios. For example, multi tenant apps running on PostgreSQL would define multiple prefixes, usually one per client, under a single database. The idea is that prefixes will provide data isolation between the different users of the application, guaranteeing either globally or at the data level that queries and commands act on a specific tenants.\n\nPrefixes may also be useful on high-traffic applications where data is partitioned upfront. For example, a gaming platform may break game data into isolated partitions, each named after a different prefix. A partition for a given player is either chosen at random or calculated based on the player information.\n\nGiven each tenant has its own database structure, multi tenancy with query prefixes is expensive to setup. For example, migrations have to run individually for each prefix. Therefore this approach is useful when there is a limited or a slowly growing number of tenants.\n\nLet's get started. Note all the examples below assume you are using PostgreSQL. Other databases engines may require slightly different solutions.","ref":"multi-tenancy-with-query-prefixes.html#multi-tenancy-and-migrations-warning"},{"type":"extras","title":"Connection prefixes - Multi tenancy with query prefixes","doc":"As a starting point, let's start with a simple scenario: your application must connect to a particular prefix when running in production. This may be due to infrastructure conditions, database administration rules or others.\n\nLet's define a repository and a schema to get started:\n\n```elixir\n# lib/repo.ex\ndefmodule MyApp.Repo do\n  use Ecto.Repo,\n    otp_app: :my_app,\n    adapter: Ecto.Adapters.Postgres\nend\n\n# lib/sample.ex\ndefmodule MyApp.Sample do\n  use Ecto.Schema\n\n  schema \"samples\" do\n    field :name\n    timestamps\n  end\nend\n```\n\nNow let's configure the repository:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Repo,\n  username: \"postgres\",\n  password: \"postgres\",\n  database: \"demo\",\n  hostname: \"localhost\",\n  pool_size: 10\n```\n\nAnd define a migration:\n\n```elixir\n# priv/repo/migrations/20160101000000_create_sample.exs\ndefmodule MyApp.Repo.Migrations.CreateSample do\n  use Ecto.Migration\n\n  def change do\n    create table(:samples) do\n      add :name, :string\n      timestamps()\n    end\n  end\nend\n```\n\nNow let's create the database, migrate it and then start an IEx session:\n\n```bash\n$ mix ecto.create\n$ mix ecto.migrate\n$ iex -S mix\nInteractive Elixir - press Ctrl+C to exit\niex(1)> MyApp.Repo.all MyApp.Sample\n[]\n```\n\nWe haven't done anything unusual so far. We created our database instance, made it up to date by running migrations and then successfully made a query against the \"samples\" table, which returned an empty list.\n\nBy default, connections to Postgres' databases run on the \"public\" prefix. When we run migrations and queries, they are all running against the \"public\" prefix. However imagine your application has a requirement to run on a particular prefix in production, let's call it \"connection_prefix\".\n\nLuckily Postgres allows us to change the prefix our database connections run on by setting the \"schema search path\". The best moment to change the search path is right after we setup the database connection, ensuring all of our queries will run on that particular prefix, throughout the connection life-cycle.\n\nTo do so, let's change our database configuration in \"config/config.exs\" and specify an `:after_connect` option. `:after_connect` expects a tuple with module, function and arguments it will invoke with the connection process, as soon as a database connection is established:\n\n```elixir\nquery_args = [\"SET search_path TO connection_prefix\", []]\n\nconfig :my_app, MyApp.Repo,\n  username: \"postgres\",\n  password: \"postgres\",\n  database: \"demo_dev\",\n  hostname: \"localhost\",\n  pool_size: 10,\n  after_connect: {Postgrex, :query!, query_args}\n```\n\nNow let's try to run the same query as before:\n\n```bash\n$ iex -S mix\nInteractive Elixir - press Ctrl+C to exit\niex(1)> MyApp.Repo.all MyApp.Sample\n** (Postgrex.Error) ERROR (undefined_table):\n   relation \"samples\" does not exist\n```\n\nOur previously successful query now fails because there is no table \"samples\" under the new prefix. Let's try to fix that by running migrations:\n\n```bash\n$ mix ecto.migrate\n** (Postgrex.Error) ERROR (invalid_schema_name):\n   no schema has been selected to create in\n```\n\nOops. Now migration says there is no such schema name. That's because Postgres automatically creates the \"public\" prefix every time we create a new database. If we want to use a different prefix, we must explicitly create it on the database we are running on:\n\n```bash\n$ psql -d demo_dev -c \"CREATE SCHEMA connection_prefix\"\n```\n\nNow we are ready to migrate and run our queries:\n\n```bash\n$ mix ecto.migrate\n$ iex -S mix\nInteractive Elixir - press Ctrl+C to exit\niex(1)> MyApp.Repo.all MyApp.Sample\n[]\n```\n\nData in different prefixes are isolated. Writing to the \"samples\" table in one prefix cannot be accessed by the other unless we change the prefix in the connection or use the Ecto conveniences we will discuss next.","ref":"multi-tenancy-with-query-prefixes.html#connection-prefixes"},{"type":"extras","title":"Schema prefixes - Multi tenancy with query prefixes","doc":"Ecto also allows you to set a particular schema to run on a specific prefix. Imagine you are building a multi-tenant application. Each client data belongs to a particular prefix, such as \"client_foo\", \"client_bar\" and so forth. Yet your application may still rely on a set of tables that are shared across all clients. One of such tables may be exactly the table that maps the Client ID to its database prefix. Let's assume we want to store this data in a prefix named \"main\":\n\n```elixir\ndefmodule MyApp.Mapping do\n  use Ecto.Schema\n\n  @schema_prefix \"main\"\n  schema \"mappings\" do\n    field :client_id, :integer\n    field :db_prefix\n    timestamps\n  end\nend\n```\n\nNow running `MyApp.Repo.all MyApp.Mapping` will by default run on the \"main\" prefix, regardless of the value configured for the connection on the `:after_connect` callback. However, we may want to override the schema prefix too and Ecto gives us the opportunity to do so, let's see how.","ref":"multi-tenancy-with-query-prefixes.html#schema-prefixes"},{"type":"extras","title":"Per-query and per-struct prefixes - Multi tenancy with query prefixes","doc":"Now, suppose that while still configured to connect to the \"connection_prefix\" on `:after_connect`, we run the following queries:\n\n```iex\niex(1)> alias MyApp.Sample\nMyApp.Sample\niex(2)> MyApp.Repo.all(Sample)\n[]\niex(3)> MyApp.Repo.insert(%Sample{name: \"mary\"})\n{:ok, %MyApp.Sample{...}}\niex(4)> MyApp.Repo.all(Sample)\n[%MyApp.Sample{...}]\n```\n\nThe operations above ran on the \"connection_prefix\". So what happens if we try to run the sample query on the \"public\" prefix? All Ecto repository operations support the `:prefix` option. So let's set it to public.\n\n```iex\niex(7)> MyApp.Repo.all(Sample)\n[%MyApp.Sample{...}]\niex(8)> MyApp.Repo.all(Sample, prefix: \"public\")\n[]\n```\n\nNotice how we were able to change the prefix the query runs on. Back in the default \"public\" prefix, there is no data.\n\nOne interesting aspect of prefixes in Ecto is that the prefix information is carried along each struct returned by a query:\n\n```iex\niex(9)> [sample] = MyApp.Repo.all(Sample)\n[%MyApp.Sample{}]\niex(10)> Ecto.get_meta(sample, :prefix)\nnil\n```\n\nThe example above returned nil, which means no prefix was specified by Ecto, and therefore the database connection default will be used. In this case, \"connection_prefix\" will be used because of the `:after_connect` callback we added at the beginning of this guide.\n\nSince the prefix data is carried in the struct, we can use such to copy data from one prefix to the other. Let's copy the sample above from the \"connection_prefix\" to the \"public\" one:\n\n```iex\niex(11)> new_sample = Ecto.put_meta(sample, prefix: \"public\")\n%MyApp.Sample{}\niex(12)> MyApp.Repo.insert(new_sample)\n{:ok, %MyApp.Sample{}}\niex(13)> [sample] = MyApp.Repo.all(Sample, prefix: \"public\")\n[%MyApp.Sample{}]\niex(14)> Ecto.get_meta(sample, :prefix)\n\"public\"\n```\n\nNow we have data inserted in both prefixes. Note how we passed the `:prefix` option to `MyApp.Repo.all`. Almost all Repo operations accept `:prefix` as an option, with one important distinction:\n\n  * the `:prefix` option in query operations (`all/2`, `update_all/2`, and `delete_all/2`) is a fallback. It will only be used when a `@schema_prefix` or a query prefix was not previously specified\n\n  * the `:prefix` option in schema operations (`insert_all/3`, `insert/2`, `update/2`, etc) will override the `@schema_prefix` as well as any prefix in the struct/changeset\n\nThis difference in behaviour is by design: we want to allow flexibility when writing queries but we want to enforce struct/changeset operations to always work isolated within a given prefix. In fact, if call `MyApp.Repo.insert(post)` or `MyApp.Repo.update(post)`, and the post includes associations, the associated data will also be inserted/updated in the same prefix as `post`.","ref":"multi-tenancy-with-query-prefixes.html#per-query-and-per-struct-prefixes"},{"type":"extras","title":"Per from/join prefixes - Multi tenancy with query prefixes","doc":"Finally, Ecto allows you to set the prefix individually for each `from` and `join` expression. Here's an example:\n\n```elixir\nfrom p in Post, prefix: \"foo\",\n  join: c in Comment, prefix: \"bar\"\n```\n\nThose will take precedence over all other prefixes we have defined so far. For each join/from in the query, the prefix used will be determined by the following order:\n\n  1. If the prefix option is given exclusively to join/from\n  2. If the `@schema_prefix` is set in the related schema\n  3. If the `:prefix` field given to the repo operation (i.e. `Repo.all(query, prefix: prefix)`)\n  4. The connection prefix","ref":"multi-tenancy-with-query-prefixes.html#per-from-join-prefixes"},{"type":"extras","title":"Migration prefixes - Multi tenancy with query prefixes","doc":"When the connection prefix is set, it also changes the prefix migrations run on. However it is also possible to set the prefix through the command line or per table in the migration itself.\n\nFor example, imagine you are a gaming company where the game is broken in 128 partitions, named \"prefix_1\", \"prefix_2\", \"prefix_3\" up to \"prefix_128\". Now, whenever you need to migrate data, you need to migrate data on all different 128 prefixes. There are two ways of achieve that.\n\nThe first mechanism is to invoke `mix ecto.migrate` multiple times, once per prefix, passing the `--prefix` option:\n\n```bash\n$ mix ecto.migrate --prefix \"prefix_1\"\n$ mix ecto.migrate --prefix \"prefix_2\"\n$ mix ecto.migrate --prefix \"prefix_3\"\n...\n$ mix ecto.migrate --prefix \"prefix_128\"\n```\n\nThe other approach is by changing each desired migration to run across multiple prefixes. For example:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.CreateSample do\n  use Ecto.Migration\n\n  def change do\n    for i <- 1..128 do\n      prefix = \"prefix_#{i}\"\n      create table(:samples, prefix: prefix) do\n        add :name, :string\n        timestamps()\n      end\n\n      # Execute the commands on the current prefix\n      # before moving on to the next prefix\n      flush()\n    end\n  end\nend\n```","ref":"multi-tenancy-with-query-prefixes.html#migration-prefixes"},{"type":"extras","title":"Summing up - Multi tenancy with query prefixes","doc":"Ecto provides many conveniences for working with querying prefixes. Those conveniences allow developers to configure prefixes with different precedence, starting with the highest one. When executing queries with `all`, `update_all` or `delete_all`, the prefix is computed as follows:\n\n  1. from/join prefixes\n  2. schema prefixes\n  3. the `:prefix` option\n  4. connection prefixes\n\nWhen working with schemas and changesets in `insert_all`, `insert`, `update`, and so forth, the precedence is:\n\n  1. the `:prefix` option\n  2. changeset prefixes\n  3. schema prefixes\n  4. connection prefixes\n\nThis way developers can tackle different scenarios from production requirements to multi-tenant applications.","ref":"multi-tenancy-with-query-prefixes.html#summing-up"},{"type":"extras","title":"Multi tenancy with foreign keys","doc":"# Multi tenancy with foreign keys\n\nIn [Multi tenancy with query prefixes](Multi tenancy with query prefixes.md), we have learned how to set up multi tenant applications by using separate query prefixes, known as DDL Schemas in PostgreSQL and MSSQL and simply a separate database in MySQL.\n\nEach query prefix is isolated, having their own tables and data, which provides the security guarantees we need. On the other hand, such approach for multi tenancy may be too expensive, as each schema needs to be created, migrated, and versioned separately.\n\nTherefore, some applications may prefer a cheaper mechanism for multi tenancy, by relying on foreign keys. The idea here is that most - if not all - resources in the system belong to a tenant. The tenant is typically an organization or a user and all resources have an `org_id` (or `user_id`) foreign key pointing directly to it.\n\nIn this guide, we will show how to leverage Ecto constructs to guarantee that all Ecto queries in your application are properly scoped to a chosen `org_id`.","ref":"multi-tenancy-with-foreign-keys.html"},{"type":"extras","title":"Adding org_id to read operations - Multi tenancy with foreign keys","doc":"The first step in our implementation is to make the repository aware of `org_id`. We want to allow commands such as:\n\n```elixir\nMyApp.Repo.all Post, org_id: 13\n```\n\nWhere the repository will automatically scope all posts to the organization with `ID=13`. We can achieve this with the `c:Ecto.Repo.prepare_query/3` repository callback:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n\n  require Ecto.Query\n\n  @impl true\n  def prepare_query(_operation, query, opts) do\n    cond do\n      opts[:skip_org_id] || opts[:schema_migration] ->\n        {query, opts}\n\n      org_id = opts[:org_id] ->\n        {Ecto.Query.where(query, org_id: ^org_id), opts}\n\n      true ->\n        raise \"expected org_id or skip_org_id to be set\"\n    end\n  end\nend\n```\n\nNow we can pass `:org_id` to all READ operations, such as `get`, `get_by`, `preload`, etc and all query operations, such `all`, `update_all`, and `delete_all`. Note we have intentionally made the `:org_id` required, with the exception of two scenarios:\n\n  * if you explicitly set `:skip_org_id` to true, it won't require an `:org_id`. This reduces the odds of a developer forgetting to scope their queries, which can accidentally expose private data to other users\n\n  * if the `:schema_migration` option is set. This means the repository operation was issued by Ecto itself when migrating our database and we don't want to apply an `org_id` to them\n\nStill, setting the `org_id` for every operation is cumbersome and error prone. We will be better served if all operations attempt to set an `org_id`.","ref":"multi-tenancy-with-foreign-keys.html#adding-org_id-to-read-operations"},{"type":"extras","title":"Setting `org_id` by default - Multi tenancy with foreign keys","doc":"To make sure our read operations use the `org_id` by default, we will make two additional changes to the repository.\n\nFirst, we will store the `org_id` in the process dictionary. The process dictionary is a storage that is exclusive to each process. For example, each test in your project runs in a separate process. Each request in a web application runs in a separate process too. Each of these processes have their own dictionary which we will store and read from. Let's add these functions:\n\n```elixir\ndefmodule MyApp.Repo do\n  ...\n\n  @tenant_key {__MODULE__, :org_id}\n\n  def put_org_id(org_id) do\n    Process.put(@tenant_key, org_id)\n  end\n\n  def get_org_id() do\n    Process.get(@tenant_key)\n  end\nend\n```\n\nWe added two new functions. The first, `put_org_id`, stores the organization id in the process dictionary. `get_org_id` reads the value in the process dictionary.\n\nYou will want to call `put_org_id` on every process before you use the repository. For example, on every request in a web application, as soon as you read the current organization from the request parameter or the session, you should call `MyApp.Repo.put_org_id(params_org_id)`. In tests, you want to explicitly set the `put_org_id` or pass the `:org_id` option as in the previous section.\n\nThe second change we need to do is to set the `org_id` as a default option on all repository operations. The value of `org_id` will be precisely the value in the process dictionary. We can do so trivially by implementing the `default_options` callback:\n\n```elixir\ndefmodule MyApp.Repo do\n  ...\n\n  @impl true\n  def default_options(_operation) do\n    [org_id: get_org_id()]\n  end\nend\n```\n\nWith these changes, we will always set the `org_id` field in our Ecto queries, unless we explicitly set `skip_org_id: true` when calling the repository. The only remaining step is to make sure the `org_id` field is not null in your database tables and make sure the `org_id` is set whenever inserting into the database.\n\nTo better understand how our database schema should look like, let's discuss some other techniques that we can use to tighten up multi tenant support, especially in regards to associations.","ref":"multi-tenancy-with-foreign-keys.html#setting-org_id-by-default"},{"type":"extras","title":"Working with multi tenant associations - Multi tenancy with foreign keys","doc":"Let's expand our data domain a little bit.\n\nSo far we have assumed there is an organization schema. However, instead of naming its primary key `id`, we will name it `org_id`, so `Repo.one(Org, org_id: 13)` just works:\n\n```elixir\ndefmodule MyApp.Organization do\n  use Ecto.Schema\n\n  @primary_key {:org_id, :id, autogenerate: true}\n  schema \"orgs\" do\n    field :name\n    timestamps()\n  end\nend\n```\n\nLet's also say that you may have multiple posts in an organization and the posts themselves may have multiple comments:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  schema \"posts\" do\n    field :title\n    field :org_id, :integer\n    has_many :comments, MyApp.Comment\n    timestamps()\n  end\nend\n\ndefmodule MyApp.Comment do\n  use Ecto.Schema\n\n  schema \"comments\" do\n    field :body\n    field :org_id, :integer\n    belongs_to :post, MyApp.Post\n    timestamps()\n  end\nend\n```\n\nOne thing to have in mind is that, our `prepare_query` callback will apply to all queries, but it won't apply to joins inside the same query. Therefore, if you write this query:\n\n```elixir\nMyApp.Repo.put_org_id(some_org_id)\n\nMyApp.Repo.all(\n  from p in Post, join: c in assoc(p, :comments)\n)\n```\n\n`prepare_query` will apply the `org_id` only to posts but not to the `join`. While this may seem problematic, in practice it is not an issue, because when you insert posts and comments in the database, **they will always have the same `org_id`**. If posts and comments do not have the same `org_id`, then there is a bug: the data either got corrupted or there is a bug in our software when inserting data.\n\nLuckily, we can leverage database's foreign keys to guarantee that the `org_id`s always match between posts and comments. Our first stab at defining these schema migrations would look like this:\n\n```elixir\ncreate table(:orgs, primary_key: false) do\n  add :org_id, :bigserial, primary_key: true\n  add :name, :string\n  timestamps()\nend\n\ncreate table(:posts) do\n  add :title, :string\n\n  add :org_id,\n      references(:orgs, column: :org_id),\n      null: false\n\n  timestamps()\nend\n\ncreate table(:comments) do\n  add :body, :string\n  add :org_id, references(:orgs), null: false\n  add :post_id, references(:posts), null: false\n  timestamps()\nend\n```\n\nSo far the only noteworthy change compared to a regular migration is the `primary_key: false` option to the `:orgs` table, as we want to mirror the primary key of `org_id` given to the schema. While the schema above works and guarantees that posts references an existing organization and that comments references existing posts and organizations, it does not guarantee that all posts and their related comments belong to the same organization.\n\nWe can tighten up this requirement by using composite foreign keys with the following changes:\n\n```elixir\ncreate unique_index(:posts, [:id, :org_id])\n\ncreate table(:comments) do\n  add :body, :string\n\n  # There is no need to define a reference for org_id\n  add :org_id, :integer, null: false\n\n  # Instead define a composite foreign key\n  add :post_id,\n      references(:posts, with: [org_id: :org_id]),\n      null: false\n\n  timestamps()\nend\n```\n\nInstead of defining both `post_id` and `org_id` as individual foreign keys, we define `org_id` as a regular integer and then we define `post_id+org_id` as a composite foreign key by passing the `:with` option to `Ecto.Migration.references/2`. This makes sure comments point to posts which point to orgs, where all `org_id`s match.\n\nGiven composite foreign keys require the referenced keys to be unique, we also defined a unique index on the posts table **before** we defined the composite foreign key.\n\nIf you are using PostgreSQL and you want to tighten these guarantees even further, you can pass the `match: :full` option to `references`:\n\n```elixir\nreferences(:posts, with: [org_id: :org_id], match: :full)\n```\n\nwhich will help enforce none of the columns in the foreign key can be `nil`.","ref":"multi-tenancy-with-foreign-keys.html#working-with-multi-tenant-associations"},{"type":"extras","title":"Summary - Multi tenancy with foreign keys","doc":"In this guide, we have changed our repository interface to guarantee our queries are always scoped to an `org_id`, unless we explicitly opt out. We also learned how to leverage database features to enforce the data is always valid.\n\nWhen it comes to associations, you will want to apply composite foreign keys whenever possible. For example, imagine comments belongs to posts (which belong to an organization) and also to user (which belong to an organization). The comments schema migration should be defined like this:\n\n```elixir\ncreate table(:comments) do\n  add :body, :string\n  add :org_id, :integer, null: false\n\n  add :post_id,\n      references(:posts, with: [org_id: :org_id]),\n      null: false\n\n  add :user_id,\n      references(:users, with: [org_id: :org_id]),\n      null: false\n\n  timestamps()\nend\n```\n\nAs long as all schemas have an `org_id`, all operations will be safely contained by the current tenant.\n\nIf by any chance you have schemas that are not tied to an `org_id`, you can even consider keeping them in a separate query prefix or in a separate database altogether, so you keep non-tenant data completely separated from tenant-specific data.","ref":"multi-tenancy-with-foreign-keys.html#summary"},{"type":"extras","title":"Self-referencing many to many","doc":"# Self-referencing many to many\n\n`Ecto.Schema.many_to_many/3` is used to establish the association between two schemas with a join table (or a join schema) tracking the relationship between them. But, what if we want the same table to reference itself? This is commonly used for symmetric relationships and is often referred to as a self-referencing `many_to_many` association.","ref":"self-referencing-many-to-many.html"},{"type":"extras","title":"People relationships - Self-referencing many to many","doc":"Let's imagine we are building a system that supports a model for relationships between people.\n\n```elixir\ndefmodule MyApp.Accounts.Person do\n  use Ecto.Schema\n\n  alias MyApp.Accounts.Person\n  alias MyApp.Relationships.Relationship\n\n  schema \"people\" do\n    field :name, :string\n\n    many_to_many :relationships,\n                 Person,\n                 join_through: Relationship,\n                 join_keys: [person_id: :id, relation_id: :id]\n\n    many_to_many :reverse_relationships,\n                 Person,\n                 join_through: Relationship,\n                 join_keys: [relation_id: :id, person_id: :id]\n\n    timestamps()\n  end\nend\n\ndefmodule MyApp.Relationships.Relationship do\n  use Ecto.Schema\n\n  schema \"relationships\" do\n    field :person_id, :id\n    field :relation_id, :id\n    timestamps()\n  end\nend\n```\n\nIn our example, we implement an intermediate schema, `MyApp.Relationships.Relationship`, on our `:join_through` option and pass in a pair of ids that we will be creating a unique index on in our database migration. By implementing an intermediate schema, we make it easy to add additional attributes and functionality to relationships in the future.\n\nWe had to create an additional `many_to_many` `:reverse_relationships` call with an inverse of the `:join_keys` in order to finish the other half of the association. This ensures that both sides of the relationship will get added in the database when either side completes a successful relationship request.\n\nThe person who is the inverse of the relationship will have the relationship struct stored in a list under the \"reverse_relationships\" key. We can then construct queries for both `:relationships` and `:reverse_relationships` with the proper `:preload`:\n\n```elixir\niex> preloads = [:relationships, :reverse_relationships]\niex> people = Repo.all from p in Person, preload: preloads\n[\n  MyApp.Accounts.Person \n    ]\n  >,\n  MyApp.Accounts.Person \n    ]\n  >\n]\n```\n\nIn the example query above, we are assuming that we have two \"people\" that have entered into a relationship. Our query illustrates how one person is added on the `:relationships` side and the other on the `:reverse_relationships` side.\n\nIt is also worth noticing that we are implementing separate parent modules for both our `Person` and `Relationship` modules. This separation of concerns helps improve code organization and maintainability by allowing us to isolate core functions for relationships in the `MyApp.Relationships` context and vice-versa.\n\nLet's take a look at our Ecto migration:\n\n```elixir\ndef change do\n  create table(:relationships) do\n    add :person_id, references(:people)\n    add :relation_id, references(:people)\n    timestamps()\n  end\n\n  create index(:relationships, [:person_id])\n  create index(:relationships, [:relation_id])\n\n  create unique_index(\n    :relationships,\n    [:person_id, :relation_id],\n    name: :relationships_person_id_relation_id_index\n  )\nend\n```\n\nWe create indexes on both the `:person_id` and `:relation_id` for quicker access in the future. Then, we create one unique index on the `:relationships` to ensure that people cannot have duplicate relationships. Lastly, we pass a name to the `:name` option to help clarify the unique constraint when working with our changeset.\n\n```elixir\n# In MyApp.Relationships.Relationship\n@attrs [:person_id, :relation_id]\n\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, @attrs)\n  |> Ecto.Changeset.unique_constraint(\n    [:person_id, :relation_id],\n    name: :relationships_person_id_relation_id_index\n  )\nend\n```\n\nDue to the self-referential nature, we will only need to cast the `:join_keys` in order for Ecto to correctly associate the two records in the database. When considering production applications, we will most likely want to add additional attributes and validations. This is where our isolation of modules will help us maintain and organize the increasing complexity.","ref":"self-referencing-many-to-many.html#people-relationships"},{"type":"extras","title":"Summary - Self-referencing many to many","doc":"In this guide we used `many_to_many` associations to implement a self-referencing symmetric relationship.\n\nOur goal was to allow \"people\" to associate to different \"people\". Further, we wanted to lay a strong foundation for code organization and maintainability into the future. We have done this by creating intermediate tables, two separate functional core modules, a clear naming strategy, an inverse association, and by using `many_to_many` `:join_keys` to automatically manage those join tables.\n\nOverall, our code contains a small structural modification, when compared with a typical `many_to_many`, in order to implement an inverse join between our self-referenced table and schema.\n\nWhere we go from here will depend greatly on the specific needs of our application. If we remember to adhere to our clear naming strategy with a strong separation of concerns, we will go a long way in keeping our self-referencing `many_to_many` association organized and easier to maintain.","ref":"self-referencing-many-to-many.html#summary"},{"type":"extras","title":"Polymorphic associations with many to many","doc":"# Polymorphic associations with many to many\n\nBesides `belongs_to`, `has_many`, `has_one` and `:through` associations, Ecto also includes `many_to_many`. `many_to_many` relationships, as the name says, allows a record from table X to have many associated entries from table Y and vice-versa. Although `many_to_many` associations can be written as `has_many :through`, using `many_to_many` may considerably simplify some workflows.\n\nIn this guide, we will talk about polymorphic associations and how `many_to_many` can remove boilerplate from certain approaches compared to `has_many :through`.","ref":"polymorphic-associations-with-many-to-many.html"},{"type":"extras","title":"Todo lists v65131 - Polymorphic associations with many to many","doc":"The internet has seen its share of todo list applications. But that won't stop us from creating our own!\n\nIn our case, there is one aspect of todo list applications we are interested in, which is the relationship where the todo list has many todo items. This exact scenario is explored in detail in a post about [nested associations and embeds](https://dashbit.co/blog/working-with-ecto-associations-and-embeds) from Dashbit's blog. Let's recap the important points.\n\nOur todo list app has two schemas, `Todo.List` and `Todo.Item`:\n\n```elixir\ndefmodule MyApp.TodoList do\n  use Ecto.Schema\n\n  schema \"todo_lists\" do\n    field :title\n    has_many :todo_items, MyApp.TodoItem\n    timestamps()\n  end\nend\n\ndefmodule MyApp.TodoItem do\n  use Ecto.Schema\n\n  schema \"todo_items\" do\n    field :description\n    timestamps()\n  end\nend\n```\n\nOne of the ways to introduce a todo list with multiple items into the database is to couple our UI representation to our schemas. That's the approach we took in the blog post with Phoenix. Roughly:\n\n```eex\n  %>\n   \n    %>\n    ...\n   \n \n```\n\nWhen such a form is submitted in Phoenix, it will send parameters with the following shape:\n\n```elixir\n%{\n  \"todo_list\" => %{\n    \"title\" => \"shopping list\",\n    \"todo_items\" => %{\n      0 => %{\"description\" => \"bread\"},\n      1 => %{\"description\" => \"eggs\"}\n    }\n  }\n}\n```\n\nWe could then retrieve those parameters and pass it to an Ecto changeset and Ecto would automatically figure out what to do:\n\n```elixir\n# In MyApp.TodoList\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:title])\n  |> Ecto.Changeset.cast_assoc(:todo_items, required: true)\nend\n\n# And then in MyApp.TodoItem\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:description])\nend\n```\n\nBy calling `Ecto.Changeset.cast_assoc/3`, Ecto will look for a \"todo_items\" key inside the parameters given on cast, and compare those parameters with the items stored in the todo list struct. Ecto will automatically generate instructions to insert, update or delete todo items such that:\n\n  * if a todo item sent as parameter has an ID and it matches an existing associated todo item, we consider that todo item should be updated\n  * if a todo item sent as parameter does not have an ID (nor a matching ID), we consider that todo item should be inserted\n  * if a todo item is currently associated but its ID was not sent as parameter, we consider the todo item is being replaced and we act according to the `:on_replace` callback. By default `:on_replace` will raise so you choose a behaviour between replacing, deleting, ignoring or nilifying the association\n\nThe advantage of using `cast_assoc/3` is that Ecto is able to do all of the hard work of keeping the entries associated, **as long as we pass the data exactly in the format that Ecto expects**. However, such approach is not always preferable and in many situations it is better to design our associations differently or decouple our UIs from our database representation.","ref":"polymorphic-associations-with-many-to-many.html#todo-lists-v65131"},{"type":"extras","title":"Polymorphic todo items - Polymorphic associations with many to many","doc":"To show an example of where using `cast_assoc/3` is just too complicated to be worth it, let's imagine you want your \"todo items\" to be polymorphic. For example, you want to be able to add todo items not only to \"todo lists\" but to many other parts of your application, such as projects, milestones, you name it.\n\nFirst of all, it is important to remember Ecto does not provide the same type of polymorphic associations available in frameworks such as Rails and Laravel. In such frameworks, a polymorphic association uses two columns, the `parent_id` and `parent_type`. For example, one todo item would have `parent_id` of 1 with `parent_type` of \"TodoList\" while another would have `parent_id` of 1 with `parent_type` of \"Project\".\n\nThe issue with the design above is that it breaks database references. The database is no longer capable of guaranteeing the item you associate to exists or will continue to exist in the future. This leads to an inconsistent database which end-up pushing workarounds to your application.\n\nThe design above is also extremely inefficient, especially if you're working with large tables. Bear in mind that if that's your case, you might be forced to remove such polymorphic references in the future when frequent polymorphic queries start grinding the database to a halt even after adding indexes and optimizing the database.\n\nLuckily, the documentation for the `Ecto.Schema.belongs_to/3` macro includes a section named \"Polymorphic associations\" with some examples on how to design sane and performant associations. One of those approaches consists in using several join tables. Besides the \"todo_lists\" and \"projects\" tables and the \"todo_items\" table, we would create \"todo_list_items\" and \"project_items\" to associate todo items to todo lists and todo items to projects respectively. In terms of migrations, we are looking at the following:\n\n```elixir\ncreate table(:todo_lists)  do\n  add :title\n  timestamps()\nend\n\ncreate table(:projects)  do\n  add :name\n  timestamps()\nend\n\ncreate table(:todo_items)  do\n  add :description\n  timestamps()\nend\n\ncreate table(:todo_list_items) do\n  add :todo_item_id, references(:todo_items)\n  add :todo_list_id, references(:todo_lists)\n  timestamps()\nend\n\ncreate table(:project_items) do\n  add :todo_item_id, references(:todo_items)\n  add :project_id, references(:projects)\n  timestamps()\nend\n```\n\nBy adding one table per association pair, we keep database references and can efficiently perform queries that relies on indexes.\n\nFirst let's see how to implement this functionality in Ecto using a `has_many :through` and then use `many_to_many` to remove a lot of the boilerplate we were forced to introduce.","ref":"polymorphic-associations-with-many-to-many.html#polymorphic-todo-items"},{"type":"extras","title":"Polymorphism with has_many :through - Polymorphic associations with many to many","doc":"Given we want our todo items to be polymorphic, we can no longer associate a todo list to todo items directly. Instead we will create an intermediate schema to tie `MyApp.TodoList` and `MyApp.TodoItem` together.\n\n```elixir\ndefmodule MyApp.TodoList do\n  use Ecto.Schema\n\n  schema \"todo_lists\" do\n    field :title\n    has_many :todo_list_items, MyApp.TodoListItem\n    has_many :todo_items,\n      through: [:todo_list_items, :todo_item]\n    timestamps()\n  end\nend\n\ndefmodule MyApp.TodoListItem do\n  use Ecto.Schema\n\n  schema \"todo_list_items\" do\n    belongs_to :todo_list, MyApp.TodoList\n    belongs_to :todo_item, MyApp.TodoItem\n    timestamps()\n  end\nend\n\ndefmodule MyApp.TodoItem do\n  use Ecto.Schema\n\n  schema \"todo_items\" do\n    field :description\n    timestamps()\n  end\nend\n```\n\nAlthough we introduced `MyApp.TodoListItem` as an intermediate schema, `has_many :through` allows us to access all todo items for any todo list transparently:\n\n```elixir\ntodo_lists |> Repo.preload(:todo_items)\n```\n\nThe trouble is that `:through` associations are **read-only** since Ecto does not have enough information to fill in the intermediate schema. This means that, if we still want to use `cast_assoc` to insert a todo list with many todo items directly from the UI, we cannot use the `:through` association and instead must go step by step. We would need to first `cast_assoc(:todo_list_items)` from `TodoList` and then call `cast_assoc(:todo_item)` from the `TodoListItem` schema:\n\n```elixir\n# In MyApp.TodoList\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:title])\n  |> Ecto.Changeset.cast_assoc(\n    :todo_list_items,\n    required: true\n  )\nend\n\n# And then in the MyApp.TodoListItem\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast_assoc(:todo_item, required: true)\nend\n\n# And then in MyApp.TodoItem\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:description])\nend\n```\n\nTo further complicate things, remember `cast_assoc` expects a particular shape of data that reflects your associations. In this case, because of the intermediate schema, the data sent through your forms in Phoenix would have to look as follows:\n\n```elixir\n%{\"todo_list\" => %{\n  \"title\" => \"shipping list\",\n  \"todo_list_items\" => %{\n    0 => %{\"todo_item\" => %{\"description\" => \"bread\"}},\n    1 => %{\"todo_item\" => %{\"description\" => \"eggs\"}},\n  }\n}}\n```\n\nTo make matters worse, you would have to duplicate this logic for every intermediate schema, and introduce `MyApp.TodoListItem` for todo lists, `MyApp.ProjectItem` for projects, etc.\n\nLuckily, `many_to_many` allows us to remove all of this boilerplate.","ref":"polymorphic-associations-with-many-to-many.html#polymorphism-with-has_many-through"},{"type":"extras","title":"Polymorphism with many_to_many - Polymorphic associations with many to many","doc":"In a way, the idea behind `many_to_many` associations is that it allows us to associate two schemas via an intermediate schema while automatically taking care of all details about the intermediate schema. Let's rewrite the schemas above to use `many_to_many`:\n\n```elixir\ndefmodule MyApp.TodoList do\n  use Ecto.Schema\n\n  schema \"todo_lists\" do\n    field :title\n    many_to_many :todo_items, MyApp.TodoItem,\n      join_through: MyApp.TodoListItem\n    timestamps()\n  end\nend\n\ndefmodule MyApp.TodoListItem do\n  use Ecto.Schema\n\n  schema \"todo_list_items\" do\n    belongs_to :todo_list, MyApp.TodoList\n    belongs_to :todo_item, MyApp.TodoItem\n    timestamps()\n  end\nend\n\ndefmodule MyApp.TodoItem do\n  use Ecto.Schema\n\n  schema \"todo_items\" do\n    field :description\n    timestamps()\n  end\nend\n```\n\nNotice `MyApp.TodoList` no longer needs to define a `has_many` association pointing to the `MyApp.TodoListItem` schema and instead we can just associate to `:todo_items` using `many_to_many`.\n\nDifferently from `has_many :through`, `many_to_many` associations are also writable. This means we can send data through our forms exactly as we did at the beginning of this guide:\n\n```elixir\n%{\"todo_list\" => %{\n  \"title\" => \"shipping list\",\n  \"todo_items\" => %{\n    0 => %{\"description\" => \"bread\"},\n    1 => %{\"description\" => \"eggs\"},\n  }\n}}\n```\n\nAnd we no longer need to define a changeset function in the intermediate schema:\n\n```elixir\n# In MyApp.TodoList\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:title])\n  |> Ecto.Changeset.cast_assoc(:todo_items, required: true)\nend\n\n# And then in MyApp.TodoItem\ndef changeset(struct, params \\\\ %{}) do\n  struct\n  |> Ecto.Changeset.cast(params, [:description])\nend\n```\n\nIn other words, we can use exactly the same code we had in the \"todo lists has_many todo items\" case. So even when external constraints require us to use a join table, `many_to_many` associations can automatically manage them for us. Everything you know about associations will just work with `many_to_many` associations as well.\n\nFinally, even though we have specified a schema as the `:join_through` option in `many_to_many`, `many_to_many` can also work without intermediate schemas altogether by simply giving it a table name:\n\n```elixir\ndefmodule MyApp.TodoList do\n  use Ecto.Schema\n\n  schema \"todo_lists\" do\n    field :title\n    many_to_many :todo_items, MyApp.TodoItem,\n      join_through: \"todo_list_items\"\n    timestamps()\n  end\nend\n```\n\nIn this case, you can completely remove the `MyApp.TodoListItem` schema from your application and the code above will still work. The only difference is that when using tables, any autogenerated value that is filled by Ecto schema, such as timestamps, won't be filled as we no longer have a schema. To solve this, you can either drop those fields from your migrations or set a default at the database level.","ref":"polymorphic-associations-with-many-to-many.html#polymorphism-with-many_to_many"},{"type":"extras","title":"Summary - Polymorphic associations with many to many","doc":"In this guide we used `many_to_many` associations to drastically improve a polymorphic association design that relied on `has_many :through`. Our goal was to allow \"todo_items\" to associate to different entities in our code base, such as \"todo_lists\" and \"projects\". We have done this by creating intermediate tables and by using `many_to_many` associations to automatically manage those join tables.\n\nAt the end, our schemas may look like:\n\n```elixir\ndefmodule MyApp.TodoList do\n  use Ecto.Schema\n\n  schema \"todo_lists\" do\n    field :title\n    many_to_many :todo_items, MyApp.TodoItem,\n      join_through: \"todo_list_items\"\n    timestamps()\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:title])\n    |> Ecto.Changeset.cast_assoc(\n      :todo_items,\n      required: true\n    )\n  end\nend\n\ndefmodule MyApp.Project do\n  use Ecto.Schema\n\n  schema \"projects\" do\n    field :name\n    many_to_many :todo_items, MyApp.TodoItem,\n      join_through: \"project_items\"\n    timestamps()\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:name])\n    |> Ecto.Changeset.cast_assoc(\n      :todo_items,\n      required: true\n    )\n  end\nend\n\ndefmodule MyApp.TodoItem do\n  use Ecto.Schema\n\n  schema \"todo_items\" do\n    field :description\n    timestamps()\n  end\n\n  def changeset(struct, params \\\\ %{}) do\n    struct\n    |> Ecto.Changeset.cast(params, [:description])\n  end\nend\n```\n\nAnd the database migration:\n\n```elixir\ncreate table(\"todo_lists\")  do\n  add :title\n  timestamps()\nend\n\ncreate table(\"projects\")  do\n  add :name\n  timestamps()\nend\n\ncreate table(\"todo_items\")  do\n  add :description\n  timestamps()\nend\n\n# Primary key and timestamps are not required if\n# using many_to_many without schemas\ncreate table(\"todo_list_items\", primary_key: false) do\n  add :todo_item_id, references(:todo_items)\n  add :todo_list_id, references(:todo_lists)\n  # timestamps()\nend\n\n# Primary key and timestamps are not required if\n# using many_to_many without schemas\ncreate table(\"project_items\", primary_key: false) do\n  add :todo_item_id, references(:todo_items)\n  add :project_id, references(:projects)\n  # timestamps()\nend\n```\n\nOverall our code looks structurally the same as `has_many` would, although at the database level our relationships are expressed with join tables.\n\nWhile in this guide we changed our code to cope with the parameter format required by `cast_assoc`, in [Constraints and Upserts](Constraints and Upserts.md) we drop `cast_assoc` altogether and use `put_assoc` which brings more flexibilities when working with associations.","ref":"polymorphic-associations-with-many-to-many.html#summary"},{"type":"extras","title":"Replicas and dynamic repositories","doc":"# Replicas and dynamic repositories\n\nWhen applications reach a certain scale, a single database may not be enough to sustain the required throughput. In such scenarios, it is very common to introduce read replicas: all write operations are sent to the primary database and most of the read operations are performed against the replicas. The credentials of the primary and replica databases are typically known upfront by the time the code is compiled.\n\nIn other cases, you may need a single Ecto repository to interact with different database instances which are not known upfront. For instance, you may need to communicate with hundreds of databases very sporadically, so instead of opening up a connection to each of those hundreds of databases when your application starts, you want to quickly start a connection, perform some queries, and then shut down, while still leveraging Ecto's APIs as a whole.\n\nThis guide will cover how to tackle both approaches.","ref":"replicas-and-dynamic-repositories.html"},{"type":"extras","title":"Primary and Replicas - Replicas and dynamic repositories","doc":"Since the credentials of the primary and replicas databases are known upfront, adding support for primary and replica databases in your Ecto application is relatively straightforward. Imagine you have a `MyApp.Repo` and you want to add four read replicas. This could be done in three steps.\n\nFirst, define the primary and replicas repositories in `lib/my_app/repo.ex`:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo,\n    otp_app: :my_app,\n    adapter: Ecto.Adapters.Postgres\n\n  @replicas [\n    MyApp.Repo.Replica1,\n    MyApp.Repo.Replica2,\n    MyApp.Repo.Replica3,\n    MyApp.Repo.Replica4\n  ]\n\n  def replica do\n    Enum.random(@replicas)\n  end\n\n  for repo <- @replicas do\n    defmodule repo do\n      use Ecto.Repo,\n        otp_app: :my_app,\n        adapter: Ecto.Adapters.Postgres,\n        read_only: true\n    end\n  end\nend\n```\n\nThe code above defines a regular `MyApp.Repo` and four replicas, called `MyApp.Repo.Replica1` up to `MyApp.Repo.Replica4`. We pass the `:read_only` option to the replica repositories, so operations such as `insert`, `update` and friends are not made accessible. We also define a function called `replica` with the purpose of returning a random replica.\n\nNext we need to make sure both primary and replicas are configured properly in your `config/config.exs` files. In development and test, you can likely use the same database credentials for all repositories, all pointing to the same database address:\n\n```elixir\nreplicas = [\n  MyApp.Repo,\n  MyApp.Repo.Replica1,\n  MyApp.Repo.Replica2,\n  MyApp.Repo.Replica3,\n  MyApp.Repo.Replica4\n]\n\nfor repo <- replicas do\n  config :my_app, repo,\n    username: \"postgres\",\n    password: \"postgres\",\n    database: \"my_app_prod\",\n    hostname: \"localhost\",\n    pool_size: 10\nend\n```\n\nIn production, you want each database to connect to a different hostname:\n\n```elixir\nrepos = %{\n  MyApp.Repo => \"prod-primary\",\n  MyApp.Repo.Replica1 => \"prod-replica-1\",\n  MyApp.Repo.Replica2 => \"prod-replica-2\",\n  MyApp.Repo.Replica3 => \"prod-replica-3\",\n  MyApp.Repo.Replica4 => \"prod-replica-4\"\n}\n\nfor {repo, hostname} <- repos do\n  config :my_app, repo,\n    username: \"postgres\",\n    password: \"postgres\",\n    database: \"my_app_prod\",\n    hostname: hostname,\n    pool_size: 10\nend\n```\n\nFinally, make sure to start all repositories in your supervision tree:\n\n```elixir\nchildren = [\n  MyApp.Repo,\n  MyApp.Repo.Replica1,\n  MyApp.Repo.Replica2,\n  MyApp.Repo.Replica3,\n  MyApp.Repo.Replica4\n]\n```\n\nNow that all repositories are configured, we can safely use them in your application code. Every time you are performing a read operation, you can call the `replica/0` function that we have added to return a random replica we will send the query to:\n\n```elixir\nMyApp.Repo.replica().all(query)\n```\n\nAnd now you are ready to work with primary and replicas, no hacks or complex dependencies required!","ref":"replicas-and-dynamic-repositories.html#primary-and-replicas"},{"type":"extras","title":"Testing replicas - Replicas and dynamic repositories","doc":"While all of the work we have done so far should fully work in development and production, it may not be enough for tests. Most developers testing Ecto applications are using a sandbox, such as the [Ecto SQL Sandbox](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html).\n\nWhen using a sandbox, each of your tests run in an isolated and independent transaction. Once the test is done, the transaction is rolled back. Which means we can trivially revert all of the changes done in a test in a very performant way.\n\nUnfortunately, even if you configure your primary and replicas to have the same credentials and point to the same hostname, each Ecto repository will open up their own pool of database connections. This means that, once you move to a primary + replicas setup, a simple test like this one won't pass:\n\n```elixir\nuser = Repo.insert!(%User{name: \"jane doe\"})\nassert Repo.replica().get!(User, user.id)\n```\n\nThat's because `Repo.insert!` will write to one database connection and the repository returned by `Repo.replica()` will perform the read in another connection. Since the write is done in a transaction, its contents won't be available to other connections until the transaction commits, which will never happen for test connections.\n\nThere are two options to tackle this problem: one is to change replicas and the other is to use dynamic repos.\n\n#","ref":"replicas-and-dynamic-repositories.html#testing-replicas"},{"type":"extras","title":"A custom `replica` definition - Replicas and dynamic repositories","doc":"One simple solution to the problem above is to use a custom `replica` implementation during tests that always return the primary repository, like this:\n\n```elixir\nif Mix.env() == :test do\n  def replica, do: __MODULE__\nelse\n  def replica, do: Enum.random(@replicas)\nend\n```\n\nNow during tests, the replica will always return the repository primary repository itself. While this approach works fine, it has the downside that, if you accidentally invoke a write function in a replica, the test will pass, since the `replica` function is returning the primary repo, while the code will fail in production.\n\n#","ref":"replicas-and-dynamic-repositories.html#a-custom-replica-definition"},{"type":"extras","title":"Using `:default_dynamic_repo` - Replicas and dynamic repositories","doc":"Another approach to testing is to set the `:default_dynamic_repo` option when defining the repository. Let's see what we mean by that.\n\nWhen you list a repository in your supervision tree, such as `MyApp.Repo`, behind the scenes it will start a supervision tree with a process named `MyApp.Repo`. By default, the process has the same name as the repository module itself. Now every time you invoke a function in `MyApp.Repo`, such as `MyApp.Repo.insert/2`, Ecto will use the connection pool from the process named `MyApp.Repo`.\n\nFrom v3.0, Ecto has the ability to start multiple processes from the same repository. The only requirement is that they must have different process names, like this:\n\n```elixir\nchildren = [\n  MyApp.Repo,\n  {MyApp.Repo, name: :another_instance_of_repo}\n]\n```\n\nWhile the particular example doesn't make much sense (we will cover an actual use case for this feature next), the idea is that now you have two repositories running: one is named `MyApp.Repo` and the other one is named `:another_instance_of_repo`. Each of those processes have their own connection pool. You can tell Ecto which process you want to use in your repo operations by calling:\n\n```elixir\nMyApp.Repo.put_dynamic_repo(MyApp.Repo)\nMyApp.Repo.put_dynamic_repo(:another_instance_of_repo)\n```\n\nOnce you call `MyApp.Repo.put_dynamic_repo(name)`, all invocations made on `MyApp.Repo` will use the connection pool denoted by `name`.\n\nHow can this help with our replica tests? If we look back to the supervision tree we defined earlier in this guide, you will find this:\n\n```elixir\nchildren = [\n  MyApp.Repo,\n  MyApp.Repo.Replica1,\n  MyApp.Repo.Replica2,\n  MyApp.Repo.Replica3,\n  MyApp.Repo.Replica4\n]\n```\n\nWe are starting five different repositories and five different connection pools. Since we want the replica repositories to use the `MyApp.Repo`, we can achieve this by doing the following on the setup of each test:\n\n```elixir\n@replicas [\n  MyApp.Repo.Replica1,\n  MyApp.Repo.Replica2,\n  MyApp.Repo.Replica3,\n  MyApp.Repo.Replica4\n]\n\nsetup do\n  for replica <- @replicas do\n    replica.put_dynamic_repo(MyApp.Repo)\n  end\n\n  :ok\nend\n```\n\nNote `put_dynamic_repo` is per process. So every time you spawn a new process, the `dynamic_repo` value will reset to its default until you call `put_dynamic_repo` again.\n\nLuckily, there is even a better way! We can pass a `:default_dynamic_repo` option when we define the repository. In this case, we want to set the `:default_dynamic_repo` to `MyApp.Repo` only during the test environment. In your `lib/my_app/repo.ex`, do this:\n\n```elixir\n  for repo <- @replicas do\n    default_dynamic_repo =\n      if Mix.env() == :test do\n        MyApp.Repo\n      else\n        repo\n      end\n\n    defmodule repo do\n      use Ecto.Repo,\n        otp_app: :my_app,\n        adapter: Ecto.Adapters.Postgres,\n        read_only: true,\n        default_dynamic_repo: default_dynamic_repo\n    end\n  end\n```\n\nAnd now your tests should work as before, while still being able to detect if you accidentally perform a write operation in a replica.","ref":"replicas-and-dynamic-repositories.html#using-default_dynamic_repo"},{"type":"extras","title":"Dynamic repositories - Replicas and dynamic repositories","doc":"At this point, we have learned that Ecto allows you to start multiple connections based on the same repository. This is typically useful when you have to connect multiple databases or perform short-lived database connections.\n\nFor example, you can start a repository with a given set of credentials dynamically, like this:\n\n```elixir\nMyApp.Repo.start_link(\n  name: :some_client,\n  hostname: \"client.example.com\",\n  username: \"...\",\n  password: \"...\",\n  pool_size: 1\n)\n```\n\nIn other words, `start_link` accepts the same options as the database configuration. Now let's do a query on the dynamically started repository. If you attempt to simply perform `MyApp.Repo.all(Post)`, it may fail, as by default it will try to use a process named `MyApp.Repo`, which may or may not be running. So don't forget to call `put_dynamic_repo/1` before:\n\n```elixir\nMyApp.Repo.put_dynamic_repo(:some_client)\nMyApp.Repo.all(Post)\n```\n\nEcto also allows you to start a repository with no name (just like that famous horse). In such cases, you need to explicitly pass `name: nil` and match on the result of `MyApp.Repo.start_link/1` to retrieve the PID, which should be given to `put_dynamic_repo`. Let's also use this opportunity and perform proper database clean-up, by shutting up the new repository and reverting the value of `put_dynamic_repo`:\n\n```elixir\ndefault_dynamic_repo = MyApp.Repo.get_dynamic_repo()\n\n{:ok, repo} =\n  MyApp.Repo.start_link(\n    name: nil,\n    hostname: \"client.example.com\",\n    username: \"...\",\n    password: \"...\",\n    pool_size: 1\n  )\n\ntry do\n  MyApp.Repo.put_dynamic_repo(repo)\n  MyApp.Repo.all(Post)\nafter\n  MyApp.Repo.put_dynamic_repo(default_dynamic_repo)\n  Supervisor.stop(repo)\nend\n```\n\nWe can encapsulate all of this in a function too, which you could define in your repository:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, ...\n\n  def with_dynamic_repo(credentials, callback) do\n    default_dynamic_repo = get_dynamic_repo()\n    start_opts = [name: nil, pool_size: 1] ++ credentials\n    {:ok, repo} = MyApp.Repo.start_link(start_opts)\n\n    try do\n      MyApp.Repo.put_dynamic_repo(repo)\n      callback.()\n    after\n      MyApp.Repo.put_dynamic_repo(default_dynamic_repo)\n      Supervisor.stop(repo)\n    end\n  end\nend\n```\n\nAnd now use it as:\n\n```elixir\ncredentials = [\n  hostname: \"client.example.com\",\n  username: \"...\",\n  password: \"...\"\n]\n\nMyApp.Repo.with_dynamic_repo(credentials, fn ->\n  MyApp.Repo.all(Post)\nend)\n```\n\nAnd that's it! Now you can have dynamic connections, all properly encapsulated in a single function and built on top of the dynamic repo API.","ref":"replicas-and-dynamic-repositories.html#dynamic-repositories"},{"type":"extras","title":"Schemaless queries","doc":"# Schemaless queries\n\nMost queries in Ecto are written using schemas. For example, to retrieve all posts in a database, one may write:\n\n```elixir\nMyApp.Repo.all(Post)\n```\n\nIn the construct above, Ecto knows all fields and their types in the schema, rewriting the query above to:\n\n```elixir\nquery =\n  from p in Post,\n    select: %Post{title: p.title, body: p.body, ...}\n\nMyApp.Repo.all(query)\n```\n\nAlthough you might use schemas for most of your queries, Ecto also adds the ability to write regular schemaless queries when preferred.\n\nOne example is this ability to select all desired fields without duplication:\n\n```elixir\nfrom \"posts\", select: [:title, :body]\n```\n\nWhen a list of fields is given, Ecto will automatically convert the list of fields to a map or a struct.\n\nSupport for passing a list of fields or keyword lists is available to almost all query constructs. For example, we can use an update query to change the title of a given post without a schema:\n\n```elixir\ndef update_title(post, new_title) do\n  query =\n    from \"posts\",\n      where: [id: ^post.id],\n      update: [set: [title: ^new_title]]\n\n  MyApp.Repo.update_all(query, [])\nend\n```\n\nThe `Ecto.Query.update/3` construct supports four commands:\n\n  * `:set` - sets the given column to the given values\n  * `:inc` - increments the given column by the given value\n  * `:push` - pushes (appends) the given value to the end of an array column\n  * `:pull` - pulls (removes) the given value from an array column\n\nFor example, we can increment a column atomically by using the `:inc` command, with or without schemas:\n\n```elixir\ndef increment_page_views(post) do\n  query =\n    from \"posts\",\n      where: [id: ^post.id],\n      update: [inc: [page_views: 1]]\n\n  MyApp.Repo.update_all(query, [])\nend\n```\n\nLet's take a look at another example. Imagine you are writing a reporting view, it may be counter-productive to think how your existing application schemas relate to the report being generated. It is often simpler to write a query that returns only the data you need, without trying to fit the data into existing schemas:\n\n```elixir\nimport Ecto.Query\n\ndef running_activities(start_at, end_at) do\n  query =\n    from u in \"users\",\n      join: a in \"activities\",\n      on: a.user_id == u.id,\n      where:\n        a.start_at > type(^start_at, :naive_datetime) and\n          a.end_at < type(^end_at, :naive_datetime),\n      group_by: a.user_id,\n      select: %{\n        user_id: a.user_id,\n        interval: a.end_at - a.start_at,\n        count: count(u.id)\n      }\n\n  MyApp.Repo.all(query)\nend\n```\n\nThe function above does not rely on schemas. It returns only the data that matters for building the report. Notice how we use the `type/2` function to specify what is the expected type of the argument we are interpolating, benefiting from the same type casting guarantees a schema would give.\n\nBy allowing regular data structures to be given to most query operations, Ecto makes queries with and without schemas more accessible. Not only that, it also enables developers to write dynamic queries, where fields, filters, ordering cannot be specified upfront.","ref":"schemaless-queries.html"},{"type":"extras","title":"insert_all, update_all and delete_all - Schemaless queries","doc":"Ecto allows all database operations to be expressed without a schema. One of the functions provided is `c:Ecto.Repo.insert_all/3`. With `insert_all`, developers can insert multiple entries at once into a repository using the source and a list of fields and values to be passed directly to the adapter:\n\n```elixir\nMyApp.Repo.insert_all(\n  \"posts\",\n  [\n    [title: \"hello\", body: \"world\"],\n    [title: \"another\", body: \"post\"]\n  ]\n)\n```\n\nUpdates and deletes can also be done without schemas via `c:Ecto.Repo.update_all/3` and `c:Ecto.Repo.delete_all/2` respectively:\n\n```elixir\n# Use the ID to trigger updates\npost = from p in \"posts\", where: [id: ^id]\n\n# Update the title for all matching posts\n{1, _} =\n  MyApp.Repo.update_all post, set: [title: \"new title\"]\n\n# Delete all matching posts\n{1, _} =\n  MyApp.Repo.delete_all post\n```\n\nIt is not hard to see how these operations directly map to their SQL variants, keeping the database at your fingertips without the need to intermediate all operations through schemas.","ref":"schemaless-queries.html#insert_all-update_all-and-delete_all"},{"type":"extras","title":"Test factories","doc":"# Test factories\n\nMany projects depend on external libraries to build their test data. Some of those libraries are called factories because they provide convenience functions for producing different groups of data. However, given Ecto is able to manage complex data trees, we can implement such functionality without relying on third-party projects.\n\nTo get started, let's create a file at \"test/support/factory.ex\" with the following contents:\n\n```elixir\ndefmodule MyApp.Factory do\n  alias MyApp.Repo\n\n  # Factories\n\n  def build(:post) do\n    %MyApp.Post{title: \"hello world\"}\n  end\n\n  def build(:comment) do\n    %MyApp.Comment{body: \"good post\"}\n  end\n\n  def build(:post_with_comments) do\n    %MyApp.Post{\n      title: \"hello with comments\",\n      comments: [\n        build(:comment, body: \"first\"),\n        build(:comment, body: \"second\")\n      ]\n    }\n  end\n\n  def build(:user) do\n    %MyApp.User{\n      email: \"hello#{System.unique_integer()}\",\n      username: \"hello#{System.unique_integer()}\"\n    }\n  end\n\n  # Convenience API\n\n  def build(factory_name, attributes) do\n    factory_name |> build() |> struct!(attributes)\n  end\n\n  def insert!(factory_name, attributes \\\\ []) do\n    factory_name |> build(attributes) |> Repo.insert!()\n  end\nend\n```\n\nOur factory module defines four \"factories\" as different clauses to the build function: `:post`, `:comment`, `:post_with_comments` and `:user`. Each clause defines structs with the fields that are required by the database. In certain cases, the generated struct also needs to generate unique fields, such as the user's email and username. We did so by calling Elixir's `System.unique_integer()` - you could call `System.unique_integer([:positive])` if you need a strictly positive number.\n\nAt the end, we defined two functions, `build/2` and `insert!/2`, which are conveniences for building structs with specific attributes and for inserting data directly in the repository respectively.\n\nThat's literally all that is necessary for building our factories. We are now ready to use them in our tests. First, open up your \"mix.exs\" and make sure the \"test/support/factory.ex\" file is compiled:\n\n```elixir\ndef project do\n  [...,\n   elixirc_paths: elixirc_paths(Mix.env),\n   ...]\nend\n\ndefp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\ndefp elixirc_paths(_), do: [\"lib\"]\n```\n\nNow in any of the tests that need to generate data, we can import the `MyApp.Factory` module and use its functions:\n\n```elixir\nimport MyApp.Factory\n\nbuild(:post)\n#=> %MyApp.Post{id: nil, title: \"hello world\", ...}\n\nbuild(:post, title: \"custom title\")\n#=> %MyApp.Post{id: nil, title: \"custom title\", ...}\n\ninsert!(:post, title: \"custom title\")\n#=> %MyApp.Post{id: ..., title: \"custom title\"}\n```\n\nBy building the functionality we need on top of Ecto capabilities, we are able to extend and improve our factories on whatever way we desire, without being constrained to third-party limitations.","ref":"test-factories.html"}],"content_type":"text/markdown"}